MODULE YOM_YGFL

USE PARKIND1  ,ONLY : JPIM
USE TYPE_GFLS ,ONLY : TYPE_GFLD,TYPE_GFL_COMP,TYPE_GFL_NAML

IMPLICIT NONE
SAVE

!-------------------------------------------------------------------------
! Contains the descriptors of GFL arrays

! YGFL  : GFL general descriptor, for info about content see comments
!         in type declaration module : type_gfls.F90

! JPGFL : Max number of GFL fields
! JPNAMED_GFL : Number of currently pre-defined components of GFL
! JPGHG : Number of greenhouse gas fields
! JPGRG : Number of reactive gas fields
! JPAERO : Number of active aerosol fields
! JPAEROUT: Number of output aerosol fields
! JPUVP : Number of output from UV processor
! JPTRAC : Number of tracers for diagnostics
! JPERA40 : Number of ERA40 diagnostic fields
! JPCH4S  : Number of added fields related to methane
! JPNOGW  : Number of diagnostic fields for NORO GWD SCHEME
! JPSLDIA : Number of SL dynamics diagnostic fields
!-------------------------------------------------------------------------

INTEGER(KIND=JPIM),PARAMETER :: JPGFL=450
INTEGER(KIND=JPIM),PARAMETER :: JPNAMED_GFL=27
INTEGER(KIND=JPIM),PARAMETER :: JPGHG=3
INTEGER(KIND=JPIM),PARAMETER :: JPTRAC=10
INTEGER(KIND=JPIM),PARAMETER :: JPGRG=5       ! naj data assimilation code still assumes JPGRG=5 , to be replaced with JPGRGVAR
INTEGER(KIND=JPIM),PARAMETER :: JPGRGVAR=5
INTEGER(KIND=JPIM),PARAMETER :: JPAERO=15
INTEGER(KIND=JPIM),PARAMETER :: JPFORC=180
INTEGER(KIND=JPIM),PARAMETER :: JPERA40=14
INTEGER(KIND=JPIM),PARAMETER :: JPSLDIA=7
INTEGER(KIND=JPIM),PARAMETER :: JPEZDIAG=50
INTEGER(KIND=JPIM),PARAMETER :: JPCH4S=2
INTEGER(KIND=JPIM),PARAMETER :: JPNOGW=2
INTEGER(KIND=JPIM),PARAMETER :: JPAEROUT=8
INTEGER(KIND=JPIM),PARAMETER :: JPUVP=2
INTEGER(KIND=JPIM) :: NGFL_EXT
INTEGER(KIND=JPIM) :: NGFL_FORC
INTEGER(KIND=JPIM) :: NGFL_EZDIAG
INTEGER(KIND=JPIM) :: NGHG
INTEGER(KIND=JPIM) :: NTRAC
INTEGER(KIND=JPIM) :: NGRG
INTEGER(KIND=JPIM) :: NGRG_CPLO4
INTEGER(KIND=JPIM) :: NGRG_ASSIM
INTEGER(KIND=JPIM) :: NAERO
INTEGER(KIND=JPIM) :: NACTAERO
INTEGER(KIND=JPIM) :: NDDHAERO
INTEGER(KIND=JPIM) :: NERA40
INTEGER(KIND=JPIM) :: NNOGW
INTEGER(KIND=JPIM) :: NAEROUT
INTEGER(KIND=JPIM) :: NUVP
INTEGER(KIND=JPIM) :: NSLDIA
INTEGER(KIND=JPIM) :: NSLDIAGP
LOGICAL :: LCO2SFC
LOGICAL :: LCH4SFC
LOGICAL :: LAEROSFC
LOGICAL :: LSF6SFC
LOGICAL :: LFIRE
LOGICAL :: LAERODIU
LOGICAL :: LGRGSFC
LOGICAL :: LTRCMFIX
LOGICAL :: LAEROUT
LOGICAL :: LUVPOUT

TYPE(TYPE_GFLD) :: YGFL
TYPE(TYPE_GFL_COMP), TARGET  :: YGFLC(JPGFL)    ! General descriptor of all components

TYPE(TYPE_GFL_COMP),POINTER  :: YQ          => NULL() ! Specific humidity
TYPE(TYPE_GFL_COMP),POINTER  :: YI          => NULL() ! Ice water
TYPE(TYPE_GFL_COMP),POINTER  :: YL          => NULL() ! Liquid water
TYPE(TYPE_GFL_COMP),POINTER  :: YLCONV      => NULL() ! Liquid water (CONV. PART)
TYPE(TYPE_GFL_COMP),POINTER  :: YICONV      => NULL() ! Ice    water (CONV. PART)
TYPE(TYPE_GFL_COMP),POINTER  :: YRCONV      => NULL() ! Rain         (CONV. PART)
TYPE(TYPE_GFL_COMP),POINTER  :: YSCONV      => NULL() ! Snow         (CONV. PART)
TYPE(TYPE_GFL_COMP),POINTER  :: YIRAD       => NULL() ! Radiative cloud Ice water
TYPE(TYPE_GFL_COMP),POINTER  :: YLRAD       => NULL() ! Radiative cloud Liquid water
TYPE(TYPE_GFL_COMP),POINTER  :: YS          => NULL() ! Snow
TYPE(TYPE_GFL_COMP),POINTER  :: YR          => NULL() ! Rain
TYPE(TYPE_GFL_COMP),POINTER  :: YG          => NULL() ! Graupel
TYPE(TYPE_GFL_COMP),POINTER  :: YH          => NULL() ! Hail
TYPE(TYPE_GFL_COMP),POINTER  :: YTKE        => NULL() ! Turbulent Kinetic Energy
TYPE(TYPE_GFL_COMP),POINTER  :: YTTE        => NULL() ! Turbulent Total Energy
TYPE(TYPE_GFL_COMP),POINTER  :: YMXL        => NULL() ! Prognostic mixing length
TYPE(TYPE_GFL_COMP),POINTER  :: YEFB1       => NULL() ! First variable EFB scheme
TYPE(TYPE_GFL_COMP),POINTER  :: YEFB2       => NULL() ! Second variable EFB scheme
TYPE(TYPE_GFL_COMP),POINTER  :: YEFB3       => NULL() ! Third variable EFB scheme
TYPE(TYPE_GFL_COMP),POINTER  :: YA          => NULL() ! Cloud fraction
TYPE(TYPE_GFL_COMP),POINTER  :: YO3         => NULL() ! Ozone
TYPE(TYPE_GFL_COMP),POINTER  :: YSRC        => NULL() ! Second-order flux for AROME
                                                      ! s'rc'/2Sigma_s2
                                                      ! multiplied by Lambda_3
TYPE(TYPE_GFL_COMP),POINTER  :: YCPF        => NULL() ! Convective precipitation flux
TYPE(TYPE_GFL_COMP),POINTER  :: YSPF        => NULL() ! Stratiform precipitation flux
TYPE(TYPE_GFL_COMP),POINTER  :: YCVGQ       => NULL() ! Moisture Convergence for french physics
TYPE(TYPE_GFL_COMP),POINTER  :: YQVA        => NULL() ! total humidity variation
TYPE(TYPE_GFL_COMP),POINTER  :: YGHG(:)     => NULL() ! Greenhouse Gases
TYPE(TYPE_GFL_COMP),POINTER  :: YGRG(:)     => NULL() ! Reactive Gases
TYPE(TYPE_GFL_COMP),POINTER  :: YGRGTEND(:) => NULL() ! Reactive Gases Tendecies
TYPE(TYPE_GFL_COMP),POINTER  :: YAERO(:)    => NULL() ! Aerosols
TYPE(TYPE_GFL_COMP),POINTER  :: YTRAC(:)    => NULL() ! tracers for diagnostics
TYPE(TYPE_GFL_COMP),POINTER  :: YLRCH4      => NULL() ! CH4 loss rate (instantaneous field)
TYPE(TYPE_GFL_COMP),POINTER  :: YCH4S       => NULL() ! CH4 atmospheric sink (accumulated field)
TYPE(TYPE_GFL_COMP),POINTER  :: YFORC(:)    => NULL() ! large scale forcing
TYPE(TYPE_GFL_COMP),POINTER  :: YEZDIAG(:)  => NULL() ! easy diagnostics
TYPE(TYPE_GFL_COMP),POINTER  :: YERA40(:)   => NULL() ! ERA40 diagnostic fields
TYPE(TYPE_GFL_COMP),POINTER  :: YNOGW(:)    => NULL() ! NORO GWD SCHEME
TYPE(TYPE_GFL_COMP),POINTER  :: YSLDIA(:)   => NULL() ! SL dynamics diagnostics
TYPE(TYPE_GFL_COMP),POINTER  :: YAEROUT(:)  => NULL() ! Aerosol outputs
TYPE(TYPE_GFL_COMP),POINTER  :: YUVP(:)     => NULL() ! UV-processor output

TYPE(TYPE_GFL_COMP),POINTER  :: YSDSAT      => NULL() ! Standard Deviation of the
                                                      ! SATuration Depression (Sigma_s)
TYPE(TYPE_GFL_COMP),POINTER  :: YCVV        => NULL() ! Convective Vertical Velocity
TYPE(TYPE_GFL_COMP),POINTER  :: YRKTH       => NULL() ! Rasch-Kristjansson H tendency
TYPE(TYPE_GFL_COMP),POINTER  :: YRKTQV      => NULL() ! Rasch-Kristjansson Qv tendency
TYPE(TYPE_GFL_COMP),POINTER  :: YRKTQC      => NULL() ! Rasch-Kristjansson Qc tendency

! Prognostic convection variables: add 6 named components
TYPE(TYPE_GFL_COMP),POINTER  :: YUOM        => NULL() ! Updraught vert velocity
TYPE(TYPE_GFL_COMP),POINTER  :: YUAL        => NULL() ! Updraught mesh fraction
TYPE(TYPE_GFL_COMP),POINTER  :: YDOM        => NULL() ! Downdraught vert velocity
TYPE(TYPE_GFL_COMP),POINTER  :: YDAL        => NULL() ! Downdraught mesh fraction
TYPE(TYPE_GFL_COMP),POINTER  :: YUEN        => NULL() ! Updraught entrainment
TYPE(TYPE_GFL_COMP),POINTER  :: YUNEBH      => NULL() ! pseudo-historic convective

! Extra fields

TYPE(TYPE_GFL_COMP),POINTER  :: YEXT(:)     => NULL() ! Extra fields

TYPE(TYPE_GFL_NAML)  :: YQ_NL                 ! Specific humidity
TYPE(TYPE_GFL_NAML)  :: YI_NL                 ! Ice water
TYPE(TYPE_GFL_NAML)  :: YL_NL                 ! Liquid water
TYPE(TYPE_GFL_NAML)  :: YLCONV_NL             ! Liquid water (CONV. PART)
TYPE(TYPE_GFL_NAML)  :: YICONV_NL             ! Ice    water (CONV. PART)
TYPE(TYPE_GFL_NAML)  :: YRCONV_NL             ! Rain         (CONV. PART)
TYPE(TYPE_GFL_NAML)  :: YSCONV_NL             ! Snow         (CONV. PART)
TYPE(TYPE_GFL_NAML)  :: YIRAD_NL              ! Radiative cloud Ice water
TYPE(TYPE_GFL_NAML)  :: YLRAD_NL              ! Radiative cloud Liquid water
TYPE(TYPE_GFL_NAML)  :: YS_NL                 ! Snow
TYPE(TYPE_GFL_NAML)  :: YR_NL                 ! Rain
TYPE(TYPE_GFL_NAML)  :: YG_NL                 ! Graupels
TYPE(TYPE_GFL_NAML)  :: YH_NL                 ! Hail
TYPE(TYPE_GFL_NAML)  :: YTKE_NL               ! Turbulent Kinetic Energy
TYPE(TYPE_GFL_NAML)  :: YTTE_NL               ! Turbulent Total Energy
TYPE(TYPE_GFL_NAML)  :: YMXL_NL               ! Prognostic mixing length
TYPE(TYPE_GFL_NAML)  :: YEFB1_NL              ! First variable EFB scheme
TYPE(TYPE_GFL_NAML)  :: YEFB2_NL              ! Second variable EFB scheme
TYPE(TYPE_GFL_NAML)  :: YEFB3_NL              ! Third variable EFB scheme
TYPE(TYPE_GFL_NAML)  :: YA_NL                 ! Cloud fraction
TYPE(TYPE_GFL_NAML)  :: YO3_NL                ! Ozone
TYPE(TYPE_GFL_NAML)  :: YSRC_NL               ! Second-order flux for AROME
                                              ! s'rc'/2Sigma_s2
                                              ! multiplied by Lambda_3
TYPE(TYPE_GFL_NAML)  :: YCPF_NL               ! Convective precipitation flux
TYPE(TYPE_GFL_NAML)  :: YSPF_NL               ! Stratiform precipitation flux
TYPE(TYPE_GFL_NAML)  :: YCVGQ_NL              ! Moisture Convergence for french physics
TYPE(TYPE_GFL_NAML)  :: YQVA_NL               ! Total humidity variation

TYPE(TYPE_GFL_NAML)  :: YGHG_NL(JPGHG)        ! Greenhouse Gases
TYPE(TYPE_GFL_NAML)  :: YGRG_NL(JPGRG)        ! Reactive Gases
TYPE(TYPE_GFL_NAML)  :: YGRGTEND_NL(JPGRG)    ! Reactive Gases Tendecies
TYPE(TYPE_GFL_NAML)  :: YAERO_NL(JPAERO)      ! Aerosol fields
TYPE(TYPE_GFL_NAML)  :: YTRAC_NL(JPTRAC)      ! Tracers for diagnostics
TYPE(TYPE_GFL_NAML)  :: YERA40_NL(JPERA40)    ! ERA40 diagnostic fields
TYPE(TYPE_GFL_NAML)  :: YNOGW_NL(JPNOGW)      ! NORO GWD SCHEME
TYPE(TYPE_GFL_NAML)  :: YSLDIA_NL(JPSLDIA)    ! SL dynamics diagnostics
TYPE(TYPE_GFL_NAML)  :: YLRCH4_NL             ! CH4 loss rate
TYPE(TYPE_GFL_NAML)  :: YCH4S_NL              ! CH4 atmospheric sink
TYPE(TYPE_GFL_NAML)  :: YAEROUT_NL(JPAEROUT)  ! Aerosol outputs
TYPE(TYPE_GFL_NAML)  :: YUVP_NL(JPUVP)        ! UV-processor outputs
TYPE(TYPE_GFL_NAML)  :: YRKTH_NL              ! Rasch-Kristjansson H tendency
TYPE(TYPE_GFL_NAML)  :: YRKTQV_NL             ! Rasch-Kristjansson Qv tendency
TYPE(TYPE_GFL_NAML)  :: YRKTQC_NL             ! Rasch-Kristjansson Qc tendency

! Extra fields

TYPE(TYPE_GFL_NAML)  :: YSDSAT_NL             ! Standard Deviation of the
                                              ! SATuration Depression (Sigma_s)
TYPE(TYPE_GFL_NAML)  :: YCVV_NL               ! Convective Vertical Velocity
TYPE(TYPE_GFL_NAML)  :: YFORC_NL(JPFORC)      ! Forcing precursor
TYPE(TYPE_GFL_NAML)  :: YEZDIAG_NL(JPEZDIAG)  ! Easy diagnostics
TYPE(TYPE_GFL_NAML)  :: YEXT_NL(JPGFL-JPNAMED_GFL-JPGHG-JPGRG-JPFORC-JPEZDIAG-JPAERO-JPTRAC-JPERA40-&
 &                              JPNOGW-JPSLDIA-JPCH4S-JPAEROUT-JPUVP) ! Extra fields

! Prognostic convection variables: 6 more namelist components
TYPE(TYPE_GFL_NAML)  :: YUOM_NL               ! Updraught vert velocity
TYPE(TYPE_GFL_NAML)  :: YUAL_NL               ! Updraught mesh fraction
TYPE(TYPE_GFL_NAML)  :: YDOM_NL               ! Downdraught vert velocity
TYPE(TYPE_GFL_NAML)  :: YDAL_NL               ! Downdraught mesh fraction
TYPE(TYPE_GFL_NAML)  :: YUEN_NL               ! Updraught entrainment
TYPE(TYPE_GFL_NAML)  :: YUNEBH_NL             ! Pseudi Hist Conv cloud fraction

!------------------------------------------------------------------

TYPE TYPE_YOM_YGFL
  PRIVATE
  INTEGER(KIND=JPIM) :: NGFL_EXT
  INTEGER(KIND=JPIM) :: NGFL_FORC
  INTEGER(KIND=JPIM) :: NGFL_EZDIAG
  INTEGER(KIND=JPIM) :: NGHG
  INTEGER(KIND=JPIM) :: NTRAC
  INTEGER(KIND=JPIM) :: NGRG
  INTEGER(KIND=JPIM) :: NGRG_CPLO4
  INTEGER(KIND=JPIM) :: NGRG_ASSIM
  INTEGER(KIND=JPIM) :: NAERO
  INTEGER(KIND=JPIM) :: NACTAERO
  INTEGER(KIND=JPIM) :: NDDHAERO
  INTEGER(KIND=JPIM) :: NERA40
  INTEGER(KIND=JPIM) :: NNOGW
  INTEGER(KIND=JPIM) :: NAEROUT
  INTEGER(KIND=JPIM) :: NUVP
  INTEGER(KIND=JPIM) :: NSLDIA
  INTEGER(KIND=JPIM) :: NSLDIAGP
  LOGICAL :: LCO2SFC
  LOGICAL :: LCH4SFC
  LOGICAL :: LAEROSFC
  LOGICAL :: LSF6SFC
  LOGICAL :: LFIRE
  LOGICAL :: LAERODIU
  LOGICAL :: LGRGSFC
  LOGICAL :: LTRCMFIX
  LOGICAL :: LAEROUT
  LOGICAL :: LUVPOUT

  TYPE(TYPE_GFLD) :: YGFL
  TYPE(TYPE_GFL_COMP) :: YGFLC(JPGFL)
  TYPE(TYPE_GFL_COMP),POINTER  :: YQ          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YI          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YL          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YIRAD       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YLRAD       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YS          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YR          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YG          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YH          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YTKE        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YTTE        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YMXL        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YEFB1       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YEFB2       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YEFB3       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YA          => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YO3         => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YSRC        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YCPF        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YSPF        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YCVGQ       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YQVA        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YGHG(:)     => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YGRG(:)     => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YGRGTEND(:) => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YAERO(:)    => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YTRAC(:)    => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YLRCH4      => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YCH4S       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YFORC(:)    => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YEZDIAG(:)  => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YERA40(:)   => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YNOGW(:)    => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YSLDIA(:)   => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YAEROUT(:)  => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YUVP(:)     => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YSDSAT      => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YCVV        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YRKTH       => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YRKTQV      => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YRKTQC      => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YUOM        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YUAL        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YDOM        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YDAL        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YUEN        => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YUNEBH      => NULL()
  TYPE(TYPE_GFL_COMP),POINTER  :: YEXT(:)     => NULL()
END TYPE TYPE_YOM_YGFL

!     -------------------------------------------------------------------------

CONTAINS

SUBROUTINE SET_YOM_YGFL(T)

  IMPLICIT NONE
  TYPE(TYPE_YOM_YGFL), INTENT(IN) :: T

  NGFL_EXT    =  T%NGFL_EXT
  NGFL_FORC   =  T%NGFL_FORC
  NGFL_EZDIAG =  T%NGFL_EZDIAG
  NGHG        =  T%NGHG
  NTRAC       =  T%NTRAC
  NGRG        =  T%NGRG
  NGRG_CPLO4  =  T%NGRG_CPLO4
  NGRG_ASSIM  =  T%NGRG_ASSIM
  NAERO       =  T%NAERO
  NACTAERO    =  T%NACTAERO
  NDDHAERO    =  T%NDDHAERO
  NERA40      =  T%NERA40
  NNOGW       =  T%NNOGW
  NAEROUT     =  T%NAEROUT
  NUVP        =  T%NUVP
  NSLDIA      =  T%NSLDIA
  NSLDIAGP    =  T%NSLDIAGP
  LCO2SFC     =  T%LCO2SFC
  LCH4SFC     =  T%LCH4SFC
  LAEROSFC    =  T%LAEROSFC
  LSF6SFC     =  T%LSF6SFC
  LFIRE       =  T%LFIRE
  LAERODIU    =  T%LAERODIU
  LGRGSFC     =  T%LGRGSFC
  LTRCMFIX    =  T%LTRCMFIX
  LAEROUT     =  T%LAEROUT
  LUVPOUT     =  T%LUVPOUT

  YGFL        =  T%YGFL
  YGFLC       =  T%YGFLC
  YQ          => T%YQ
  YI          => T%YI
  YL          => T%YL
  YIRAD       => T%YIRAD
  YLRAD       => T%YLRAD
  YS          => T%YS
  YR          => T%YR
  YG          => T%YG
  YH          => T%YH
  YTKE        => T%YTKE
  YTTE        => T%YTTE
  YMXL        => T%YMXL
  YEFB1       => T%YEFB1
  YEFB2       => T%YEFB2
  YEFB3       => T%YEFB3
  YA          => T%YA
  YO3         => T%YO3
  YSRC        => T%YSRC
  YCPF        => T%YCPF
  YSPF        => T%YSPF
  YCVGQ       => T%YCVGQ
  YQVA        => T%YQVA
  YGHG        => T%YGHG
  YGRG        => T%YGRG
  YGRGTEND    => T%YGRGTEND
  YAERO       => T%YAERO
  YTRAC       => T%YTRAC
  YLRCH4      => T%YLRCH4
  YCH4S       => T%YCH4S
  YFORC       => T%YFORC
  YEZDIAG     => T%YEZDIAG
  YERA40      => T%YERA40
  YNOGW       => T%YNOGW
  YSLDIA      => T%YSLDIA
  YAEROUT     => T%YAEROUT
  YUVP        => T%YUVP
  YSDSAT      => T%YSDSAT
  YCVV        => T%YCVV
  YRKTH       => T%YRKTH
  YRKTQV      => T%YRKTQV
  YRKTQC      => T%YRKTQC
  YUOM        => T% YUOM
  YUAL        => T%YUAL
  YDOM        => T%YDOM
  YDAL        => T%YDAL
  YUEN        => T%YUEN
  YUNEBH      => T%YUNEBH
  YEXT        => T%YEXT

END SUBROUTINE SET_YOM_YGFL

!     -------------------------------------------------------------------------

SUBROUTINE SAVE_YOM_YGFL(T)

  IMPLICIT NONE
  TYPE(TYPE_YOM_YGFL), INTENT(OUT) :: T

  T%NGFL_EXT    =  NGFL_EXT
  T%NGFL_FORC   =  NGFL_FORC
  T%NGFL_EZDIAG =  NGFL_EZDIAG
  T%NGHG        =  NGHG
  T%NTRAC       =  NTRAC
  T%NGRG        =  NGRG
  T%NGRG_CPLO4  =  NGRG_CPLO4
  T%NGRG_ASSIM  =  NGRG_ASSIM
  T%NAERO       =  NAERO
  T%NACTAERO    =  NACTAERO
  T%NDDHAERO    =  NDDHAERO
  T%NERA40      =  NERA40
  T%NNOGW       =  NNOGW
  T%NAEROUT     =  NAEROUT
  T%NUVP        =  NUVP
  T%NSLDIA      =  NSLDIA
  T%NSLDIAGP    =  NSLDIAGP
  T%LCO2SFC     =  LCO2SFC
  T%LCH4SFC     =  LCH4SFC
  T%LAEROSFC    =  LAEROSFC
  T%LSF6SFC     =  LSF6SFC
  T%LFIRE       =  LFIRE
  T%LAERODIU    =  LAERODIU
  T%LGRGSFC     =  LGRGSFC
  T%LTRCMFIX    =  LTRCMFIX
  T%LAEROUT     =  LAEROUT
  T%LUVPOUT     =  LUVPOUT

  T%YGFL        =  YGFL
  T%YGFLC       =  YGFLC
  T%YQ          => YQ
  T%YI          => YI
  T%YL          => YL
  T%YIRAD       => YIRAD
  T%YLRAD       => YLRAD
  T%YS          => YS
  T%YR          => YR
  T%YG          => YG
  T%YH          => YH
  T%YTKE        => YTKE
  T%YTTE        => YTTE
  T%YMXL        => YMXL
  T%YEFB1       => YEFB1
  T%YEFB2       => YEFB2
  T%YEFB3       => YEFB3
  T%YA          => YA
  T%YO3         => YO3
  T%YSRC        => YSRC
  T%YCPF        => YCPF
  T%YSPF        => YSPF
  T%YCVGQ       => YCVGQ
  T%YQVA        => YQVA
  T%YGHG        => YGHG
  T%YGRG        => YGRG
  T%YGRGTEND    => YGRGTEND
  T%YAERO       => YAERO
  T%YTRAC       => YTRAC
  T%YLRCH4      => YLRCH4
  T%YCH4S       => YCH4S
  T%YFORC       => YFORC
  T%YEZDIAG     => YEZDIAG
  T%YERA40      => YERA40
  T%YNOGW       => YNOGW
  T%YSLDIA      => YSLDIA
  T%YAEROUT     => YAEROUT
  T%YUVP        => YUVP
  T%YSDSAT      => YSDSAT
  T%YCVV        => YCVV
  T%YRKTH       => YRKTH
  T%YRKTQV      => YRKTQV
  T%YRKTQC      => YRKTQC
  T%YUOM        =>  YUOM
  T%YUAL        => YUAL
  T%YDOM        => YDOM
  T%YDAL        => YDAL
  T%YUEN        => YUEN
  T%YUNEBH      => YUNEBH
  T%YEXT        => YEXT

END SUBROUTINE SAVE_YOM_YGFL

END MODULE YOM_YGFL
