!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACDIFV3 ( KIDIA,KFDIA,KLON,KTDIA,KLEV,KSTEP,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPRS,PAPHI,PAPHIF,PCP,PDELP,PKTROV,PXTROV,&
 & PR,PRDELP,PCHROV,PXHROV,&
 & PNEBQ,PQ,PQL,PQI,PU,PV,PT,PALPH,PLNPR,&
 & PLML, PTH_FUN,PWW_FUN,PF_EPS,&
 ! - INPUT/ OUTPUT 2D .
 & PDIFTS,PDIFTQ,PTKE,PTENDPTKE, &
 & PMN2_ES,PMN2_EQ,PMN2_DS,PMN2_DQ, &
 ! - OUTPUT 2D .
 & PDIFTQL,PDIFTQI,&
 ! - INPUT/OUTPUT 1D .
 & PDIFWQ,PDIFWS,&  
 & PFCLL,PFCLN,PFCS,PFEVI,PFEVL,PFEVN,&
 ! - INPUT 1D .
 & PGZ0,PRTI,&
 & PSC_FEVI,PSC_FEVN,PSC_FCLL,PSC_FCLN,&
 & PTSTAR,PTSTAR2,PTSTARQ,PTSTAR2Q)

!**** *ACDIFV3 * - TOM TERMS CORRECTION

!     Sujet.
!     ------

!**   Interface.
!     ----------
!        *CALL* *ACDIFV3*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA      : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL).
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".
! KSTEP      : NUMERO DU PAS DE TEMPS.                       

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPRS      : PRESSION AUX NIVEAUX DES INTER-COUCHES.
! PAPHI      : GEOPOTENTIEL AUX DEMI-NIVEAUX.
! PKTROV     : COEFFICIENT D'ECHANGE VERTICAL DE T ET Q EN KG/(M*M*S).
! PTH_FUN    : T_h - stability functions for TOMs par.
! PWW_FUN    : F_ww - stability functions for TOMs par.
! PF_EPS     : TERM ASOCIATED WITH MIX. LENGTH CONVERSION L-lm: F_epsilon
! PNEBQ      : Q FOR q_li diffusion

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIEL AUX NIVEAUX DES COUCHES.
! PCP        : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR.
! PDELP      : EPAISSEUR EN PRESSION DE LA COUCHE.
! PR         : CONSTANTE DES GAZ POUR L'AIR.
! PRDELP     : INVERSE DE L'EPAISSEUR EN PRESSION DE LA COUCHE.
! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PQL        : QUANTITE DE SPECIFIQUE CONDENSAT LIQUIDE.
! PQI        : QUANTITE SPECIFIQUE DE CONDENSAT SOLIDE.
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.
! PT         : TEMPERATURE.
! PALPH      : LOG(PAPRS(JLEV)/PAPRSF(JLEV)) (FOR HYDROSTATICS).
! PLNPR      : LOG(PAPRS(JLEV)/PAPRS(JLEV-1)) (FOR HYDROSTATICS).

! - 1D (DIAGNOSTIQUE) .
! PRTI       : INVERSE DE R*T AT SURFACE
! PGZ0       : G FOIS LA LONGUEUR DE RUGOSITE COURANTE.

!-----------------------------------------------------------------------

! -   INPUT/OUTPUT ARGUMENTS 
!     -----------------------

! (0:KLEV)
! PDIFTS     : FLUX TURBULENT D'ENTHALPIE (OU D'ENERGIE STATIQUE SECHE).
! PDIFTQ     : FLUX TURBULENT (ET Q NEGATIF) D'HUMIDITE SPECIFIQUE.
! (1:KLEV)
! PTKE       : TURBULENT KINETIC ENERGY
! PTENDPTKE  : TKE TENDENCY (USED BY PSEUDO-PROGNOSTIC TKE SCHEME).


! -   OUTPUT ARGUMENTS 
!     -----------------

! (0:KLEV)
! PDIFTQL    : FLUX TURBULENT D'EAU LIQUIDE.
! PDIFTQI    : FLUX TURBULENT D'EAU SOLIDE.

! - 1D 
!INPUT/OUTPUT
! PDIFWQ     : FLUX TURBULENT (ET Q NEGATIF) D'HUMIDITE SPECIFIQUE.
! PDIFWS     : FLUX TURBULENT D'ENTHALPIE (OU D'ENERGIE STATIQUE SECHE).
! PFCLL      : FLUX DE CHALEUR LATENTE SUR EAU LIQUIDE (OU SOL HUMIDE).
! PFCLN      : FLUX DE CHALEUR LATENTE SUR NEIGE (OU GLACE).
! PFCS       : FLUX DE CHALEUR SENSIBLE EN SURFACE.
! PFEVI      : FLUX DE VAPEUR D'EAU SUR SOL GELE.
! PFEVL      : FLUX DE VAPEUR D'EAU SUR EAU LIQUIDE (OU SOL HUMIDE).
! PFEVN      : FLUX DE VAPEUR D'EAU SUR NEIGE (OU GLACE).

!INPUT
!PSC_*       : multiplicators for corresponding fluxes - used after TOMs for update of fluxes

!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMCST /
! COMMON/YOMPHY/
! COMMON/YOMPHY0/
! COMMON/YOMPHY2/

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     Methode.
!     --------

!     Auteur.
!     -------

!     Modifications.
!     --------------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOMCST   , ONLY :  RG       ,RD       ,&
 & RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD    ,RDT      ,RCPD     ,&
 & RATM     ,RKAPPA
USE YOMPHY   , ONLY :  LDIFCONS,NDIFFNEB,LCOEFK_FLX,LCOEFK_TOMS
USE YOMPHY0  , ONLY : VKARMN,C_EPSILON,ALMAV,&
 &           ETKE_KA1,ETKE_KA2,ETKE_KA3,ETKE_CG01,ETKE_CG02,ETKE_CG03,&
 &           GCISMIN, ETKE_BETA_EPS,ETKE_GAMMA_EPS,ENT_LAMBDA, RDTFAC,&
 &           NUPTKE, C_EPSILON 
USE YOMPHY2  , ONLY :  TSPHY

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKTROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXTROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCHROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXHROV(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEBQ(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQL(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQI(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTH_FUN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWW_FUN   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PF_EPS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMN2_ES   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMN2_EQ   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMN2_DS (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMN2_DQ (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLML(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTQ(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTKE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENDPTKE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFTQI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCLL(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCLN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEVI(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEVL(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEVN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFWQ(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFWS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSC_FEVI(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSC_FEVN(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSC_FCLL(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSC_FCLN(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0(KLON)  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTI(KLON)  
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSTAR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSTAR2(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSTARQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTSTAR2Q(KLON,0:KLEV) 


!-----------------------------------------------------------------------

REAL(KIND=JPRB) :: ZT_INSS(KLON,KLEV),ZT_INSS_2(KLON,0:KLEV),&
& ZAH_FUN(KLON,0:KLEV),ZTAU_TKE(KLON,0:KLEV),&
& ZT_INS0S(KLON,KLEV),ZT_INS0Q(KLON,KLEV),ZT_INS0S_2(KLON,0:KLEV),&
& ZT_INS0Q_2(KLON,0:KLEV),ZT_INSSQ(KLON,KLEV)
REAL(KIND=JPRB) :: ZIPOI(KLON,KLEV), ZPOID(KLON,KLEV)
REAL(KIND=JPRB) :: ZSUB1(KLON,KLEV), ZN1(KLON,KLEV), ZN2(KLON,KLEV)
REAL(KIND=JPRB) :: ZSCG0(KLON,KLEV), ZQCG0(KLON,KLEV)
REAL(KIND=JPRB) :: ZZZ(KLON,0:KLEV), ZTSTAR(KLON,KLEV)
REAL(KIND=JPRB) :: ZDIFSO(KLON,KLEV), ZDIFQO(KLON,KLEV)
REAL(KIND=JPRB) :: ZDIFSI(KLON,KLEV), ZDIFQI(KLON,KLEV)
REAL(KIND=JPRB) :: ZRHO(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZXSTAM(KLON,KLEV), ZXSTAP(KLON,KLEV)
REAL(KIND=JPRB) :: ZDIFTSTOM(KLON,0:KLEV), ZDIFTQTOM(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZKTROV(KLON,0:KLEV),ZKTROV2(KLON,0:KLEV), ZCORS(KLON,KLEV), ZCORQ(KLON,KLEV)
REAL(KIND=JPRB) :: ZCOEFA(KLON,0:KLEV), ZALPHA1(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZLVT(KLON,0:KLEV), ZPICE(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZQL(KLON), ZQI(KLON), ZDUTQL(KLON,0:KLEV), &
   & ZDUTQI(KLON,0:KLEV), ZDDTQL(KLON,0:KLEV), ZDDTQI(KLON,0:KLEV), &
   & ZDTQFL(KLON), ZDTQFI(KLON)
REAL(KIND=JPRB) :: ZTSTAR2(KLON,0:KLEV),ZDIFTS2(KLON,0:KLEV),ZDIFTQ2(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZXSTEST(KLON),ZDELPM(KLON,KLEV), ZDELPP(KLON,KLEV)
REAL(KIND=JPRB) :: ZRAP,ZSEC,ZACVSST,ZIACVSST
REAL(KIND=JPRB) :: ZPT_INS0_3
REAL(KIND=JPRB) :: ZPT_INS0S_2,ZPT_INS0Q_2,ZPT_INS0_3Q
REAL(KIND=JPRB) :: ZT_INS0S_2A1(KLON,0:KLEV),ZT_INS0Q_2A1(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZT_INS0S_2CR(KLON,0:KLEV),ZT_INS0Q_2CR(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZPT_INS0_3II(KLON,0:KLEV),ZPT_INS0_1II(KLON,0:KLEV),&
&ZPT_INS0_1(KLON,KLEV)
REAL(KIND=JPRB)  :: ZFTENDPTKE(KLON,KLEV)
REAL(KIND=JPRB)  :: ZLML(KLON,0:KLEV)
REAL(KIND=JPRB)  :: ZQTOT(KLON,KLEV)


INTEGER(KIND=JPIM) :: JLEV, JLON, JITER
INTEGER(KIND=JPIM) :: INITER

REAL(KIND=JPRB) :: ZELIM, ZMUL, ZEPS, ZGDT, ZGDTI, ZGLU 
REAL(KIND=JPRB) :: ZTH, ZLV, ZLS, ZCPH, ZTLI, ZICE, ZLIQ, ZEISP, ZELSP
REAL(KIND=JPRB) :: ZQSATI, ZQSATL, ZDQSATI, ZDQSATL, ZDIFTQC, ZSIGN
REAL(KIND=JPRB) :: ZGAMMA
REAL(KIND=JPRB) :: ZTKE_CEPS2, ZZ, ZCIS,ZTKES,ZRTI
REAL(KIND=JPRB) :: ZD_DIFWQ,ZD_DIFWS,ZDPHI

REAL(KIND=JPRB) :: ZTSTAR2Q(KLON,0:KLEV),ZTSTARQ(KLON,KLEV),ZSUB1Q(KLON,KLEV)
REAL(KIND=JPRB) :: ZCROSS(KLON,KLEV),ZCROSSQ(KLON,KLEV)
REAL(KIND=JPRB) :: ZXSTAMQ(KLON,KLEV), ZXSTAPQ(KLON,KLEV),ZKTROV2Q(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZXSTESTQ(KLON),ZDELPMQ(KLON,KLEV), ZDELPPQ(KLON,KLEV)
REAL(KIND=JPRB) :: ZRAPQ,ZSECQ
REAL(KIND=JPRB) :: ZELIMQ, ZMULQ
REAL(KIND=JPRB) :: ZCNEBDIFF
REAL(KIND=JPRB) :: ZTAU3(KLON,0:KLEV),ZL2DFEPS2(KLON,0:KLEV),&
&ZIL2DFEPS2(KLON,0:KLEV),ZTAUFEPSDL2(KLON,0:KLEV)


REAL(KIND=JPRB) :: ZRKONV
REAL(KIND=JPRB) :: ZTERM_II_1, ZTERM_II_2

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttrm.h"
#include "fctdoi.h"

!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ACDIFV3',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------

!     I - Initialization of constants

ZGLU=RG*ALMAV

ZTKE_CEPS2=C_EPSILON*C_EPSILON*0.25_JPRB

ZGDT=RG*TSPHY
ZGDTI=1.0_JPRB/ZGDT

ZEPS=EPSILON(1.0_JPRB)*1.E+4_JPRB
ZACVSST=2.0_JPRB
ZIACVSST=1.0_JPRB/ZACVSST

ZGAMMA=1.0_JPRB

ZRKONV= C_EPSILON/(NUPTKE*NUPTKE*NUPTKE)

!Switch for diffusion of q_li
IF(NDIFFNEB==0) THEN
 ZCNEBDIFF=0.0_JPRB
ELSE
 ZCNEBDIFF=1.0_JPRB
ENDIF

!FOR DIFCONS and TKE correction
ZQTOT(KIDIA:KFDIA,KTDIA:KLEV)=PQ(KIDIA:KFDIA,KTDIA:KLEV)+&
& PQL(KIDIA:KFDIA,KTDIA:KLEV)+PQI(KIDIA:KFDIA,KTDIA:KLEV)


! Number of the TOM solver iterations
INITER=2

IF(LCOEFK_TOMS) THEN
!DENSITY-half - SURFACE
DO JLON=KIDIA,KFDIA
   ZRHO(JLON,KLEV)=PAPRS(JLON,KLEV)*PRTI(JLON)
ENDDO

ZFTENDPTKE(KIDIA:KFDIA,KTDIA:KLEV)=PTENDPTKE(KIDIA:KFDIA,KTDIA:KLEV)*TSPHY+&
&                                  PTKE(KIDIA:KFDIA,KTDIA:KLEV)

! HALF LEVEL TAU,L - without surface
DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
   ZZ=VKARMN*(0.5_JPRB*(PAPHIF(JLON,JLEV)+PAPHIF(JLON,JLEV+1)) &
     & -PAPHI(JLON,KLEV)+PGZ0(JLON))
   ZDPHI=PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)

   ZCIS=(PU(JLON,JLEV)-PU(JLON,JLEV+1))**2 &
     &  +(PV(JLON,JLEV)-PV(JLON,JLEV+1))**2 &
     &  +(GCISMIN*ZDPHI/(RG*(1.0_JPRB+ZZ/ZGLU)))**2

   ZTKES=0.5_JPRB*(ZFTENDPTKE(JLON,JLEV)+ZFTENDPTKE(JLON,JLEV+1))

   ZLML(JLON,JLEV)=PLML(JLON,JLEV)

   ZTAU_TKE(JLON,JLEV)=2.0_JPRB*ZLML(JLON,JLEV)&
   &  /(sqrt(ZTKES)*C_EPSILON*PF_EPS(JLON,JLEV))
        
   !security for small tau_epsilon
   ZTAU_TKE(JLON,JLEV)=ZTAU_TKE(JLON,JLEV)+ETKE_BETA_EPS*TSPHY
        
   !security for large tau_epsilon
   ZTAU_TKE(JLON,JLEV)=ZTAU_TKE(JLON,JLEV)/(1.0_JPRB+ETKE_GAMMA_EPS/TSPHY*ZTAU_TKE(JLON,JLEV))
   ZLML(JLON,JLEV)=ZTAU_TKE(JLON,JLEV)&
    & *sqrt(ZTKES)*C_EPSILON/2.0_JPRB*PF_EPS(JLON,JLEV)

  ENDDO
ENDDO

! SURFACE
DO JLON=KIDIA,KFDIA
 
   ZDPHI=PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)
   ZCIS=PU(JLON,KLEV)**2+PV(JLON,KLEV)**2+(GCISMIN*ZDPHI/RG)**2

   ! Here to gain higher consistency between L and E for the lower
   !  TOMs solver boundary the lowest half level is suplied by L and e
   !   of the lowest full level (ZLML and ZFTENDPTKE).
   ZTKES=ZFTENDPTKE(JLON,KLEV)
  ZLML(JLON,KLEV)=PLML(JLON,KLEV)*(PGZ0(JLON)+PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)) &
  &  /PGZ0(JLON)

   ZTAU_TKE(JLON,KLEV)=2.0_JPRB*ZLML(JLON,KLEV)&
   & /(sqrt(ZTKES)*C_EPSILON*PF_EPS(JLON,KLEV))

   !security for small tau_epsilon
   ZTAU_TKE(JLON,KLEV)=ZTAU_TKE(JLON,KLEV)+ETKE_BETA_EPS*TSPHY
   !security for large tau_epsilon
   ZTAU_TKE(JLON,KLEV)=ZTAU_TKE(JLON,KLEV)/(1.0_JPRB+ETKE_GAMMA_EPS/TSPHY*ZTAU_TKE(JLON,KLEV))
   ZLML(JLON,KLEV)=ZTAU_TKE(JLON,KLEV)&
    &*sqrt(ZTKES)*C_EPSILON/2.0_JPRB*PF_EPS(JLON,KLEV)

ENDDO
!TOP
ZTAU_TKE(KIDIA:KFDIA,KTDIA-1)=ZTAU_TKE(KIDIA:KFDIA,KTDIA)
ZLML(KIDIA:KFDIA,KTDIA-1)=ZLML(KIDIA:KFDIA,KTDIA)


!DENSITY COMPUTATION - STANDATD HALF LEVELS
DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
   ZRHO(JLON,JLEV)=2._JPRB/(PR(JLON,JLEV)*PT(JLON,JLEV)+PR(JLON,JLEV+1)*PT(JLON,JLEV+1))&
   &               *PAPRS(JLON,JLEV)
  ENDDO
ENDDO
ZRHO(KIDIA:KFDIA,KTDIA-1)=ZRHO(KIDIA:KFDIA,KTDIA)

! HALF LEVEL QUANTITIES - T** terms - ALL LEVELS 
DO JLEV=KTDIA-1,KLEV
  DO JLON=KIDIA,KFDIA
     ZTAU3(JLON,JLEV)=ZTAU_TKE(JLON,JLEV)**3
     ZL2DFEPS2(JLON,JLEV)=(ZLML(JLON,JLEV)/PF_EPS(JLON,JLEV))**2
     ZIL2DFEPS2(JLON,JLEV)=1.0_JPRB/ZL2DFEPS2(JLON,JLEV)
     ZTAUFEPSDL2(JLON,JLEV)=(ZTAU_TKE(JLON,JLEV)*&
     &PF_EPS(JLON,JLEV)/ZLML(JLON,JLEV))**2
  ENDDO
ENDDO

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA

     ZT_INSS_2(JLON,JLEV)=ETKE_CG02*ETKE_KA2*ZTKE_CEPS2/ZRHO(JLON,JLEV)*ZTKE_CEPS2&
     & *(ZTAU3(JLON,JLEV)*ZTAU_TKE(JLON,JLEV))*ZIL2DFEPS2(JLON,JLEV)
   
     ZT_INS0S_2(JLON,JLEV)=PWW_FUN(JLON,JLEV)&
     & *( -0.06_JPRB*ETKE_CG01*ETKE_KA1*PMN2_ES(JLON,JLEV)*PMN2_DS(JLON,JLEV)&
     & *ZTAU3(JLON,JLEV)&
     & +0.3_JPRB*ETKE_CG03*ETKE_KA3*ZTAU_TKE(JLON,JLEV))
     ZT_INS0Q_2(JLON,JLEV)=PWW_FUN(JLON,JLEV)&
     & *(-0.06_JPRB*ETKE_CG01*ETKE_KA1*PMN2_EQ(JLON,JLEV)*PMN2_DQ(JLON,JLEV)&
     & *ZTAU3(JLON,JLEV)&
     & +0.3_JPRB*ETKE_CG03*ETKE_KA3*ZTAU_TKE(JLON,JLEV))

     ZT_INS0S_2A1(JLON,JLEV)=(-0.06_JPRB*ETKE_CG01*ETKE_KA1*PMN2_DQ(JLON,JLEV))&
     & *ZTAU3(JLON,JLEV)*ZIL2DFEPS2(JLON,JLEV)
     ZT_INS0Q_2A1(JLON,JLEV)=(-0.06_JPRB*ETKE_CG01*ETKE_KA1*PMN2_DS(JLON,JLEV))&
     & *ZTAU3(JLON,JLEV)*ZIL2DFEPS2(JLON,JLEV)

     ZT_INS0S_2CR(JLON,JLEV)=PWW_FUN(JLON,JLEV)*PMN2_ES(JLON,JLEV)&
     & *ZL2DFEPS2(JLON,JLEV)
     ZT_INS0Q_2CR(JLON,JLEV)=PWW_FUN(JLON,JLEV)*PMN2_EQ(JLON,JLEV)&
     & *ZL2DFEPS2(JLON,JLEV)

    ENDDO
  ENDDO
  ZT_INS0S_2CR(KIDIA:KFDIA,KTDIA-1)=ZT_INS0S_2CR(KIDIA:KFDIA,KTDIA)
  ZT_INS0Q_2CR(KIDIA:KFDIA,KTDIA-1)=ZT_INS0Q_2CR(KIDIA:KFDIA,KTDIA)

  ZT_INS0S_2(KIDIA:KFDIA,KTDIA-1)=0.0_JPRB
  ZT_INS0S_2A1(KIDIA:KFDIA,KTDIA-1)=0.0_JPRB
  ZT_INS0Q_2(KIDIA:KFDIA,KTDIA-1)=0.0_JPRB
  ZT_INS0Q_2A1(KIDIA:KFDIA,KTDIA-1)=0.0_JPRB

  ZT_INSS_2(KIDIA:KFDIA,KTDIA-1)=0.0_JPRB

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
  ZRTI=2._JPRB/(PR(JLON,JLEV)*PT(JLON,JLEV)+PR(JLON,JLEV+1)*PT(JLON,JLEV+1))
  ZDPHI=PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)
  ZAH_FUN(JLON,JLEV)=ETKE_CG03*ETKE_KA3*PTH_FUN(JLON,JLEV)&
  & *ZTKE_CEPS2*ZTAU_TKE(JLON,JLEV)/(ZLML(JLON,JLEV))**2&
  & *PKTROV(JLON,JLEV)/(PAPRS(JLON,JLEV)*ZRTI*RG/ZDPHI)
  ENDDO
ENDDO
ZAH_FUN (KIDIA:KFDIA,KTDIA-1)=ZAH_FUN (KIDIA:KFDIA,KTDIA)
ZAH_FUN (KIDIA:KFDIA,KLEV)=ZAH_FUN (KIDIA:KFDIA,KLEV-1)

! HALF LEVEL QUANTITIES
DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
   ZPT_INS0_3II(JLON,JLEV)=ZRHO(JLON,JLEV)*&
   &PWW_FUN(JLON,JLEV)*ZLML(JLON,JLEV)**2
   ZPT_INS0_1II(JLON,JLEV)=ZPT_INS0_3II(JLON,JLEV)/&
   & ZTAU_TKE(JLON,JLEV)
  ENDDO
ENDDO
ZPT_INS0_3II (KIDIA:KFDIA,KTDIA-1)=ZPT_INS0_3II (KIDIA:KFDIA,KTDIA)
ZPT_INS0_1II (KIDIA:KFDIA,KTDIA-1)=ZPT_INS0_1II (KIDIA:KFDIA,KTDIA)

! FULL LEVEL QUANTITY
DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
   ZPT_INS0_1(JLON,JLEV)=(ZPT_INS0_1II(JLON,JLEV)-&
   &ZPT_INS0_1II(JLON,JLEV-1))*PRDELP(JLON,JLEV)
  ENDDO
ENDDO

! FULL LEVEL QUANTITIES
  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA

     ZPT_INS0_3=-(ZPT_INS0_3II(JLON,JLEV-1)*PMN2_ES(JLON,JLEV-1)&
     &- ZPT_INS0_3II(JLON,JLEV)*PMN2_ES(JLON,JLEV))*PRDELP(JLON,JLEV)
     ZPT_INS0_3Q=-(ZPT_INS0_3II(JLON,JLEV-1)*PMN2_EQ(JLON,JLEV-1)&
     &-ZPT_INS0_3II(JLON,JLEV)*PMN2_EQ(JLON,JLEV)&
     &)*PRDELP(JLON,JLEV)
     
     
     ZPT_INS0S_2=PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*                    &
     & PMN2_DS(JLON,JLEV-1)*ZTAU3(JLON,JLEV-1)*ZIL2DFEPS2(JLON,JLEV-1)+&
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*                   &
     & PMN2_DS(JLON,JLEV)*ZTAU3(JLON,JLEV)*ZIL2DFEPS2(JLON,JLEV)      

     ZPT_INS0Q_2=PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*                    &
     & PMN2_DQ(JLON,JLEV-1)*ZTAU3(JLON,JLEV-1)*ZIL2DFEPS2(JLON,JLEV-1)+&
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*                   &
     & PMN2_DQ(JLON,JLEV)*ZTAU3(JLON,JLEV)*ZIL2DFEPS2(JLON,JLEV)      


     ZT_INS0S(JLON,JLEV)=RG*RG*                                         &
     &  (-0.06_JPRB*ETKE_CG01*ETKE_KA1*ZPT_INS0S_2*ZPT_INS0_3           &
     &  +0.3_JPRB*ETKE_CG03*ETKE_KA3*ZPT_INS0_1(JLON,JLEV)              &
     & *(PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*                             &
     & ZTAUFEPSDL2(JLON,JLEV-1)+                                        &
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*                    &
     & ZTAUFEPSDL2(JLON,JLEV)))

     ZT_INS0Q(JLON,JLEV)=RG*RG*                                         &
     &  (-0.06_JPRB*ETKE_CG01*ETKE_KA1*ZPT_INS0Q_2*ZPT_INS0_3Q          &
     &  +0.3_JPRB*ETKE_CG03*ETKE_KA3*ZPT_INS0_1(JLON,JLEV)              &
     & *(PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*                             &
     & ZTAUFEPSDL2(JLON,JLEV-1)+                                        &
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*                    &
     & ZTAUFEPSDL2(JLON,JLEV)))


     ZT_INSS(JLON,JLEV)=ETKE_CG02*ETKE_KA2*ZTKE_CEPS2*&
     & (PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*            &
     & ZIL2DFEPS2(JLON,JLEV-1)                        &
     & *ZTAU3(JLON,JLEV-1)*PMN2_ES(JLON,JLEV-1)+      &
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*  &
     & ZIL2DFEPS2(JLON,JLEV)                          &
     & *ZTAU3(JLON,JLEV)*PMN2_ES(JLON,JLEV))*         &
     & RG*RG*RG

     ZT_INSSQ(JLON,JLEV)=ETKE_CG02*ETKE_KA2*ZTKE_CEPS2*&
     & (PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*             &
     & ZIL2DFEPS2(JLON,JLEV-1)                         &
     & *ZTAU3(JLON,JLEV-1)*PMN2_EQ(JLON,JLEV-1)+       &
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*   &
     & ZIL2DFEPS2(JLON,JLEV)                           &
     & *ZTAU3(JLON,JLEV)*PMN2_EQ(JLON,JLEV))*          &
     & RG*RG*RG

    ENDDO
  ENDDO

!  END OF TOM's TERMS CALCULATIONS--------------------------

!*
!     ------------------------------------------------------------------
!     II - Preliminary computation 

! - TEMPORAIRE(S) 2D (1:KLEV) .


! Exchange coefficient consistently prolongated till surface
!  being aware of the dry anti-fibrillation (the monotonicity
!  of PXTROV has been already ensured in ACDIFV1)

DO JLON=KIDIA,KFDIA
  ZKTROV(JLON,KTDIA-1)=0.0_JPRB
  ZKTROV2(JLON,KTDIA-1)=0.0_JPRB
ENDDO

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
   ZKTROV(JLON,JLEV)=PKTROV(JLON,JLEV)*PXTROV(JLON,JLEV)
   !scaling
   ZKTROV(JLON,JLEV) =ZKTROV(JLON,JLEV)& 
   & *TSPHY/(TSPHY+ZGAMMA*ZTAU_TKE(JLON,JLEV)*ZAH_FUN(JLON,JLEV))
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZKTROV(JLON,KLEV)=PCHROV(JLON)*PXHROV(JLON)
   !scaling
  ZKTROV(JLON,KLEV) =ZKTROV(JLON,KLEV)& 
   & *TSPHY/(TSPHY+ZGAMMA*ZTAU_TKE(JLON,KLEV)*ZAH_FUN(JLON,KLEV))
ENDDO

ENDIF
!End of TOMs 1st loop

! ZPOID     : DP/(RG*DT) POUR UNE COUCHE ET UN PAS DE TEMPS DONNES.
!           : DP/(RG*DT) FOR A GIVEN LAYER AND A GIVEN TIME STEP.
! ZIPOI     : INVERSE DE ZPOID.
!           : INVERSE OF ZPOID.

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZPOID(JLON,JLEV)=PDELP(JLON,JLEV)*ZGDTI
    ZIPOI(JLON,JLEV)=PRDELP(JLON,JLEV)*ZGDT
  ENDDO
ENDDO

IF (LDIFCONS) THEN

! Preparation for retrieving the condensates fluxes.

  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZTH               =(PT(JLON,JLEV)+PT(JLON,JLEV+1))*0.5_JPRB
      ZLV               = FOLH(ZTH,0.0_JPRB)
      ZLS               = FOLH(ZTH,1.0_JPRB)
      ZPICE(JLON,JLEV)  = FONICE(ZTH)
      ZCPH              =(PCP(JLON,JLEV)+PCP(JLON,JLEV+1))*0.5_JPRB
      ZTLI              =ZTH-0.5_JPRB*(ZLV*(PQL(JLON,JLEV) &
       &  +PQL(JLON,JLEV+1))+ZLS*(PQI(JLON,JLEV)+PQI(JLON,JLEV+1)))/ZCPH
      ZICE              =ZPICE(JLON,JLEV)
      ZLIQ              =1.0_JPRB-ZICE
      ZLVT(JLON,JLEV)   =ZLV*ZLIQ+ZLS*ZICE
      ZEISP             = FOEW(ZTLI,1.0_JPRB)/PAPRS(JLON,JLEV) 
      ZELSP             = FOEW(ZTLI,0.0_JPRB)/PAPRS(JLON,JLEV) 
      ZQSATI            = FOQS(ZEISP)
      ZQSATL            = FOQS(ZELSP)
      ZDQSATI           =ZQSATI* FOLH(ZTLI,1.0_JPRB)/(RV*ZTLI*ZTLI*ZCPH)
      ZDQSATL           =ZQSATL* FOLH(ZTLI,0.0_JPRB)/(RV*ZTLI*ZTLI*ZCPH)
      ZALPHA1(JLON,JLEV)=(ZLIQ*ZDQSATL+ZICE*ZDQSATI)
      ZCOEFA(JLON,JLEV) =ZCNEBDIFF*PNEBQ(JLON,JLEV) &
       &  /((1.0_JPRB+ZDQSATL*ZLV)*ZLIQ &
       &   +(1.0_JPRB+ZDQSATI*ZLS)*ZICE)
    ENDDO
  ENDDO

ENDIF


IF(LCOEFK_TOMS) THEN
ZDIFTS2(KIDIA:KFDIA,KTDIA-1:KLEV)   =PDIFTS(KIDIA:KFDIA,KTDIA:KLEV)
ZDIFTQ2(KIDIA:KFDIA,KTDIA-1:KLEV)   =PDIFTQ(KIDIA:KFDIA,KTDIA:KLEV)


!*
!     ------------------------------------------------------------------
!     III - Preparation for the TOM solvers

!     T*^-1                           : ZTSTAR (KLEV)
!     T**                             : ZTSTAR2(0:KLEV)
!     s* - s-                         : ZDIFSO (KLEV)
!     q* - q-                         : ZDIFQO (KLEV)
DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZDIFSO(JLON,JLEV)=ZIPOI(JLON,JLEV)*(PDIFTS(JLON,JLEV-1)-PDIFTS(JLON,JLEV))
    ZDIFQO(JLON,JLEV)=ZIPOI(JLON,JLEV)*(PDIFTQ(JLON,JLEV-1)-PDIFTQ(JLON,JLEV))

    ZDIFSI(JLON,JLEV)=ZDIFSO(JLON,JLEV)
    ZDIFQI(JLON,JLEV)=ZDIFQO(JLON,JLEV)
  END DO
END DO

DO JITER=1,INITER

!cdir unroll=8
DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZDIFSO(JLON,JLEV)=ZIPOI(JLON,JLEV)*(PDIFTS(JLON,JLEV-1)-PDIFTS(JLON,JLEV))
    ZDIFQO(JLON,JLEV)=ZIPOI(JLON,JLEV)*(PDIFTQ(JLON,JLEV-1)-PDIFTQ(JLON,JLEV))

    ZDIFSI(JLON,JLEV)=ZDIFSO(JLON,JLEV)
    ZDIFQI(JLON,JLEV)=ZDIFQO(JLON,JLEV)
  ENDDO
ENDDO

!cdir unroll=8
  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
    
    ZTSTAR(JLON,JLEV)=ZT_INS0S(JLON,JLEV)+&
    & PRDELP(JLON,JLEV)*&
      & (ZT_INSS(JLON,JLEV) *(ZDIFTS2(JLON,JLEV-1)*ZTAU_TKE(JLON,JLEV-1) -&
      & ZDIFTS2(JLON,JLEV)*ZTAU_TKE(JLON,JLEV)))
    ZTSTARQ(JLON,JLEV)=ZT_INS0Q(JLON,JLEV)+&
    & PRDELP(JLON,JLEV)*&
      & (ZT_INSSQ(JLON,JLEV) *(ZDIFTQ2(JLON,JLEV-1)*ZTAU_TKE(JLON,JLEV-1) -&
      & ZDIFTQ2(JLON,JLEV)*ZTAU_TKE(JLON,JLEV)))


    ZXSTAP(JLON,JLEV)=ZTSTAR(JLON,JLEV)+ABS(ZTSTAR(JLON,JLEV))
    ZXSTAM(JLON,JLEV)=ZTSTAR(JLON,JLEV)-ABS(ZTSTAR(JLON,JLEV))
    ZXSTAPQ(JLON,JLEV)=ZTSTARQ(JLON,JLEV)+ABS(ZTSTARQ(JLON,JLEV))
    ZXSTAMQ(JLON,JLEV)=ZTSTARQ(JLON,JLEV)-ABS(ZTSTARQ(JLON,JLEV))

          
!AVAREGING TO FULL LEVELS
     ZCROSS(JLON,JLEV)=(PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)&
     & *ZT_INS0S_2CR(JLON,JLEV-1)+                           &
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))          &
     & *ZT_INS0S_2CR(JLON,JLEV))

     ZCROSSQ(JLON,JLEV)=(PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)&
     & *ZT_INS0Q_2CR(JLON,JLEV-1)+                           &
     & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))          &
     & *ZT_INS0Q_2CR(JLON,JLEV))
     
    
    ZTSTAR2(JLON,JLEV)=MAX(0._JPRB,ZT_INS0S_2(JLON,JLEV)- &
      & ZT_INSS_2(JLON,JLEV)*PMN2_ES(JLON,JLEV)*ZDIFTS2(JLON,JLEV))
    ZTSTAR2Q(JLON,JLEV)=MAX(0._JPRB,ZT_INS0Q_2(JLON,JLEV)- &
      & ZT_INSS_2(JLON,JLEV)*PMN2_EQ(JLON,JLEV)*ZDIFTQ2(JLON,JLEV))
    ENDDO
  ENDDO

!ZT_INS0Q_2A1(KIDIA:KFDIA,KLEV)=0.0_JPRB
ZT_INS0Q_2A1(KIDIA:KFDIA,KTDIA-1)=ZT_INS0Q_2A1(KIDIA:KFDIA,KTDIA)
!ZT_INS0S_2A1(KIDIA:KFDIA,KLEV)=0.0_JPRB
ZT_INS0S_2A1(KIDIA:KFDIA,KTDIA-1)=ZT_INS0S_2A1(KIDIA:KFDIA,KTDIA)

ZTSTAR2(KIDIA:KFDIA,KTDIA-1)=ZTSTAR2(KIDIA:KFDIA,KTDIA)
ZTSTAR2(KIDIA:KFDIA,KLEV)=0.0_JPRB
ZTSTAR2Q(KIDIA:KFDIA,KTDIA-1)=ZTSTAR2Q(KIDIA:KFDIA,KTDIA)
ZTSTAR2Q(KIDIA:KFDIA,KLEV)=0.0_JPRB

!diag
PTSTAR2(:,:)=ZTSTAR2(:,:)
PTSTAR(:,:)=ZTSTAR(:,:)
PTSTAR2Q(:,:)=ZTSTAR2Q(:,:)
PTSTARQ(:,:)=ZTSTARQ(:,:)

!  Half level term to simplify solver  (HL) : ZZZ

DO JLON=KIDIA,KFDIA
  ZZZ(JLON,KTDIA-1)=0.0_JPRB
ENDDO

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
!   ZZZ(JLON,JLEV)=0.5_JPRB*ZKTROV(JLON,JLEV) &
!    &  *(PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)) &
!    &  /RG*PTH_FUN(JLON,JLEV)
!cheat ?!    
   ZZZ(JLON,JLEV)=0.5_JPRB*ZKTROV(JLON,JLEV) &
    &  *(PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)) &
    &  /RG*PTH_FUN(JLON,JLEV)/TSPHY

   ZKTROV2(JLON,JLEV)=ZKTROV(JLON,JLEV)&
    & *(1.0_JPRB+PTH_FUN(JLON,JLEV)*ZTSTAR2(JLON,JLEV)/TSPHY)
   ZKTROV2Q(JLON,JLEV)=ZKTROV(JLON,JLEV)&
    & *(1.0_JPRB+PTH_FUN(JLON,JLEV)*ZTSTAR2Q(JLON,JLEV)/TSPHY)
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
! ZZZ(JLON,KLEV)=0.5_JPRB*ZKTROV(JLON,KLEV) &
!  &  *(PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)) &
!  &  /RG*PTH_FUN(JLON,KLEV)

!cheat ?!    
 ZZZ(JLON,KLEV)=0.5_JPRB*ZKTROV(JLON,KLEV) &
  &  *(PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)) &
  &  /RG*PTH_FUN(JLON,KLEV)/TSPHY

 ZKTROV2(JLON,KLEV)=ZKTROV(JLON,KLEV)&
  & *(1.0_JPRB+PTH_FUN(JLON,KLEV)*ZTSTAR2(JLON,KLEV)/TSPHY)
 ZKTROV2Q(JLON,KLEV)=ZKTROV(JLON,KLEV)&
  & *(1.0_JPRB+PTH_FUN(JLON,KLEV)*ZTSTAR2Q(JLON,KLEV)/TSPHY)
ENDDO

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZXSTAP(JLON,JLEV)=ZXSTAP(JLON,JLEV)*ZZZ(JLON,JLEV-1)
    ZXSTAM(JLON,JLEV)=ZXSTAM(JLON,JLEV)*ZZZ(JLON,JLEV)
    ZXSTAPQ(JLON,JLEV)=ZXSTAPQ(JLON,JLEV)*ZZZ(JLON,JLEV-1)
    ZXSTAMQ(JLON,JLEV)=ZXSTAMQ(JLON,JLEV)*ZZZ(JLON,JLEV)
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZXSTAP(JLON,KLEV)=ZXSTAP(JLON,KLEV)*ZZZ(JLON,KLEV-1)
  ZXSTAM(JLON,JLEV)=ZXSTAM(JLON,KLEV)*ZZZ(JLON,KLEV)
  ZXSTAPQ(JLON,KLEV)=ZXSTAPQ(JLON,KLEV)*ZZZ(JLON,KLEV-1)
  ZXSTAMQ(JLON,JLEV)=ZXSTAMQ(JLON,KLEV)*ZZZ(JLON,KLEV)
ENDDO


! protection against n.-l. instability; sharing the dp resource via a guess

DO JLON=KIDIA,KFDIA
  ZXSTEST(JLON)=0.0_JPRB
  ZXSTESTQ(JLON)=0.0_JPRB
ENDDO

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZXSTEST(JLON)      =ZXSTEST(JLON)+(ZXSTAM(JLON,JLEV) &
     &  -ZXSTEST(JLON))/(1.0_JPRB+(MAX(0.0_JPRB,ZXSTEST(JLON) &
     &  -ZXSTAM(JLON,JLEV))*PRDELP(JLON,JLEV+1)))
    ZDELPM(JLON,JLEV+1)=MAX(ZEPS*PDELP(JLON,JLEV+1) &
     &  ,-ZXSTAM(JLON,JLEV)+ZXSTEST(JLON))
    ZXSTESTQ(JLON)      =ZXSTESTQ(JLON)+(ZXSTAMQ(JLON,JLEV) &
     &  -ZXSTESTQ(JLON))/(1.0_JPRB+(MAX(0.0_JPRB,ZXSTESTQ(JLON) &
     &  -ZXSTAMQ(JLON,JLEV))*PRDELP(JLON,JLEV+1)))
    ZDELPMQ(JLON,JLEV+1)=MAX(ZEPS*PDELP(JLON,JLEV+1) &
     &  ,-ZXSTAMQ(JLON,JLEV)+ZXSTESTQ(JLON))
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZXSTEST(JLON)=0.0_JPRB
  ZXSTESTQ(JLON)=0.0_JPRB
ENDDO

DO JLEV=KTDIA+1,KLEV
  DO JLON=KIDIA,KFDIA
    ZXSTEST(JLON)      =ZXSTEST(JLON)+(ZXSTAP(JLON,JLEV) &
     &  -ZXSTEST(JLON))/(1.0_JPRB+(MAX(0.0_JPRB,ZXSTAP(JLON,JLEV) &
     &  -ZXSTEST(JLON))*PRDELP(JLON,JLEV-1)))
    ZDELPP(JLON,JLEV-1)=MAX(ZEPS*PDELP(JLON,JLEV-1) &
     &  ,ZXSTAP(JLON,JLEV)-ZXSTEST(JLON))
    ZXSTESTQ(JLON)      =ZXSTESTQ(JLON)+(ZXSTAPQ(JLON,JLEV) &
     &  -ZXSTESTQ(JLON))/(1.0_JPRB+(MAX(0.0_JPRB,ZXSTAPQ(JLON,JLEV) &
     &  -ZXSTESTQ(JLON))*PRDELP(JLON,JLEV-1)))
    ZDELPPQ(JLON,JLEV-1)=MAX(ZEPS*PDELP(JLON,JLEV-1) &
     &  ,ZXSTAPQ(JLON,JLEV)-ZXSTESTQ(JLON))

  ENDDO
ENDDO

DO JLEV=KTDIA+1,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZRAP             =MAX(ZEPS,MIN(1.0_JPRB-ZEPS,ZDELPM(JLON,JLEV) &
     &  /(ZDELPM(JLON,JLEV)+ZDELPP(JLON,JLEV))))
    ZDELPM(JLON,JLEV)=PDELP(JLON,JLEV)*ZRAP
    ZDELPP(JLON,JLEV)=PDELP(JLON,JLEV)*(1.0_JPRB-ZRAP)
    ZRAPQ             =MAX(ZEPS,MIN(1.0_JPRB-ZEPS,ZDELPMQ(JLON,JLEV) &
     &  /(ZDELPMQ(JLON,JLEV)+ZDELPPQ(JLON,JLEV))))
    ZDELPMQ(JLON,JLEV)=PDELP(JLON,JLEV)*ZRAPQ
    ZDELPPQ(JLON,JLEV)=PDELP(JLON,JLEV)*(1.0_JPRB-ZRAPQ)

  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  ZDELPP(JLON,KTDIA)=PDELP(JLON,KTDIA)
  ZDELPM(JLON,KLEV) =PDELP(JLON,KLEV)
  ZDELPPQ(JLON,KTDIA)=PDELP(JLON,KTDIA)
  ZDELPMQ(JLON,KLEV) =PDELP(JLON,KLEV)
ENDDO

! protection against n.-l. instability; actual symmetric implementation

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZSEC             =MAX(0.0_JPRB,(ZXSTAM(JLON,JLEV+1)-ZXSTAM(JLON,JLEV)) &
     &  /ZDELPM(JLON,JLEV+1))
!   ZSEC             =(1.0_JPRB+ZSEC**ZACVSST)**ZIACVSST
    ZSEC             =SQRT(1.0_JPRB+ZSEC*ZSEC)
    ZXSTAM(JLON,JLEV)=ZXSTAM(JLON,JLEV+1)+(ZXSTAM(JLON,JLEV) &
     &  -ZXSTAM(JLON,JLEV+1))/ZSEC
    ZSECQ             =MAX(0.0_JPRB,(ZXSTAMQ(JLON,JLEV+1)-ZXSTAMQ(JLON,JLEV)) &
     &  /ZDELPMQ(JLON,JLEV+1))
!   ZSECQ             =(1.0_JPRB+ZSECQ**ZACVSST)**ZIACVSST
    ZSECQ             =SQRT(1.0_JPRB+ZSECQ*ZSECQ)
    ZXSTAMQ(JLON,JLEV)=ZXSTAMQ(JLON,JLEV+1)+(ZXSTAMQ(JLON,JLEV) &
     &  -ZXSTAMQ(JLON,JLEV+1))/ZSECQ

  ENDDO
ENDDO

DO JLEV=KTDIA+1,KLEV
  DO JLON=KIDIA,KFDIA
    ZSEC             =MAX(0.0_JPRB,(ZXSTAP(JLON,JLEV)-ZXSTAP(JLON,JLEV-1)) &
     &  /ZDELPP(JLON,JLEV-1))
!   ZSEC             =(1.0_JPRB+ZSEC**ZACVSST)**ZIACVSST
    ZSEC             =SQRT(1.0_JPRB+ZSEC*ZSEC)
    ZXSTAP(JLON,JLEV)=ZXSTAP(JLON,JLEV-1)+(ZXSTAP(JLON,JLEV) &
     &  -ZXSTAP(JLON,JLEV-1))/ZSEC
    ZSECQ             =MAX(0.0_JPRB,(ZXSTAPQ(JLON,JLEV)-ZXSTAPQ(JLON,JLEV-1)) &
     &  /ZDELPPQ(JLON,JLEV-1))
!   ZSECQ             =(1.0_JPRB+ZSECQ**ZACVSST)**ZIACVSST
    ZSECQ             =SQRT(1.0_JPRB+ZSECQ*ZSECQ)
    ZXSTAPQ(JLON,JLEV)=ZXSTAPQ(JLON,JLEV-1)+(ZXSTAPQ(JLON,JLEV) &
     &  -ZXSTAPQ(JLON,JLEV-1))/ZSECQ

  ENDDO
ENDDO


!  (S_cg)_0 term (RHS of the solver) : ZSCG0
!  (Q_cg)_0 term (RHS of the solver) : ZQCG0

DO JLON=KIDIA,KFDIA

  ZSCG0(JLON,KTDIA)=PRDELP(JLON,KTDIA)              &
   &  *(ZXSTAM(JLON,KTDIA)*ZDIFSO(JLON,KTDIA)       &
   &  +ZXSTAP(JLON,KTDIA+1)*ZDIFSO(JLON,KTDIA+1))   &
   &  -RG**2*PRDELP(JLON,KTDIA)                     &
   & *(ZTSTAR2(JLON,KTDIA)*ZZZ(JLON,KTDIA)*2.0_JPRB &
   & * (ZDIFSO(JLON,KTDIA)-ZDIFSO(JLON,KTDIA+1))    &
   & /(PAPHIF(JLON,KTDIA)-PAPHIF(JLON,KTDIA+1)))    &
   & -RG**2*PRDELP(JLON,KTDIA)                      &
   & *(ZT_INS0Q_2A1(JLON,KTDIA)*ZZZ(JLON,KTDIA)*2.0_JPRB&
   & * (ZDIFQI(JLON,KTDIA)*ZCROSSQ(JLON,KTDIA)  &
   & -ZDIFQI(JLON,KTDIA+1)*ZCROSSQ(JLON,KTDIA+1)) &
   & /(PAPHIF(JLON,KTDIA)-PAPHIF(JLON,KTDIA+1)))
  ZQCG0(JLON,KTDIA)=PRDELP(JLON,KTDIA)              &
   &  *(ZXSTAMQ(JLON,KTDIA)*ZDIFQO(JLON,KTDIA)       &
   &  +ZXSTAPQ(JLON,KTDIA+1)*ZDIFQO(JLON,KTDIA+1))   &
   & -RG**2*PRDELP(JLON,KTDIA)                      &
   & *(ZTSTAR2Q(JLON,KTDIA)*ZZZ(JLON,KTDIA)*2.0_JPRB &
   & * (ZDIFQO(JLON,KTDIA)-ZDIFQO(JLON,KTDIA+1))    &
   & /(PAPHIF(JLON,KTDIA)-PAPHIF(JLON,KTDIA+1)))    &
   &  -RG**2*PRDELP(JLON,KTDIA)                     &
   & *(ZT_INS0S_2A1(JLON,KTDIA)*ZZZ(JLON,KTDIA)*2.0_JPRB &
   & * (ZDIFSI(JLON,KTDIA)*ZCROSS(JLON,KTDIA)   &
   & -ZDIFSI(JLON,KTDIA+1)*ZCROSS(JLON,KTDIA+1)) &
   & /(PAPHIF(JLON,KTDIA)-PAPHIF(JLON,KTDIA+1)))    
ENDDO

!cdir unroll=8
DO JLEV=KTDIA+1,KLEV-1
  DO JLON=KIDIA,KFDIA

    ZSCG0(JLON,JLEV)=-PRDELP(JLON,JLEV)                          &
     &  *(ZXSTAM(JLON,JLEV-1)*ZDIFSO(JLON,JLEV-1)                &
     &  +(ZXSTAP(JLON,JLEV)-ZXSTAM(JLON,JLEV))*ZDIFSO(JLON,JLEV) &
     &  -ZXSTAP(JLON,JLEV+1)*ZDIFSO(JLON,JLEV+1))                &
     &  +RG**2*PRDELP(JLON,JLEV)                                 &
     & *(ZTSTAR2(JLON,JLEV-1)*ZZZ(JLON,JLEV-1)*2.0_JPRB          &
     & *(ZDIFSO(JLON,JLEV-1)-ZDIFSO(JLON,JLEV))                  &
     & /(PAPHIF(JLON,JLEV-1)-PAPHIF(JLON,JLEV))                  &
     & - ZTSTAR2(JLON,JLEV)*ZZZ(JLON,JLEV)*2.0_JPRB              &
     & * (ZDIFSO(JLON,JLEV)-ZDIFSO(JLON,JLEV+1))                 &
     & /(PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)))                 &
     &  +RG**2*PRDELP(JLON,JLEV)                                 &
     & *(ZT_INS0Q_2A1(JLON,JLEV-1)*ZZZ(JLON,JLEV-1)*2.0_JPRB     &
     & *(ZDIFQI(JLON,JLEV-1)*ZCROSSQ(JLON,JLEV-1)                 &
     & -ZDIFQI(JLON,JLEV)*ZCROSSQ(JLON,JLEV))                     &
     & /(PAPHIF(JLON,JLEV-1)-PAPHIF(JLON,JLEV))                  &
     & - ZT_INS0Q_2A1(JLON,JLEV)*ZZZ(JLON,JLEV)*2.0_JPRB         &
     & * (ZDIFQI(JLON,JLEV)*ZCROSSQ(JLON,JLEV)               &
     & -ZDIFQI(JLON,JLEV+1)*ZCROSSQ(JLON,JLEV+1))            &
     & /(PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)))
    ZQCG0(JLON,JLEV)=-PRDELP(JLON,JLEV)                          &
     &  *(ZXSTAMQ(JLON,JLEV-1)*ZDIFQO(JLON,JLEV-1)                &
     &  +(ZXSTAPQ(JLON,JLEV)-ZXSTAMQ(JLON,JLEV))*ZDIFQO(JLON,JLEV) &
     &  -ZXSTAPQ(JLON,JLEV+1)*ZDIFQO(JLON,JLEV+1))                &
     &  +RG**2*PRDELP(JLON,JLEV)                                 &
     & *(ZTSTAR2Q(JLON,JLEV-1)*ZZZ(JLON,JLEV-1)*2.0_JPRB          &
     & *(ZDIFQO(JLON,JLEV-1)-ZDIFQO(JLON,JLEV))                  &
     & /(PAPHIF(JLON,JLEV-1)-PAPHIF(JLON,JLEV))                  &
     & - ZTSTAR2Q(JLON,JLEV)*ZZZ(JLON,JLEV)*2.0_JPRB              &
     & * (ZDIFQO(JLON,JLEV)-ZDIFQO(JLON,JLEV+1))                 &
     & /(PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)))                 &
     &  +RG**2*PRDELP(JLON,JLEV)                                 &
     & *(ZT_INS0S_2A1(JLON,JLEV-1)*ZZZ(JLON,JLEV-1)*2.0_JPRB     &
     & *(ZDIFSI(JLON,JLEV-1)*ZCROSS(JLON,JLEV-1)             &
     & -ZDIFSI(JLON,JLEV)*ZCROSS(JLON,JLEV))                  &
     & /(PAPHIF(JLON,JLEV-1)-PAPHIF(JLON,JLEV))                  &
     & - ZT_INS0S_2A1(JLON,JLEV)*ZZZ(JLON,JLEV)*2.0_JPRB         &
     & * (ZDIFSI(JLON,JLEV)*ZCROSS(JLON,JLEV)                &
     & -ZDIFSI(JLON,JLEV+1)*ZCROSS(JLON,JLEV+1))             &
     & /(PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)))                 

  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
   
  ZSCG0(JLON,KLEV)=-PRDELP(JLON,KLEV)                  &
   &  *(ZXSTAM(JLON,KLEV-1)*ZDIFSO(JLON,KLEV-1)        &
   &  +ZXSTAP(JLON,KLEV)*ZDIFSO(JLON,KLEV))            &
   &  +RG**2*PRDELP(JLON,KLEV)                         &
   &  *(ZTSTAR2(JLON,KLEV-1)*ZZZ(JLON,KLEV-1)*2.0_JPRB &
   &  *(ZDIFSO(JLON,KLEV-1)-ZDIFSO(JLON,KLEV))         &
   &  /(PAPHIF(JLON,KLEV-1)-PAPHIF(JLON,KLEV)))        &
   &  +RG**2*PRDELP(JLON,KLEV)                         &
   &  *(ZT_INS0Q_2A1(JLON,KLEV-1)*ZZZ(JLON,KLEV-1)*2.0_JPRB &
   &  *(ZDIFQI(JLON,KLEV-1)*ZCROSSQ(JLON,KLEV-1)   &
   &  -ZDIFQI(JLON,KLEV)*ZCROSSQ(JLON,KLEV))     &
   &  /(PAPHIF(JLON,KLEV-1)-PAPHIF(JLON,KLEV)))
  ZQCG0(JLON,KLEV)=-PRDELP(JLON,KLEV)                  &
   &  *(ZXSTAMQ(JLON,KLEV-1)*ZDIFQO(JLON,KLEV-1)        &
   &  +ZXSTAPQ(JLON,KLEV)*ZDIFQO(JLON,KLEV))            &
   &  +RG**2*PRDELP(JLON,KLEV)                         &
   &  *(ZTSTAR2Q(JLON,KLEV-1)*ZZZ(JLON,KLEV-1)*2.0_JPRB &
   &  *(ZDIFQO(JLON,KLEV-1)-ZDIFQO(JLON,KLEV))         &
   &  /(PAPHIF(JLON,KLEV-1)-PAPHIF(JLON,KLEV)))        &
   &  +RG**2*PRDELP(JLON,KLEV)                         &
   &  *(ZT_INS0S_2A1(JLON,KLEV-1)*ZZZ(JLON,KLEV-1)*2.0_JPRB &
   &  *(ZDIFSI(JLON,KLEV-1)*ZCROSS(JLON,KLEV-1)    &
   &  -ZDIFSI(JLON,KLEV)*ZCROSS(JLON,KLEV))         &
   &  /(PAPHIF(JLON,KLEV-1)-PAPHIF(JLON,KLEV)))

ENDDO

!*
!     ------------------------------------------------------------------
!     IV - TOM solvers for S and Q

!     ELIMINATION AU SOMMET 
!     ELIMINATION AT THE TOP

DO JLON=KIDIA,KFDIA
  ZMUL             =1.0_JPRB/(1.0_JPRB &
   &  +ZIPOI(JLON,KTDIA)*ZKTROV2(JLON,KTDIA) &
   &  -PRDELP(JLON,KTDIA)*ZXSTAM(JLON,KTDIA))
  ZSUB1(JLON,KTDIA)=ZMUL*(ZIPOI(JLON,KTDIA)*ZKTROV2(JLON,KTDIA) &
   &  +PRDELP(JLON,KTDIA)*ZXSTAP(JLON,KTDIA+1))

  ZMULQ             =1.0_JPRB/(1.0_JPRB &
   &  +ZIPOI(JLON,KTDIA)*ZKTROV2Q(JLON,KTDIA) &
   &  -PRDELP(JLON,KTDIA)*ZXSTAMQ(JLON,KTDIA))
  ZSUB1Q(JLON,KTDIA)=ZMULQ*(ZIPOI(JLON,KTDIA)*ZKTROV2Q(JLON,KTDIA) &
   &  +PRDELP(JLON,KTDIA)*ZXSTAPQ(JLON,KTDIA+1))
   
  ZN1(JLON,KTDIA)  =ZMUL*ZSCG0(JLON,KTDIA)
  ZN2(JLON,KTDIA)  =ZMULQ*ZQCG0(JLON,KTDIA)
ENDDO

!     ELIMINATION POUR UNE COUCHE STANDARD
!     ELIMINATION FOR A STANDARD LEVEL

DO JLEV=KTDIA+1,KLEV-1
  DO JLON=KIDIA,KFDIA
    ZELIM           =ZIPOI(JLON,JLEV)*ZKTROV2(JLON,JLEV-1) &
     &  -PRDELP(JLON,JLEV)*ZXSTAM(JLON,JLEV-1)
    ZMUL            =1.0_JPRB/(1.0_JPRB-ZELIM*ZSUB1(JLON,JLEV-1) &
     &  +ZIPOI(JLON,JLEV)*(ZKTROV2(JLON,JLEV-1)+ZKTROV2(JLON,JLEV)) &
     &  -PRDELP(JLON,JLEV)*(ZXSTAM(JLON,JLEV)-ZXSTAP(JLON,JLEV)))
    ZSUB1(JLON,JLEV)=ZMUL*(ZIPOI(JLON,JLEV)*ZKTROV2(JLON,JLEV) &
     &  +PRDELP(JLON,JLEV)*ZXSTAP(JLON,JLEV+1))

    ZELIMQ           =ZIPOI(JLON,JLEV)*ZKTROV2Q(JLON,JLEV-1) &
     &  -PRDELP(JLON,JLEV)*ZXSTAMQ(JLON,JLEV-1)
    ZMULQ            =1.0_JPRB/(1.0_JPRB-ZELIMQ*ZSUB1Q(JLON,JLEV-1) &
     &  +ZIPOI(JLON,JLEV)*(ZKTROV2Q(JLON,JLEV-1)+ZKTROV2Q(JLON,JLEV)) &
     &  -PRDELP(JLON,JLEV)*(ZXSTAMQ(JLON,JLEV)-ZXSTAPQ(JLON,JLEV)))
    ZSUB1Q(JLON,JLEV)=ZMULQ*(ZIPOI(JLON,JLEV)*ZKTROV2Q(JLON,JLEV) &
     &  +PRDELP(JLON,JLEV)*ZXSTAPQ(JLON,JLEV+1))

    ZN1(JLON,JLEV)  =ZMUL*(ZSCG0(JLON,JLEV)+ZELIM*ZN1(JLON,JLEV-1))
    ZN2(JLON,JLEV)  =ZMULQ*(ZQCG0(JLON,JLEV)+ZELIMQ*ZN2(JLON,JLEV-1))
  ENDDO
ENDDO

!      CALCULS EN SURFACE.
!      SURFACE CALCULATIONS.

DO JLON=KIDIA,KFDIA
  ZELIM           =ZIPOI(JLON,KLEV)*ZKTROV2(JLON,KLEV-1) &
   &  -PRDELP(JLON,KLEV)*ZXSTAM(JLON,KLEV-1)
  ZMUL            =1.0_JPRB/(1.0_JPRB-ZELIM*ZSUB1(JLON,KLEV-1) &
   &  +ZIPOI(JLON,KLEV)*(ZKTROV2(JLON,KLEV-1)+ZKTROV2(JLON,KLEV)) &
   &  +PRDELP(JLON,KLEV)*ZXSTAP(JLON,KLEV))

  ZELIMQ           =ZIPOI(JLON,KLEV)*ZKTROV2Q(JLON,KLEV-1) &
   &  -PRDELP(JLON,KLEV)*ZXSTAMQ(JLON,KLEV-1)
  ZMULQ            =1.0_JPRB/(1.0_JPRB-ZELIMQ*ZSUB1Q(JLON,KLEV-1) &
   &  +ZIPOI(JLON,KLEV)*(ZKTROV2Q(JLON,KLEV-1)+ZKTROV2Q(JLON,KLEV)) &
   &  +PRDELP(JLON,KLEV)*ZXSTAPQ(JLON,KLEV))

  ZN1(JLON,KLEV)  =ZMUL*(ZSCG0(JLON,KLEV)+ZELIM*ZN1(JLON,KLEV-1))
  ZN2(JLON,KLEV)  =ZMULQ*(ZQCG0(JLON,KLEV)+ZELIMQ*ZN2(JLON,KLEV-1))
  ZCORS(JLON,KLEV)=ZN1(JLON,KLEV)
  ZCORQ(JLON,KLEV)=ZN2(JLON,KLEV)
ENDDO

!     SUBSTITUTION POUR UNE COUCHE STANDARD ET AU SOMMET.
!     BACK-SUBSTITUTION FOR A STANDART LAYER AND AT THE TOP.

DO JLEV=KLEV-1,KTDIA,-1
  DO JLON=KIDIA,KFDIA
    ZN1(JLON,JLEV)  =ZN1(JLON,JLEV)+ZSUB1(JLON,JLEV)*ZN1(JLON,JLEV+1)
    ZN2(JLON,JLEV)  =ZN2(JLON,JLEV)+ZSUB1Q(JLON,JLEV)*ZN2(JLON,JLEV+1)
    ZCORS(JLON,JLEV)=ZN1(JLON,JLEV)
    ZCORQ(JLON,JLEV)=ZN2(JLON,JLEV)
  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     V - Converting the TOM corrections of S and q back to fluxes.
!          Those fluxes are then added to the fluxes of vert. diffusion

!  Upper boundary condition and surface


 ZDIFTSTOM(KIDIA:KFDIA,KTDIA-1)=0.0_JPRB
 ZDIFTQTOM(KIDIA:KFDIA,KTDIA-1)=0.0_JPRB
 
 
 !  Standard levels
 
 DO JLEV=KTDIA,KLEV
   DO JLON=KIDIA,KFDIA
     ZDIFTSTOM(JLON,JLEV)=ZDIFTSTOM(JLON,JLEV-1) &
      &  -ZPOID(JLON,JLEV)*ZCORS(JLON,JLEV)
     ZDIFTQTOM(JLON,JLEV)=ZDIFTQTOM(JLON,JLEV-1) &
      &  -ZPOID(JLON,JLEV)*ZCORQ(JLON,JLEV)
     ZDIFTS2(JLON,JLEV)   =PDIFTS(JLON,JLEV)+ZDIFTSTOM(JLON,JLEV)
     ZDIFTQ2(JLON,JLEV)   =PDIFTQ(JLON,JLEV)+ZDIFTQTOM(JLON,JLEV)
     !store increments for next iteration
     ZDIFSI(JLON,JLEV)=ZCORS(JLON,JLEV)+ZDIFSO(JLON,JLEV)
     ZDIFQI(JLON,JLEV)=ZCORQ(JLON,JLEV)+ZDIFQO(JLON,JLEV)
     !cheat?
     ZDIFSI(JLON,JLEV)=ZDIFSO(JLON,JLEV)
     ZDIFQI(JLON,JLEV)=ZDIFQO(JLON,JLEV)
  ENDDO
 ENDDO

ENDDO ! JITER

 ! update for surface fluxes
 DO JLON=KIDIA,KFDIA
  ZD_DIFWQ=(ZDIFTQ2(JLON,KLEV)-PDIFTQ(JLON,KLEV))
  ZD_DIFWS=(ZDIFTS2(JLON,KLEV)-PDIFTS(JLON,KLEV))
 
  PDIFWS(JLON)=PDIFWS(JLON)+ZD_DIFWS
  PFCS(JLON)=PFCS(JLON)+ZD_DIFWS
 
  PDIFWQ(JLON)=PDIFWQ(JLON)+ZD_DIFWQ
  PFCLL(JLON)=PFCLL(JLON)+ZD_DIFWQ*PSC_FCLL(JLON)
  PFCLN(JLON)=PFCLN(JLON)+ZD_DIFWQ*PSC_FCLN(JLON)
  PFEVI(JLON)=PFEVI(JLON)+ZD_DIFWQ*PSC_FEVI(JLON)
  PFEVN(JLON)=PFEVN(JLON)+ZD_DIFWQ*PSC_FEVN(JLON)
  PFEVL(JLON)=PFEVL(JLON)+ZD_DIFWQ*(1.0_JPRB-PSC_FEVN(JLON))
 ENDDO
 
!update for fluxes in atmosphere
PDIFTS(KIDIA:KFDIA,KTDIA:KLEV)=ZDIFTS2(KIDIA:KFDIA,KTDIA:KLEV)
PDIFTQ(KIDIA:KFDIA,KTDIA:KLEV)=ZDIFTQ2(KIDIA:KFDIA,KTDIA:KLEV)

ENDIF

!*
!     ------------------------------------------------------------------
!     VI Retrieving the condensates fluxes.
!          The basic fluxes go at the same time from [s_L,q_t] to [s,q_v]

IF (LDIFCONS) THEN

  !     ON PASSE DES VARIABLES "CONSERVATIVES" AUX VARIABLES "SECHES".
  !     FROM "MOIST AND  CONSERVATIVE" VARIABLES TO THE "DRY" ONES.

  DO JLEV=KLEV-1,KTDIA,-1
    DO JLON=KIDIA, KFDIA
      ZDIFTQC           =ZCOEFA(JLON,JLEV)*(PDIFTQ(JLON,JLEV) &
       &  -ZALPHA1(JLON,JLEV)*PDIFTS(JLON,JLEV))
      PDIFTS(JLON,JLEV) =PDIFTS(JLON,JLEV)+ZDIFTQC*ZLVT(JLON,JLEV)
      PDIFTQ(JLON,JLEV) =PDIFTQ(JLON,JLEV)-ZDIFTQC
      PDIFTQL(JLON,JLEV)=ZDIFTQC*(1.0_JPRB-ZPICE(JLON,JLEV))
      PDIFTQI(JLON,JLEV)=ZDIFTQC*ZPICE(JLON,JLEV)
    ENDDO
  ENDDO

  DO JLON=KIDIA,KFDIA
    PDIFTQL(JLON,KTDIA-1)=0.0_JPRB
    PDIFTQI(JLON,KTDIA-1)=0.0_JPRB
    PDIFTQL(JLON,KLEV)   =0.0_JPRB
    PDIFTQI(JLON,KLEV)   =0.0_JPRB
  ENDDO

  DO JLON=KIDIA,KFDIA
    ZDUTQL(JLON,KLEV)=0.0_JPRB
    ZDUTQI(JLON,KLEV)=0.0_JPRB
  ENDDO

  DO JLEV=KLEV,KTDIA+1,-1
    DO JLON=KIDIA,KFDIA
      ZQL(JLON)          =PQL(JLON,JLEV)-ZIPOI(JLON,JLEV) &
       &  *(PDIFTQL(JLON,JLEV)+ZDUTQL(JLON,JLEV)-PDIFTQL(JLON,JLEV-1))
      ZSIGN              =MAX(0.0_JPRB,SIGN(1.0_JPRB,-PDIFTQL(JLON,JLEV-1)))
      ZDUTQL(JLON,JLEV-1)=ZSIGN*MIN(MAX(0.0_JPRB,-ZQL(JLON) &
       &  *ZPOID(JLON,JLEV)),MAX(0.0_JPRB,-PDIFTQL(JLON,JLEV-1)))
      ZQI(JLON)          =PQI(JLON,JLEV)-ZIPOI(JLON,JLEV) &
       &  *(PDIFTQI(JLON,JLEV)+ZDUTQI(JLON,JLEV)-PDIFTQI(JLON,JLEV-1))
      ZSIGN              =MAX(0.0_JPRB,SIGN(1.0_JPRB,-PDIFTQI(JLON,JLEV-1)))
      ZDUTQI(JLON,JLEV-1)=ZSIGN*MIN(MAX(0.0_JPRB,-ZQI(JLON) &
       &  *ZPOID(JLON,JLEV)),MAX(0.0_JPRB,-PDIFTQI(JLON,JLEV-1)))
    ENDDO
  ENDDO

  DO JLON=KIDIA,KFDIA
    ZDDTQL(JLON,KTDIA-1)=0.0_JPRB
    ZDDTQI(JLON,KTDIA-1)=0.0_JPRB
  ENDDO

  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZQL(JLON)        =PQL(JLON,JLEV)-ZIPOI(JLON,JLEV) &
       &  *(PDIFTQL(JLON,JLEV)-PDIFTQL(JLON,JLEV-1)-ZDDTQL(JLON,JLEV-1))
      ZSIGN            =MAX(0.0_JPRB,SIGN(1.0_JPRB,PDIFTQL(JLON,JLEV)))
      ZDDTQL(JLON,JLEV)=ZSIGN*MAX(MIN(0.0_JPRB,ZQL(JLON) &
       &  *ZPOID(JLON,JLEV)),MIN(0.0_JPRB,-PDIFTQL(JLON,JLEV)))
      ZQI(JLON)        =PQI(JLON,JLEV)-ZIPOI(JLON,JLEV) &
       &  *(PDIFTQI(JLON,JLEV)-PDIFTQI(JLON,JLEV-1)-ZDDTQI(JLON,JLEV-1))
      ZSIGN            =MAX(0.0_JPRB,SIGN(1.0_JPRB,PDIFTQI(JLON,JLEV)))
      ZDDTQI(JLON,JLEV)=ZSIGN*MAX(MIN(0.0_JPRB,ZQI(JLON) &
       &  *ZPOID(JLON,JLEV)),MIN(0.0_JPRB,-PDIFTQI(JLON,JLEV)))
    ENDDO
  ENDDO

  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZTH               =(PT(JLON,JLEV)+PT(JLON,JLEV+1))*0.5_JPRB
      ZDTQFL(JLON)      =ZDUTQL(JLON,JLEV)+ZDDTQL(JLON,JLEV)
      PDIFTQL(JLON,JLEV)=PDIFTQL(JLON,JLEV)+ZDTQFL(JLON)
      PDIFTQ(JLON,JLEV) =PDIFTQ(JLON,JLEV)-ZDTQFL(JLON)
      PDIFTS(JLON,JLEV) =PDIFTS(JLON,JLEV)+ZDTQFL(JLON)*FOLH(ZTH,0.0_JPRB)
      ZDTQFI(JLON)      =ZDUTQI(JLON,JLEV)+ZDDTQI(JLON,JLEV)
      PDIFTQI(JLON,JLEV)=PDIFTQI(JLON,JLEV)+ZDTQFI(JLON)
      PDIFTQ(JLON,JLEV) =PDIFTQ(JLON,JLEV)-ZDTQFI(JLON)
      PDIFTS(JLON,JLEV) =PDIFTS(JLON,JLEV)+ZDTQFI(JLON)*FOLH(ZTH,1.0_JPRB)
    ENDDO
  ENDDO

ELSE

  DO JLEV=KTDIA-1,KLEV
    DO JLON=KIDIA,KFDIA
      PDIFTQL(JLON,JLEV)=0.0_JPRB
      PDIFTQI(JLON,JLEV)=0.0_JPRB
    ENDDO
  ENDDO

ENDIF

!*
!     ------------------------------------------------------------------
!     VII  Upgrading the TKE increment with TOMs corrections influence

IF((.NOT.LCOEFK_FLX).AND.LCOEFK_TOMS.AND.(KSTEP>1)) THEN
 DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZTERM_II_1=-(PMN2_ES(JLON,JLEV-1)*ZDIFTSTOM(JLON,JLEV-1)+&
     & PMN2_EQ(JLON,JLEV-1)*ZDIFTQTOM(JLON,JLEV-1))/ZRHO(JLON,JLEV-1)
    ZTERM_II_2=-(PMN2_ES(JLON,JLEV)*ZDIFTSTOM(JLON,JLEV)+&
     & PMN2_EQ(JLON,JLEV)*ZDIFTQTOM(JLON,JLEV))/ZRHO(JLON,JLEV)

    PTENDPTKE(JLON,JLEV)=PTENDPTKE(JLON,JLEV)+&
    & TSPHY*(PALPH(JLON,JLEV)/PLNPR(JLON,JLEV)*&
    & ZTERM_II_1 + &
    & (1.0_JPRB-PALPH(JLON,JLEV)/PLNPR(JLON,JLEV))*&
    & ZTERM_II_2) 

  ENDDO
 ENDDO
ENDIF
!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ACDIFV3',1,ZHOOK_HANDLE)
END SUBROUTINE ACDIFV3
