!OPTIONS XOPT(NOEVAL)
!OPTION! -O nochg
SUBROUTINE ACMRIP( KIDIA,KFDIA,KLON,KTDIA,KLEV,KSTEP,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI,PAPHIF,PAPRS,PQ,PQL,PQI,PCP,&
 & PR,PT,PU,PV,PLMU,PFPLSH,PFPLCH,PNEB_SC, &
 & PDIFTQ,PDIFTS,PSHEAR, &
 & PTKE,PTTE, &
 ! - INPUT  1D .
 & PTS,PQSH,PRS,PCPS,PRTI,PGZ0,PCDN, &
 ! - INPUT  LOGIQUE .
 & LDCLS,&
 ! - OUTPUT 2D .
 & PMRIPP,PMRIFPP,&
 & PNEBCVPP,PNEBQ,PNBVNO,&
 & PFMTKE,PFTTKE,PF_EPS,PFUN_TTE,&
 & PFMTKE_S1,PIPRT_S1,PCHI3_S1,PPHIQ_S1,&
 & PAUTKE,PATTKE,PRCORR,&
 & PFHORM,PFHORH,&
 & PTH_FUN,PWW_FUN,&
 & PMRIMC,PMRICTERM,PMN2PP,&
 & PMN2_ES,PMN2_EQ,PMN2_DS,PMN2_DQ)

!**** *ACMRIP*  moist Ri' computation
!        *CALL* *ACMRIP*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA      : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL).
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".
! KSTEP      : NUMERO DU PAS DE TEMPS.

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPHI      : GEOPOTENTIEL AUX DEMI-NIVEAUX.
! PAPRS      : PRESSION AUX DEMI-NIVEAUX.
! PLMU       : LONGUEUR DE MELANGE POUR LA QUANTITE DE MOUVEMENT.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIEL AUX NIVEAUX DES COUCHES.
! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PQL        : EAU LIQUIDE
! PQI        : GLACE
! PCP        : SPECIFIC HEAT AT CONSTANT PRESSURE FOR AIR.
! PR         : CONSTANTE DES GAZ POUR L'AIR.
! PT         : TEMPERATURE.
! PTKE       : TKE
! PTTE       : TTE
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.
! - 1D (PROGNOSTIQUE) .

! PTS        : TEMPERATURE DE SURFACE.
! PQSH       : PSEUDO-HISTORICAL ARRAY FOR SURFACE MOISTURE
! PCPS       : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR AU SOL.

! - 1D (DIAGNOSTIQUE) .

! PGZ0       : G FOIS LA LONGUEUR DE RUGOSITE COURANTE.
! PCDN       : COEFFICIENT NEUTRE D'ECHANGE EN SURFACE.
! PRS        : CONSTANTE DES GAZ POUR L'AIR AU SOL.
! PRTI       : INVERSE DE R*T.
! - LOGIQUES .

! LDCLS      : DETERMINATION COMPLETE DES CARACTERISTIQUES DE SURFACE.

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------
! PMRIPP     : MOIST Ri '
! PMRIFPP     : MOIST Rif '
! PFMTKE     : STABILITY FUNCTION F_m
! PFMS1TKE     : STABILITY FUNCTION F_m computed from Ri_s1
! PFTTKE     : STABILITY FUNCTION F_h
! PAUTKE     : alpha_u for dry AF scheme
! PATTKE     : alpha_theta for dry AF scheme
! PRCORR     : MOIST GUSTINESS COEFFICIENT
! PWW_FUN    : F_ww - stability functions for TOMs par.
! PTH_FUN    : T_h - stability functions for TOMs par.
! PMRIPRT    : inverse Prt computed from Ri_s1 - for K_h computation after TKE solver
! PMRIMC     : M(C) from P. Marquet's moist Ri computation - for TKE correction after TOMs
! PMRICTERM  : Rv/R.F(C)-1/M(C).T/Tv from P. Marquet's moist Ri computation - for TKE correction after TOMs
! PFHORM     : STABILITY FUNCTION FOR HORIZONTAL DIFFUSION - MOMENTUM
! PFHORH     : STABILITY FUNCTION FOR HORIZONTAL DIFFUSION - HEAT
!---------------------------------------------------------------


! -   ARGUMENTS IMPLICITES.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY1/
! COMMON/YOMPHY2/
! COMMON/FCTTRM /

!-----------------------------------------------------------------------

!     Methode.
!     --------

!     Auteur.
!     -------
!        2012-07, I. Bastak-Duran

!     Modifications.
!     --------------
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : LRRGUST  ,L3MT  ,LSTRAPRO,&
 & LCOEFK_RIS,&
 & LCOEFK_RIH, LCOEFK_PTTE,LCOEFK_SCQ,&
 & CGTURS,LCOEFK_FLX,LCOEFK_F1
USE YOMCST   , ONLY : RG       ,RD       ,RKAPPA    ,&
 & RV       ,RCPD     ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD, RATM, RPI
USE YOMPHY0  , ONLY : ALMAV    , VKARMN  ,ENT_LAMBDA,&
 & UTILGUST ,RRGAMMA  ,RRSCALE  , GCISMIN ,   NUPTKE,&
 & C_EPSILON,ETKE_RIFC    ,ETKE_R  , C3TKEFREE      ,&
 & ETKE_OLAM,ETKE_LAM1,ETKE_LAM2,ETKE_LAM3,ETKE_LAM4,&
 & ETKE_LAM0,EFB_UR,EFB_AZ0,REFB_1,REFB_2,REFB_3
USE YOMPHY2  , ONLY : XMULAF
USE YOMQNSE  , ONLY :CHIHQNSE_S1,CHIHQNSE_S2,CHIHQNSE_S3,CHIHQNSE_S4, &
 & CHIHQNSE_U1,CHIHQNSE_U2,CHIHQNSE_U3,CHIHQNSE_U4, &
 & PHIHQNSE_S1,PHIHQNSE_S2,PHIHQNSE_S3,PHIHQNSE_S4,PHIHQNSE_S5,PHIHQNSE_S6, &
 & PHIHQNSE_U1,PHIHQNSE_U2,PHIHQNSE_U3,PHIHQNSE_U4,&
 & RQNSE_S0,RQNSE_GS1,RQNSE_GS2,RQNSE_GS3,RQNSE_GS4,&
 & RQNSE_GU1,RQNSE_GU2
USE YOMLOUIS  , ONLY :                                   &
 & PLOUIS_S0,PLOUIS_GS1,PLOUIS_GS2,PLOUIS_GS3,PLOUIS_GS4,&
 & PLOUIS_GU1,PLOUIS_GU2,                                &
 & RLOUIS_S0,RLOUIS_GS1,RLOUIS_GS2,RLOUIS_GS3,RLOUIS_GS4,&
 & RLOUIS_GU1,RLOUIS_GU2
 
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI    (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF   (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS    (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ       (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQL      (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQI      (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP      (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR       (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT       (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTKE     (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTTE     (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU       (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV       (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSH   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCH   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEB_SC  (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSHEAR  (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTS  (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIFTQ  (KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLMU     (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS      (KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSH     (KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRS      (KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPS     (KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTI     (KLON) 
LOGICAL           ,INTENT(IN)    :: LDCLS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0     (KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDN     (KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMRIPP   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMRIFPP   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFMTKE_S1 (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PIPRT_S1(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCHI3_S1 (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PPHIQ_S1(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMN2PP   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMN2_ES   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMN2_EQ   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMN2_DS (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMN2_DQ (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFMTKE(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFTTKE(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAUTKE(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PATTKE(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PF_EPS   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFUN_TTE   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRCORR   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWW_FUN   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTH_FUN   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNEBCVPP (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNEBQ (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNBVNO   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMRIMC  (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMRICTERM  (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHORM   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHORH   (KLON,0:KLEV)
!-----------------------------------------------------------------------

REAL(KIND=JPRB) :: ZU(KLON,0:KLEV),ZRTI(KLON,0:KLEV),ZDPHIA(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZCISA(KLON,0:KLEV),ZQTOTA(KLON,KLEV),ZSLAMA(KLON,KLEV)
REAL(KIND=JPRB) :: ZPHI3TA(KLON,0:KLEV),ZCHI3TA(KLON,0:KLEV),ZPHIQTA(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZRITKE_S1(KLON,0:KLEV),ZRIFTKE_S1(KLON,0:KLEV)

INTEGER(KIND=JPIM) :: JLEV, JLON, ITTYPE, JITER
INTEGER(KIND=JPIM) :: INITER

LOGICAL :: LLAFGD, LL3DTURB, LLM123, LLRISH

REAL(KIND=JPRB) :: ZDPHI, ZGLU, ZZ, ZC0,ZC1
REAL(KIND=JPRB) :: ZEPS, ZQTOT, ZQTOTI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: ZRITKE,ZRIFTKE,ZSTA,ZRITKE_U,ZRITKE_F(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZPHI3TKE,ZCHI3TKE,ZPHI3HTKE,ZFSFTKE
REAL(KIND=JPRB) :: ZFMTKE_DX(KLON,0:KLEV),ZFHTKE_DX(KLON,0:KLEV),&
&ZFMTKE(KLON,0:KLEV),ZFTTKE(KLON,0:KLEV),ZSIG_AF(KLON,0:KLEV),&
&ZFFTKE_DX(KLON,0:KLEV),ZFFTKE(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZDCHI3TKE(KLON,0:KLEV), ZDPHI3TKE(KLON,0:KLEV),&
& ZDRIFTKE(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZDCHI3TKE_U, ZDPHI3TKE_U,ZDRIFTKE_U,ZDRRI_U,ZDPRI_U
REAL(KIND=JPRB) :: Z_ETKE_R,Z_ETKE_P,Z_ETKE_RU,Z_ETKE_PU
REAL(KIND=JPRB) :: ZETKE_R(KLON,0:KLEV),ZETKE_P(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZCD,ZKUROV,ZFP
REAL(KIND=JPRB) :: ZAU,ZAT

REAL(KIND=JPRB) :: ZS_FC, ZS_MC
REAL(KIND=JPRB) :: ZS_LV, &
 &  ZS_LS, ZS_TS, ZS_L, &
 &  ZS_Q,ZS_QT,&
 &  ZS_EV,ZS_ES,ZS_E,&
 &  ZS_N2_0,ZS_N2_1, ZS_QLI
REAL(KIND=JPRB) :: ZLAMCP,ZS_T,ZS_DLEV,ZS_DLES,ZS_DLE,ZS_RV
REAL(KIND=JPRB) :: ZS_DC(KLON,0:KLEV),ZS_NX4(KLON,0:KLEV),ZS_B3X(KLON,0:KLEV),&
 &ZS_A1(KLON,0:KLEV),ZS_AR(KLON,0:KLEV),ZS_X0(KLON,0:KLEV),ZS_X1(KLON,0:KLEV),&
 &ZS_X2(KLON,0:KLEV),ZS_X3(KLON,0:KLEV),ZFCN(KLON,0:KLEV),ZS_N2(KLON,0:KLEV) 

REAL(KIND=JPRB) :: ZCP, ZNEBQ
REAL(KIND=JPRB) :: ZS_A,ZS_B,ZS_C,  &
 & ZS_AA1, ZS_AA2, ZS_AA3, ZS_AA4
REAL(KIND=JPRB) :: ZS_CTHIRD,ZS_CW,ZS_CP,ZS_CQ,ZS_CDIS,ZS_CPHI,ZS_ZSOL,&
 & ZS_CU1,ZS_CU2,ZS_CU3, &
 & ZS_CX1,ZS_CX2,ZS_CX3
REAL(KIND=JPRB) ::ZPHI3_S1(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZMRIFPP,ZMRIFPPI
REAL(KIND=JPRB) :: ZN2S_CP,ZTKE_LAM5T,Z2AZ0,Z2THIRD
REAL(KIND=JPRB) :: ZPHIQTKE,ZDPHIQTKE,ZDRRI,ZDPRI,ZEFB_UP
REAL(KIND=JPRB) :: ZCN, ZCN_D,ZN2DM,ZN2DMDC
REAL(KIND=JPRB) :: ZEPS4,ZEPS8,ZSIG

#include "fcttrm.h"


!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACMRIP',0,ZHOOK_HANDLE)

!*
!     ------------------------------------------------------------------
!     I - CALCUL DES PARAMETRES DERIVES ET CONSTANTE DE SECURITE

!         COMPUTATION OF DERIVED PARAMETERS AND SECURITY CONSTANTS 

LLAFGD=XMULAF /= 0.0_JPRB
LL3DTURB=.FALSE.

ZGLU=RG*ALMAV

Z2AZ0=2.0_JPRB/3.0_JPRB*(1.0_JPRB-3.0_JPRB*ETKE_LAM3-ETKE_LAM2)

ZCP=ETKE_OLAM/ETKE_LAM0/C3TKEFREE
ZS_CTHIRD=1.0_JPRB/3.0_JPRB
Z2THIRD=2.0_JPRB/3.0_JPRB

ZLAMCP=RCPV/RCPD-1.0_JPRB
ZC0=RCPD-RCPV
ZC1=RCPD-RCPV+ENT_LAMBDA*RCPD
! security constant
ZEPS=EPSILON(1.0_JPRB)*100.0_JPRB
ZEPS4=1e-4_JPRB
ZEPS8=1e-8_JPRB

ZQTOT=0.0_JPRB
ZQTOTI=0.0_JPRB

!full levels
DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZQTOTA(JLON,JLEV)=PQ(JLON,JLEV)+PQL(JLON,JLEV)+PQI(JLON,JLEV)
    ZSLAMA(JLON,JLEV)=RCPD*PT(JLON,JLEV)*(1.0_JPRB+ZQTOTA(JLON,JLEV)*ZLAMCP)+&
    & PAPHIF(JLON,JLEV)-&
    & (PQL(JLON,JLEV)*RLV(PT(JLON,JLEV))+PQI(JLON,JLEV)*RLS(PT(JLON,JLEV)))
  ENDDO
ENDDO

IF     (TRIM(CGTURS) == 'RMC') THEN
  ITTYPE=1
ELSEIF (TRIM(CGTURS) == 'MD1') THEN
  ITTYPE=2
ELSEIF (TRIM(CGTURS) == 'MD2') THEN
  ITTYPE=3
ELSEIF (TRIM(CGTURS) == 'QNSE') THEN
  ITTYPE=4
ELSEIF (TRIM(CGTURS) == 'EFB') THEN
  ITTYPE=5
ELSEIF (TRIM(CGTURS) == 'LOUIS') THEN
  ITTYPE=6
ENDIF

LLM123=(ITTYPE == 1).OR.(ITTYPE == 2).OR.(ITTYPE == 3)
LLRISH=(.NOT.LCOEFK_RIS).OR.LCOEFK_RIH
! Number of the SCC solver iterations
INITER=3

!*
!     ------------------------------------------------------------------
!     II - SURFACE CALCULATION OF RICHARDSON NUMBER

DO JLON=KIDIA,KFDIA
! ZDPHIA    : EPAISSEUR EN GEOPOTENTIEL DU NIVEAU DE SURFACE.
!           : GEOPOTENTIAL THICKNESS OF THE SURFACE LEVEL.
  ZDPHI=PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)
  ZDPHIA(JLON,KLEV)=ZDPHI

! ZU        : MODULE DU VENT DE SURFACE.
!            : SURFACE WIND SPEED.

  ZCISA(JLON,KLEV)=PU(JLON,KLEV)**2+PV(JLON,KLEV)**2+&
   &(GCISMIN*ZDPHI/RG)**2
  ZU(JLON,KLEV)=SQRT(ZCISA(JLON,KLEV))

! ZSTA      : APPROXIMATION DE DELTA(PHI)*DELTA(LN(THETA)).
!            : APPROXIMATION OF DELTA(PHI)*DELTA(LN(THETA)).
  ZSTA=ZDPHI*(PR(JLON,KLEV)*PT(JLON,KLEV)+RKAPPA &
   & *ZDPHI-PRS(JLON)*PTS(JLON))*PRTI(JLON)

!Ri_dry = Ri*,Ri** at surface
  PMRIPP(JLON,KLEV)=ZSTA/ZCISA(JLON,KLEV)
ENDDO

IF(LLRISH.AND.KSTEP > 0) THEN
  DO JLON=KIDIA,KFDIA
    ZDPHI=ZDPHIA(JLON,KLEV)
    ZS_Q =(PQ(JLON,KLEV)+PQSH(JLON))/2.0_JPRB
    ZS_QT =(PQ(JLON,KLEV)+PQL(JLON,KLEV)+PQI(JLON,KLEV)+PQSH(JLON))/2.0_JPRB
    ZS_QLI =PQL(JLON,KLEV)+PQI(JLON,KLEV)
    !Ri_m
    ZSTA=ZDPHI*((PT(JLON,KLEV)*PCP(JLON,KLEV)+&
     & ZDPHI-PCPS(JLON)*PTS(JLON))/&
     & (PT(JLON,KLEV)*PCP(JLON,KLEV)+&
     & ZDPHI+PCPS(JLON)*PTS(JLON))*2.0_JPRB+&
     & (ZC0/PCP(JLON,KLEV)-1.0_JPRB/(1.0_JPRB-ZS_QT)+&
     & 2.0_JPRB*RV/(PR(JLON,KLEV)+PRS(JLON))*&
     & (1.0_JPRB-ZS_QLI)/(1.0_JPRB-ZS_Q-ZS_QLI))*&
     & (PQ(JLON,KLEV)-PQSH(JLON)))

    PMRIPP(JLON,KLEV)=ZSTA/ZCISA(JLON,KLEV)
    !Ri_s
    ZSTA=ZDPHI*((PT(JLON,KLEV)*PCP(JLON,KLEV)+&
    & ZDPHI-PCPS(JLON)*PTS(JLON))/&
    & (PT(JLON,KLEV)*PCP(JLON,KLEV)+&
    & ZDPHI+PCPS(JLON)*PTS(JLON))*2.0_JPRB+&
    & (ZC1*2.0_JPRB/&
    & (PCP(JLON,KLEV)+PCPS(JLON))-1.0_JPRB/(1.0_JPRB-ZS_QT))*&
    & (PQ(JLON,KLEV)-PQSH(JLON)))

    ZRITKE_S1(JLON,KLEV)=ZSTA/ZCISA(JLON,KLEV)
    !protection against division by 0
    ZRITKE_S1(JLON,KLEV)=SIGN(MAX(ZEPS,&
    &ABS(ZRITKE_S1(JLON,KLEV))),ZRITKE_S1(JLON,KLEV))
  ENDDO
ENDIF

DO JLON=KIDIA,KFDIA
 !protection against division by 0
  PMRIPP(JLON,KLEV)=SIGN(MAX(ZEPS,&
  &ABS(PMRIPP(JLON,KLEV))),PMRIPP(JLON,KLEV))

  ZDPHI=ZDPHIA(JLON,KLEV)
  PMN2_ES(JLON,KLEV)=RG/&
      & (PT(JLON,KLEV)*PCP(JLON,KLEV)+&
      & ZDPHI+PCPS(JLON)*PTS(JLON))*2.0_JPRB
  PMN2_EQ(JLON,KLEV)=RG*(ZC0/PCP(JLON,KLEV)&
      & +2.0_JPRB*(RV-RD)/(PR(JLON,KLEV)+PRS(JLON)))

  PMN2_DS(JLON,KLEV)=RG/ZDPHI*(PT(JLON,KLEV)*PCP(JLON,KLEV)+&
      & ZDPHI-PCPS(JLON)*PTS(JLON))
  PMN2_DQ(JLON,KLEV)=RG/ZDPHI*&
      & (PQ(JLON,KLEV)-PQSH(JLON))

  PNEBCVPP(JLON,KLEV)=0.0_JPRB
  PNEBQ(JLON,KLEV)=0.0_JPRB
  PMRIMC(JLON,KLEV)=1.0_JPRB
  PMRICTERM(JLON,KLEV)=1.0_JPRB
  PMRIMC(JLON,KTDIA-1)=1.0_JPRB
  PMRICTERM(JLON,KTDIA-1)=1.0_JPRB
ENDDO

IF(LLRISH .AND. KSTEP == 0) THEN
  DO JLON=KIDIA,KFDIA
    ZRITKE_S1(JLON,KLEV)=PMRIPP(JLON,KLEV)
  ENDDO
ENDIF

IF (LLM123) THEN
  !modified CCH02 or RMC01
  Z_ETKE_R=ETKE_R
  Z_ETKE_P=ETKE_RIFC
  IF(LCOEFK_RIH) THEN
    DO JLON=KIDIA,KFDIA
    ! Shallow convection with hybrid Ri - Ri_m/Ri_s1
    ! Computation Rif_s1 from Ri_s1
    !Ri=Ri_s1
      ZRITKE=ZRITKE_S1(JLON,KLEV)
      ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB
      ZRIFTKE_S1(JLON,KLEV)=-(sqrt((Z_ETKE_R&
     & *(ZRITKE*C3TKEFREE+Z_ETKE_P))**2&
     & -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
     & -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
    !security
      ZRIFTKE_S1(JLON,KLEV)=MIN(ZRIFTKE_S1(JLON,KLEV),ETKE_RIFC-ZEPS8)
      PCHI3_S1(JLON,KLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV)/Z_ETKE_R)&
     & /(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV))
      ZPHI3_S1(JLON,KLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV)/Z_ETKE_P)&
     & /(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV))
    ENDDO
  ENDIF
  !Ri ->Rif surface
  DO JLON=KIDIA,KFDIA
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

    PMRIFPP(JLON,KLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
   & -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2)    &
   & -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
    !security
    PMRIFPP(JLON,KLEV)=MIN(PMRIFPP(JLON,KLEV),ETKE_RIFC-ZEPS8)
    ZETKE_R(JLON,KLEV)=Z_ETKE_R
    ZETKE_P(JLON,KLEV)=Z_ETKE_P

    PMN2PP(JLON,KLEV)=PMRIPP(JLON,KLEV)*(ZCISA(JLON,KLEV) &
     & /(ZDPHIA(JLON,KLEV)/RG)**2)
    ZRITKE_F(JLON,KLEV)=PMRIPP(JLON,KLEV)
  ENDDO
ENDIF

IF (ITTYPE == 4) THEN
  IF (LCOEFK_RIH) THEN
    ! Shallow convection with hybrid Ri - Ri_m/Ri_s1
    ! Computation Rif_s1 from Ri_s1
    !Ri=Ri_s1
    DO JLON=KIDIA,KFDIA
      ZRITKE=ZRITKE_S1(JLON,KLEV)
      ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB
      !QNSE fit
      !UNSTABLE
      Z_ETKE_RU=(RQNSE_GU1*ZRITKE+RQNSE_S0)/&
       &(RQNSE_GU2*ZRITKE+1.0_JPRB)
      !STABLE
      Z_ETKE_R=(ZRITKE*(RQNSE_GS1*ZRITKE+&
       & RQNSE_GS2)+RQNSE_S0)/&
       &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
      Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
      Z_ETKE_P=ETKE_RIFC

      ZRIFTKE_S1(JLON,KLEV)=-(sqrt((Z_ETKE_R&
       & *(ZRITKE*C3TKEFREE+Z_ETKE_P))**2&
       & -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
       & -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
      !security
      ZRIFTKE_S1(JLON,KLEV)=MIN(ZRIFTKE_S1(JLON,KLEV),ETKE_RIFC-ZEPS8)

      PCHI3_S1(JLON,KLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV)/Z_ETKE_R)&
       & /(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV))
      ZPHI3_S1(JLON,KLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV)/Z_ETKE_P)&
       & /(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV))
    ENDDO
  ENDIF ! LCOEFK_RIH

  DO JLON=KIDIA,KFDIA
    !Ri ->Rif surface
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB  
    !  UNSTABLE
    Z_ETKE_RU=(RQNSE_GU1*ZRITKE+RQNSE_S0)/&
     &(RQNSE_GU2*ZRITKE+1.0_JPRB)
    !  STABLE
    Z_ETKE_R=(ZRITKE*(RQNSE_GS1*ZRITKE+&
     & RQNSE_GS2)+RQNSE_S0)/&
     &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
    Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
    Z_ETKE_P=ETKE_RIFC

    PMRIFPP(JLON,KLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
     &              -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2)    &
     &   -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
    !security
    PMRIFPP(JLON,KLEV)=MIN(PMRIFPP(JLON,KLEV),ETKE_RIFC-ZEPS8)
    ZETKE_R(JLON,KLEV)=Z_ETKE_R
    ZETKE_P(JLON,KLEV)=Z_ETKE_P

    PMN2PP(JLON,KLEV)=PMRIPP(JLON,KLEV)*(ZCISA(JLON,KLEV)/(ZDPHIA(JLON,KLEV)/RG)**2)
    ZRITKE_F(JLON,KLEV)=PMRIPP(JLON,KLEV)
  ENDDO

ENDIF ! ITTYPE==4 QNSE

IF (ITTYPE == 5) THEN
  !EFB
  !  UNSTABLE
  Z_ETKE_RU=EFB_UR
  !  STABLE
  ZS_AA4 = EFB_AZ0*REFB_2

  IF (LCOEFK_RIH) THEN
    ! Shallow convection with hybrid Ri - Ri_m/Ri_s1
    ! Computation Rif_s1 from Ri_s1
    !Ri=Ri_s1
    DO JLON=KIDIA,KFDIA
      ZRITKE=ZRITKE_S1(JLON,KLEV)
      ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

      ZS_AA3 = EFB_AZ0*(REFB_3-REFB_1-ZRITKE*REFB_2*C3TKEFREE)&
       &      - ETKE_OLAM*REFB_1*ZRITKE
      ZS_AA2 = C3TKEFREE*EFB_AZ0*ZRITKE*(REFB_1-REFB_3)&
       &      - REFB_3*EFB_UR*(ETKE_OLAM*ZRITKE+EFB_AZ0)
      ZS_AA1 = ZRITKE*EFB_AZ0*EFB_UR*REFB_3*C3TKEFREE

      ! cubic problem only
      ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
      ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
      ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
      ZS_CDIS=ZS_CQ**2+ZS_CP

      ZS_ZSOL = +1.E+99_JPRB
      IF (ZS_CDIS < 0.0_JPRB) THEN
        !         three real solutions!
        !         Confine the argument of ACOS to the interval [-1;1]!
        ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
        ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
        ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
        ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
        ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
        ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
        ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
         &  MIN(ZS_CU2,ZS_CU3))
        ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
        ZS_ZSOL = MIN(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
      ELSE
        !      only one real solution!
        ZS_CPHI=SQRT(ZS_CDIS)
        ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
         &  SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
        ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
      ENDIF

      ZRIFTKE=ZS_ZSOL
      Z_ETKE_R=(ZRIFTKE*REFB_1+EFB_UR*REFB_3)/&
       &(ZRIFTKE*REFB_2+REFB_3)
      Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
      Z_ETKE_P=Z_ETKE_R&
       & /(ETKE_OLAM*Z_ETKE_R/(EFB_AZ0*C3TKEFREE)+1.0_JPRB)

      ZRIFTKE_S1(JLON,KLEV)=-(sqrt((Z_ETKE_R&
       & *(ZRITKE*C3TKEFREE+Z_ETKE_P))**2&
       & -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
       & -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
      !security
      ZRIFTKE_S1(JLON,KLEV)=MIN(ZRIFTKE_S1(JLON,KLEV),ETKE_RIFC-ZEPS8)

      PCHI3_S1(JLON,KLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV)/Z_ETKE_R)&
       & /(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV))
      ZPHI3_S1(JLON,KLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV)/Z_ETKE_P)&
       & /(1.0_JPRB-ZRIFTKE_S1(JLON,KLEV))
    ENDDO
  ENDIF ! LCOEFK_RIH

  DO JLON=KIDIA,KFDIA
    !Ri ->Rif surface
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB  

    ZS_AA3 = EFB_AZ0*(REFB_3-REFB_1-ZRITKE*REFB_2*C3TKEFREE)&
     &      - ETKE_OLAM*REFB_1*ZRITKE
    ZS_AA2 = C3TKEFREE*EFB_AZ0*ZRITKE*(REFB_1-REFB_3)&
     &      - REFB_3*EFB_UR*(ETKE_OLAM*ZRITKE+EFB_AZ0)
    ZS_AA1 = ZRITKE*EFB_AZ0*EFB_UR*REFB_3*C3TKEFREE

    ! cubic problem only
    ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
    ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
    ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
    ZS_CDIS=ZS_CQ**2+ZS_CP

    ZS_ZSOL = +1.E+99_JPRB

    IF (ZS_CDIS < 0.0_JPRB) THEN
    !         three real solutions!
    !         Confine the argument of ACOS to the interval [-1;1]!
      ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
      ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
      ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
      ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
      ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
      ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
      ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
       &  MIN(ZS_CU2,ZS_CU3))
      ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
      ZS_ZSOL = MIN(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
    ELSE
    !      only one real solution!
      ZS_CPHI=SQRT(ZS_CDIS)
      ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
       &  SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
      ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
    ENDIF

    ZRIFTKE=ZS_ZSOL
    Z_ETKE_R=(ZRIFTKE*REFB_1+EFB_UR*REFB_3)/&
     &(ZRIFTKE*REFB_2+REFB_3)
    Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
    Z_ETKE_P=Z_ETKE_R&
     & /(ETKE_OLAM*Z_ETKE_R/(EFB_AZ0*C3TKEFREE)+1.0_JPRB)

    PMRIFPP(JLON,KLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
     &              -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2)    &
     &   -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
    !security
    PMRIFPP(JLON,KLEV)=MIN(PMRIFPP(JLON,KLEV),ETKE_RIFC-ZEPS8)
    ZETKE_R(JLON,KLEV)=Z_ETKE_R
    ZETKE_P(JLON,KLEV)=Z_ETKE_P

    PMN2PP(JLON,KLEV)=PMRIPP(JLON,KLEV)*(ZCISA(JLON,KLEV)/(ZDPHIA(JLON,KLEV)/RG)**2)
    ZRITKE_F(JLON,KLEV)=PMRIPP(JLON,KLEV)
  ENDDO

ENDIF ! ITTYPE==5 EFB

IF (ITTYPE == 6) THEN
  IF (LCOEFK_RIH) THEN
    CALL ABOR1('ACMRIP: LCOEFK_RIH=T AND CGTURS=LOUIS is not coded') 
    !DO JLON=KIDIA,KFDIA
    !ENDDO
  ENDIF ! LCOEFK_RIH

  DO JLON=KIDIA,KFDIA
    !Ri ->Rif surface
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB  
    !  UNSTABLE
    Z_ETKE_RU=(RLOUIS_GU1*ZRITKE+RLOUIS_S0)/&
     &(RLOUIS_GU2*ZRITKE+1.0_JPRB)
    !  STABLE
    Z_ETKE_R=(ZRITKE*(RLOUIS_GS1*ZRITKE+&
     & RLOUIS_GS2)+RLOUIS_S0)/&
     &(ZRITKE*(RLOUIS_GS3*ZRITKE+RLOUIS_GS4)+1.0_JPRB)
    Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU

    !UNSTABLE
    Z_ETKE_PU=(PLOUIS_GU1*ZRITKE+PLOUIS_S0)/&
     &(PLOUIS_GU2*ZRITKE+1.0_JPRB)
    !STABLE
    Z_ETKE_P=(ZRITKE*(PLOUIS_GS1*ZRITKE+&
     & PLOUIS_GS2)+PLOUIS_S0)/&
     &(ZRITKE*(PLOUIS_GS3*ZRITKE+PLOUIS_GS4)+1.0_JPRB)
    Z_ETKE_P=ZSIG*Z_ETKE_P+(1.0_JPRB-ZSIG)*Z_ETKE_PU

    PMRIFPP(JLON,KLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
     &              -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2)    &
     &   -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
    !security
    PMRIFPP(JLON,KLEV)=MIN(PMRIFPP(JLON,KLEV),ETKE_RIFC-ZEPS8)
    ZETKE_R(JLON,KLEV)=Z_ETKE_R
    ZETKE_P(JLON,KLEV)=Z_ETKE_P

    PMN2PP(JLON,KLEV)=PMRIPP(JLON,KLEV)*(ZCISA(JLON,KLEV)/(ZDPHIA(JLON,KLEV)/RG)**2)
    ZRITKE_F(JLON,KLEV)=PMRIPP(JLON,KLEV)
  ENDDO

ENDIF ! ITTYPE==6 LOUIS

!     CALCULS DE UN SUR G FOIS LA FREQUENCE DE BRUNT-VAISALA DIVISEE PAR
!     LA DENSITE LE TOUT AU CARRE DE MEME QUE LA DENSITE (POUR G.W.D.).
!     COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY DIVIDED
!     BY DENSITY THE WHOLE BEING SQUARED AS WELL AS DENSITY (FOR G.W.D.)
IF (LDCLS) THEN
  DO JLON=KIDIA,KFDIA
    PNBVNO(JLON,KLEV)=PMRIPP(JLON,KLEV)*ZCISA(JLON,KLEV)/(PAPRS(JLON,KLEV) &
   &   *PRTI(JLON)*ZDPHIA(JLON,KLEV))**2
  ENDDO
ENDIF

!new code
!*
!     ------------------------------------------------------------------
!     III - BOUCLE PASSIVE SUR LES NIVEAUX VERTICAUX.

!           PASSIVE LOOP ON VERTICAL LEVELS.

DO JLEV=KTDIA,KLEV-1

!*
!     ------------------------------------------------------------------
!     IV - RICHARDSON NUMBER COMPUTATION

  DO JLON=KIDIA,KFDIA
!     CISAILLEMENT DE VENT.
!     WIND SHEAR.

    ZRTI(JLON,JLEV)=2.0_JPRB/(PR(JLON,JLEV)*PT(JLON,JLEV)&
    &+PR(JLON,JLEV+1)*PT(JLON,JLEV+1))

    ZZ=VKARMN*(0.5_JPRB*(PAPHIF(JLON,JLEV)+PAPHIF(JLON,JLEV+1)) &
     & -PAPHI(JLON,KLEV)+PGZ0(JLON))
    ZDPHIA(JLON,JLEV)=PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)
    ZCISA(JLON,JLEV)=(PU(JLON,JLEV)-PU(JLON,JLEV+1))**2 &
     &  +(PV(JLON,JLEV)-PV(JLON,JLEV+1))**2 &
     &  +(GCISMIN*ZDPHIA(JLON,JLEV)/(RG*(1.0_JPRB+ZZ/ZGLU)))**2
    ZU(JLON,JLEV)=SQRT(ZCISA(JLON,JLEV))

    ZS_T =(PT(JLON,JLEV)+PT(JLON,JLEV+1))/2.0_JPRB
    ZRITKE_S1(JLON,JLEV)=(1.0_JPRB/((PCP(JLON,JLEV)+PCP(JLON,JLEV+1))/&
    & 2.0_JPRB*ZS_T)*&
    & (ZSLAMA(JLON,JLEV)-ZSLAMA(JLON,JLEV+1)+ZS_T*ZC1*&
    & (ZQTOTA(JLON,JLEV)-ZQTOTA(JLON,JLEV+1))))/&
    & (ZCISA(JLON,JLEV)/ZDPHIA(JLON,JLEV))
    !protection against division by 0
    ZRITKE_S1(JLON,JLEV)=SIGN(MAX(ZEPS,&
    &ABS(ZRITKE_S1(JLON,JLEV))),ZRITKE_S1(JLON,JLEV))
  ENDDO
ENDDO

!preparation for SCC compuation
DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA


    ZS_T =(PT(JLON,JLEV)+PT(JLON,JLEV+1))*0.5_JPRB
    ZS_Q =(PQ(JLON,JLEV)+PQ(JLON,JLEV+1))*0.5_JPRB
    ZS_QT =(ZQTOTA(JLON,JLEV)+ZQTOTA(JLON,JLEV+1))*0.5_JPRB
    ZS_RV=ZS_Q/(1.0_JPRB-ZS_QT)

    ZS_TS=SIGN(0.5_JPRB,ZS_T-RTT)+0.5_JPRB

    ZS_LV=RLV(ZS_T)
    ZS_LS=RLS(ZS_T)
    ZS_L=ZS_LV*ZS_TS+(1.0_JPRB-ZS_TS)*ZS_LS

    ZS_EV=FOEW(ZS_T,0.0_JPRB)
    ZS_ES=FOEW(ZS_T,1.0_JPRB)
    ZS_E=ZS_EV*ZS_TS+(1.0_JPRB-ZS_TS)*ZS_ES

    ZS_DLEV=FODLEW(ZS_T,0.0_JPRB)
    ZS_DLES=FODLEW(ZS_T,1.0_JPRB)
    ZS_DLE=ZS_DLEV*ZS_TS+(1.0_JPRB-ZS_TS)*ZS_DLES

    ZS_DC(JLON,JLEV)=MAX(ZS_T/(PAPRS(JLON,JLEV)-ZS_E)&
    & *ZS_E*ZS_DLE,ZEPS8)

    ZS_X0(JLON,JLEV)=-ZC0/(PCP(JLON,JLEV)+PCP(JLON,JLEV+1))*2.0_JPRB
    ZS_X1(JLON,JLEV) =(PCP(JLON,JLEV)*PT(JLON,JLEV)&
    & +PCP(JLON,JLEV+1)*PT(JLON,JLEV+1))*0.5_JPRB
    ZS_X2(JLON,JLEV)=1.0_JPRB/(ZS_QT-1.0_JPRB)
    ZS_X3(JLON,JLEV)=(1.0_JPRB+ZS_RV)*RV&
    & /(RD*(1.0_JPRB-ZS_QT)+RV*ZS_Q)

    ZS_AR(JLON,JLEV)=(RV-RD)/(RD*(1.0_JPRB-ZS_QT)+RV*ZS_Q)
    ZS_A1(JLON,JLEV)=(ZSLAMA(JLON,JLEV)-ZSLAMA(JLON,JLEV+1))   &
    &/ZS_X1(JLON,JLEV)+(ZS_AR(JLON,JLEV)-ZS_X0(JLON,JLEV))*&
    & (ZQTOTA(JLON,JLEV)-ZQTOTA(JLON,JLEV+1))
    ZS_B3X(JLON,JLEV)=(ZS_AR(JLON,JLEV)-ZS_X2(JLON,JLEV)&
    &      /(1.0_JPRB+ZS_DC(JLON,JLEV)))                &
    &      *(ZQTOTA(JLON,JLEV)-ZQTOTA(JLON,JLEV+1))

    ZS_NX4(JLON,JLEV)=MAX(ZS_L*(RD*(1.0_JPRB-ZS_QT)+RV*ZS_Q)/&
    & (ZS_X1(JLON,JLEV)*RV)-1.0_JPRB,ZEPS8)

  ENDDO
ENDDO

!f(C_n) computation - Q^=C^f(Cn)
IF(LCOEFK_SCQ) THEN
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA

        !Cn
        ZCN_D=PDIFTQ(JLON,JLEV)*ZS_NX4(JLON,JLEV)&
        & *(ZS_AR(JLON,JLEV)-ZS_X2(JLON,JLEV)/(1.0_JPRB+ZS_DC(JLON,JLEV)))
        ZCN=MIN(-(PDIFTS(JLON,JLEV)/ZS_X1(JLON,JLEV)+PDIFTQ(JLON,JLEV)&
        & *(ZS_AR(JLON,JLEV)-ZS_X0(JLON,JLEV)))&
         & /SIGN(MAX(ABS(ZCN_D),ZEPS8),ZCN_D),10.0_JPRB)
        ZFCN(JLON,JLEV)=MIN(MAX(0.5_JPRB*(sqrt((6.25_JPRB*ZCN)**2+4.0_JPRB)-6.25_JPRB*ZCN),&
         & 1E-2_JPRB),5.0_JPRB)
  
    ENDDO
  ENDDO
ELSE
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZFCN(JLON,JLEV)=1.0_JPRB
    ENDDO
  ENDDO
ENDIF

IF (LCOEFK_RIS) THEN
  !Shallow convection cloudiness in case of Ri*/Ri**
  !Q^=C also first guess for Q^=C^F(Cn)
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
        ! no cloudiness N2'/g^2*dphi
        ZS_N2_0=ZS_A1(JLON,JLEV)

        ! cloudiness=1.0 N2'/g^2*dphi
        ZS_FC=1.0_JPRB+ZS_NX4(JLON,JLEV)
        ZS_MC=(1.0_JPRB+ZS_DC(JLON,JLEV))&
        & /(1.0_JPRB+ZS_DC(JLON,JLEV)*ZS_FC)
        ZS_N2_1=ZS_MC*(ZS_A1(JLON,JLEV)+ZS_NX4(JLON,JLEV)&
        &*ZS_B3X(JLON,JLEV))
        ! N2'/g^2*dphi
        ZS_N2(JLON,JLEV)=PMRIPP(JLON,JLEV)&
        & *ZCISA(JLON,JLEV)/ZDPHIA(JLON,JLEV)
        !limitation
        ZS_N2(JLON,JLEV)=min(max(ZS_N2(JLON,JLEV)&
        & ,ZS_N2_1),ZS_N2_0)

       !Q^=C also first guess for Q^=C^f(Cn)       
!       PNEBCVPP(JLON,JLEV)=(-(ZS_DC(JLON,JLEV)+1)&
        PNEBQ(JLON,JLEV)=(-(ZS_DC(JLON,JLEV)+1)&
        &*(ZS_N2(JLON,JLEV)-ZS_A1(JLON,JLEV)))&
        &/(ZS_NX4(JLON,JLEV)*(ZS_DC(JLON,JLEV)&
        &*(ZS_N2(JLON,JLEV)-ZS_B3X(JLON,JLEV))-ZS_B3X(JLON,JLEV)))

        !if (N2_0=N2 => SCC=0) & (N2/g^2*dphi>2e-3 => SCC=0)
        !ZS_N2_0 is equal to ZS_A1(JLON,JLEV)
        ZSIG= (1.0_JPRB-SIGN(1.0_JPRB,ZS_N2(JLON,JLEV)-ZS_A1(JLON,JLEV)))*&
                & (1.0_JPRB-SIGN(1.0_JPRB,ZS_N2(JLON,JLEV)-2E-3_JPRB))*0.25_JPRB
!       PNEBCVPP(JLON,JLEV)=ZSIG*PNEBCVPP(JLON,JLEV)
        PNEBQ(JLON,JLEV)=ZSIG*PNEBQ(JLON,JLEV)
        !limitation
!       PNEBCVPP(JLON,JLEV)=max(min(1.0_JPRB,PNEBCVPP(JLON,JLEV)),ZEPS4)
        PNEBQ(JLON,JLEV)=max(min(1.0_JPRB,PNEBQ(JLON,JLEV)),ZEPS4)
        PNEBCVPP(JLON,JLEV)=PNEBQ(JLON,JLEV)**(1.0_JPRB/ZFCN(JLON,JLEV))

    ENDDO
  ENDDO

  IF(LCOEFK_SCQ) THEN

    !Q^=C^F(Cn) iteration
    DO JLEV=KTDIA,KLEV-1
      DO JITER=1,INITER
        DO JLON=KIDIA,KFDIA

          ZS_FC=1.0_JPRB+PNEBCVPP(JLON,JLEV)*ZS_NX4(JLON,JLEV)
          ZS_MC=(1.0_JPRB+ZS_DC(JLON,JLEV))/(1.0_JPRB+ZS_DC(JLON,JLEV)*ZS_FC)

          !Fx(C)
          ZN2DM=ZS_N2(JLON,JLEV)-ZS_MC*(ZS_A1(JLON,JLEV)&
          &+ZS_NX4(JLON,JLEV)*ZS_B3X(JLON,JLEV)*PNEBQ(JLON,JLEV))
   
          !dervative of -Fx(C) in C
          ZN2DMDC=((ZS_DC(JLON,JLEV)+1.0_JPRB)*ZS_NX4(JLON,JLEV)*(PNEBCVPP(JLON,JLEV)*                 &
           &ZFCN(JLON,JLEV)*ZS_B3X(JLON,JLEV)*ZS_DC(JLON,JLEV)*ZS_NX4(JLON,JLEV)*PNEBQ(JLON,JLEV)-     &
           &PNEBCVPP(JLON,JLEV)*ZS_B3X(JLON,JLEV)*ZS_DC(JLON,JLEV)*ZS_NX4(JLON,JLEV)*PNEBQ(JLON,JLEV)+ &
           &ZFCN(JLON,JLEV)*ZS_B3X(JLON,JLEV)*ZS_DC(JLON,JLEV)*PNEBQ(JLON,JLEV)+ZFCN(JLON,JLEV)*       &
           &ZS_B3X(JLON,JLEV)*PNEBQ(JLON,JLEV)-PNEBCVPP(JLON,JLEV)*ZS_A1(JLON,JLEV)*ZS_DC(JLON,JLEV)))/&
           &MAX(ZFCN(JLON,JLEV)*(PNEBCVPP(JLON,JLEV)*ZS_DC(JLON,JLEV)*ZS_NX4(JLON,JLEV)+               &
           &ZS_DC(JLON,JLEV)+1.0_JPRB)**2*PNEBQ(JLON,JLEV),ZEPS8)

          PNEBQ(JLON,JLEV)=PNEBQ(JLON,JLEV)+ZN2DM/SIGN(MAX(ABS(ZN2DMDC),ZEPS8),ZN2DMDC)

          !limitation
          PNEBQ(JLON,JLEV)=max(min(1.0_JPRB,PNEBQ(JLON,JLEV)),ZEPS4)

          PNEBCVPP(JLON,JLEV)=PNEBQ(JLON,JLEV)**(1.0_JPRB/ZFCN(JLON,JLEV))

        ENDDO
      ENDDO !end of iteration
    ENDDO
  ENDIF
ELSE
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      !External shallow convection cloudiness for Ri_m/Ri_s1
      PNEBCVPP(JLON,JLEV)=PNEB_SC(JLON,JLEV)
      PNEBQ(JLON,JLEV)=PNEBCVPP(JLON,JLEV)
    ENDDO
  ENDDO
ENDIF

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
     
    ZSIG= (1.0_JPRB-SIGN(1.0_JPRB,ZS_N2(JLON,JLEV)-ZS_A1(JLON,JLEV)))*&
    & (1.0_JPRB-SIGN(1.0_JPRB,ZS_N2(JLON,JLEV)-2E-3_JPRB))*0.25_JPRB
    !limitation
    PNEBQ(JLON,JLEV)=max(min(1.0_JPRB,PNEBQ(JLON,JLEV)*ZSIG),ZEPS4)

    ZSIG=(1.0_JPRB-SIGN(1.0_JPRB,ZEPS4-PNEBQ(JLON,JLEV)))*0.5_JPRB
    PNEBCVPP(JLON,JLEV)=MAX(PNEBQ(JLON,JLEV)**(1.0_JPRB/ZFCN(JLON,JLEV))*ZSIG,ZEPS4)

    ZS_FC=1.0_JPRB+PNEBCVPP(JLON,JLEV)*ZS_NX4(JLON,JLEV)
    ZS_MC=(1.0_JPRB+ZS_DC(JLON,JLEV))&
    &/(1.0_JPRB+ZS_DC(JLON,JLEV)*ZS_FC)

    PMN2PP(JLON,JLEV)=ZS_MC*(ZS_A1(JLON,JLEV)+ZS_NX4(JLON,JLEV)&
    &*ZS_B3X(JLON,JLEV)*PNEBQ(JLON,JLEV))/ZDPHIA(JLON,JLEV)*RG**2

    PNBVNO(JLON,JLEV)=PMN2PP(JLON,JLEV)/(PAPRS(JLON,JLEV) &
          &   *ZRTI(JLON,JLEV)*RG)**2
   
    PMRIPP(JLON,JLEV)=PMN2PP(JLON,JLEV)/(ZCISA(JLON,JLEV)*RG**2)&
     & *ZDPHIA(JLON,JLEV)**2

    PMN2_EQ(JLON,JLEV)=RG*ZS_MC*((ZS_AR(JLON,JLEV)-ZS_X0(JLON,JLEV))+&
    & PNEBQ(JLON,JLEV)*ZS_NX4(JLON,JLEV)*&
    & (ZS_AR(JLON,JLEV)-ZS_X2(JLON,JLEV)/(1.0_JPRB+ZS_DC(JLON,JLEV))))

    PMN2_DQ(JLON,JLEV)=(ZQTOTA(JLON,JLEV)-ZQTOTA(JLON,JLEV+1))&
    & *RG/ZDPHIA(JLON,JLEV)

    PMN2_ES(JLON,JLEV)=RG*ZS_MC/ZS_X1(JLON,JLEV)
    PMN2_DS(JLON,JLEV)=(ZSLAMA(JLON,JLEV)-ZSLAMA(JLON,JLEV+1))&
    & *RG/ZDPHIA(JLON,JLEV)
    
  ENDDO
ENDDO
PMN2_ES(KIDIA:KFDIA,KTDIA-1)=PMN2_ES(KIDIA:KFDIA,KTDIA)
PMN2_EQ(KIDIA:KFDIA,KTDIA-1)=PMN2_EQ(KIDIA:KFDIA,KTDIA)
PMN2_DS(KIDIA:KFDIA,KTDIA-1)=PMN2_DS(KIDIA:KFDIA,KTDIA)
PMN2_DQ(KIDIA:KFDIA,KTDIA-1)=PMN2_DQ(KIDIA:KFDIA,KTDIA)

IF(LLM123) THEN
  !modified CCH02 or RMC01
  Z_ETKE_R=ETKE_R
  Z_ETKE_P=ETKE_RIFC
  
  IF (LCOEFK_RIH) THEN

    IF(LCOEFK_FLX.AND.KSTEP>0) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
      !Rif_s1 computation
          ZN2S_CP =(PCP(JLON,JLEV)+PCP(JLON,JLEV+1))/2.0_JPRB
          ZS_T =(PT(JLON,JLEV)+PT(JLON,JLEV+1))/2.0_JPRB

          ZRIFTKE_S1(JLON,JLEV)=RG*PMRIMC(JLON,JLEV)&
           & /(MAX(PSHEAR(JLON,JLEV),ZEPS4)*RG/ZDPHIA(JLON,JLEV))&
           &*(PDIFTS(JLON,JLEV)/(ZN2S_CP*ZS_T)+&
           & (ENT_LAMBDA-(RCPV-RCPD)/ZN2S_CP)*PDIFTQ(JLON,JLEV))
          !security
          ZRIFTKE_S1(JLON,JLEV)=MIN(ZRIFTKE_S1(JLON,JLEV),ETKE_RIFC-ZEPS8)

          PCHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_R)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
          ZPHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_P)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
        ENDDO
      ENDDO

    ELSE  ! FLX is false or KSTEP = 0

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          !Ri=Ri_s1
          ZRITKE=ZRITKE_S1(JLON,JLEV)

          ZRIFTKE_S1(JLON,JLEV)=-(sqrt((Z_ETKE_R&
           & *(ZRITKE*C3TKEFREE+Z_ETKE_P))**2&
           & -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
           & -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
          !security
          ZRIFTKE_S1(JLON,JLEV)=MIN(ZRIFTKE_S1(JLON,JLEV),ETKE_RIFC-ZEPS8)

          PCHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_R)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
          ZPHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_P)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))

        ENDDO
      ENDDO

    ENDIF ! FLX and KSTEP

  ENDIF ! RIH true

  IF(.NOT.(LCOEFK_PTTE.AND.KSTEP>0)) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          ZRITKE=PMRIPP(JLON,JLEV)
          PMRIFPP(JLON,JLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
           &   -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
           &   -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
          !security
          PMRIFPP(JLON,JLEV)=MIN(PMRIFPP(JLON,JLEV),ETKE_RIFC-ZEPS8)
          ZETKE_R(JLON,JLEV)=Z_ETKE_R
          ZETKE_P(JLON,JLEV)=Z_ETKE_P

        ENDDO
      ENDDO

  ENDIF ! PTTE false

  IF(LCOEFK_PTTE.AND.KSTEP>0) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          !Rif_m computation from TKE and TTE
          ZMRIFPP=MIN(ETKE_RIFC-ZEPS8,&
           & (MAX(PTTE(JLON,JLEV),ZEPS4)-PTKE(JLON,JLEV))&
           & /MAX(ZEPS8,(1.0_JPRB-ZCP)*PTKE(JLON,JLEV)+MAX(PTTE(JLON,JLEV),ZEPS4)))
          ZMRIFPPI=MIN(ETKE_RIFC-ZEPS8,&
           & (MAX(PTTE(JLON,JLEV+1),ZEPS4)-PTKE(JLON,JLEV+1))&
           & /MAX(ZEPS8,(1.0_JPRB-ZCP)*PTKE(JLON,JLEV+1)+MAX(PTTE(JLON,JLEV+1),ZEPS4)))
          PMRIFPP(JLON,JLEV)=0.5_JPRB*(ZMRIFPP+ZMRIFPPI)
          !security
          PMRIFPP(JLON,JLEV)=MIN(PMRIFPP(JLON,JLEV),ETKE_RIFC-ZEPS8)

          !compute Ri from Rif
          ZRIFTKE=PMRIFPP(JLON,JLEV)

          !update Ri
          ! keep gradient Ri
          ZRITKE_F(JLON,JLEV)=ZRIFTKE/C3TKEFREE*&
           &(1.0_JPRB-ZRIFTKE/Z_ETKE_R)/&
           &(1.0_JPRB-ZRIFTKE/Z_ETKE_P)
          !protection against division by 0
          ZRITKE_F(JLON,JLEV)=SIGN(MAX(ZEPS,&
           & ABS(ZRITKE_F(JLON,JLEV))),ZRITKE_F(JLON,JLEV))
          ZETKE_R(JLON,JLEV)=Z_ETKE_R
          ZETKE_P(JLON,JLEV)=Z_ETKE_P

        ENDDO
      ENDDO

  ELSE ! not PTTE or KSTEP = 0

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
          ZRITKE_F(JLON,JLEV)=PMRIPP(JLON,JLEV)
        ENDDO
      ENDDO

  ENDIF ! PTTE true

ENDIF ! models 1 or 2 or 3

IF(ITTYPE == 4) THEN
  
  IF (LCOEFK_RIH) THEN

    IF(LCOEFK_FLX.AND.KSTEP>0) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
          !Rif_s1 computation
          ZN2S_CP =(PCP(JLON,JLEV)+PCP(JLON,JLEV+1))/2.0_JPRB
          ZS_T =(PT(JLON,JLEV)+PT(JLON,JLEV+1))/2.0_JPRB

          ZRIFTKE_S1(JLON,JLEV)=RG*PMRIMC(JLON,JLEV)&
           & /(MAX(PSHEAR(JLON,JLEV),ZEPS4)*RG/ZDPHIA(JLON,JLEV))&
           &*(PDIFTS(JLON,JLEV)/(ZN2S_CP*ZS_T)+&
           & (ENT_LAMBDA-(RCPV-RCPD)/ZN2S_CP)*PDIFTQ(JLON,JLEV))
          !security
          ZRIFTKE_S1(JLON,JLEV)=MIN(ZRIFTKE_S1(JLON,JLEV),ETKE_RIFC-ZEPS8)

          !compute R from Rif_s1
          ZRIFTKE=ZRIFTKE_S1(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRIFTKE)+0.5_JPRB

          !UNSTABLE
          ZS_A = C3TKEFREE*RQNSE_GU1*(ZRIFTKE-ETKE_RIFC)
          ZS_B = ZRIFTKE*RQNSE_S0*C3TKEFREE&
          &+ETKE_RIFC*(ZRIFTKE*(RQNSE_GU1-ZRIFTKE*RQNSE_GU2)-RQNSE_S0*C3TKEFREE)
          ZS_C = ETKE_RIFC*ZRIFTKE*(RQNSE_S0-ZRIFTKE)

          ZS_CDIS=ZS_B**2-4*ZS_A*ZS_C
          ZRITKE_U=(-ZS_B-SQRT(ZS_CDIS))/(2.0_JPRB*ZS_A)
          Z_ETKE_RU=(RQNSE_GU1*ZRITKE_U+RQNSE_S0)/&
          &(RQNSE_GU2*ZRITKE_U+1.0_JPRB)
          !UNSTABLE
          ZS_AA4 = C3TKEFREE*RQNSE_GS1*(ZRIFTKE-ETKE_RIFC)
          ZS_AA3 = ZRIFTKE*RQNSE_GS2*C3TKEFREE&
          &+ETKE_RIFC*(ZRIFTKE*(RQNSE_GS1-ZRIFTKE*RQNSE_GS3)-RQNSE_GS2*C3TKEFREE)
          ZS_AA2 = ZRIFTKE*RQNSE_S0*C3TKEFREE&
          &+ETKE_RIFC*(ZRIFTKE*(RQNSE_GS2-ZRIFTKE*RQNSE_GS4)-RQNSE_S0*C3TKEFREE)
          ZS_AA1 = ETKE_RIFC*ZRIFTKE*(RQNSE_S0-ZRIFTKE)

         ! cubic problem only
          ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
          ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
          ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
          ZS_CDIS=ZS_CQ**2+ZS_CP

          ZS_ZSOL = -1.E+99_JPRB
          IF (ZS_CDIS < 0.0_JPRB) THEN
            !         three real solutions!
            !         Confine the argument of ACOS to the interval [-1;1]!
            ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
            ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
            ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
             &  MIN(ZS_CU2,ZS_CU3))
            ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
          ELSE
            !      only one real solution!
            ZS_CPHI=SQRT(ZS_CDIS)
            ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
             &       SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
            ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
          ENDIF
          ZRITKE=ZS_ZSOL
          Z_ETKE_R=(ZRITKE*(RQNSE_GS1*ZRITKE+&
           & RQNSE_GS2)+RQNSE_S0)/&
           &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
          ZRITKE=ZSIG*ZRITKE+(1.0_JPRB-ZSIG)*ZRITKE_U
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=ETKE_RIFC

          PCHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_R)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
          ZPHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_P)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
        ENDDO
      ENDDO

    ELSE  ! FLX is false or KSTEP = 0

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          !Ri=Ri_s1
          ZRITKE=ZRITKE_S1(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

          !UNSTABLE
          Z_ETKE_RU=(RQNSE_GU1*ZRITKE+RQNSE_S0)/&
           &(RQNSE_GU2*ZRITKE+1.0_JPRB)
          !STABLE
          Z_ETKE_R=(ZRITKE*(RQNSE_GS1*ZRITKE+&
           & RQNSE_GS2)+RQNSE_S0)/&
           &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=ETKE_RIFC


          ZRIFTKE_S1(JLON,JLEV)=-(sqrt((Z_ETKE_R&
           & *(ZRITKE*C3TKEFREE+Z_ETKE_P))**2&
           & -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
           & -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
          !security
          ZRIFTKE_S1(JLON,JLEV)=MIN(ZRIFTKE_S1(JLON,JLEV),ETKE_RIFC-ZEPS8)

          PCHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_R)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
          ZPHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_P)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))

        ENDDO
      ENDDO

    ENDIF ! FLX and KSTEP

  ENDIF ! RIH true

  IF(.NOT.(LCOEFK_PTTE.AND.KSTEP>0)) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          ZRITKE=PMRIPP(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

          !UNSTABLE
          Z_ETKE_RU=(RQNSE_GU1*ZRITKE+RQNSE_S0)/&
          &(RQNSE_GU2*ZRITKE+1.0_JPRB)
          !STABLE
          Z_ETKE_R=(ZRITKE*(RQNSE_GS1*ZRITKE+&
          & RQNSE_GS2)+RQNSE_S0)/&
          &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=ETKE_RIFC

          PMRIFPP(JLON,JLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
           &   -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
           &   -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
          !security
          PMRIFPP(JLON,JLEV)=MIN(PMRIFPP(JLON,JLEV),ETKE_RIFC-ZEPS8)
          ZETKE_R(JLON,JLEV)=Z_ETKE_R
          ZETKE_P(JLON,JLEV)=Z_ETKE_P

        ENDDO
      ENDDO

  ENDIF ! PTTE false

  IF(LCOEFK_PTTE.AND.KSTEP>0) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          !Rif_m computation from TKE and TTE
          ZMRIFPP=MIN(ETKE_RIFC-ZEPS8,&
           & (MAX(PTTE(JLON,JLEV),ZEPS4)-PTKE(JLON,JLEV))&
           & /MAX(ZEPS8,(1.0_JPRB-ZCP)*PTKE(JLON,JLEV)+MAX(PTTE(JLON,JLEV),ZEPS4)))
          ZMRIFPPI=MIN(ETKE_RIFC-ZEPS8,&
           & (MAX(PTTE(JLON,JLEV+1),ZEPS4)-PTKE(JLON,JLEV+1))&
           & /MAX(ZEPS8,(1.0_JPRB-ZCP)*PTKE(JLON,JLEV+1)+MAX(PTTE(JLON,JLEV+1),ZEPS4)))
          PMRIFPP(JLON,JLEV)=0.5_JPRB*(ZMRIFPP+ZMRIFPPI)
          !security
          PMRIFPP(JLON,JLEV)=MIN(PMRIFPP(JLON,JLEV),ETKE_RIFC-ZEPS8)

          !compute Ri from Rif
          ZRIFTKE=PMRIFPP(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRIFTKE)+0.5_JPRB

          !UNSTABLE
          ZS_A = C3TKEFREE*RQNSE_GU1*(ZRIFTKE-ETKE_RIFC)
          ZS_B = ZRIFTKE*RQNSE_S0*C3TKEFREE&
           &+ETKE_RIFC*(ZRIFTKE*(RQNSE_GU1-ZRIFTKE*RQNSE_GU2)-RQNSE_S0*C3TKEFREE)
          ZS_C = ETKE_RIFC*ZRIFTKE*(RQNSE_S0-ZRIFTKE)

          ZS_CDIS=ZS_B**2-4*ZS_A*ZS_C
          ZRITKE_U=(-ZS_B-SQRT(ZS_CDIS))/(2.0_JPRB*ZS_A)
          Z_ETKE_RU=(RQNSE_GU1*ZRITKE_U+RQNSE_S0)/&
           &(RQNSE_GU2*ZRITKE_U+1.0_JPRB)
          !UNSTABLE
          ZS_AA4 = C3TKEFREE*RQNSE_GS1*(ZRIFTKE-ETKE_RIFC)
          ZS_AA3 = ZRIFTKE*RQNSE_GS2*C3TKEFREE&
           &+ETKE_RIFC*(ZRIFTKE*(RQNSE_GS1-ZRIFTKE*RQNSE_GS3)-RQNSE_GS2*C3TKEFREE)
          ZS_AA2 = ZRIFTKE*RQNSE_S0*C3TKEFREE&
           &+ETKE_RIFC*(ZRIFTKE*(RQNSE_GS2-ZRIFTKE*RQNSE_GS4)-RQNSE_S0*C3TKEFREE)
          ZS_AA1 = ETKE_RIFC*ZRIFTKE*(RQNSE_S0-ZRIFTKE)

          ! cubic problem only
          ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
          ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
          ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
          ZS_CDIS=ZS_CQ**2+ZS_CP

          ZS_ZSOL = -1.E+99_JPRB
          IF (ZS_CDIS < 0.0_JPRB) THEN
            !         three real solutions!
            !         Confine the argument of ACOS to the interval [-1;1]!
            ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
            ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
            ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
             &  MIN(ZS_CU2,ZS_CU3))
            ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
          ELSE
            !      only one real solution!
            ZS_CPHI=SQRT(ZS_CDIS)
            ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
             &  SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
            ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
          ENDIF
          ZRITKE=ZS_ZSOL
          Z_ETKE_R=(ZRITKE*(RQNSE_GS1*ZRITKE+&
          & RQNSE_GS2)+RQNSE_S0)/&
          &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
          ZRITKE=ZSIG*ZRITKE+(1.0_JPRB-ZSIG)*ZRITKE_U
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=ETKE_RIFC

          !update Ri
          ! keep gradient Ri
          ZRITKE_F(JLON,JLEV)=ZRIFTKE/C3TKEFREE*&
           &(1.0_JPRB-ZRIFTKE/Z_ETKE_R)/&
           &(1.0_JPRB-ZRIFTKE/Z_ETKE_P)
          !protection against division by 0
          ZRITKE_F(JLON,JLEV)=SIGN(MAX(ZEPS,&
           & ABS(ZRITKE_F(JLON,JLEV))),ZRITKE_F(JLON,JLEV))
          ZETKE_R(JLON,JLEV)=Z_ETKE_R
          ZETKE_P(JLON,JLEV)=Z_ETKE_P

        ENDDO
      ENDDO

  ELSE ! not PTTE or KSTEP = 0

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
          ZRITKE_F(JLON,JLEV)=PMRIPP(JLON,JLEV)
        ENDDO
      ENDDO

  ENDIF ! PTTE true

ENDIF ! model QNSE

IF(ITTYPE == 5) THEN
! model EFB
  
  IF (LCOEFK_RIH) THEN

    IF(LCOEFK_FLX.AND.KSTEP>0) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
      !Rif_s1 computation
          ZN2S_CP =(PCP(JLON,JLEV)+PCP(JLON,JLEV+1))/2.0_JPRB
          ZS_T =(PT(JLON,JLEV)+PT(JLON,JLEV+1))/2.0_JPRB

          ZRIFTKE_S1(JLON,JLEV)=RG*PMRIMC(JLON,JLEV)&
           & /(MAX(PSHEAR(JLON,JLEV),ZEPS4)*RG/ZDPHIA(JLON,JLEV))&
           &*(PDIFTS(JLON,JLEV)/(ZN2S_CP*ZS_T)+&
           & (ENT_LAMBDA-(RCPV-RCPD)/ZN2S_CP)*PDIFTQ(JLON,JLEV))
          !security
          ZRIFTKE_S1(JLON,JLEV)=MIN(ZRIFTKE_S1(JLON,JLEV),ETKE_RIFC-ZEPS8)

          !compute R from Rif_s1
          ZRIFTKE=ZRIFTKE_S1(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRIFTKE)+0.5_JPRB
          ! UNSTABLE
          Z_ETKE_RU=EFB_UR
          !  STABLE
          Z_ETKE_R=(ZRIFTKE*REFB_1+EFB_UR*REFB_3)/&
           &(ZRIFTKE*REFB_2+REFB_3)
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=MIN(Z_ETKE_R&
           & /(ETKE_OLAM*Z_ETKE_R/(EFB_AZ0*C3TKEFREE)+1.0_JPRB),ETKE_RIFC-ZEPS8)

          PCHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_R)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
          ZPHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_P)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
        ENDDO
      ENDDO

    ELSE  ! FLX is false or KSTEP = 0

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          !Ri=Ri_s1
          ZRITKE=ZRITKE_S1(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

          !  UNSTABLE
          Z_ETKE_RU=EFB_UR
          !  STABLE
          ZS_AA4 = EFB_AZ0*REFB_2
          ZS_AA3 = EFB_AZ0*(REFB_3-REFB_1-ZRITKE*REFB_2*C3TKEFREE)&
          &      - ETKE_OLAM*REFB_1*ZRITKE
          ZS_AA2 = C3TKEFREE*EFB_AZ0*ZRITKE*(REFB_1-REFB_3)&
          &      - REFB_3*EFB_UR*(ETKE_OLAM*ZRITKE+EFB_AZ0)
          ZS_AA1 = ZRITKE*EFB_AZ0*EFB_UR*REFB_3*C3TKEFREE

          ! cubic problem only
          ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
          ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
          ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
          ZS_CDIS=ZS_CQ**2+ZS_CP

          ZS_ZSOL = +1.E+99_JPRB
          IF (ZS_CDIS < 0.0_JPRB) THEN
            !         three real solutions!
            !         Confine the argument of ACOS to the interval [-1;1]!
            ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
            ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
            ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
              &  MIN(ZS_CU2,ZS_CU3))
            ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_ZSOL = MIN(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
          ELSE
            !      only one real solution!
            ZS_CPHI=SQRT(ZS_CDIS)
            ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
             &       SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
            ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
          ENDIF

          ZRIFTKE=ZS_ZSOL
          Z_ETKE_R=(ZRIFTKE*REFB_1+EFB_UR*REFB_3)/&
           &(ZRIFTKE*REFB_2+REFB_3)
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=MIN(Z_ETKE_R&
           & /(ETKE_OLAM*Z_ETKE_R/(EFB_AZ0*C3TKEFREE)+1.0_JPRB),ETKE_RIFC-ZEPS8)

          ZRIFTKE_S1(JLON,JLEV)=-(sqrt((Z_ETKE_R&
           & *(ZRITKE*C3TKEFREE+Z_ETKE_P))**2&
           & -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
           & -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
          !security
          ZRIFTKE_S1(JLON,JLEV)=MIN(ZRIFTKE_S1(JLON,JLEV),ETKE_RIFC-ZEPS8)

          PCHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_R)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))
          ZPHI3_S1(JLON,JLEV)=(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV)/Z_ETKE_P)&
           & /(1.0_JPRB-ZRIFTKE_S1(JLON,JLEV))

        ENDDO
      ENDDO

    ENDIF ! FLX and KSTEP

  ENDIF ! RIH true

  IF(.NOT.(LCOEFK_PTTE.AND.KSTEP>0)) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          ZRITKE=PMRIPP(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

          !  UNSTABLE
          Z_ETKE_RU=EFB_UR
          !  STABLE
          ZS_AA4 = EFB_AZ0*REFB_2
          ZS_AA3 = EFB_AZ0*(REFB_3-REFB_1-ZRITKE*REFB_2*C3TKEFREE)&
          &      - ETKE_OLAM*REFB_1*ZRITKE
          ZS_AA2 = C3TKEFREE*EFB_AZ0*ZRITKE*(REFB_1-REFB_3)&
          &      - REFB_3*EFB_UR*(ETKE_OLAM*ZRITKE+EFB_AZ0)
          ZS_AA1 = ZRITKE*EFB_AZ0*EFB_UR*REFB_3*C3TKEFREE

          ! cubic problem only
          ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
          ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
          ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
          ZS_CDIS=ZS_CQ**2+ZS_CP

          ZS_ZSOL = +1.E+99_JPRB
          IF (ZS_CDIS < 0.0_JPRB) THEN
            !         three real solutions!
            !         Confine the argument of ACOS to the interval [-1;1]!
            ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
            ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
            ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
            ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
              &  MIN(ZS_CU2,ZS_CU3))
            ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
            ZS_ZSOL = MIN(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
          ELSE
            !      only one real solution!
            ZS_CPHI=SQRT(ZS_CDIS)
            ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
             &       SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
            ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
          ENDIF

          ZRIFTKE=ZS_ZSOL
          Z_ETKE_R=(ZRIFTKE*REFB_1+EFB_UR*REFB_3)/&
           &(ZRIFTKE*REFB_2+REFB_3)
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=MIN(Z_ETKE_R&
           & /(ETKE_OLAM*Z_ETKE_R/(EFB_AZ0*C3TKEFREE)+1.0_JPRB),ETKE_RIFC-ZEPS8)

          PMRIFPP(JLON,JLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
           &   -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
           &   -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
          !security
          PMRIFPP(JLON,JLEV)=MIN(PMRIFPP(JLON,JLEV),ETKE_RIFC-ZEPS8)
          ZETKE_R(JLON,JLEV)=Z_ETKE_R
          ZETKE_P(JLON,JLEV)=Z_ETKE_P

        ENDDO
      ENDDO

  ENDIF ! PTTE false

  IF(LCOEFK_PTTE.AND.KSTEP>0) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          !Rif_m computation from TKE and TTE
          ZMRIFPP=MIN(ETKE_RIFC-ZEPS8,&
           & (MAX(PTTE(JLON,JLEV),ZEPS4)-PTKE(JLON,JLEV))&
           & /MAX(ZEPS8,(1.0_JPRB-ZCP)*PTKE(JLON,JLEV)+MAX(PTTE(JLON,JLEV),ZEPS4)))
          ZMRIFPPI=MIN(ETKE_RIFC-ZEPS8,&
           & (MAX(PTTE(JLON,JLEV+1),ZEPS4)-PTKE(JLON,JLEV+1))&
           & /MAX(ZEPS8,(1.0_JPRB-ZCP)*PTKE(JLON,JLEV+1)+MAX(PTTE(JLON,JLEV+1),ZEPS4)))
          PMRIFPP(JLON,JLEV)=0.5_JPRB*(ZMRIFPP+ZMRIFPPI)
          !security
          PMRIFPP(JLON,JLEV)=MIN(PMRIFPP(JLON,JLEV),ETKE_RIFC-ZEPS8)

          !compute Ri from Rif
          ZRIFTKE=PMRIFPP(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRIFTKE)+0.5_JPRB

          !  UNSTABLE
          Z_ETKE_RU=EFB_UR
          !  STABLE
          Z_ETKE_R=(ZRIFTKE*REFB_1+EFB_UR*REFB_3)/&
           &(ZRIFTKE*REFB_2+REFB_3)
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
          Z_ETKE_P=MIN(Z_ETKE_R&
           & /(ETKE_OLAM*Z_ETKE_R/(EFB_AZ0*C3TKEFREE)+1.0_JPRB),ETKE_RIFC-ZEPS8)

          !update Ri
          ! keep gradient Ri
          ZRITKE_F(JLON,JLEV)=ZRIFTKE/C3TKEFREE*&
           &(1.0_JPRB-ZRIFTKE/Z_ETKE_R)/&
           &(1.0_JPRB-ZRIFTKE/Z_ETKE_P)
          !protection against division by 0
          ZRITKE_F(JLON,JLEV)=SIGN(MAX(ZEPS,&
           & ABS(ZRITKE_F(JLON,JLEV))),ZRITKE_F(JLON,JLEV))
          ZETKE_R(JLON,JLEV)=Z_ETKE_R
          ZETKE_P(JLON,JLEV)=Z_ETKE_P

        ENDDO
      ENDDO

  ELSE ! not PTTE or KSTEP = 0

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
          ZRITKE_F(JLON,JLEV)=PMRIPP(JLON,JLEV)
        ENDDO
      ENDDO

  ENDIF ! PTTE true

ENDIF ! EFB

IF(ITTYPE == 6) THEN
  
  IF (LCOEFK_RIH) THEN
    CALL ABOR1('ACMRIP: LCOEFK_RIH=T AND CGTURS=LOUIS is not coded') 

!    IF(LCOEFK_FLX.AND.KSTEP>0) THEN

!      DO JLEV=KTDIA,KLEV-1
!        DO JLON=KIDIA,KFDIA
!        ENDDO
!      ENDDO

!    ELSE  ! FLX is false or KSTEP = 0

!      DO JLEV=KTDIA,KLEV-1
!        DO JLON=KIDIA,KFDIA


!        ENDDO
!      ENDDO

 !   ENDIF ! FLX and KSTEP

  ENDIF ! RIH true

  IF(.NOT.(LCOEFK_PTTE.AND.KSTEP>0)) THEN

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA

          ZRITKE=PMRIPP(JLON,JLEV)
          ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

          !  UNSTABLE
          Z_ETKE_RU=(RLOUIS_GU1*ZRITKE+RLOUIS_S0)/&
          &(RLOUIS_GU2*ZRITKE+1.0_JPRB)
          !  STABLE
          Z_ETKE_R=(ZRITKE*(RLOUIS_GS1*ZRITKE+&
          & RLOUIS_GS2)+RLOUIS_S0)/&
          &(ZRITKE*(RLOUIS_GS3*ZRITKE+RLOUIS_GS4)+1.0_JPRB)
          Z_ETKE_R=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU

          !UNSTABLE
          Z_ETKE_PU=(PLOUIS_GU1*ZRITKE+PLOUIS_S0)/&
          &(PLOUIS_GU2*ZRITKE+1.0_JPRB)
         !STABLE
          Z_ETKE_P=(ZRITKE*(PLOUIS_GS1*ZRITKE+&
          & PLOUIS_GS2)+PLOUIS_S0)/&
          &(ZRITKE*(PLOUIS_GS3*ZRITKE+PLOUIS_GS4)+1.0_JPRB)
          Z_ETKE_P=ZSIG*Z_ETKE_P+(1.0_JPRB-ZSIG)*Z_ETKE_PU

          PMRIFPP(JLON,JLEV)=-(sqrt((Z_ETKE_R*(ZRITKE*C3TKEFREE+Z_ETKE_P))**2 &
           &   -4.0_JPRB*ZRITKE*C3TKEFREE*Z_ETKE_R*Z_ETKE_P**2) &
           &   -(ZRITKE*C3TKEFREE+Z_ETKE_P)*Z_ETKE_R)/(2.0_JPRB*Z_ETKE_P)
          !security
          PMRIFPP(JLON,JLEV)=MIN(PMRIFPP(JLON,JLEV),ETKE_RIFC-ZEPS8)
          ZETKE_R(JLON,JLEV)=Z_ETKE_R
          ZETKE_P(JLON,JLEV)=Z_ETKE_P

        ENDDO
      ENDDO

  ENDIF ! PTTE false

  IF(LCOEFK_PTTE.AND.KSTEP>0) THEN
    CALL ABOR1('ACMRIP: LCOEFK_PTTE=T AND CGTURS=LOUIS is not coded') 

!      DO JLEV=KTDIA,KLEV-1
!        DO JLON=KIDIA,KFDIA

!        ENDDO
!      ENDDO

  ELSE ! not PTTE or KSTEP = 0

      DO JLEV=KTDIA,KLEV-1
        DO JLON=KIDIA,KFDIA
          ZRITKE_F(JLON,JLEV)=PMRIPP(JLON,JLEV)
        ENDDO
      ENDDO

  ENDIF ! PTTE true

ENDIF ! model LOUIS

!*
!     ------------------------------------------------------------------
!     V - BOUCLE PASSIVE SUR LES NIVEAUX VERTICAUX.

!           PASSIVE LOOP ON VERTICAL LEVELS.
!*
!     ------------------------------------------------------------------
!     VI - COMPUTATION OF RI RELATED QUANTITIES - STABILITY FUNCTIONS AND ETC.

! new code
IF (ITTYPE==1) THEN

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA

      ZCHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_R(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))
      ZPHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_P(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))

      !RMC01
      ZPHIQTA(JLON,JLEV)=1.0_JPRB
      PPHIQ_S1(JLON,JLEV)=1.0_JPRB
      PWW_FUN(JLON,JLEV)=Z2THIRD
      PTH_FUN(JLON,JLEV)=1.0_JPRB/(2.0_JPRB*PWW_FUN(JLON,JLEV))

    ENDDO
  ENDDO

ENDIF
IF (ITTYPE==2) THEN

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA

      ZCHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_R(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))
      ZPHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_P(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))

      !Model I
      PWW_FUN(JLON,JLEV)=Z2THIRD*(1.0_JPRB&
       & -(3.0_JPRB*ETKE_LAM3-ETKE_LAM2     &
       & +4.0_JPRB*ETKE_LAM4*PMRIFPP(JLON,JLEV))/(1.0_JPRB-PMRIFPP(JLON,JLEV)))
      ZPHIQTA(JLON,JLEV)=PWW_FUN(JLON,JLEV)/Z2AZ0
      PTH_FUN(JLON,JLEV)=1.0_JPRB/(2.0_JPRB*PWW_FUN(JLON,JLEV))
      IF (LCOEFK_RIH) THEN
        PWW_FUN(JLON,JLEV)=Z2THIRD*(1.0_JPRB&
        & -(3.0_JPRB*ETKE_LAM3-ETKE_LAM2     &
        & +4.0_JPRB*ETKE_LAM4*ZRIFTKE_S1(JLON,JLEV))/(1-ZRIFTKE_S1(JLON,JLEV)))
        PPHIQ_S1(JLON,JLEV)=PWW_FUN(JLON,JLEV)/Z2AZ0
      ENDIF

    ENDDO
  ENDDO

ENDIF
!IF (ITTYPE==3.OR.ITTYPE==4) THEN
IF (ITTYPE==3.OR.ITTYPE==4.OR.ITTYPE==6) THEN

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA

      ZCHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_R(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))
      ZPHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_P(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))

      !Model II or QNSE
      PWW_FUN(JLON,JLEV)=Z2THIRD*(1.0_JPRB&
       & -(3.0_JPRB*ETKE_LAM3-ETKE_LAM2     &
       & +4.0_JPRB*ETKE_LAM4*PMRIFPP(JLON,JLEV))/(1.0_JPRB-PMRIFPP(JLON,JLEV)))
      ZTKE_LAM5T=(C3TKEFREE+ZCHI3TA(JLON,JLEV)/ZPHI3TA(JLON,JLEV))&
       & /(C3TKEFREE+1.0_JPRB)
       ZPHIQTA(JLON,JLEV)=PWW_FUN(JLON,JLEV)/ZTKE_LAM5T/Z2AZ0
       PTH_FUN(JLON,JLEV)=1.0_JPRB/(2.0_JPRB*PWW_FUN(JLON,JLEV))
       IF (LCOEFK_RIH) THEN
         PWW_FUN(JLON,JLEV)=Z2THIRD*(1.0_JPRB&
          & -(3.0_JPRB*ETKE_LAM3-ETKE_LAM2     &
          & +4.0_JPRB*ETKE_LAM4*ZRIFTKE_S1(JLON,JLEV))/(1-ZRIFTKE_S1(JLON,JLEV)))
         ZTKE_LAM5T=(C3TKEFREE+PCHI3_S1(JLON,JLEV)/ZPHI3_S1(JLON,JLEV))&
          & /(C3TKEFREE+1.0_JPRB)
         PPHIQ_S1(JLON,JLEV)=PWW_FUN(JLON,JLEV)/ZTKE_LAM5T/Z2AZ0
       ENDIF

    ENDDO
  ENDDO

ENDIF
IF (ITTYPE==5) THEN

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA

      ZCHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_R(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))
      ZPHI3TA(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_P(JLON,JLEV))&
       & /(1.0_JPRB-PMRIFPP(JLON,JLEV))

      !EFB
      ZPHIQTA(JLON,JLEV)=ZCHI3TA(JLON,JLEV)
      PWW_FUN(JLON,JLEV)=2.0_JPRB*EFB_AZ0*ZCHI3TA(JLON,JLEV)
      PTH_FUN(JLON,JLEV)=1.0_JPRB/(2.0_JPRB*PWW_FUN(JLON,JLEV))
      IF (LCOEFK_RIH) THEN
        PPHIQ_S1(JLON,JLEV)=PCHI3_S1(JLON,JLEV)
        PWW_FUN(JLON,JLEV)=2.0_JPRB*EFB_AZ0*PCHI3_S1(JLON,JLEV)
      ENDIF

    ENDDO
  ENDDO

ENDIF

IF (LCOEFK_RIH) THEN
  !hybrid mode
  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
      ZPHI3TKE=ZPHI3TA(JLON,JLEV)
      ZCHI3TKE=ZCHI3TA(JLON,JLEV)
      ZPHI3HTKE=MAX((2.0_JPRB*PPHIQ_S1(JLON,JLEV)&
       & -(PPHIQ_S1(JLON,JLEV)-ZPHI3TKE))/&
       &(1.0_JPRB+PPHIQ_S1(JLON,JLEV)/ZPHI3TKE),0.0_JPRB)
      PFMTKE_S1(JLON,JLEV)=PCHI3_S1(JLON,JLEV)&
       &*SQRT(MAX(ZCHI3TKE-C3TKEFREE*ZPHI3TKE*PMRIPP(JLON,JLEV),ZEPS8))
      PIPRT_S1(JLON,JLEV)=ZPHI3HTKE/PCHI3_S1(JLON,JLEV)
      PF_EPS(JLON,JLEV)=SQRT((SQRT(ZCHI3TKE*(1.0_JPRB-PMRIFPP(JLON,JLEV)))&
       & /PCHI3_S1(JLON,JLEV))**3)
      PTH_FUN(JLON,JLEV)=MAX(PTH_FUN(JLON,JLEV)*ZPHIQTA(JLON,JLEV)&
       & /MAX(PPHIQ_S1(JLON,JLEV)-0.5_JPRB*(ZPHIQTA(JLON,JLEV)-ZPHI3TKE),ZEPS8)&
       & ,0.0_JPRB)
      PFUN_TTE(JLON,JLEV)=ZCHI3TKE**2/SQRT(ZCHI3TKE*(1.0_JPRB-PMRIFPP(JLON,JLEV)))
    ENDDO
  ENDDO

ELSE

  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
      ZCHI3TKE=ZCHI3TA(JLON,JLEV)
      PF_EPS(JLON,JLEV)=SQRT((SQRT(ZCHI3TKE*(1.0_JPRB-PMRIFPP(JLON,JLEV)))&
       &                      /ZCHI3TKE)**3)
      PFUN_TTE(JLON,JLEV)=ZCHI3TKE**2/SQRT(ZCHI3TKE*(1.0_JPRB-PMRIFPP(JLON,JLEV)))
      PTH_FUN(JLON,JLEV)=MAX(2.0_JPRB*PTH_FUN(JLON,JLEV)*ZPHIQTA(JLON,JLEV)&
       & /(ZPHIQTA(JLON,JLEV)+ZPHI3TA(JLON,JLEV)),0.0_JPRB)
    ENDDO
  ENDDO

ENDIF

IF(ITTYPE == 6) THEN
  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
      PF_EPS(JLON,JLEV)=1.0_JPRB
    ENDDO
  ENDDO
ENDIF

PF_EPS(KIDIA:KFDIA,KTDIA-1)=PF_EPS(KIDIA:KFDIA,KTDIA)

IF(LCOEFK_F1) THEN
 DO JLON=KIDIA,KFDIA
   ZSIG=MAX(0._JPRB,SIGN(1._JPRB,PMRIFPP(JLON,KLEV)))
   ZFSFTKE=(1.0_JPRB-ZSIG)+ZSIG*SQRT(ZCHI3TA(JLON,KLEV)-&
   & C3TKEFREE*ZPHI3TA(JLON,KLEV)*PMRIPP(JLON,KLEV))
   PFMTKE(JLON,KLEV)=ZCHI3TA(JLON,KLEV)*ZFSFTKE
   PFTTKE(JLON,KLEV)=ZPHI3TA(JLON,KLEV)*ZFSFTKE
   ZFMTKE(JLON,KLEV)=PFMTKE(JLON,KLEV)
   ZFTTKE(JLON,KLEV)=PFTTKE(JLON,KLEV)
 ENDDO
ELSE
 DO JLON=KIDIA,KFDIA
   PFMTKE(JLON,KLEV)=ZCHI3TA(JLON,KLEV)*SQRT(ZCHI3TA(JLON,KLEV)&
    &*(1._JPRB-PMRIFPP(JLON,KLEV)))
   PFTTKE(JLON,KLEV)=PFMTKE(JLON,KLEV)*ZPHI3TA(JLON,KLEV)/ZCHI3TA(JLON,KLEV)
   ZFMTKE(JLON,KLEV)=PFMTKE(JLON,KLEV)
   ZFTTKE(JLON,KLEV)=PFTTKE(JLON,KLEV)
 ENDDO
ENDIF


DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    PFMTKE(JLON,JLEV)=ZCHI3TA(JLON,JLEV)*(ZCHI3TA(JLON,JLEV)-C3TKEFREE*ZPHI3TA(JLON,JLEV)&
     &  *PMRIPP(JLON,JLEV))/SQRT(ZCHI3TA(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV)))
    PFTTKE(JLON,JLEV)=PFMTKE(JLON,JLEV)*ZPHI3TA(JLON,JLEV)/ZCHI3TA(JLON,JLEV)
    ZFMTKE(JLON,JLEV)=ZCHI3TA(JLON,JLEV)*SQRT(ZCHI3TA(JLON,JLEV) &
     &  *(1._JPRB-PMRIFPP(JLON,JLEV)))
    ZFTTKE(JLON,JLEV)=ZFMTKE(JLON,JLEV)*ZPHI3TA(JLON,JLEV)/ZCHI3TA(JLON,JLEV)
  ENDDO
ENDDO

!     CALCUL ANTI-FIBRILLATION "GIRARD-DELAGE".
!     ANTI-FIBRILLATION "GIRARD-DELAGE" COMPUTATION.
IF (LLAFGD) THEN
  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
        ZSIG_AF(JLON,JLEV)=SIGN(0.5_JPRB,PMRIFPP(JLON,JLEV))+0.5_JPRB
    ENDDO
  ENDDO

  IF (ITTYPE==1) THEN

    DO JLEV=KTDIA,KLEV
      DO JLON=KIDIA,KFDIA

        ZDRIFTKE(JLON,JLEV)=(ETKE_RIFC-PMRIFPP(JLON,JLEV))*C3TKEFREE*ETKE_R/&
         & ((ZRITKE_F(JLON,JLEV)*C3TKEFREE+ETKE_RIFC)*ETKE_R-2.0_JPRB*      &
         & PMRIFPP(JLON,JLEV)*ETKE_RIFC)
        ZDPHI3TKE(JLON,JLEV)=(ETKE_RIFC-1.0_JPRB)/&
        & ((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*ETKE_RIFC)*ZDRIFTKE(JLON,JLEV)
        ZDCHI3TKE(JLON,JLEV)=(ETKE_R-1.0_JPRB)/&
        & ((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*ETKE_R)*ZDRIFTKE(JLON,JLEV)
!!        ZDPHIQTKE(JLON,JLEV)=0.0_JPRB

        !IF(LCOEFK_RIH) THEN
        !  ZDPHI3TKE(JLON,JLEV)=(ZDPHI3TKE(JLON,JLEV)-ZDPHIQTKE(JLON,JLEV))/(ZPHIQTA(JLON,JLEV)/ZPHI3TA(JLON,JLEV)+1)&
        !   & -((-ZPHIQTA(JLON,JLEV)+ZPHI3TA(JLON,JLEV)+2*PPHIQ_S1(JLON,JLEV))*(ZDPHIQTKE/ZPHI3TA(JLON,JLEV)&
        !   & -(ZPHIQTA(JLON,JLEV)*ZDPHI3TKE(JLON,JLEV))/ZPHI3TA(JLON,JLEV)**2))/(ZPHIQTA(JLON,JLEV)/ZPHI3TA(JLON,JLEV)+1)**2
        !  ZDCHI3TKE(JLON,JLEV)=0.0_JPRB
        !ENDIF
      ENDDO
    ENDDO

  ENDIF ! type 1

  IF (ITTYPE==2) THEN
    DO JLEV=KTDIA,KLEV
      DO JLON=KIDIA,KFDIA

        !Model I
        ZDRIFTKE(JLON,JLEV)=(ETKE_RIFC-PMRIFPP(JLON,JLEV))*C3TKEFREE*ETKE_R/&
         & ((ZRITKE_F(JLON,JLEV)*C3TKEFREE+ETKE_RIFC)*ETKE_R-2.0_JPRB*      &
         & PMRIFPP(JLON,JLEV)*ETKE_RIFC)
        ZDPHI3TKE(JLON,JLEV)=(ETKE_RIFC-1.0_JPRB)/&
        & ((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*ETKE_RIFC)*ZDRIFTKE(JLON,JLEV)
        ZDCHI3TKE(JLON,JLEV)=(ETKE_R-1.0_JPRB)/&
        & ((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*ETKE_R)*ZDRIFTKE(JLON,JLEV)
!!        ZDPHIQTKE(JLON,JLEV)=-(2.0_JPRB*(3.0_JPRB*ETKE_LAM3-ETKE_LAM2)*&
!!        & ZDRIFTKE(JLON,JLEV))/(3.0_JPRB*Z2AZ0*(1.0_JPRB-PMRIFPP(JLON,JLEV))**2)

!        IF(LCOEFK_RIH) THEN
!          ZDPHI3TKE=(ZDPHI3TKE-ZDPHIQTKE)/(ZPHIQTKE/ZPHI3TKE+1)&
!           & -((-ZPHIQTKE+ZPHI3TKE+2*PPHIQ_S1(JLON,JLEV))*(ZDPHIQTKE/ZPHI3TKE&
!           & -(ZPHIQTKE*ZDPHI3TKE)/ZPHI3TKE**2))/(ZPHIQTKE/ZPHI3TKE+1)**2
!          ZDCHI3TKE=0.0_JPRB
!        ENDIF
      ENDDO
    ENDDO

  ENDIF ! type 2

  IF (ITTYPE==3) THEN

    DO JLEV=KTDIA,KLEV
      DO JLON=KIDIA,KFDIA

        !Model II
        ZDRIFTKE(JLON,JLEV)=(ETKE_RIFC-PMRIFPP(JLON,JLEV))*C3TKEFREE*ETKE_R/&
         & ((ZRITKE_F(JLON,JLEV)*C3TKEFREE+ETKE_RIFC)*ETKE_R-2.0_JPRB*      &
         &PMRIFPP(JLON,JLEV)*ETKE_RIFC)
        ZDPHI3TKE(JLON,JLEV)=(ETKE_RIFC-1.0_JPRB)/&
        &((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*ETKE_RIFC)*ZDRIFTKE(JLON,JLEV)
        ZDCHI3TKE(JLON,JLEV)=(ETKE_R-1.0_JPRB)/&
        & ((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*ETKE_R)*ZDRIFTKE(JLON,JLEV)
!!        ZDPHIQTKE(JLON,JLEV)=(2.0_JPRB*(-((3.0_JPRB*ETKE_LAM3-ETKE_LAM2)*ZDRIFTKE(JLON,JLEV)      &
!!         & *((4.0_JPRB*ETKE_LAM4*PMRIFPP(JLON,JLEV))/(3.0_JPRB*ETKE_LAM3-ETKE_LAM2)     &
!!         & +1.0_JPRB))/(1.0_JPRB-PMRIFPP(JLON,JLEV))**2-(4.0_JPRB*ETKE_LAM4*ZDRIFTKE(JLON,JLEV))   &
!!         & /(1.0_JPRB-PMRIFPP(JLON,JLEV))))/(3.0_JPRB*(C3TKEFREE+1.0_JPRB)*Z2AZ0        &
!!         & *((C3TKEFREE*ZRITKE_F(JLON,JLEV))/PMRIFPP(JLON,JLEV)+C3TKEFREE))-(2.0_JPRB*(1.0_JPRB      &
!!         & -((3.0_JPRB*ETKE_LAM3-ETKE_LAM2)*((4.0_JPRB*ETKE_LAM4*PMRIFPP(JLON,JLEV))    &
!!         & /(3.0_JPRB*ETKE_LAM3-ETKE_LAM2)+1.0_JPRB))/(1.0_JPRB-PMRIFPP(JLON,JLEV)))    &
!!         & *(C3TKEFREE/PMRIFPP(JLON,JLEV)-(C3TKEFREE*ZDRIFTKE(JLON,JLEV)*&
!!         & ZRITKE_F(JLON,JLEV))/PMRIFPP(JLON,JLEV)**2))&
!!         & /(3.0_JPRB*(C3TKEFREE+1.0_JPRB)*Z2AZ0*((C3TKEFREE*ZRITKE_F(JLON,JLEV))/PMRIFPP(JLON,JLEV) &
!!         & +C3TKEFREE)**2)

!        IF(LCOEFK_RIH) THEN
!          ZDPHI3TKE=(ZDPHI3TKE-ZDPHIQTKE)/(ZPHIQTKE/ZPHI3TKE+1)&
!           & -((-ZPHIQTKE+ZPHI3TKE+2*PPHIQ_S1(JLON,JLEV))*(ZDPHIQTKE/ZPHI3TKE&
!           & -(ZPHIQTKE*ZDPHI3TKE)/ZPHI3TKE**2))/(ZPHIQTKE/ZPHI3TKE+1)**2
!          ZDCHI3TKE=0.0_JPRB
!        ENDIF
      ENDDO
    ENDDO

  ENDIF ! type 3
  IF (ITTYPE==4) THEN
    DO JLEV=KTDIA,KLEV
      DO JLON=KIDIA,KFDIA

        !Unstable
        ZDRRI_U=RQNSE_GU1/(RQNSE_GU2*ZRITKE_F(JLON,JLEV)+1.0_JPRB) &
         & -(RQNSE_GU2*(RQNSE_GU1*ZRITKE_F(JLON,JLEV)+RQNSE_S0))/  &
         & (RQNSE_GU2*ZRITKE_F(JLON,JLEV)+1)**2
        !Stable
        ZDRRI=(2.0_JPRB*RQNSE_GS1*ZRITKE_F(JLON,JLEV)+RQNSE_GS2)/&
         & (RQNSE_GS3*ZRITKE_F(JLON,JLEV)**2+ZRITKE_F(JLON,JLEV)*RQNSE_GS4+1.0_JPRB)- &
         & ((2.0_JPRB*RQNSE_GS3*ZRITKE_F(JLON,JLEV)+RQNSE_GS4)                        &
         & *(RQNSE_GS1*ZRITKE_F(JLON,JLEV)**2+RQNSE_S0+ZRITKE_F(JLON,JLEV)*RQNSE_GS2))&
         & /(RQNSE_GS3*ZRITKE_F(JLON,JLEV)**2+ZRITKE_F(JLON,JLEV)*RQNSE_GS4+1.0_JPRB)**2
        ZDRRI=ZSIG_AF(JLON,JLEV)*ZDRRI+(1.0_JPRB-ZSIG_AF(JLON,JLEV))*ZDRRI_U


        ZDRIFTKE(JLON,JLEV)=(ZETKE_P(JLON,JLEV)*ZDRRI*PMRIFPP(JLON,JLEV)**3&
        & +(C3TKEFREE*ZETKE_R(JLON,JLEV)**2-ZETKE_P(JLON,JLEV)**2*ZDRRI)*  &
        & PMRIFPP(JLON,JLEV)**2-2.0_JPRB*C3TKEFREE*ZETKE_P(JLON,JLEV)*     &
        & ZETKE_R(JLON,JLEV)**2*PMRIFPP(JLON,JLEV)+C3TKEFREE*              &
        & ZETKE_P(JLON,JLEV)**2*ZETKE_R(JLON,JLEV)**2)                     &
        & /(ZETKE_P(JLON,JLEV)*ZETKE_R(JLON,JLEV)*PMRIFPP(JLON,JLEV)**2    &
        & -2.0_JPRB*ZETKE_P(JLON,JLEV)**2*ZETKE_R(JLON,JLEV)*              &
        & PMRIFPP(JLON,JLEV)+ZETKE_P(JLON,JLEV)**2*ZETKE_R(JLON,JLEV)**2)

!!        ZDPHIQTKE=(2.0_JPRB*(-((3.0_JPRB*ETKE_LAM3-ETKE_LAM2)*ZDRIFTKE      &
!!        & *((4.0_JPRB*ETKE_LAM4*ZRIFTKE)/(3.0_JPRB*ETKE_LAM3-ETKE_LAM2)     &
!!        & +1.0_JPRB))/(1.0_JPRB-ZRIFTKE)**2-(4.0_JPRB*ETKE_LAM4*ZDRIFTKE)   &
!!        & /(1.0_JPRB-ZRIFTKE)))/(3.0_JPRB*(C3TKEFREE+1.0_JPRB)*Z2AZ0        &
!!        & *((C3TKEFREE*ZRITKE)/ZRIFTKE+C3TKEFREE))-(2.0_JPRB*(1.0_JPRB      &
!!        & -((3.0_JPRB*ETKE_LAM3-ETKE_LAM2)*((4.0_JPRB*ETKE_LAM4*ZRIFTKE)    &
!!        & /(3.0_JPRB*ETKE_LAM3-ETKE_LAM2)+1.0_JPRB))/(1.0_JPRB-ZRIFTKE))    &
!!        & *(C3TKEFREE/ZRIFTKE-(C3TKEFREE*ZDRIFTKE*ZRITKE)/ZRIFTKE**2))      &
!!        & /(3.0_JPRB*(C3TKEFREE+1.0_JPRB)*Z2AZ0*((C3TKEFREE*ZRITKE)/ZRIFTKE &
!!        & +C3TKEFREE)**2)

        ZDCHI3TKE(JLON,JLEV)=((ZDRRI*PMRIFPP(JLON,JLEV))/ZETKE_R(JLON,JLEV)**2    &
        & -ZDRIFTKE(JLON,JLEV)/ZETKE_R(JLON,JLEV))/(1.0_JPRB-PMRIFPP(JLON,JLEV))  &
        & +(ZDRIFTKE(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_R(JLON,JLEV))) &
        & /(1.0_JPRB-PMRIFPP(JLON,JLEV))**2
        ZDPHI3TKE(JLON,JLEV)=(ZDRIFTKE(JLON,JLEV)*              &
        & (1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_P(JLON,JLEV)))     &
        & /(1.0_JPRB-PMRIFPP(JLON,JLEV))**2-ZDRIFTKE(JLON,JLEV)/&
        & (ZETKE_P(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV)))

!        IF(LCOEFK_RIH) THEN
!          ZDPHI3TKE=(ZDPHI3TKE-ZDPHIQTKE)/(ZPHIQTKE/ZPHI3TKE+1)&
!           & -((-ZPHIQTKE+ZPHI3TKE+2*PPHIQ_S1(JLON,JLEV))*(ZDPHIQTKE/ZPHI3TKE&
!           & -(ZPHIQTKE*ZDPHI3TKE)/ZPHI3TKE**2))/(ZPHIQTKE/ZPHI3TKE+1)**2
!          ZDCHI3TKE=0.0_JPRB
!        ENDIF
      ENDDO
    ENDDO

  ENDIF ! type 4

  IF (ITTYPE==5) THEN

    DO JLEV=KTDIA,KLEV
      DO JLON=KIDIA,KFDIA

       !Unstable
        ZEFB_UP=EFB_UR&
        & /(ETKE_OLAM*EFB_UR/(EFB_AZ0*C3TKEFREE)+1.0_JPRB)
        ZDRIFTKE_U=(ZEFB_UP-PMRIFPP(JLON,JLEV))*C3TKEFREE*EFB_UR/&
        &         ((ZRITKE_F(JLON,JLEV)*C3TKEFREE+ZEFB_UP)*EFB_UR&
        & -2.0_JPRB*PMRIFPP(JLON,JLEV)*ZEFB_UP)
        ZDPHI3TKE_U=(ZEFB_UP-1.0_JPRB)/&
        & ((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*ZEFB_UP)*ZDRIFTKE_U
        ZDCHI3TKE_U=(EFB_UR-1.0_JPRB)/ &
        & ((PMRIFPP(JLON,JLEV)-1.0_JPRB)**2*EFB_UR)*ZDRIFTKE_U

       !Stable

        ZDRIFTKE(JLON,JLEV)=(C3TKEFREE*EFB_AZ0*REFB_2*PMRIFPP(JLON,JLEV)**2+     &
        & ETKE_OLAM*REFB_1*PMRIFPP(JLON,JLEV)**2+EFB_UR*ETKE_OLAM*REFB_3*        &
        & PMRIFPP(JLON,JLEV)+C3TKEFREE*EFB_AZ0*REFB_3*PMRIFPP(JLON,JLEV)-        &
        & C3TKEFREE*EFB_AZ0*REFB_1*PMRIFPP(JLON,JLEV)-                           &
        & C3TKEFREE*EFB_AZ0*EFB_UR*REFB_3)**2/(EFB_AZ0*(C3TKEFREE*EFB_AZ0*       &
        & REFB_2**2*PMRIFPP(JLON,JLEV)**4+ETKE_OLAM*REFB_1*REFB_2                &
        &*PMRIFPP(JLON,JLEV)**4+2.0_JPRB*EFB_UR*ETKE_OLAM*REFB_2*REFB_3*         &
        &PMRIFPP(JLON,JLEV)**3+2.0_JPRB*C3TKEFREE*EFB_AZ0*REFB_2*REFB_3*         &
        &PMRIFPP(JLON,JLEV)**3-2.0_JPRB*C3TKEFREE*EFB_AZ0                        &
        &*REFB_1*REFB_2*PMRIFPP(JLON,JLEV)**3+EFB_UR*ETKE_OLAM*REFB_3**2*        &
        &PMRIFPP(JLON,JLEV)**2+C3TKEFREE*EFB_AZ0*REFB_3**2*PMRIFPP(JLON,JLEV)**2-&
        & 2.0_JPRB*C3TKEFREE*EFB_AZ0*EFB_UR*REFB_2*REFB_3*PMRIFPP(JLON,JLEV)**2- &
        & 2.0_JPRB*C3TKEFREE*EFB_AZ0*REFB_1*REFB_3*PMRIFPP(JLON,JLEV)**2         &
        &+C3TKEFREE*EFB_AZ0*REFB_1**2*PMRIFPP(JLON,JLEV)**2-2.0_JPRB*            &
        & C3TKEFREE*EFB_AZ0*EFB_UR*REFB_3**2*PMRIFPP(JLON,JLEV)+2.0_JPRB*        &
        & C3TKEFREE*EFB_AZ0*EFB_UR*REFB_1*REFB_3*                                &
        & PMRIFPP(JLON,JLEV)+C3TKEFREE*EFB_AZ0*EFB_UR**2*REFB_3**2))

        ZDRRI=-((EFB_UR*REFB_2-REFB_1)*REFB_3*ZDRIFTKE(JLON,JLEV))&
        & /(REFB_2**2*PMRIFPP(JLON,JLEV)**2+2.0_JPRB*REFB_2*      &
        & REFB_3*PMRIFPP(JLON,JLEV)+REFB_3**2)

        ZDPRI=-((C3TKEFREE**2*EFB_AZ0**2*EFB_UR*REFB_2-C3TKEFREE**2    &
        & *EFB_AZ0**2*REFB_1)*REFB_3*ZDRIFTKE(JLON,JLEV))/             &
        & ((C3TKEFREE**2*EFB_AZ0**2*REFB_2**2+2.0_JPRB*C3TKEFREE       &
        &*EFB_AZ0*ETKE_OLAM*REFB_1*REFB_2+ETKE_OLAM**2*REFB_1**2)*     &
        & PMRIFPP(JLON,JLEV)**2+((2.0_JPRB*C3TKEFREE*EFB_AZ0*EFB_UR*   &
        & ETKE_OLAM+2*C3TKEFREE**2*EFB_AZ0**2)*REFB_2+(2.0_JPRB*EFB_UR*&
        & ETKE_OLAM**2+2.0_JPRB*C3TKEFREE*EFB_AZ0*ETKE_OLAM)*REFB_1)   &
        &*REFB_3*PMRIFPP(JLON,JLEV)+ (EFB_UR**2*ETKE_OLAM**2+2.0_JPRB* &
        & C3TKEFREE*EFB_AZ0*EFB_UR*ETKE_OLAM+C3TKEFREE**2*EFB_AZ0**2)*REFB_3**2)

        ZDCHI3TKE(JLON,JLEV)=((ZDRRI*PMRIFPP(JLON,JLEV))/ZETKE_R(JLON,JLEV)**2&
        & -ZDRIFTKE(JLON,JLEV)/ZETKE_R(JLON,JLEV))/                           &
        & (1.0_JPRB-PMRIFPP(JLON,JLEV))                                       &
        & +(ZDRIFTKE(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV)/                 &
        &ZETKE_R(JLON,JLEV)))/(1.0_JPRB-PMRIFPP(JLON,JLEV))**2

        ZDPHI3TKE(JLON,JLEV)=((ZDPRI*PMRIFPP(JLON,JLEV))/ZETKE_P(JLON,JLEV)**2 &
        & -ZDRIFTKE(JLON,JLEV)/ZETKE_P(JLON,JLEV))/                            &
        & (1.0_JPRB-PMRIFPP(JLON,JLEV))+(ZDRIFTKE(JLON,JLEV)*                  &
        & (1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_P(JLON,JLEV)))/                   &
        & (1.0_JPRB-PMRIFPP(JLON,JLEV))**2

        ZDRIFTKE(JLON,JLEV)=ZSIG_AF(JLON,JLEV)*ZDRIFTKE(JLON,JLEV)+&
        & (1.0_JPRB-ZSIG_AF(JLON,JLEV))*ZDRIFTKE_U
        ZDCHI3TKE(JLON,JLEV)=ZSIG_AF(JLON,JLEV)* ZDCHI3TKE(JLON,JLEV)+&
        &(1.0_JPRB-ZSIG_AF(JLON,JLEV))* ZDCHI3TKE_U
        ZDPHI3TKE(JLON,JLEV)=ZSIG_AF(JLON,JLEV)* ZDPHI3TKE(JLON,JLEV)+&
        & (1.0_JPRB-ZSIG_AF(JLON,JLEV))* ZDPHI3TKE_U
!!        ZDPHIQTKE=ZDCHI3TKE

!        IF(LCOEFK_RIH) THEN
!          ZDPHI3TKE=(ZDPHI3TKE-ZDPHIQTKE)/(ZPHIQTKE/ZPHI3TKE+1)&
!           & -((-ZPHIQTKE+ZPHI3TKE+2*PPHIQ_S1(JLON,JLEV))*(ZDPHIQTKE/ZPHI3TKE&
!           & -(ZPHIQTKE*ZDPHI3TKE)/ZPHI3TKE**2))/(ZPHIQTKE/ZPHI3TKE+1)**2
!          ZDCHI3TKE=0.0_JPRB
!        ENDIF
      ENDDO
    ENDDO

  ENDIF ! type 5
  IF (ITTYPE==6) THEN

    DO JLEV=KTDIA,KLEV
      DO JLON=KIDIA,KFDIA
        !Unstable
        ZDRRI_U=RLOUIS_GU1/(RLOUIS_GU2*ZRITKE_F(JLON,JLEV)+1.0_JPRB) &
         & -(RLOUIS_GU2*(RLOUIS_GU1*ZRITKE_F(JLON,JLEV)+RLOUIS_S0))/ &
         & (RLOUIS_GU2*ZRITKE_F(JLON,JLEV)+1)**2
        !Stable
        ZDRRI=(2.0_JPRB*RLOUIS_GS1*ZRITKE_F(JLON,JLEV)+RLOUIS_GS2)/      &
        & (RLOUIS_GS3*ZRITKE_F(JLON,JLEV)**2+ZRITKE_F(JLON,JLEV)*        &
        & RLOUIS_GS4+1.0_JPRB)-((2.0_JPRB*RLOUIS_GS3*ZRITKE_F(JLON,JLEV)+&
        & RLOUIS_GS4)*(RLOUIS_GS1*ZRITKE_F(JLON,JLEV)**2+RLOUIS_S0+      &
        & ZRITKE_F(JLON,JLEV)*RLOUIS_GS2))/(RLOUIS_GS3*                  &
        & ZRITKE_F(JLON,JLEV)**2+ZRITKE_F(JLON,JLEV)*RLOUIS_GS4+1.0_JPRB)**2
        ZDRRI=ZSIG_AF(JLON,JLEV)*ZDRRI+(1.0_JPRB-ZSIG_AF(JLON,JLEV))*ZDRRI_U

        !Unstable
        ZDPRI_U=PLOUIS_GU1/(PLOUIS_GU2*ZRITKE_F(JLON,JLEV)+1.0_JPRB)   &
         & -(PLOUIS_GU2*(PLOUIS_GU1*ZRITKE_F(JLON,JLEV)+PLOUIS_S0))/   &
         & (PLOUIS_GU2*ZRITKE_F(JLON,JLEV)+1)**2
        !Stable
        ZDPRI=(2.0_JPRB*PLOUIS_GS1*ZRITKE_F(JLON,JLEV)+PLOUIS_GS2)/      &
        & (PLOUIS_GS3*ZRITKE_F(JLON,JLEV)**2+ZRITKE_F(JLON,JLEV)*        &
        & PLOUIS_GS4+1.0_JPRB)-((2.0_JPRB*PLOUIS_GS3*ZRITKE_F(JLON,JLEV)+&
        & PLOUIS_GS4)*(PLOUIS_GS1*ZRITKE_F(JLON,JLEV)**2+PLOUIS_S0+      &
        & ZRITKE_F(JLON,JLEV)*PLOUIS_GS2))/(PLOUIS_GS3*                  &
        & ZRITKE_F(JLON,JLEV)**2+ZRITKE_F(JLON,JLEV)*PLOUIS_GS4+1.0_JPRB)**2
        ZDPRI=ZSIG_AF(JLON,JLEV)*ZDPRI+(1.0_JPRB-ZSIG_AF(JLON,JLEV))*ZDPRI_U

        ZDRIFTKE(JLON,JLEV)=(PMRIFPP(JLON,JLEV)**3*(ZETKE_P(JLON,JLEV)*        &
        & ZDRRI-ZETKE_R(JLON,JLEV)*ZDPRI)+PMRIFPP(JLON,JLEV)**2*               &
        & (-ZETKE_P(JLON,JLEV)**2*ZDRRI+ZETKE_R(JLON,JLEV)**2*(ZDPRI+1.0_JPRB))&
        &-2.0_JPRB*ZETKE_P(JLON,JLEV)*ZETKE_R(JLON,JLEV)**2*PMRIFPP(JLON,JLEV)+&
        & ZETKE_P(JLON,JLEV)**2*ZETKE_R(JLON,JLEV)**2)/                        &
        &(ZETKE_P(JLON,JLEV)*ZETKE_R(JLON,JLEV)*                               &
        & (PMRIFPP(JLON,JLEV)**2-2*ZETKE_P(JLON,JLEV)*PMRIFPP(JLON,JLEV)+      &
        & ZETKE_P(JLON,JLEV)*ZETKE_R(JLON,JLEV)))

        ZDCHI3TKE(JLON,JLEV)=((ZDRRI*PMRIFPP(JLON,JLEV))/ZETKE_R(JLON,JLEV)**2    &
        & -ZDRIFTKE(JLON,JLEV)/ZETKE_R(JLON,JLEV))/(1.0_JPRB-PMRIFPP(JLON,JLEV))  &
        & +(ZDRIFTKE(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV)/ZETKE_R(JLON,JLEV))) &
        & /(1.0_JPRB-PMRIFPP(JLON,JLEV))**2
        ZDPHI3TKE(JLON,JLEV)=(ZDRIFTKE(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV)/     &
        & ZETKE_P(JLON,JLEV)))/(1.0_JPRB-PMRIFPP(JLON,JLEV))**2-ZDRIFTKE(JLON,JLEV)/&
        & (ZETKE_P(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV)))

      ENDDO
    ENDDO

  ENDIF ! type 6

  !derivatives of Fm and Fh- surface
  IF(LCOEFK_F1) THEN
    DO JLON=KIDIA,KFDIA
      ZFFTKE(JLON,KLEV)=ZCHI3TA(JLON,KLEV)*(1.0_JPRB-PMRIFPP(JLON,KLEV))
      ZFFTKE_DX(JLON,KLEV)=MAX((1.0_JPRB-PMRIFPP(JLON,KLEV))*ZDCHI3TKE(JLON,KLEV)&
        & -ZCHI3TA(JLON,KLEV)*ZDRIFTKE(JLON,KLEV),ZEPS8)
      ZSIG=MAX(0._JPRB,SIGN(1._JPRB,PMRIFPP(JLON,KLEV)))
      !ZFMTKE_DX(JLON,KLEV)=(1.0_JPRB-ZSIG)*ZDCHI3TKE(JLON,KLEV)+ZSIG*&
      ! &((1.0_JPRB-PMRIFPP(JLON,KLEV))*ZDCHI3TKE(JLON,KLEV)&
      ! & -ZCHI3TA(JLON,KLEV)*ZDRIFTKE(JLON,KLEV))
      !ZFHTKE_DX(JLON,KLEV)=(1.0_JPRB-ZSIG)*ZDPHI3TKE(JLON,KLEV)+ZSIG*&
      ! &((1.0_JPRB-PMRIFPP(JLON,KLEV))*ZDPHI3TKE(JLON,KLEV)&
      ! & -ZPHI3TA(JLON,KLEV)*ZDRIFTKE(JLON,KLEV))
      !ZFMTKE_DX(JLON,KLEV)=ZDCHI3TKE(JLON,KLEV)
      !ZFHTKE_DX(JLON,KLEV)=ZDPHI3TKE(JLON,KLEV)
       ZFMTKE_DX(JLON,KLEV)=(1.0_JPRB-ZSIG)*ZDCHI3TKE(JLON,KLEV)+ZSIG*&
       &((ZFFTKE(JLON,KLEV)*ZDCHI3TKE(JLON,KLEV)+&
       & 0.5_JPRB*ZCHI3TA(JLON,KLEV)*ZFFTKE_DX(JLON,KLEV))&
       &/SQRT(ZFFTKE(JLON,KLEV)))
       ZFHTKE_DX(JLON,KLEV)=(1.0_JPRB-ZSIG)*ZDPHI3TKE(JLON,KLEV)+ZSIG*&
       &((ZFFTKE(JLON,KLEV)*ZDCHI3TKE(JLON,KLEV)+&
       & 0.5_JPRB*ZCHI3TA(JLON,KLEV)*ZFFTKE_DX(JLON,KLEV))&
       &/SQRT(ZFFTKE(JLON,KLEV)))
    ENDDO
  ELSE
    DO JLON=KIDIA,KFDIA
      !ZFMTKE_DX(JLON,KLEV)=(1.0_JPRB-PMRIFPP(JLON,KLEV))*ZDCHI3TKE(JLON,KLEV)&
      ! & -ZCHI3TA(JLON,KLEV)*ZDRIFTKE(JLON,KLEV)
      !ZFHTKE_DX(JLON,KLEV)=(1.0_JPRB-PMRIFPP(JLON,KLEV))*ZDPHI3TKE(JLON,KLEV)&
      ! & -ZPHI3TA(JLON,KLEV)*ZDRIFTKE(JLON,KLEV)
       ZFFTKE(JLON,KLEV)=ZCHI3TA(JLON,KLEV)*(1.0_JPRB-PMRIFPP(JLON,KLEV))
       ZFFTKE_DX(JLON,KLEV)=MAX((1.0_JPRB-PMRIFPP(JLON,KLEV))*ZDCHI3TKE(JLON,KLEV)&
        & -ZCHI3TA(JLON,KLEV)*ZDRIFTKE(JLON,KLEV),ZEPS8)
       ZFMTKE_DX(JLON,KLEV)=(ZFFTKE(JLON,KLEV)*ZDCHI3TKE(JLON,KLEV)+&
       & 0.5_JPRB*ZCHI3TA(JLON,KLEV)*ZFFTKE_DX(JLON,KLEV))&
       &/SQRT(ZFFTKE(JLON,KLEV))
       ZFHTKE_DX(JLON,KLEV)=(ZFFTKE(JLON,KLEV)*ZDPHI3TKE(JLON,KLEV)+&
       & 0.5_JPRB*ZPHI3TA(JLON,KLEV)*ZFFTKE_DX(JLON,KLEV))&
       &/SQRT(ZFFTKE(JLON,KLEV))
    ENDDO
  ENDIF

  !derivatives of Fm and Fh- above surface
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      !derivatives of Fm and Fh
!      ZFMTKE_DX(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV))*ZDCHI3TKE(JLON,JLEV)&
!       & -ZCHI3TA(JLON,JLEV)*ZDRIFTKE(JLON,JLEV)
!      ZFHTKE_DX(JLON,JLEV)=(1.0_JPRB-PMRIFPP(JLON,JLEV))*ZDPHI3TKE(JLON,JLEV)&
!       & -ZPHI3TA(JLON,JLEV)*ZDRIFTKE(JLON,JLEV)
      ZFFTKE(JLON,JLEV)=ZCHI3TA(JLON,JLEV)*(1.0_JPRB-PMRIFPP(JLON,JLEV))
      ZFFTKE_DX(JLON,JLEV)=MAX((1.0_JPRB-PMRIFPP(JLON,JLEV))*ZDCHI3TKE(JLON,JLEV)&
       & -ZCHI3TA(JLON,JLEV)*ZDRIFTKE(JLON,JLEV),ZEPS8)
      ZFMTKE_DX(JLON,JLEV)=(ZFFTKE(JLON,JLEV)*ZDCHI3TKE(JLON,JLEV)+&
       & 0.5_JPRB*ZCHI3TA(JLON,JLEV)*ZFFTKE_DX(JLON,JLEV))&
       &/SQRT(ZFFTKE(JLON,JLEV))
      ZFHTKE_DX(JLON,JLEV)=(ZFFTKE(JLON,JLEV)*ZDPHI3TKE(JLON,JLEV)+&
       & 0.5_JPRB*ZPHI3TA(JLON,JLEV)*ZFFTKE_DX(JLON,JLEV))&
       &/SQRT(ZFFTKE(JLON,JLEV))
    ENDDO
  ENDDO

  ! COMPUTAION OF alpha_u and alpha_theta - PAUTKE,PATTKE
  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
      PAUTKE(JLON,JLEV)=0.5_JPRB*ZRITKE_F(JLON,JLEV)*&
      & ZFMTKE_DX(JLON,JLEV)/ZFMTKE(JLON,JLEV)
      PATTKE(JLON,JLEV)=0.5_JPRB*ZRITKE_F(JLON,JLEV)*&
      &ZFHTKE_DX(JLON,JLEV)/ZFTTKE(JLON,JLEV)
    ENDDO
  ENDDO
ENDIF ! LLAFGD

PTH_FUN (KIDIA:KFDIA,KTDIA-1)=PTH_FUN (KIDIA:KFDIA,KTDIA)

!*
!     ------------------------------------------------------------------
! VII -COMUTATION of moist gustiness correction PRRCOR=sqrt(gamma_PRC)
! computed from Ri'
IF (LRRGUST) THEN
  !SURFACE
  DO JLON=KIDIA,KFDIA
    ZCD=ZFMTKE(JLON,KLEV)*PCDN(JLON)
    ZFP=MAX(0.0_JPRB,PFPLSH(JLON,KLEV)+PFPLCH(JLON,KLEV))
    PRCORR(JLON,KLEV)=SQRT(SQRT(1.0_JPRB+((((ZFP/(ZFP+RRSCALE))**RRGAMMA)*UTILGUST)**2) &
     & /(ZCD*ZU(JLON,KLEV)**2)))
  ENDDO

  !REMAINING LEVELS
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZDPHI=PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)
      ZKUROV=MAX(ZFMTKE(JLON,JLEV)*ZU(JLON,JLEV)* (PLMU(JLON,JLEV)*RG)**2 &
       & *PAPRS(JLON,JLEV)*ZRTI(JLON,JLEV)/ZDPHI**2,ZEPS)
      ZFP=MAX(0.0_JPRB,PFPLSH(JLON,JLEV)+PFPLCH(JLON,JLEV))
      PRCORR(JLON,JLEV)=SQRT(SQRT(1.0_JPRB+((((ZFP/(ZFP+RRSCALE))**RRGAMMA) &
       & *UTILGUST)**2)*PAPRS(JLON,JLEV)*ZRTI(JLON,JLEV)/(ZU(JLON,JLEV)*ZKUROV)))  
    ENDDO
  ENDDO

ENDIF

!*
!     ------------------------------------------------------------------
!     VIII - HORIZONTAL FUNCTIONS FOR 3D-TURBULENCE

IF (LL3DTURB.AND.(ITTYPE==4)) THEN
!3DTURB + QNSE

  ! QNSE fit for CHI_H and PHI_H*C3
  DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
      IF (ZRITKE_F(JLON,JLEV) < 0.0_JPRB ) THEN
        ZAU= 1._JPRB+ZRITKE_F(JLON,JLEV)                        &
         & *(CHIHQNSE_U4+ZRITKE_F(JLON,JLEV)*CHIHQNSE_U3 )
        ZAU=SIGN(MAX(ZEPS,ABS(ZAU)),ZAU)
        PFHORM(JLON,JLEV)=(1._JPRB+ZRITKE_F(JLON,JLEV)          &
         & *(CHIHQNSE_U2+ZRITKE_F(JLON,JLEV)*CHIHQNSE_U1 ))/ZAU
        ZAT= 1._JPRB+ZRITKE_F(JLON,JLEV)                        &
         & *(PHIHQNSE_U4+ZRITKE_F(JLON,JLEV)*PHIHQNSE_U3 )
        ZAT=SIGN(MAX(ZEPS,ABS(ZAT)),ZAT)
        PFHORH(JLON,JLEV)=(1._JPRB+ZRITKE_F(JLON,JLEV)          &
         & *(PHIHQNSE_U2+ZRITKE_F(JLON,JLEV)*PHIHQNSE_U1 ))/ZAT &
         & *C3TKEFREE
      ELSE
        ZAU= 1._JPRB+ZRITKE_F(JLON,JLEV)                        &
         & *(CHIHQNSE_S4+ZRITKE_F(JLON,JLEV)*CHIHQNSE_S3 )
        ZAU=SIGN(MAX(ZEPS,ABS(ZAU)),ZAU)
        PFHORM(JLON,JLEV)=(1._JPRB+ZRITKE_F(JLON,JLEV)          &
         & *(CHIHQNSE_S2+ZRITKE_F(JLON,JLEV)*CHIHQNSE_S1 ))/ZAU 
        ZAT= 1._JPRB+ZRITKE_F(JLON,JLEV)                        &
         & *(PHIHQNSE_S6+ZRITKE_F(JLON,JLEV)*(PHIHQNSE_S5       &
         & +ZRITKE_F(JLON,JLEV)*PHIHQNSE_S4 ))
        ZAT=SIGN(MAX(ZEPS,ABS(ZAT)),ZAT)
        PFHORH(JLON,JLEV)=(1._JPRB+ZRITKE_F(JLON,JLEV)          &
         & *(PHIHQNSE_S3+ZRITKE_F(JLON,JLEV)*(PHIHQNSE_S2       &
         & +ZRITKE_F(JLON,JLEV)*PHIHQNSE_S1 )))/ZAT             &
         & *C3TKEFREE
      ENDIF
    ENDDO
  ENDDO
  ! Upper bound 
  PFHORM(KIDIA:KFDIA,KTDIA-1)=PFHORM(KIDIA:KFDIA,KTDIA)
  PFHORH(KIDIA:KFDIA,KTDIA-1)=PFHORH(KIDIA:KFDIA,KTDIA)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ACMRIP',1,ZHOOK_HANDLE)
END SUBROUTINE ACMRIP
