!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACPTKE ( KIDIA,KFDIA,KLON,KTDIA,KLEV,KSTEP,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI,PAPHIF,PDELP,PRDELP,PKNROV,&
 & PT,PU,PV,PALPH,&
 & PAPRS,PR,PTKE,PLMU,PLNPR,PF_EPS,PFUN_TTE,&
 & PMRIPP,PMRIFPP,PFMTKE_S1,PIPRT_S1,PCHI3_S1,PPHIQ_S1,PRHS,&
 & PMN2_ES,PMN2_EQ,PRRCOR,PDIFTQ,PDIFTS,PSHEAR,&
 !- INPUT 1D
 & PCD,PGZ0,&
 ! - INPUT/OUTPUT 2D
 & PKUROV,PKTROV,PXPTKEROV,&
 & PLMLTILD,PMXL,PLML,&
 & PTH_FUN,PWW_FUN,&
 ! - OUTPUT 2D .
 & PTENDPTKE,PTAUITKE,PKERV,PTTE,PTKETILD)


!**** *ACPTKE * - Specific p-TKE computation.

!     Sujet.
!     ------
!     - ROUTINE DE CALCUL ACTIF .
!       CALCULS DE LA P-TKE. 

!**   Interface.
!     ----------
!        *CALL* *ACPTKE*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KTDIA      : INDICE DE DEPART DES BOUCLES VERTICALES (1 EN GENERAL).
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".
! KSTEP      : NUMERO DU PAS DE TEMPS.                       

! - NOM DES VARIABLES DE LA PHYSIQUE 
! * INPUT

! - 2D (1:KLEV) .
! PAPHI      : GEOPOTENTIEL AUX DEMI-NIVEAUX.
! PAPHIF     : GEOPOTENTIEL AUX NIVEAUX DES COUCHES. 
! PDELP      : EPAISSEUR EN PRESSION DE LA COUCHE. 
! PRDELP     : INVERSE DE L'EPAISSEUR EN PRESSION DE LA COUCHE.


! - 2D (0:KLEV) .
! PKNROV     : COEFFICIENT D'ECHANGE VERTICAL NEUTRE (U,V) EN KG/(M*M*S). 

! - 2D (1:KLEV) .
! PT         : TEMPERATURE.
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.
! PALPH      : LOG(PAPRS(JLEV)/PAPRSF(JLEV)) (FOR HYDROSTATICS).

! - 2D (0:KLEV) .
! PAPRS      : HALF LEVEL PRESSURE
! PLMU       : MIXING LENGTH FOR MOMENTUM
! PFMS1TKE   : Fm computed from Ri_s1 
! PMRIFPP    : moist Rif
! PMRIPRT    : Prt-term computed from Ri_s1 - for K_h computation after TKE solver
! PRSH       : SOURCE TERM FROM 3D TURBULENCE FOR TKE SOLVER
! PF_EPS     : TERM ASOCIATED WITH MIX. LENGTH CONVERSION L-lm: F_epsilon
! PFUN_TTE   : Function used for computation of tte_tilde in case of LCOEFK_FLX=.FALSE.

! - 2D (1:KLEV) .
! PR         : CONSTANTE DES GAZ POUR L'AIR. 
! PTKE       : TURBULENT KINETIC ENERGY
! PLNPR      : LOG(PAPRS(JLEV)/PAPRS(JLEV-1)) (FOR HYDROSTATICS).

! - 1D (DIAGNOSTIQUE) .
! PCD        : COEFFICIENT D'ECHANGE EN SURFACE POUR U ET V.
! PGZ0       : G FOIS LA LONGUEUR DE RUGOSITE COURANTE.

! * INPUT/OUTPUT ARGUMENTS

! - 2D (0:KLEV) .
! PKUROV     : COEFFICIENT D'ECHANGE VERTICAL DE U ET V EN KG/(M*M*S).
! PKTROV     : COEFFICIENT D'ECHANGE VERTICAL DE T ET Q EN KG/(M*M*S).
! PWW_FUN    : F_ww - stability functions for TOMs par.
! PTH_FUN    : T_h - stability functions for TOMs par.
! PXPTKEROV  : "ANTI-FIBRILLATION" COEFFICIENT FOR THE TKE AUTO-DIFFUSION

! * OUTPUT
 
! - 2D (1:KLEV) .
! PTENDPTKE  : TKE INCREMENT (USED BY PSEUDO-PROGNOSTIC TKE SCHEME)
! - 2D (0:KLEV) .
! PTAUITKE   : dt/tau_epsilon
! PKERV      : K_E - TKE autodiffsion coefficient
!-----------------------------------------------------------------------
!     Auteur.
!     -------
!        2008-02, R. Brozkova (moving relevant part from ACDIFUS)

!     Modifications.
!     --------------
!      F. Vana  01-Sep-2008: all pTKE specific stuff collected here.
!      F. Vana  21-Jan-2009: bug fix of PGZ0 + new stabilization of solver
!      K. Yessad (Jul 2009): remove CDLOCK + some cleanings
!      I. Bastak Duran 07-2012: pTKE solver moved to ACPTKES
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG,RKAPPA,RCPV,RCPD,RPI
USE YOMPHY   , ONLY : LCOEFK_RIH, LCOEFK_PTTE,  LCOEFK_FLX,&
& LCOEFK_PL,CGTURS
USE YOMPHY0  , ONLY : NUPTKE,VKARMN, ETKE_BETA_EPS,ETKE_GAMMA_EPS,&
& C3TKEFREE,ETKE_OLAM,ETKE_LAM0,ETKE_R,ETKE_RIFC,ETKE_TAULM,&
& ETKE_LAM2,ETKE_LAM3,ETKE_LAM4,ENT_LAMBDA,C_EPSILON,&
& EFB_UR,EFB_AZ0,REFB_1,REFB_2,REFB_3,GCISMIN,ALMAV,ETKE_KLM
USE YOMPHY2  , ONLY : TSPHY ,LMULAF
USE YOM_YGFL , ONLY : YTKE_NL
USE YOMQNSE  , ONLY : RQNSE_S0,RQNSE_GS1,RQNSE_GS2,RQNSE_GS3,RQNSE_GS4,&
 & RQNSE_GU1,RQNSE_GU2

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF(KLON,KLEV)  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKNROV(KLON,0:KLEV)  
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTKE(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLMU(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PF_EPS(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFUN_TTE(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMRIFPP  (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMRIPP  (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFMTKE_S1 (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPRT_S1(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCHI3_S1 (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHIQ_S1 (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMN2_ES   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMN2_EQ   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHS(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCD(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0(KLON)  
 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKUROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKTROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWW_FUN   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTH_FUN   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN) :: PRRCOR   (KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXPTKEROV(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTAUITKE(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKERV(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTKETILD(KLON,0:KLEV)

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTENDPTKE(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLMLTILD(KLON,0:KLEV) ! 'STATIC' TKE type mixing length
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLML(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMXL(KLON,KLEV)

!they are actually on half levels
REAL(KIND=JPRB)   ,INTENT(IN) :: PSHEAR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PDIFTS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PDIFTQ(KLON,KLEV) 

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTTE(KLON,KLEV) 
!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JLON, ITTYPE

REAL(KIND=JPRB) :: ZALTKE, ZLIMTKE, ZNU3TKE

REAL(KIND=JPRB) :: ZRTI, ZTKES
REAL(KIND=JPRB) :: ZRLM_L(KLON,0:KLEV),ZLIMTKE_L,&
& ZKERV_L(KLON,0:KLEV),ZTAUITKE_L(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZEPSTKE, ZEPSKRV, ZRELAXB, ZDRELAXB
REAL(KIND=JPRB) :: ZTERM_II,ZCP

REAL(KIND=JPRB) :: ZTKE1(KLON,KLEV)
REAL(KIND=JPRB) :: ZTKETILD(KLON,0:KLEV),ZCONVERT(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZTTETILD(KLON,0:KLEV),ZTAUITTE(KLON,0:KLEV),&
& ZKTTERV(KLON,0:KLEV),ZTTE1(KLON,KLEV)
REAL(KIND=JPRB) :: ZKESROV(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZSH0,&
& ZPHI3HTKE,ZTKE_LAM5T,ZNU4TKE,ZTKE_OLAMC3,&
& ZRITKE,Z_ETKE_RU,ZS_CDIS,ZRITKE_U
REAL(KIND=JPRB) :: Z_ETKE_R(KLON,0:KLEV),Z_ETKE_P(KLON,0:KLEV),&
& ZCHI3TKE(KLON,0:KLEV),ZPHI3TKE(KLON,0:KLEV),&
& ZPHIQTKE(KLON,0:KLEV),ZFMTKE(KLON,0:KLEV),ZPRT_INV(KLON,0:KLEV),&
& ZRIFTKE(KLON,0:KLEV),ZRHO(KLON,0:KLEV),ZTERM_I(KLON,0:KLEV),&
& ZDPHIA(KLON,0:KLEV),ZRLM(KLON,0:KLEV),ZWW_FUN_S1(KLON,0:KLEV),&
& ZTAUICONV(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZMRIFPP,ZMRIFPPI
REAL(KIND=JPRB) :: ZRHS(KLON,KLEV), ZLML1(KLON,KLEV),ZCK
REAL(KIND=JPRB) :: ZS_AA1, ZS_AA2, ZS_AA3, ZS_AA4,ZS_CW,&
 & ZS_CTHIRD,ZS_ZSOL,ZS_A,ZS_B,ZS_C,&
 & ZS_CP, ZS_CQ, ZS_CPHI, ZS_CU1,ZS_CU2,ZS_CU3,ZS_CX1,&
 & ZS_CX2,ZS_CX3,Z2THIRD,Z2AZ0
REAL(KIND=JPRB) :: ZEPS4,ZEPS8,ZSIG(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZGLU
LOGICAL         :: LLM123

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "acptkes.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ACPTKE',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

! * Initialization of constants
ZGLU=RG*ALMAV

ZRELAXB=1.5_JPRB
ZDRELAXB=1._JPRB - ZRELAXB

ZEPSKRV=1.E-8_JPRB
ZEPSTKE=1.E-8_JPRB  ! not really needed, it should be secured in ALPLAR

ZALTKE=1.0_JPRB/(NUPTKE*NUPTKE)
ZNU3TKE=NUPTKE**3
ZLIMTKE=0.5_JPRB*NUPTKE*ZNU3TKE*ZRELAXB/(RG*RG)
ZLIMTKE_L=0.5_JPRB*NUPTKE*ZNU3TKE*ZRELAXB/(RG*RG)/(ETKE_TAULM*ETKE_KLM)

ZCP=ETKE_OLAM/ETKE_LAM0/C3TKEFREE
ZNU4TKE=NUPTKE**4
ZSH0=ZNU4TKE/2._JPRB*C3TKEFREE
ZTKE_OLAMC3=0.75_JPRB*ETKE_OLAM/C3TKEFREE
ZCK=ZNU4TKE/C_EPSILON

ZS_CTHIRD=1.0_JPRB/3.0_JPRB
Z2THIRD=2.0_JPRB/3.0_JPRB
Z2AZ0=2.0_JPRB/3.0_JPRB*(1.0_JPRB-3.0_JPRB*ETKE_LAM3-ETKE_LAM2)

ZEPS4=1e-4_JPRB
ZEPS8=1e-8_JPRB

IF     (TRIM(CGTURS) == 'RMC') THEN
  ITTYPE=1
ELSEIF (TRIM(CGTURS) == 'MD1') THEN
  ITTYPE=2
ELSEIF (TRIM(CGTURS) == 'MD2') THEN
  ITTYPE=3
ELSEIF (TRIM(CGTURS) == 'QNSE') THEN
  ITTYPE=4
ELSEIF (TRIM(CGTURS) == 'EFB') THEN
  ITTYPE=5
ELSEIF (TRIM(CGTURS) == 'LOUIS') THEN
  ITTYPE=6
ENDIF
LLM123=(ITTYPE == 1).OR.(ITTYPE == 2).OR.(ITTYPE == 3)
!     ------------------------------------------------------------------

IF (LMULAF) THEN
  DO JLEV=KTDIA,KLEV-1
     DO JLON=KIDIA,KFDIA
        PXPTKEROV(JLON,JLEV)=SQRT(PXPTKEROV(JLON,JLEV))
     ENDDO
  ENDDO
  
  DO JLEV=KTDIA+1,KLEV-1
    DO JLON=KIDIA,KFDIA
       PXPTKEROV(JLON,JLEV)=MAX(PXPTKEROV(JLON,JLEV),PXPTKEROV(JLON,JLEV-1))
    ENDDO
  ENDDO
ENDIF

!*
!     -----------------------------------------------------------------
!     COMPUTATION OF PTKE RELATED VALUES

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA

    ZRTI=2._JPRB/(PR(JLON,JLEV)*PT(JLON,JLEV)+PR(JLON,JLEV+1)*PT(JLON,JLEV+1))
    ZDPHIA(JLON,JLEV)=PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)
    ZRLM(JLON,JLEV)= ((ZLIMTKE*ZDPHIA(JLON,JLEV)/(PXPTKEROV(JLON,JLEV)&
     & *PAPRS(JLON,JLEV)*ZRTI))*MAX(PDELP(JLON,JLEV)*(1._JPRB-(PALPH(JLON,JLEV)&
     & /PLNPR(JLON,JLEV))),PDELP(JLON,JLEV+1)*PALPH(JLON,JLEV+1)&
     & /PLNPR(JLON,JLEV+1)))/(PLMU(JLON,JLEV)*PLMU(JLON,JLEV))
    ZRLM_L(JLON,JLEV)= ((ZLIMTKE_L*ZDPHIA(JLON,JLEV)/(PXPTKEROV(JLON,JLEV)&
     & *PAPRS(JLON,JLEV)*ZRTI))*MAX(PDELP(JLON,JLEV)*(1._JPRB-(PALPH(JLON,JLEV)&
     & /PLNPR(JLON,JLEV))),PDELP(JLON,JLEV+1)*PALPH(JLON,JLEV+1)&
     & /PLNPR(JLON,JLEV+1)))/(PLMU(JLON,JLEV)*PLMU(JLON,JLEV))
    ZCONVERT(JLON,JLEV)=PAPRS(JLON,JLEV)*PLMU(JLON,JLEV)*RG*ZRTI*NUPTKE/&
    & ZDPHIA(JLON,JLEV)
    ZTAUICONV(JLON,JLEV)=TSPHY*ZNU3TKE/(PLMU(JLON,JLEV)*ZCONVERT(JLON,JLEV))
    ZKESROV(JLON,JLEV)=SQRT(PKNROV(JLON,JLEV)*MAX(PKUROV(JLON,JLEV),ZEPS8))
  ENDDO
ENDDO

IF ((KSTEP == 0).AND.(YTKE_NL%NREQIN == 0)) THEN
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA

    ! CHOICE OF COEFFICIENTS FOR TKE DIFUSION AND RELAXATION
    ! THEY CAN BE COMPUTED FROM THE PREVIOUS TIMESTEP TKE VALUE
    ! OR FROM THE TILDED TKE VALUE (DIAGNOSED AT THE INSTANT
    ! TIMESTEP FROM THE EXCHANGE COEFFICIENT)

      ! COEFFICIENT FOR RELAXATION (times dt)
      ! CORRESPONDS TO TILDED VALUES
      PTAUITKE(JLON,JLEV)=ZTAUICONV(JLON,JLEV)*ZKESROV(JLON,JLEV)*PF_EPS(JLON,JLEV)
      ZTAUITKE_L(JLON,JLEV)=ZTAUICONV(JLON,JLEV)*ZKESROV(JLON,JLEV)/ETKE_TAULM

      ! COEFFICIENT FOR DIFFUSION CORRESPONDS TO TILDED VALUES (with
      !  the security for small tau_epsilon)
      PKERV(JLON,JLEV)=ZALTKE*ZKESROV(JLON,JLEV) &
        &* MAX(1._JPRB,PF_EPS(JLON,JLEV)) &
        &* MAX(1._JPRB,ZRLM(JLON,JLEV)/(1._JPRB+ETKE_BETA_EPS*PTAUITKE(JLON,JLEV)))

      ZKERV_L(JLON,JLEV)=ETKE_KLM*ZALTKE*ZKESROV(JLON,JLEV) &
        &* MAX(1._JPRB,PF_EPS(JLON,JLEV)) &
        &* MAX(1._JPRB,ZRLM_L(JLON,JLEV)/(1._JPRB+ETKE_BETA_EPS*PTAUITKE(JLON,JLEV)))
        
      !security for small tau_epsilon
      PTAUITKE(JLON,JLEV)=PTAUITKE(JLON,JLEV)&
        &/(1._JPRB+ETKE_BETA_EPS*PTAUITKE(JLON,JLEV))
      !security for large tau_epsilon
      PTAUITKE(JLON,JLEV)=PTAUITKE(JLON,JLEV)&
      & *(1._JPRB+ETKE_GAMMA_EPS/PTAUITKE(JLON,JLEV))

      !security for small tau_L
      ZTAUITKE_L(JLON,JLEV)=ZTAUITKE_L(JLON,JLEV)&
        &/(1._JPRB+ETKE_BETA_EPS*ZTAUITKE_L(JLON,JLEV))
      !security for large tau_epsilon
      ZTAUITKE_L(JLON,JLEV)=ZTAUITKE_L(JLON,JLEV)&
      & *(1._JPRB+ETKE_GAMMA_EPS/ZTAUITKE_L(JLON,JLEV))
    ENDDO
  ENDDO
ELSE
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZTKES=0.5_JPRB*(PTKE(JLON,JLEV)+PTKE(JLON,JLEV+1))
      ! COEFFICIENT FOR RELAXATION (times dt)
      ! CORRESPONDS TO PREVIOUS TKE VALUE
      PTAUITKE(JLON,JLEV)=ZTAUICONV(JLON,JLEV)*ZCONVERT(JLON,JLEV) &
       & *SQRT(MAX(ZEPSTKE,ZTKES))*PF_EPS(JLON,JLEV)

      ZTAUITKE_L(JLON,JLEV)=ZTAUICONV(JLON,JLEV)*ZCONVERT(JLON,JLEV) &
       & *SQRT(MAX(ZEPSTKE,ZTKES))*PF_EPS(JLON,JLEV)/ETKE_TAULM
      ! COEFFICIENT FOR DIFFUSION CORRESPONDS TO PREVIOUS TKE VALUE
      !  (with the security for small tau_epsilon)
      PKERV(JLON,JLEV)=ZALTKE*ZCONVERT(JLON,JLEV) &
       & *SQRT(MAX(ZEPSTKE,ZTKES))*MAX(1._JPRB,PF_EPS(JLON,JLEV)) &
       & * MAX(1._JPRB,ZRLM(JLON,JLEV)/(1._JPRB+ETKE_BETA_EPS*PTAUITKE(JLON,JLEV)))
       
      ZKERV_L(JLON,JLEV)=ETKE_KLM*ZALTKE*ZCONVERT(JLON,JLEV) &
       & *SQRT(MAX(ZEPSTKE,ZTKES))*MAX(1._JPRB,PF_EPS(JLON,JLEV)) &
       & * MAX(1._JPRB,ZRLM_L(JLON,JLEV)/(1._JPRB+ETKE_BETA_EPS*PTAUITKE(JLON,JLEV)))
      !security for small tau_epsilon
      PTAUITKE(JLON,JLEV)=PTAUITKE(JLON,JLEV)&
        &/(1._JPRB+ETKE_BETA_EPS*PTAUITKE(JLON,JLEV))
      !security for large tau_epsilon
      PTAUITKE(JLON,JLEV)=PTAUITKE(JLON,JLEV)&
      & *(1._JPRB+ETKE_GAMMA_EPS/PTAUITKE(JLON,JLEV))
    ENDDO
  ENDDO
ENDIF

IF(LCOEFK_PTTE) THEN
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZTAUITTE(JLON,JLEV)=PTAUITKE(JLON,JLEV)/&
        &(1.0_JPRB-(1.0_JPRB-ZCP)*PMRIFPP(JLON,JLEV))
      ZKTTERV(JLON,JLEV)=PKERV(JLON,JLEV)/&
      &(1.0_JPRB-(1.0_JPRB-ZCP)*PMRIFPP(JLON,JLEV))
    ENDDO
  ENDDO
ENDIF

!source terms computed from fluxes
IF(LCOEFK_FLX .AND. KSTEP > 0) THEN
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZRHO(JLON,JLEV)=2._JPRB/(PR(JLON,JLEV)*PT(JLON,JLEV)+PR(JLON,JLEV+1)*PT(JLON,JLEV+1))&
      &               *PAPRS(JLON,JLEV)
      ZTERM_I(JLON,JLEV)= PSHEAR(JLON,JLEV)*RG/ZDPHIA(JLON,JLEV)

      ZTERM_II=-(PMN2_ES(JLON,JLEV)*PDIFTS(JLON,JLEV)+&
      & PMN2_EQ(JLON,JLEV)*PDIFTQ(JLON,JLEV))

      ZTKETILD(JLON,JLEV)=TSPHY/PTAUITKE(JLON,JLEV)/ZRHO(JLON,JLEV)*&
      & (ZTERM_I(JLON,JLEV)+ZTERM_II)
    ENDDO
  ENDDO
     
  IF(LCOEFK_PTTE) THEN
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        ZTTETILD(JLON,JLEV)=TSPHY/ZTAUITTE(JLON,JLEV)/&
        & ZRHO(JLON,JLEV)*ZTERM_I(JLON,JLEV)
      ENDDO
    ENDDO
  ENDIF
ELSE 
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZTKETILD(JLON,JLEV)=PKUROV(JLON,JLEV)*PKNROV(JLON,JLEV)/ZCONVERT(JLON,JLEV)**2
    ENDDO
  ENDDO
  IF(LCOEFK_PTTE) THEN
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
       ZTTETILD(JLON,JLEV)=PTAUITKE(JLON,JLEV)/ZTAUITTE(JLON,JLEV)*&
       &(PKNROV(JLON,JLEV)/ZCONVERT(JLON,JLEV))**2*PFUN_TTE(JLON,JLEV)
      ENDDO
    ENDDO
  ENDIF
ENDIF
    

!     CALCULS EN SURFACE.
!     SURFACE CALCULATIONS.

DO JLON=KIDIA,KFDIA

  ! FV: This is bit inconsistent as the E_tilde and mixing lenght
  !     are obtained in this case for the for the full level.
  ZTKETILD(JLON,KLEV)=ZALTKE*PCD(JLON)&
   & *(PU(JLON,KLEV)**2+PV(JLON,KLEV)**2)
  PTAUITKE(JLON,KLEV)=TSPHY*ZNU3TKE*RG*SQRT(ZTKETILD(JLON,KLEV))&
   & /(VKARMN*(PGZ0(JLON)+PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)))*PF_EPS(JLON,KLEV)

  !security for small tau_epsilon
  PTAUITKE(JLON,KLEV)=PTAUITKE(JLON,KLEV)&
   & /(1._JPRB+ETKE_BETA_EPS*PTAUITKE(JLON,KLEV))
  !security for large tau_epsilon
  PTAUITKE(JLON,KLEV)=PTAUITKE(JLON,KLEV)&
   & *(1._JPRB+ETKE_GAMMA_EPS/PTAUITKE(JLON,KLEV))
ENDDO

IF(LCOEFK_PTTE) THEN
  DO JLON=KIDIA,KFDIA
    ZTTETILD(JLON,KLEV)=ZTKETILD(JLON,KLEV)*&
    & (1.0_JPRB-(1.0_JPRB-ZCP)*PMRIFPP(JLON,KLEV))/&
    & (1.0_JPRB-PMRIFPP(JLON,KLEV))
    ZTAUITTE(JLON,KLEV)=PTAUITKE(JLON,KLEV)/&
    &(1.0_JPRB-(1.0_JPRB-ZCP)*PMRIFPP(JLON,KLEV))
  ENDDO
ENDIF


!     ------------------------------------------------------------------
!TKE solver
ZRHS (KIDIA:KFDIA,KTDIA:KLEV) = 0.0_JPRB !NO SOURCE TERM
CALL ACPTKES ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
  & PRDELP,PALPH,PLNPR,PTKE,PRHS,&
  & PTAUITKE,PKERV,PXPTKEROV,ZTKETILD,1.0_JPRB,&
  & ZTKE1)

!   TTE SOLVER
IF(LCOEFK_PTTE) THEN  
  !initialisation
  IF(KSTEP==0) THEN
   DO JLEV=KTDIA,KLEV
    DO JLON=KIDIA,KFDIA
     PTTE(JLON,JLEV)=PTKE(JLON,JLEV)*&
     & (1.0_JPRB+ZCP*PMRIFPP(JLON,JLEV)&
     & /(1.0_JPRB-PMRIFPP(JLON,JLEV)))
    ENDDO
   ENDDO
  ENDIF
  
  ZRHS (KIDIA:KFDIA,KTDIA:KLEV) = 0.0_JPRB !NO SOURCE TERM
  CALL ACPTKES ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
   & PRDELP,PALPH,PLNPR,PTTE,ZRHS,&
   & ZTAUITTE,ZKTTERV,PXPTKEROV,ZTTETILD,1.0_JPRB,&
   & ZTTE1)
  PTTE(KIDIA:KFDIA,KTDIA:KLEV)=ZTTE1(KIDIA:KFDIA,KTDIA:KLEV)
ENDIF

!   PSEUDO TKE SOLVER - PROGNOSTIC L
IF(LCOEFK_PL) THEN
  ZRHS (KIDIA:KFDIA,KTDIA:KLEV) = 0.0_JPRB !NO SOURCE TERM
  CALL ACPTKES ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
    & PRDELP,PALPH,PLNPR,PMXL,ZRHS,&
    & ZTAUITKE_L,ZKERV_L,PXPTKEROV,PLMLTILD,1.0_JPRB,&
    & ZLML1)

  PMXL(KIDIA:KFDIA,KTDIA:KLEV-1)=ZLML1(KIDIA:KFDIA,KTDIA:KLEV-1)
  !surface
  PMXL(KIDIA:KFDIA,KLEV)=(PALPH(KIDIA:KFDIA,KLEV)/PLNPR(KIDIA:KFDIA,KLEV)*&
     & PLMLTILD(KIDIA:KFDIA,KLEV-1) + &
     & (1.0_JPRB-PALPH(KIDIA:KFDIA,KLEV)/PLNPR(KIDIA:KFDIA,KLEV))*&
     & PLMLTILD(KIDIA:KFDIA,KLEV))

!update of mixing length for TOMs only
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
     PLML(JLON,JLEV)=(PMXL(JLON,JLEV)+PMXL(JLON,JLEV+1))*0.5_JPRB*PRRCOR(JLON,JLEV)
    ENDDO
  ENDDO

ENDIF


!* !     ------------------------------------------------------------------
! * Modification of exchange coefficients
IF(LCOEFK_PTTE.AND.KSTEP>0) THEN
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA

      !Rif_m computation from TKE and TTE
      ZMRIFPP=MIN(ETKE_RIFC-ZEPS8,&
      & (MAX(PTTE(JLON,JLEV),ZEPS4)-ZTKE1(JLON,JLEV))&
      & /MAX(ZEPS8,(1.0_JPRB-ZCP)*ZTKE1(JLON,JLEV)+MAX(PTTE(JLON,JLEV),ZEPS4)))
      ZMRIFPPI=MIN(ETKE_RIFC-ZEPS8,&
      & (MAX(PTTE(JLON,JLEV+1),ZEPS4)-ZTKE1(JLON,JLEV+1))&
      & /MAX(ZEPS8,(1.0_JPRB-ZCP)*ZTKE1(JLON,JLEV+1)+MAX(PTTE(JLON,JLEV+1),ZEPS4)))
      ZRIFTKE(JLON,JLEV)=0.5_JPRB*(ZMRIFPP+ZMRIFPPI)
      !security
      ZRIFTKE(JLON,JLEV)=MIN(ZRIFTKE(JLON,JLEV),ETKE_RIFC-ZEPS8)

      !Rif computation
      ZSIG(JLON,JLEV)=SIGN(0.5_JPRB,ZRIFTKE(JLON,JLEV))+0.5_JPRB
    ENDDO
  ENDDO

  IF(LLM123) THEN
  !modified CCH02 or RMC01
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        Z_ETKE_R(JLON,JLEV)=ETKE_R
        Z_ETKE_P(JLON,JLEV)=ETKE_RIFC
      ENDDO
    ENDDO
  ELSEIF (ITTYPE == 4) THEN 
    !QNSE fit
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        !UNSTABLE
          ZS_A = C3TKEFREE*RQNSE_GU1*(ZRIFTKE(JLON,JLEV)-ETKE_RIFC)
          ZS_B = ZRIFTKE(JLON,JLEV)*RQNSE_S0*C3TKEFREE                  &
          &+ETKE_RIFC*(ZRIFTKE(JLON,JLEV)*(RQNSE_GU1-ZRIFTKE(JLON,JLEV)*&
          & RQNSE_GU2)-RQNSE_S0*C3TKEFREE)
          ZS_C = ETKE_RIFC*ZRIFTKE(JLON,JLEV)*(RQNSE_S0-ZRIFTKE(JLON,JLEV))

          ZS_CDIS=ZS_B**2-4*ZS_A*ZS_C
          ZRITKE_U=(-ZS_B-SQRT(ZS_CDIS))/(2.0_JPRB*ZS_A)
          Z_ETKE_RU=(RQNSE_GU1*ZRITKE_U+RQNSE_S0)/&
          &(RQNSE_GU2*ZRITKE_U+1.0_JPRB)
        !UNSTABLE
          ZS_AA4 = C3TKEFREE*RQNSE_GS1*(ZRIFTKE(JLON,JLEV)-ETKE_RIFC)
          ZS_AA3 = ZRIFTKE(JLON,JLEV)*RQNSE_GS2*C3TKEFREE               &
          &+ETKE_RIFC*(ZRIFTKE(JLON,JLEV)*(RQNSE_GS1-ZRIFTKE(JLON,JLEV)*&
          & RQNSE_GS3)-RQNSE_GS2*C3TKEFREE)
          ZS_AA2 = ZRIFTKE(JLON,JLEV)*RQNSE_S0*C3TKEFREE                &
          &+ETKE_RIFC*(ZRIFTKE(JLON,JLEV)*(RQNSE_GS2-ZRIFTKE(JLON,JLEV)*&
          & RQNSE_GS4)-RQNSE_S0*C3TKEFREE)
          ZS_AA1 = ETKE_RIFC*ZRIFTKE(JLON,JLEV)*(RQNSE_S0-ZRIFTKE(JLON,JLEV))

          ! cubic problem only
          ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
          ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
          ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
          ZS_CDIS=ZS_CQ**2+ZS_CP

          ZS_ZSOL = -1.E+99_JPRB
          IF (ZS_CDIS < 0.0_JPRB) THEN
           !         three real solutions!
           !         Confine the argument of ACOS to the interval [-1;1]!
           ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
           ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
           ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
           ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
           ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
           ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
           ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
             &  MIN(ZS_CU2,ZS_CU3))
           ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
           ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
          ELSE
           !      only one real solution!
           ZS_CPHI=SQRT(ZS_CDIS)
           ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
           &       SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
           ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
          ENDIF
          ZRITKE=ZS_ZSOL
          Z_ETKE_R(JLON,JLEV)=(ZRITKE*(RQNSE_GS1*ZRITKE+&
          & RQNSE_GS2)+RQNSE_S0)/&
          &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
          ZRITKE=ZSIG(JLON,JLEV)*ZRITKE+(1.0_JPRB-ZSIG(JLON,JLEV))*ZRITKE_U
          Z_ETKE_R(JLON,JLEV)=ZSIG(JLON,JLEV)*Z_ETKE_R(JLON,JLEV)+&
          & (1.0_JPRB-ZSIG(JLON,JLEV))*Z_ETKE_RU
          Z_ETKE_P(JLON,JLEV)=ETKE_RIFC
      ENDDO
    ENDDO
  ELSEIF (ITTYPE == 5) THEN 
    !EFB
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
      !  UNSTABLE
          Z_ETKE_RU=EFB_UR
      !  STABLE
        Z_ETKE_R(JLON,JLEV)=(ZRIFTKE(JLON,JLEV)*REFB_1+EFB_UR*REFB_3)/&
        &(ZRIFTKE(JLON,JLEV)*REFB_2+REFB_3)
        Z_ETKE_R(JLON,JLEV)=ZSIG(JLON,JLEV)*Z_ETKE_R(JLON,JLEV)+&
        & (1.0_JPRB-ZSIG(JLON,JLEV))*Z_ETKE_RU
        Z_ETKE_P(JLON,JLEV)=Z_ETKE_R(JLON,JLEV)&
        & /(ETKE_OLAM*Z_ETKE_R(JLON,JLEV)/(EFB_AZ0*C3TKEFREE)+1.0_JPRB)
      ENDDO
    ENDDO
  ELSEIF (ITTYPE == 6) THEN 
    CALL ABOR1('ACPTKE: LCOEFK_PTTE=T AND CGTURS=LOUIS is not coded') 
  ENDIF

  !chi_3 and phi_3
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ZCHI3TKE(JLON,JLEV)=(1.0_JPRB-ZRIFTKE(JLON,JLEV)/&
      & Z_ETKE_R(JLON,JLEV))/(1.0_JPRB-ZRIFTKE(JLON,JLEV))
      ZPHI3TKE(JLON,JLEV)=(1.0_JPRB-ZRIFTKE(JLON,JLEV)/&
      &Z_ETKE_P(JLON,JLEV))/(1.0_JPRB-ZRIFTKE(JLON,JLEV))

      !Store F_ww ?
      ZWW_FUN_S1(JLON,JLEV)=PWW_FUN(JLON,JLEV)
      ENDDO
    ENDDO
! STABILITY FUNCTIONS Phi_Q, F_WW and T_H FOR TOMS COMPUTATION  
  IF (ITTYPE==1) THEN 
    !RMC01
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        ZPHIQTKE(JLON,JLEV)=1.0_JPRB
        PWW_FUN(JLON,JLEV)=Z2THIRD
        PTH_FUN(JLON,JLEV)=ZTKE_OLAMC3/(2.0_JPRB*PWW_FUN(JLON,JLEV))
      ENDDO
    ENDDO
  ELSEIF (ITTYPE==2) THEN 
    !Model I
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        PWW_FUN(JLON,JLEV)=Z2THIRD*(1.0_JPRB&
        & -(3.0_JPRB*ETKE_LAM3-ETKE_LAM2     &
        & +4.0_JPRB*ETKE_LAM4*ZRIFTKE(JLON,JLEV))/&
        & (1.0_JPRB-ZRIFTKE(JLON,JLEV)))
        ZPHIQTKE(JLON,JLEV)=PWW_FUN(JLON,JLEV)/Z2AZ0
        PTH_FUN(JLON,JLEV)=ZTKE_OLAMC3/(2.0_JPRB*PWW_FUN(JLON,JLEV))
      ENDDO
    ENDDO
  ELSEIF ((ITTYPE==3).OR.(ITTYPE==4)) THEN 
    !Model II or QNSE
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        PWW_FUN(JLON,JLEV)=Z2THIRD*(1.0_JPRB      &
        & -(3.0_JPRB*ETKE_LAM3-ETKE_LAM2          &
        & +4.0_JPRB*ETKE_LAM4*ZRIFTKE(JLON,JLEV))/&
        & (1.0_JPRB-ZRIFTKE(JLON,JLEV)))
        ZTKE_LAM5T=(C3TKEFREE+ZCHI3TKE(JLON,JLEV)/&
        & ZPHI3TKE(JLON,JLEV))/(C3TKEFREE+1.0_JPRB)
        ZPHIQTKE(JLON,JLEV)=PWW_FUN(JLON,JLEV)/ZTKE_LAM5T/Z2AZ0
        PTH_FUN(JLON,JLEV)=ZTKE_OLAMC3/(2.0_JPRB*PWW_FUN(JLON,JLEV))
      ENDDO
    ENDDO
  ELSEIF(ITTYPE==5) THEN 
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
      !EFB
       ZPHIQTKE(JLON,JLEV)=ZCHI3TKE(JLON,JLEV)
       PWW_FUN(JLON,JLEV)=2.0_JPRB*EFB_AZ0*ZCHI3TKE(JLON,JLEV)
       PTH_FUN(JLON,JLEV)=ZTKE_OLAMC3/(2.0_JPRB*PWW_FUN(JLON,JLEV))
      ENDDO
    ENDDO
  ELSEIF (ITTYPE == 6) THEN 
    CALL ABOR1('ACPTKE: LCOEFK_PTTE=T AND CGTURS=LOUIS is not coded') 
  ENDIF

  ! STABILITY FUNCTIONS FM, 1/sigma and T_h'', F_ww
  IF (LCOEFK_RIH) THEN
    !hybrid mode
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        ZPHI3HTKE=MAX((2.0_JPRB*PPHIQ_S1(JLON,JLEV)&
        & -(PPHIQ_S1(JLON,JLEV)-ZPHI3TKE(JLON,JLEV)))/&
        &(1.0_JPRB+PPHIQ_S1(JLON,JLEV)/ZPHI3TKE(JLON,JLEV)),0.0_JPRB)
        ZFMTKE(JLON,JLEV)=PCHI3_S1(JLON,JLEV)&
        &*SQRT(MAX(ZCHI3TKE(JLON,JLEV)-C3TKEFREE*ZPHI3TKE(JLON,JLEV)*&
        & PMRIPP(JLON,JLEV),ZEPS8))
        ZPRT_INV(JLON,JLEV)=C3TKEFREE*ZPHI3HTKE/PCHI3_S1(JLON,JLEV)
        PWW_FUN(JLON,JLEV)=ZWW_FUN_S1(JLON,JLEV)
        PTH_FUN(JLON,JLEV)=MAX(PTH_FUN(JLON,JLEV)*ZPHIQTKE(JLON,JLEV)&
        & /MAX(PPHIQ_S1(JLON,JLEV)-0.5_JPRB*(ZPHIQTKE(JLON,JLEV)-&
        & ZPHI3TKE(JLON,JLEV)),ZEPS8) ,0.0_JPRB)
      ENDDO
    ENDDO
  ELSE
    DO JLEV=KTDIA,KLEV-1
      DO JLON=KIDIA,KFDIA
        ZFMTKE(JLON,JLEV)=SQRT(ZCHI3TKE(JLON,JLEV)**3*&
        & (1.0_JPRB-ZRIFTKE(JLON,JLEV)))
        ZPRT_INV(JLON,JLEV)=C3TKEFREE*ZPHI3TKE(JLON,JLEV)/ZCHI3TKE(JLON,JLEV)
        PTH_FUN(JLON,JLEV)=MAX(2.0_JPRB*PTH_FUN(JLON,JLEV)*ZPHIQTKE(JLON,JLEV)&
        & /(ZPHIQTKE(JLON,JLEV)+ZPHI3TKE(JLON,JLEV)),0.0_JPRB)
      ENDDO
    ENDDO
  ENDIF
 
  !K_M/H computation
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      PKUROV(JLON,JLEV)=ZCONVERT(JLON,JLEV)*SQRT(MAX(ZEPSTKE,&
       & 0.5_JPRB*(ZTKE1(JLON,JLEV)+ZTKE1(JLON,JLEV+1))))* &
       & sqrt(ZFMTKE(JLON,JLEV))
      PKTROV(JLON,JLEV)=PKUROV(JLON,JLEV)*ZPRT_INV(JLON,JLEV)
    ENDDO
  ENDDO
ELSE IF(LCOEFK_RIH.AND.(.NOT.(LCOEFK_PTTE.AND.KSTEP>0))) THEN
  !K_M/H computation
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      PKUROV(JLON,JLEV)=ZCONVERT(JLON,JLEV)*SQRT(MAX(ZEPSTKE,&
       & 0.5_JPRB*(ZTKE1(JLON,JLEV)+ZTKE1(JLON,JLEV+1)))) &
       & *sqrt(PFMTKE_S1(JLON,JLEV))
      PKTROV(JLON,JLEV)=PKUROV(JLON,JLEV)*C3TKEFREE*PIPRT_S1(JLON,JLEV)
    ENDDO
  ENDDO
ELSE !pTKE code
  !K_M/H computation
  DO JLEV=KTDIA,KLEV-1
    DO JLON=KIDIA,KFDIA
      ! preparation only: keeps the ratio of original PKTROV, PKUROV
      PKTROV(JLON,JLEV)=(PKTROV(JLON,JLEV)/MAX(ZEPSKRV,PKUROV(JLON,JLEV)))
      ! now the PKUROV,PKTROV are redefined
      PKUROV(JLON,JLEV)=ZCONVERT(JLON,JLEV)*SQRT(MAX(ZEPSTKE,&
       & 0.5_JPRB*(ZTKE1(JLON,JLEV)+ZTKE1(JLON,JLEV+1)))) &
       & *(PKUROV(JLON,JLEV)/ZKESROV(JLON,JLEV))
      PKTROV(JLON,JLEV)=PKUROV(JLON,JLEV)*PKTROV(JLON,JLEV)
    ENDDO
  ENDDO
ENDIF
    

!     ------------------------------------------------------------------

!     pTKE TENDENCY EVALUATION (PTENDPTKE=(PTKE1-PTKE0)/DT)

!cdir unroll=8
DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    PTENDPTKE(JLON,JLEV)=(ZTKE1(JLON,JLEV)-PTKE(JLON,JLEV))/TSPHY
  ENDDO
ENDDO
!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ACPTKE',1,ZHOOK_HANDLE)
END SUBROUTINE ACPTKE
