!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACTKEHMT ( KIDIA,KFDIA,KLON,KLEV,KVCLIV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI,PAPHIF,PAPRS,PAPRSF,PCP,PQ,PR,PT,PU,PV,&
 & PFPLSH,PFPLCH,&
 ! - INPUT  1D .
 & PDPHIT,PDPHIV,PGZ0F,PGZ0HF,PGZ0RLF,PHV,PLSM,&
 & PNEIJG,PNEIJV,PSNS,PTS,PVEG0,PWFC,PWS,PWSI,&
 ! - INPUT  LOGIQUE .
 & LDCLS,LDHMT,&
 ! - OUTPUT 2D .
 & PNBVNO,PMRIPP,&
 ! - OUTPUT 1D .
 & PCD,PCDN,PCDROV,PCH,PCHROV,PCPS,PDQSTS,PGWDCS,&
 & PGZ0,PGZ0H,PHQ,PHU,PNEIJ,PQCLS,PQS,PQSATS,&
 & PRHCLS,PRS,PRTI,PSTAB,PTCLS,PUCLS,PVCLS,PUCLN,PVCLN,PZPCLS,PVEG,&
 & PXDROV,PXHROV,PURAF,PVRAF)

!**** *ACTKEHMT   * - COMPUTATION of drag coefficients PCD and PCH with AF scheme
!**** *ACTKEHMT   * - INTERP. DES PARAMETRES U,V,T,Q AUX HAUTEURS CHOISIES.

!     Sujet.
!     ------

!     - ROUTINE DE CALCUL ACTIF .
!       DETERMINATION DE 
!       LES COEFFICIENTS D'ECHANGE EN
!       SURFACE PLUS LEUR AMPLIFICATION SI "ANTI-FIBRILLATION" ACTIVEE .
!     - SI L'OPTION 'CLS' EST SELECTIONNEE :
!          DETERMINATION DE PARAMETRES SUPPLEMENTAIRES NECESSAIRES POUR
!          LES CALCULS DE DIFFUSION VERTICALE ET DE TRAINEE DES ONDES DE
!          GRAVITE OROGRAPHIQUES .
!     - SI L'OPTION 'HMT' EST SELECTIONNEE :
!          INTERPOLATION DES TEMPERATURES, HUMIDITES (SPECIFIQUES ET
!          RELATIVES) ET VENTS AUX HAUTEURS "METEO" .

!     - COMPUTATION OF 
!       SURFACE EXCHANGE COEFFICIENTS PLUS THEIR AMPLIFICATION
!       FACTOR IF "ANTI-FIBRILLATION" IS ACTIVATED .
!     - IF THE SWITCH 'CLS' IS ON :
!          COMPUTATION OF ADDITIONNAL PARAMETERS NECESSARY FOR THE
!          VERTICAL DIFFUSION AND GRAVITY WAVE DRAG PARAMETERIZATIONS .
!     - IF THE SWITCH 'HMT' IS ON :
!          TEMPERATURE, MOISTURE (SPECIFIC AND RELATIVE) AND WIND
!          INTERPOLATION AT STANDARD "METEOROLOGICAL" LEVELS .

!**   Interface.
!     ----------
!        *CALL* *ACTKEHMT*

!-----------------------------------------------------------------------
! WARNING: THE ENGLISH VERSION OF VARIABLES' NAMES IS TO BE READ IN THE
!          "APLPAR" CODE.
!-----------------------------------------------------------------------

! -   ARGUMENTS D'ENTREE.
!     -------------------

! - NOM DES PARAMETRES DE DIMENSIONNEMENT DE LA PHYSIQUE.

! KIDIA      : INDICE DE DEPART DES BOUCLES VECTORISEES SUR L'HORIZONT..
! KFDIA      : INDICE DE FIN DES BOUCLES VECTORISEES SUR L'HORIZONTALE.
! KLON       : DIMENSION HORIZONTALE DES TABLEAUX.
! KLEV       : DIMENSION VERTICALE DES TABLEAUX "FULL LEVEL".
! KVCLIV     : NOMBRE DE CHAMPS POUR LA VEGETATION (NVCLIV DANS CPG).
! KVCLIV     : NUMBER OF FIELDS FOR VEGETATION (NVCLIV IN CPG).

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PAPHI      : GEOPOTENTIEL AUX DEMI-NIVEAUX.
! PAPRS      : PRESSION AUX DEMI-NIVEAUX.
! PFPLSH     : CHAMP "HISTORIQUE" DES PRECIPITATIONS STRATIFORMES.
! PFPLCH     : CHAMP "HISTORIQUE" DES PRECIPITATIONS CONVECTIVES.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIEL AUX NIVEAUX DES COUCHES.
! PAPRSF     : PRESSION AUX NIVEAUX.
! PCP        : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR.
! PQ         : HUMIDITE SPECIFIQUE DE LA VAPEUR D'EAU.
! PR         : CONSTANTE DES GAZ POUR L'AIR.
! PT         : TEMPERATURE.
! PU         : COMPOSANTE EN X DU VENT.
! PV         : COMPOSANTE EN Y DU VENT.

! - 1D (PROGNOSTIQUE) .

! PSNS       : MASSE PAR UNITE DE SURFACE DE LA COUCHE NEIGEUSE.
! PTS        : TEMPERATURE DE SURFACE.
! PWS        : CONTENU EN EAU DU RESERVOIR DE SURFACE.
! PWSI       : CONTENU EN EAU GELEE DU RESERVOIR DE SURFACE.

! - 1D (SPECIFIQUE SELON L'UTILISATION DE ACHMT) .

! PDPHIT     : HAUTEUR D INTERPOLATION POUR T ET Q (EN GEOPOTENTIEL).
! PDPHIV     : HAUTEUR D INTERPOLATION POUR U ET V (EN GEOPOTENTIEL).
! PHV        : RESISTANCE AU FLUX D'EVAPOTRANSPIRATION.
! PWFC       : TENEUR EN EAU A LA CAPACITE AU CHAMPS.

! - 1D (DIAGNOSTIQUE) .

! PNEIJG     : PROPORTION DE SOL RECOUVERTE DE NEIGE.
! PNEIJV     : PROPORTION DE VEGETATION RECOUVERTE DE NEIGE.

! - 1D (GEOGRAPHIQUE) .

! PGZ0F      : CHAMP TEMPOREL DE G FOIS LA LONGUEUR DE RUGOSITE.
! PGZ0HF     : CHAMP TEMPOREL DE G FOIS LA LONGUEUR DE RUGOSITE THERM.
! PGZ0RLF    : GRAVITE * LONGUEUR DE RUGOSITE DUE AU RELIEF.
! PLSM       : INDICE TERRE/MER.
! PVEG0      : PROPORTION DE SOL COUVERTE DE VEGETATION.

! - LOGIQUES .

! LDCLS      : DETERMINATION COMPLETE DES CARACTERISTIQUES DE SURFACE.

!-----------------------------------------------------------------------

! -   ARGUMENTS DE SORTIE.
!     --------------------

! - NOM DES VARIABLES DE LA PHYSIQUE (PAR ORDRE ALPHABETIQUE DANS CHAQUE
!   CATEGORIE).

! - 2D (0:KLEV) .

! PNBVNO     : CARRE DE FR. BRUNT-VAISALA DIVISEE PAR G FOIS LA DENSITE.
! PMRIPP     : RICHARDSON NUMBER

! - 1D (DIAGNOSTIQUE) .

! PCD        : COEFFICIENT D'ECHANGE EN SURFACE POUR U ET V.
! PCDN       : COEFFICIENT NEUTRE D'ECHANGE EN SURFACE.
! PCDROV     : PCD RENORME EN DENSITE FOIS VITESSE.
! PCH        : COEFFICIENT D'ECHANGE EN SURFACE POUR T ET Q.
! PCHROV     : PCH RENORME EN DENSITE FOIS VITESSE.
! PCPS       : CHALEUR MASSIQUE A PRESSION CONSTANTE DE L'AIR AU SOL.
! PDQSTS     : DERIVEE DE PQSATS PAR RAPPORT A LA TEMPERATURE.
! PGWDCS     : DENSITE EN SURFACE POUR LE DRAG OROGRAPHIQUE.
! PGZ0       : G FOIS LA LONGUEUR DE RUGOSITE COURANTE.
! PGZ0H      : G*LONGUEUR DE RUGOSITE THERMIQUE COURANTE (SI KVCLIV >= 8)
! PHQ        : POIDS DE L'HUMIDITE DE L'AIR DANS L'HUMIDITE DE SURFACE.
! PHU        : POIDS DE L'HUMIDITE SATURANTE DANS L'HUMIDITE DE SURFACE.
! PNEIJ      : PROPORTION DE LA MAILLE RECOUVERTE DE NEIGE.
! PQCLS      : SORTIE DIAGNOSTIQUE DE L'HUMIDITE SPECIFIQUE A HTQ METEO.
! PQS        : HUMIDITE SPECIFIQUE DE SURFACE.
! PQSATS     : HUMIDITE SPECIFIQUE DE SATURATION EN SURFACE.
! PRHCLS     : SORTIE DIAGNOSTIQUE DE L'HUMIDITE RELATIVE A HTQ METEO.
! PRS        : CONSTANTE DES GAZ POUR L'AIR AU SOL.
! PRTI       : INVERSE DE R*T.
! PSTAB      : INDICE DE STABILITE A LA SURFACE.
! PTCLS      : SORTIE DIAGNOSTIQUE DE LA TEMPERATURE A HTQ METEO.
! PUCLS      : SORTIE DIAGNOSTIQUE DU VENT EN X A HUV METEO.
! PVCLS      : SORTIE DIAGNOSTIQUE DU VENT EN Y A HUV METEO.
! PUCLN      : SORTIE DIAGNOSTIQUE DU VENT NEUTRE EN X A HUV METEO.
! PVCLN      : SORTIE DIAGNOSTIQUE DU VENT NEUTRE EN Y A HUV METEO.
! PZPCLS     : SORTIE DIAGNOSTIQUE DE PRESSION A HTQ METEO.
! PVEG       : PROPORTION DE VEGETATION APPARENTE (NON RECOUVERTE DE NEIGE).
! PXDROV     : MULTIPLICATEUR "ANTI-FIBRILLATION" DE PCDROV.
! PXHROV     : MULTIPLICATEUR "ANTI-FIBRILLATION" DE PCHROV.
! PURAF      : SORTIE DIAGNOSTIQUE DU VENT DE RAFALES TURBULENTES EN X
! PVRAF      : SORTIE DIAGNOSTIQUE DU VENT DE RAFALES TURBULENTES EN Y

!-----------------------------------------------------------------------

! -   ARGUMENTS IMPLICITES.
!     ---------------------

!-----------------------------------------------------------------------

!     Externes.
!     ---------

!     *ACNTCLS*

!     Methode.
!     --------

!     Auteur.
!     -------

!     Modifications.
!     --------------
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY :  RG       ,RCPV     ,RETV     ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD    ,RATM     ,RCW      ,&
 & RV       ,RPI      ,RD       ,RCPD     ,RKAPPA   

USE YOMPHY   , ONLY : LNEIGE   ,LSNV     ,LSOLV    ,LFGEL    ,&
 & LZ0HSREL  ,LRRGUST, CGTURS, LCOEFK_F1
USE YOMPHY0  , ONLY : GCISMIN,VKARMN, GCISMIN,&
 &  VZIUSTAR0,C3TKEFREE, ETKE_RIFC    ,ETKE_R,&
 & EFB_UR,EFB_AZ0,REFB_1,REFB_2,REFB_3, ETKE_OLAM,&
 & UTILGUST ,RRGAMMA  ,RRSCALE
USE YOMPHY1  , ONLY : ALRCN1   ,ALRCN2   ,GCONV    ,&
 & RD1      ,WCRIN    ,WSMX     ,RZHZ0M  
USE YOMCLI   , ONLY : STHER    ,SZZ0D
USE YOMPHY2  , ONLY : TSPHY    , XMULAF  ,LRAFTUR  ,GZ0RAF   ,FACRAF
USE YOMQNSE  , ONLY : RQNSE_S0,RQNSE_GS1,RQNSE_GS2,RQNSE_GS3,RQNSE_GS4,&
 & RQNSE_GU1,RQNSE_GU2
USE YOMLOUIS  , ONLY :                                   &
 & PLOUIS_S0,PLOUIS_GS1,PLOUIS_GS2,PLOUIS_GS3,PLOUIS_GS4,&
 & PLOUIS_GU1,PLOUIS_GU2,                                &
 & RLOUIS_S0,RLOUIS_GS1,RLOUIS_GS2,RLOUIS_GS3,RLOUIS_GS4,&
 & RLOUIS_GU1,RLOUIS_GU2

!-----------------------------------------------------------------------

#define FZH(X,Y) 1.0_JPRB+X/Y

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVCLIV
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLSH(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFPLCH(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPHIT(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDPHIV(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0F(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0HF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0RLF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHV(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEIJG(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNEIJV(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEG0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWFC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSI(KLON) 
LOGICAL           ,INTENT(IN)    :: LDCLS 
LOGICAL           ,INTENT(IN)    :: LDHMT 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNBVNO(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMRIPP(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCD(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDN(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCDROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCH(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCHROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDQSTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGWDCS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0H(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHQ(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHU(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNEIJ(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQSATS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRHCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRTI(KLON)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTAB(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLN(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLN(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZPCLS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVEG(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXDROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXHROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PURAF(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVRAF(KLON) 

REAL(KIND=JPRB) :: ZDELT(KLON),ZDPHI(KLON),ZESP(KLON)&
 & ,ZU(KLON),ZCIS(KLON)


INTEGER(KIND=JPIM) :: JLON, ITTYPE

LOGICAL :: LLAFGD, LLM123
LOGICAL :: LLZ0H

REAL(KIND=JPRB) :: ZA1, ZAPRIM, ZAT, ZAU,&
 & ZB1, ZBE, ZBETA, ZBETAP, ZBPRIM, ZC1, &
 & ZCPRIM,ZCRIT1, ZCRIT2, ZDELTA1, &
 & ZDENUM,ZEW, ZGA, ZGAMDZ, ZGKT, ZGKU,  &
 & ZMULA2, ZSDPRIM, ZTAU, ZSCRAF, ZEPS,ZRCORR,ZEPS8
REAL(KIND=JPRB) :: ZSIG
REAL(KIND=JPRB) :: ZRITKE,Z_ETKE_R,Z_ETKE_P,Z_ETKE_RU,Z_ETKE_PU,ZSTA,&
& ZRIFTKE,ZDRRI,ZDPRI,ZDRRI_U,ZDPRI_U,ZDRIFTKE_U,ZEFB_UP,ZCD,ZFP,    &
& ZDCHI3TKE_U, ZDPHI3TKE_U,ZFSFTKE
REAL(KIND=JPRB) :: ZS_AA1, ZS_AA2, ZS_AA3, ZS_AA4
REAL(KIND=JPRB) :: ZS_CTHIRD,ZS_CW,ZS_CP,ZS_CQ,ZS_CDIS,ZS_CPHI,ZS_ZSOL,&
 & ZS_CU1,ZS_CU2,ZS_CU3, &
 & ZS_CX1,ZS_CX2,ZS_CX3

REAL(KIND=JPRB) :: ZCDMR(KLON),ZCDNMR(KLON)  

REAL(KIND=JPRB) :: ZAUTKE(KLON), ZATTKE(KLON),ZSIG_AF(KLON)
REAL(KIND=JPRB) :: ZMRIFPP(KLON),ZCHI3TA(KLON),ZPHI3TA(KLON),&
& ZFMTKE(KLON),ZFTTKE(KLON),ZDRIFTKE(KLON),ZDCHI3TKE(KLON),&
& ZDPHI3TKE(KLON),ZFMTKE_DX(KLON),ZFHTKE_DX(KLON),&
& ZFFTKE_DX(KLON),ZFFTKE(KLON)
REAL(KIND=JPRB) :: ZCDNH(KLON) 
REAL(KIND=JPRB) :: ZETKE_R(KLON),ZETKE_P(KLON)
REAL(KIND=JPRB) :: ZCPVMD,ZRVMD,ZGZ0N, ZGZ0MR, ZRZDMR ,&
& ZWFC, ZWS, ZDEW, ZRZD, ZRZH, Z0CR, ZUZ0CN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "actkecls.intfb.h"

#include "fcttrm.h"

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ACTKEHMT',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------
!*
!     ------------------------------------------------------------------
!     I - CALCUL DES PARAMETRES DERIVES ET CONSTANTE DE SECURITE

!         COMPUTATION OF DERIVED PARAMETERS AND SECURITY CONSTANTS 

ZCPVMD=RCPV-RCPD
ZRVMD=RV-RD


!   ZGZ0N   : VALEUR MIN. DE G FOIS LA LONG. DE RUG. DYN. SANS RELIEF.
!            : MIN VALUE OF G * DYN. ROUGHNESS L. WITHOUT TOPOGRAPHY.

ZGZ0N=RG*SZZ0D

!*
!     ------------------------------------------------------------------
!     I- PASSAGE EN GEOPOTENTIEL POUR LES CONSTANTES RELATIVES A LA
!     LONGUEUR DE RUGOSITE EN CAS DE NEIGE.

!           GOING TO GEOPOTENTIAL FOR CONSTANTS RELATIVE TO ROUGHNESS
!     LENGTH IN CASE OF SNOW COVER.

Z0CR=RG*ALRCN1
ZUZ0CN=1.0_JPRB/(RG*ALRCN2)


ZS_CTHIRD=1.0_JPRB/3.0_JPRB

!   security constant
ZEPS    = EPSILON(1.0_JPRB)*100.0_JPRB
ZEPS8=1e-8_JPRB



! LLZ0H      : DIFFERENTES LONGUEURS DE RUGOSITE DYN/THER.

LLZ0H=(KVCLIV>=8).AND.LSOLV

IF     (TRIM(CGTURS) == 'RMC') THEN
  ITTYPE=1
ELSEIF (TRIM(CGTURS) == 'MD1') THEN
  ITTYPE=2
ELSEIF (TRIM(CGTURS) == 'MD2') THEN
  ITTYPE=3
ELSEIF (TRIM(CGTURS) == 'QNSE') THEN
  ITTYPE=4
ELSEIF (TRIM(CGTURS) == 'EFB') THEN
  ITTYPE=5
ELSEIF (TRIM(CGTURS) == 'LOUIS') THEN
  ITTYPE=6
ENDIF

LLM123=(ITTYPE == 1).OR.(ITTYPE == 2).OR.(ITTYPE == 3)
LLAFGD=XMULAF /= 0.0_JPRB

!*
!     ------------------------------------------------------------------
!     II - CALCULS PROPREMENT DITS DES CARACTERISTIQUES DE SURFACE.

!          EFFECTIVE CALCULATIONS OF THE SURFACE CHARACTERISTICS.


DO JLON=KIDIA,KFDIA
! ZDPHI    : EPAISSEUR EN GEOPOTENTIEL DU NIVEAU DE SURFACE.
!            : GEOPOTENTIAL THICKNESS OF THE SURFACE LEVEL.
  ZDPHI(JLON)=PAPHIF(JLON,KLEV)-PAPHI(JLON,KLEV)
! ZU        : MODULE DU VENT DE SURFACE.
!            : SURFACE WIND SPEED.

  ZCIS(JLON)=PU(JLON,KLEV)**2+PV(JLON,KLEV)**2+&
   &(GCISMIN*ZDPHI(JLON)/RG)**2
  ZU(JLON)=SQRT(ZCIS(JLON))

!     CALCULS CONCERNANT LA PRESENCE OU NON DE NEIGE AU SOL.
!     COMPUTATIONS CONCERNING THE PRESENCE OR ABSENCE OF SNOW ON GROUND.

! - TEMPORAIRE(S) 1D .

! ZDELT     : UN POUR CALCULS DE SATURATION SUR GLACE, ZERO SINON.
!            : ONE FOR SATURATION CALCULATION FOR ICE, ZERO OTHERWISE.

  IF (LNEIGE) THEN
    IF (LSNV) THEN
      PNEIJ(JLON)=PLSM(JLON) *&
       & ((1.0_JPRB-PVEG0(JLON))*PNEIJG(JLON) + PVEG0(JLON)*PNEIJV(JLON))  
      PVEG(JLON)=(1.0_JPRB-PNEIJV(JLON))*PVEG0(JLON)
      PGZ0(JLON)=SQRT((1.0_JPRB-PNEIJV(JLON))*PGZ0F(JLON)**2+&
       & PNEIJV(JLON)*(Z0CR**2+PGZ0RLF(JLON)**2))  
    ELSE
      PNEIJ(JLON)=PLSM(JLON)*PSNS(JLON)/(PSNS(JLON)+WCRIN)
      PVEG(JLON)=PVEG0(JLON)
      PGZ0(JLON)=PGZ0F(JLON)+(Z0CR-PGZ0F(JLON))*PLSM(JLON)&
       & *PSNS(JLON)/(PSNS(JLON)+WCRIN*(1+ZUZ0CN*PGZ0F(JLON)))  
    ENDIF
    ZDELT(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PTS(JLON)))
  ELSE
    PNEIJ(JLON)=0.0_JPRB
    PVEG(JLON)=PVEG0(JLON)
    PGZ0(JLON)=PGZ0F(JLON)
    ZDELT(JLON)=0.0_JPRB
  ENDIF

!     CAS DE DEUX LONGUEURS DE RUGOSITE (SUR CONTINENT NON TOTALEMENT
!     COUVERT DE NEIGE).
!     CASE OF TWO ROUGHNESS LENGTHS (ON PARTIALLY SNOW-FREE LAND ONLY).

  IF (LLZ0H) THEN

    IF (LNEIGE) THEN
      IF (LSNV) THEN
        IF (LZ0HSREL) THEN
          PGZ0H(JLON)=SQRT((1.0_JPRB-PNEIJV(JLON))*PGZ0HF(JLON)**2 &
           &+PNEIJV(JLON)*(STHER*Z0CR)**2)
        ELSE
          PGZ0H(JLON)=SQRT((1.0_JPRB-PNEIJV(JLON))*PGZ0HF(JLON)**2+&
           & PNEIJV(JLON)   *&
           & (Z0CR**2+(PGZ0RLF(JLON)*PGZ0HF(JLON)/PGZ0F(JLON))**2))  
        ENDIF
      ELSE
        IF (LZ0HSREL) THEN
          PGZ0H(JLON)=PGZ0HF(JLON)+(Z0CR*STHER-PGZ0HF(JLON))&
           &*PLSM(JLON)*PSNS(JLON)/(PSNS(JLON)+WCRIN&
           &*(1+ZUZ0CN*PGZ0HF(JLON)))
        ELSE
          PGZ0H(JLON)=PGZ0HF(JLON)+(Z0CR-PGZ0HF(JLON))*PLSM(JLON)&
           & *PSNS(JLON)/(PSNS(JLON)+WCRIN*(1+ZUZ0CN*PGZ0HF(JLON)))  
        ENDIF
      ENDIF
    ELSE
      PGZ0H(JLON)=PGZ0HF(JLON)
    ENDIF

! PGZ0H = ZGZ0 OVER SEA UNLESS REQUIRED IN NAMPHY1 (CLIMATE RUNS)

     IF (VZIUSTAR0 > 0) THEN
       PGZ0H(JLON)=PLSM(JLON)*PGZ0H(JLON)&
        & +(1.0_JPRB-PLSM(JLON))*PGZ0HF(JLON)
     ELSE    
       PGZ0H(JLON)=MIN(PGZ0(JLON),PLSM(JLON)*PGZ0H(JLON)&
        & +(1.0_JPRB-PLSM(JLON))*PGZ0(JLON)*RZHZ0M)  
     ENDIF

  ELSE

    PGZ0H(JLON)=PGZ0(JLON)

  ENDIF

!     CALCUL DE SATURATION UTILISANT LES FONCTIONS DEFINIES.
!     SATURATION CALCULATION WITH CALL TO DEFINED FUNCTIONS.

! - TEMPORAIRE(S) 1D .

! ZESP      : PRESSION DE VAPEUR SATURANTE SUR PRESSION.
!            : SATURATION WATER VAPOUR PRESSURE OVER PRESSURE.

  ZEW= FOEW (PTS(JLON),ZDELT(JLON))
  ZESP(JLON)=ZEW/PAPRS(JLON,KLEV)
  PQSATS(JLON)= FOQS (ZESP(JLON))

!     CALCULS RELATIFS A L'HUMIDITE DE SURFACE.
!     COMPUTATIONS CONCERNING SURFACE HUMIDITY.

!     CALCULS OPTIONNELS SUIVANT LE TYPE DE SCHEMA SOL/VEG CHOISI.
!     OPTIONAL COMPUTATIONS ACCORDING TO THE CHOSEN SOIL/VEG SCHEME.

  IF (LSOLV) THEN
    ZWFC=PWFC(JLON)*RD1*GCONV
    IF (LFGEL) THEN
      ZWS=PWS(JLON)+PWSI(JLON)
    ELSE
      ZWS=PWS(JLON)
    ENDIF
    IF (LSNV) THEN
      PHU(JLON)=1.0_JPRB+PLSM(JLON)*(1.0_JPRB-PNEIJG(JLON))*&
       & (0.5_JPRB*(1.0_JPRB-COS(RPI*MIN(ZWS/ZWFC,1.0_JPRB)))-1.0_JPRB)  
      PHU(JLON)=MAX(PHU(JLON),MIN(PQ(JLON,KLEV)/PQSATS(JLON),1.0_JPRB))
      PQS(JLON)= ( PVEG(JLON)*PHV(JLON) +&
       & PVEG0(JLON)*PNEIJV(JLON) +&
       & (1.0_JPRB-PVEG0(JLON))*PHU(JLON) ) * PQSATS(JLON) +&
       & (1.0_JPRB-PHV(JLON))*PVEG(JLON) * PQ(JLON,KLEV)  
    ELSE
      PHU(JLON)=1.0_JPRB+PLSM(JLON)*(1.0_JPRB-PNEIJ(JLON))*&
       & (0.5_JPRB*(1.0_JPRB-COS(RPI*MIN(ZWS/ZWFC,1.0_JPRB)))-1.0_JPRB)  
      PHU(JLON)=MAX(PHU(JLON),MIN(PQ(JLON,KLEV)/PQSATS(JLON),1.0_JPRB))
      PQS(JLON)= ( PVEG(JLON)*PHV(JLON) +&
       & (1.0_JPRB-PVEG(JLON))*PHU(JLON)  ) * PQSATS(JLON) +&
       & (1.0_JPRB-PHV(JLON))*PVEG(JLON) * PQ(JLON,KLEV)  
    ENDIF
  ELSE
    PHU(JLON)=1.0_JPRB+PLSM(JLON)*(1.0_JPRB-PNEIJ(JLON))&
     & *(0.5_JPRB*(1.0_JPRB-COS(RPI*PWS(JLON)/WSMX))-1.0_JPRB)  
    PHQ(JLON)=(1.0_JPRB-PHU(JLON))*PVEG(JLON)
    ZDEW=MAX(0.0_JPRB,SIGN(1.0_JPRB,PQ(JLON,KLEV)-PQSATS(JLON)))
    PHU(JLON)=PHU(JLON)+ZDEW*PHQ(JLON)
    PHQ(JLON)=PHQ(JLON)-ZDEW*PHQ(JLON)
    PQS(JLON)=PHU(JLON)*PQSATS(JLON)+PHQ(JLON)*PQ(JLON,KLEV)
  ENDIF

  PCPS(JLON)=RCPD+ZCPVMD*PQS(JLON)
  PRS(JLON)=RD+ZRVMD*PQS(JLON)

!     CALCULS GEOMETRIQUES.
!     GEOMETRIC CALCULATIONS.

  ZRZD=1.0_JPRB+ZDPHI(JLON)/PGZ0(JLON)
  PCDN(JLON)=(VKARMN/LOG(ZRZD))**2

!     CALCUL PRELIMINAIRES NECESSAIRES A LA DETERMINATION DES
!     COEFFICIENTS D'ECHANGE CD ET CH DANS LE CAS DE DEUX RUGOSITES
!     DYNAMIQUE Z0 ET THERMIQUE ZOH.

!     PRELIMINARY COMPUTATIONS NECESSARY FOR THE DETERMINATION OF
!     EXCHANGE COEFFICIENTS CD AND CH IN THE CASE OF TWO ROUGHNESS
!     LENGTHS Z0 DYNAMICAL AND ZOH THERMAL.

! - TEMPORAIRE(S) 1D .

! ZCDNH     : COEFFICIENT THERMIQUE NEUTRE D'ECHANGE EN SURFACE.
!            : NEUTRAL SURFACE THERMAL EXCHANGE COEFFICIENT.

  ZRZH=FZH(ZDPHI(JLON),PGZ0H(JLON))
  IF (LZ0HSREL.AND.LSNV) THEN
    ZGZ0MR=PLSM(JLON)*SQRT(MAX(ZGZ0N**2,PGZ0(JLON)**2 &
     &-PGZ0RLF(JLON)**2))+(1.0_JPRB-PLSM(JLON))*PGZ0(JLON)
    ZGZ0MR=MAX(ZGZ0MR,PGZ0H(JLON))
    ZRZDMR=1.0_JPRB+ZDPHI(JLON)/ZGZ0MR
  ELSEIF (LZ0HSREL.AND.(.NOT.LSNV)) THEN
    ZGZ0MR=PLSM(JLON)*PGZ0H(JLON)/STHER+(1.0_JPRB-PLSM(JLON))*PGZ0(JLON)
    ZRZDMR=1.0_JPRB+ZDPHI(JLON)/ZGZ0MR
  ELSE
    ZGZ0MR=PGZ0(JLON)
    ZRZDMR=ZRZD
  ENDIF
  ZCDNH(JLON)=C3TKEFREE*VKARMN**2/(LOG(ZRZH)*LOG(ZRZDMR))

  PRTI(JLON)=2.0_JPRB/(PR(JLON,KLEV)*PT(JLON,KLEV)+RKAPPA*ZDPHI(JLON)&
  & +PRS(JLON)*PTS(JLON))  

! ZSTA      : APPROXIMATION DE DELTA(PHI)*DELTA(LN(THETA)).
!            : APPROXIMATION OF DELTA(PHI)*DELTA(LN(THETA)).
  ZSTA=ZDPHI(JLON)*(PR(JLON,KLEV)*PT(JLON,KLEV)+RKAPPA &
   & *ZDPHI(JLON)-PRS(JLON)*PTS(JLON))*PRTI(JLON)

!Ri_dry = Ri*,Ri** at surface
  PMRIPP(JLON,KLEV)=ZSTA/ZCIS(JLON)
 !protection against division by 0
  PMRIPP(JLON,KLEV)=SIGN(MAX(ZEPS,&
  &ABS(PMRIPP(JLON,KLEV))),PMRIPP(JLON,KLEV))
ENDDO


!*
!     ------------------------------------------------------------------
!     III - COMPUTATION OF STABILITY DEPENDENCY FUNCTIONS

IF (LLM123) THEN
  !Ri ->Rif surface
  DO JLON=KIDIA,KFDIA
    !modified CCH02 or RMC01
    ZETKE_R(JLON)=ETKE_R
    ZETKE_P(JLON)=ETKE_RIFC
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB

    ZMRIFPP(JLON)=-(sqrt((ZETKE_R(JLON)*                            &
    & (ZRITKE*C3TKEFREE+ZETKE_P(JLON)))**2                          &
    & -4.0_JPRB*ZRITKE*C3TKEFREE*ZETKE_R(JLON)*ZETKE_P(JLON)**2)    &
    & -(ZRITKE*C3TKEFREE+ZETKE_P(JLON))*ZETKE_R(JLON))/             &
    & (2.0_JPRB*ZETKE_P(JLON))
    !security
    ZMRIFPP(JLON)=MIN(ZMRIFPP(JLON),ETKE_RIFC-ZEPS8)
  ENDDO
ENDIF

IF (ITTYPE == 4) THEN

  DO JLON=KIDIA,KFDIA
    !Ri ->Rif surface
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB  
    !  UNSTABLE
    Z_ETKE_RU=(RQNSE_GU1*ZRITKE+RQNSE_S0)/&
     &(RQNSE_GU2*ZRITKE+1.0_JPRB)
    !  STABLE
    Z_ETKE_R=(ZRITKE*(RQNSE_GS1*ZRITKE+&
     & RQNSE_GS2)+RQNSE_S0)/&
     &(ZRITKE*(RQNSE_GS3*ZRITKE+RQNSE_GS4)+1.0_JPRB)
    ZETKE_R(JLON)=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
    ZETKE_P(JLON)=ETKE_RIFC

    ZMRIFPP(JLON)=-(sqrt((ZETKE_R(JLON)*                            &
    & (ZRITKE*C3TKEFREE+ZETKE_P(JLON)))**2                          &
    & -4.0_JPRB*ZRITKE*C3TKEFREE*ZETKE_R(JLON)*ZETKE_P(JLON)**2)    &
    & -(ZRITKE*C3TKEFREE+ZETKE_P(JLON))*ZETKE_R(JLON))/             &
    & (2.0_JPRB*ZETKE_P(JLON))
    !security
    ZMRIFPP(JLON)=MIN(ZMRIFPP(JLON),ETKE_RIFC-ZEPS8)
  ENDDO

ENDIF ! ITTYPE==4 QNSE

IF (ITTYPE == 5) THEN
  !EFB
  !  UNSTABLE
  Z_ETKE_RU=EFB_UR
  !  STABLE
  ZS_AA4 = EFB_AZ0*REFB_2

  DO JLON=KIDIA,KFDIA
    !Ri ->Rif surface
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB  

    ZS_AA3 = EFB_AZ0*(REFB_3-REFB_1-ZRITKE*REFB_2*C3TKEFREE)&
     &      - ETKE_OLAM*REFB_1*ZRITKE
    ZS_AA2 = C3TKEFREE*EFB_AZ0*ZRITKE*(REFB_1-REFB_3)&
     &      - REFB_3*EFB_UR*(ETKE_OLAM*ZRITKE+EFB_AZ0)
    ZS_AA1 = ZRITKE*EFB_AZ0*EFB_UR*REFB_3*C3TKEFREE

    ! cubic problem only
    ZS_CW=ZS_AA3/ZS_AA4*ZS_CTHIRD
    ZS_CP=(ZS_AA2/ZS_AA4*ZS_CTHIRD-ZS_CW**2)**3
    ZS_CQ=-0.5_JPRB*(2.0_JPRB*ZS_CW**3-(ZS_AA2*ZS_CW-ZS_AA1)/ZS_AA4)
    ZS_CDIS=ZS_CQ**2+ZS_CP

    ZS_ZSOL = +1.E+99_JPRB

    IF (ZS_CDIS < 0.0_JPRB) THEN
    !         three real solutions!
    !         Confine the argument of ACOS to the interval [-1;1]!
      ZS_CPHI=ACOS(MIN(1._JPRB,MAX(-1._JPRB,ZS_CQ/SQRT(-ZS_CP))))
      ZS_CP=2.0_JPRB*(-ZS_CP)**(0.5_JPRB*ZS_CTHIRD)
      ZS_CU1=ZS_CP*COS((ZS_CPHI+2.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
      ZS_CU2=ZS_CP*COS((ZS_CPHI+4.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
      ZS_CU3=ZS_CP*COS((ZS_CPHI+6.0_JPRB*RPI)*ZS_CTHIRD)-ZS_CW
      ZS_CX1=MIN(ZS_CU1,ZS_CU2,ZS_CU3)
      ZS_CX2=MAX(MIN(ZS_CU1,ZS_CU2),MIN(ZS_CU1,ZS_CU3), &
       &  MIN(ZS_CU2,ZS_CU3))
      ZS_CX3=MAX(ZS_CU1,ZS_CU2,ZS_CU3)
      ZS_ZSOL = MIN(ZS_ZSOL,ZS_CX1,ZS_CX2,ZS_CX3)
    ELSE
    !      only one real solution!
      ZS_CPHI=SQRT(ZS_CDIS)
      ZS_CX1=SIGN(ABS(ZS_CQ+ZS_CPHI)**ZS_CTHIRD,ZS_CQ+ZS_CPHI)+ &
       &  SIGN(ABS(ZS_CQ-ZS_CPHI)**ZS_CTHIRD,ZS_CQ-ZS_CPHI)-ZS_CW
      ZS_ZSOL = MAX(ZS_ZSOL,ZS_CX1)
    ENDIF

    ZRIFTKE=ZS_ZSOL
    Z_ETKE_R=(ZRIFTKE*REFB_1+EFB_UR*REFB_3)/&
     &(ZRIFTKE*REFB_2+REFB_3)
    ZETKE_R(JLON)=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU
    ZETKE_P(JLON)=ZETKE_R(JLON)&
     & /(ETKE_OLAM*ZETKE_R(JLON)/(EFB_AZ0*C3TKEFREE)+1.0_JPRB)

    ZMRIFPP(JLON)=-(sqrt((ZETKE_R(JLON)*                            &
    & (ZRITKE*C3TKEFREE+ZETKE_P(JLON)))**2                          &
    & -4.0_JPRB*ZRITKE*C3TKEFREE*ZETKE_R(JLON)*ZETKE_P(JLON)**2)    &
    & -(ZRITKE*C3TKEFREE+ZETKE_P(JLON))*ZETKE_R(JLON))/             &
    & (2.0_JPRB*ZETKE_P(JLON))
    !security
    ZMRIFPP(JLON)=MIN(ZMRIFPP(JLON),ETKE_RIFC-ZEPS8)
  ENDDO

ENDIF ! ITTYPE==5 EFB

IF (ITTYPE == 6) THEN

  DO JLON=KIDIA,KFDIA
    !Ri ->Rif surface
    ZRITKE=PMRIPP(JLON,KLEV)
    ZSIG=SIGN(0.5_JPRB,ZRITKE)+0.5_JPRB  
    !  UNSTABLE
    Z_ETKE_RU=(RLOUIS_GU1*ZRITKE+RLOUIS_S0)/&
     &(RLOUIS_GU2*ZRITKE+1.0_JPRB)
    !  STABLE
    Z_ETKE_R=(ZRITKE*(RLOUIS_GS1*ZRITKE+&
     & RLOUIS_GS2)+RLOUIS_S0)/&
     &(ZRITKE*(RLOUIS_GS3*ZRITKE+RLOUIS_GS4)+1.0_JPRB)
    ZETKE_R(JLON)=ZSIG*Z_ETKE_R+(1.0_JPRB-ZSIG)*Z_ETKE_RU

    !UNSTABLE
    Z_ETKE_PU=(PLOUIS_GU1*ZRITKE+PLOUIS_S0)/&
     &(PLOUIS_GU2*ZRITKE+1.0_JPRB)
    !STABLE
    Z_ETKE_P=(ZRITKE*(PLOUIS_GS1*ZRITKE+&
     & PLOUIS_GS2)+PLOUIS_S0)/&
     &(ZRITKE*(PLOUIS_GS3*ZRITKE+PLOUIS_GS4)+1.0_JPRB)
    ZETKE_P(JLON)=ZSIG*Z_ETKE_P+(1.0_JPRB-ZSIG)*Z_ETKE_PU

    ZMRIFPP(JLON)=-(sqrt((ZETKE_R(JLON)*                            &
    & (ZRITKE*C3TKEFREE+ZETKE_P(JLON)))**2                          &
    & -4.0_JPRB*ZRITKE*C3TKEFREE*ZETKE_R(JLON)*ZETKE_P(JLON)**2)    &
    & -(ZRITKE*C3TKEFREE+ZETKE_P(JLON))*ZETKE_R(JLON))/             &
    & (2.0_JPRB*ZETKE_P(JLON))
    !security
    ZMRIFPP(JLON)=MIN(ZMRIFPP(JLON),ETKE_RIFC-ZEPS8)
  ENDDO

ENDIF ! ITTYPE==6 LOUIS

!     CALCULS DE UN SUR G FOIS LA FREQUENCE DE BRUNT-VAISALA DIVISEE PAR
!     LA DENSITE LE TOUT AU CARRE DE MEME QUE LA DENSITE (POUR G.W.D.).
!     COMPUTATION OF ONE OVER G TIME THE BRUNT-VAISALA FREQUENCY DIVIDED
!     BY DENSITY THE WHOLE BEING SQUARED AS WELL AS DENSITY (FOR G.W.D.)
IF (LDCLS) THEN
  DO JLON=KIDIA,KFDIA
    PNBVNO(JLON,KLEV)=PMRIPP(JLON,KLEV)*ZCIS(JLON)/(PAPRS(JLON,KLEV) &
   &   *PRTI(JLON)*ZDPHI(JLON))**2
  ENDDO
ENDIF

DO JLON=KIDIA,KFDIA
  ZCHI3TA(JLON)=(1.0_JPRB-ZMRIFPP(JLON)/ZETKE_R(JLON))&
  & /(1.0_JPRB-ZMRIFPP(JLON))
  ZPHI3TA(JLON)=(1.0_JPRB-ZMRIFPP(JLON)/ZETKE_P(JLON))&
  & /(1.0_JPRB-ZMRIFPP(JLON))
ENDDO

IF(LCOEFK_F1) THEN
  DO JLON=KIDIA,KFDIA
   ZSIG=MAX(0._JPRB,SIGN(1._JPRB,ZMRIFPP(JLON)))
   ZFSFTKE=(1.0_JPRB-ZSIG)+ZSIG*SQRT(ZCHI3TA(JLON)-&
   & C3TKEFREE*ZPHI3TA(JLON)*PMRIPP(JLON,KLEV))
   ZFMTKE(JLON)=ZCHI3TA(JLON)*ZFSFTKE
   ZFTTKE(JLON)=ZPHI3TA(JLON)*ZFSFTKE
  ENDDO
ELSE
  DO JLON=KIDIA,KFDIA
    ZFMTKE(JLON)=ZCHI3TA(JLON)*SQRT(ZCHI3TA(JLON)*(1._JPRB-ZMRIFPP(JLON)))
    ZFTTKE(JLON)=ZFMTKE(JLON)*ZPHI3TA(JLON)/ZCHI3TA(JLON)
  ENDDO
ENDIF

!*
!     ------------------------------------------------------------------
!     IV - COMPUTATION OF DRAG COEFFICIENTS


DO JLON=KIDIA,KFDIA
  ! COMPUTAION OF DRAG COEFFICIENTS PCD,PCH
  !     CALCULS POUR LES COMPOSANTES DU VENT DE CD ET DE SON PRODUIT PAR
  !     DENSITE FOIS MODULE DU VENT.
  !     COMPUTATIONS FOR MOMENTUM OF CD AND OF ITS PRODUCT BY DENSITY
  !     TIME WIND SPEED.

  PCD(JLON)=ZFMTKE(JLON)*PCDN(JLON)

  !     IDEM POUR LA TEMPERATURE ET L'HUMIDITE.
  !     THE SAME FOR TEMPERATURE AND HUMIDITY.

  PCH(JLON)=ZFTTKE(JLON)*ZCDNH(JLON)
  PSTAB(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,PMRIPP(JLON,KLEV)))
ENDDO

!*
!     ------------------------------------------------------------------
!     V - CALCUL ANTI-FIBRILLATION "GIRARD-DELAGE".
!     ANTI-FIBRILLATION "GIRARD-DELAGE" COMPUTATION.
IF (LLAFGD) THEN
  DO JLON=KIDIA,KFDIA
        ZSIG_AF(JLON)=SIGN(0.5_JPRB,ZMRIFPP(JLON))+0.5_JPRB
  ENDDO

  IF (LLM123) THEN
    DO JLON=KIDIA,KFDIA
      ZDRIFTKE(JLON)=(ETKE_RIFC-ZMRIFPP(JLON))*C3TKEFREE*ETKE_R/&
       & ((PMRIPP(JLON,KLEV)*C3TKEFREE+ETKE_RIFC)*ETKE_R-2.0_JPRB*      &
       & ZMRIFPP(JLON)*ETKE_RIFC)
      ZDPHI3TKE(JLON)=(ETKE_RIFC-1.0_JPRB)/&
       & ((ZMRIFPP(JLON)-1.0_JPRB)**2*ETKE_RIFC)*ZDRIFTKE(JLON)
      ZDCHI3TKE(JLON)=(ETKE_R-1.0_JPRB)/&
       & ((ZMRIFPP(JLON)-1.0_JPRB)**2*ETKE_R)*ZDRIFTKE(JLON)
    ENDDO
  ENDIF 

  IF (ITTYPE==4) THEN
    DO JLON=KIDIA,KFDIA

        !Unstable
        ZDRRI_U=RQNSE_GU1/(RQNSE_GU2*PMRIPP(JLON,KLEV)+1.0_JPRB) &
         & -(RQNSE_GU2*(RQNSE_GU1*PMRIPP(JLON,KLEV)+RQNSE_S0))/  &
         & (RQNSE_GU2*PMRIPP(JLON,KLEV)+1)**2
        !Stable
        ZDRRI=(2.0_JPRB*RQNSE_GS1*PMRIPP(JLON,KLEV)+RQNSE_GS2)/&
         & (RQNSE_GS3*PMRIPP(JLON,KLEV)**2+PMRIPP(JLON,KLEV)*RQNSE_GS4+1.0_JPRB)- &
         & ((2.0_JPRB*RQNSE_GS3*PMRIPP(JLON,KLEV)+RQNSE_GS4)                        &
         & *(RQNSE_GS1*PMRIPP(JLON,KLEV)**2+RQNSE_S0+PMRIPP(JLON,KLEV)*RQNSE_GS2))&
         & /(RQNSE_GS3*PMRIPP(JLON,KLEV)**2+PMRIPP(JLON,KLEV)*RQNSE_GS4+1.0_JPRB)**2
        ZDRRI=ZSIG_AF(JLON)*ZDRRI+(1.0_JPRB-ZSIG_AF(JLON))*ZDRRI_U

        ZDRIFTKE(JLON)=(ZETKE_P(JLON)*ZDRRI*ZMRIFPP(JLON)**3&
        & +(C3TKEFREE*ZETKE_R(JLON)**2-ZETKE_P(JLON)**2*ZDRRI)*  &
        & ZMRIFPP(JLON)**2-2.0_JPRB*C3TKEFREE*ZETKE_P(JLON)*     &
        & ZETKE_R(JLON)**2*ZMRIFPP(JLON)+C3TKEFREE*              &
        & ZETKE_P(JLON)**2*ZETKE_R(JLON)**2)                     &
        & /(ZETKE_P(JLON)*ZETKE_R(JLON)*ZMRIFPP(JLON)**2    &
        & -2.0_JPRB*ZETKE_P(JLON)**2*ZETKE_R(JLON)*              &
        & ZMRIFPP(JLON)+ZETKE_P(JLON)**2*ZETKE_R(JLON)**2)

        ZDCHI3TKE(JLON)=((ZDRRI*ZMRIFPP(JLON))/ZETKE_R(JLON)**2    &
        & -ZDRIFTKE(JLON)/ZETKE_R(JLON))/(1.0_JPRB-ZMRIFPP(JLON))  &
        & +(ZDRIFTKE(JLON)*(1.0_JPRB-ZMRIFPP(JLON)/ZETKE_R(JLON))) &
        & /(1.0_JPRB-ZMRIFPP(JLON))**2
        ZDPHI3TKE(JLON)=(ZDRIFTKE(JLON)*              &
        & (1.0_JPRB-ZMRIFPP(JLON)/ZETKE_P(JLON)))     &
        & /(1.0_JPRB-ZMRIFPP(JLON))**2-ZDRIFTKE(JLON)/&
        & (ZETKE_P(JLON)*(1.0_JPRB-ZMRIFPP(JLON)))

    ENDDO

  ENDIF ! type 4

  IF (ITTYPE==5) THEN
    DO JLON=KIDIA,KFDIA

       !Unstable
        ZEFB_UP=EFB_UR&
        & /(ETKE_OLAM*EFB_UR/(EFB_AZ0*C3TKEFREE)+1.0_JPRB)
        ZDRIFTKE_U=(ZEFB_UP-ZMRIFPP(JLON))*C3TKEFREE*EFB_UR/&
        &         ((PMRIPP(JLON,KLEV)*C3TKEFREE+ZEFB_UP)*EFB_UR&
        & -2.0_JPRB*ZMRIFPP(JLON)*ZEFB_UP)
        ZDPHI3TKE_U=(ZEFB_UP-1.0_JPRB)/&
        & ((ZMRIFPP(JLON)-1.0_JPRB)**2*ZEFB_UP)*ZDRIFTKE_U
        ZDCHI3TKE_U=(EFB_UR-1.0_JPRB)/ &
        & ((ZMRIFPP(JLON)-1.0_JPRB)**2*EFB_UR)*ZDRIFTKE_U

       !Stable

        ZDRIFTKE(JLON)=(C3TKEFREE*EFB_AZ0*REFB_2*ZMRIFPP(JLON)**2+          &
        & ETKE_OLAM*REFB_1*ZMRIFPP(JLON)**2+EFB_UR*ETKE_OLAM*REFB_3*        &
        & ZMRIFPP(JLON)+C3TKEFREE*EFB_AZ0*REFB_3*ZMRIFPP(JLON)-             &
        & C3TKEFREE*EFB_AZ0*REFB_1*ZMRIFPP(JLON)-                           &
        & C3TKEFREE*EFB_AZ0*EFB_UR*REFB_3)**2/(EFB_AZ0*(C3TKEFREE*EFB_AZ0*  &
        & REFB_2**2*ZMRIFPP(JLON)**4+ETKE_OLAM*REFB_1*REFB_2                &
        &*ZMRIFPP(JLON)**4+2.0_JPRB*EFB_UR*ETKE_OLAM*REFB_2*REFB_3*         &
        &ZMRIFPP(JLON)**3+2.0_JPRB*C3TKEFREE*EFB_AZ0*REFB_2*REFB_3*         &
        &ZMRIFPP(JLON)**3-2.0_JPRB*C3TKEFREE*EFB_AZ0*                       &
        &REFB_1*REFB_2*ZMRIFPP(JLON)**3+EFB_UR*ETKE_OLAM*REFB_3**2*         &
        &ZMRIFPP(JLON)**2+C3TKEFREE*EFB_AZ0*REFB_3**2*ZMRIFPP(JLON)**2-     &
        & 2.0_JPRB*C3TKEFREE*EFB_AZ0*EFB_UR*REFB_2*REFB_3*ZMRIFPP(JLON)**2- &
        & 2.0_JPRB*C3TKEFREE*EFB_AZ0*REFB_1*REFB_3*ZMRIFPP(JLON)**2         &
        &+C3TKEFREE*EFB_AZ0*REFB_1**2*ZMRIFPP(JLON)**2-2.0_JPRB*            &
        & C3TKEFREE*EFB_AZ0*EFB_UR*REFB_3**2*ZMRIFPP(JLON)+2.0_JPRB*        &
        & C3TKEFREE*EFB_AZ0*EFB_UR*REFB_1*REFB_3*                           &
        & ZMRIFPP(JLON)+C3TKEFREE*EFB_AZ0*EFB_UR**2*REFB_3**2))

        ZDRRI=-((EFB_UR*REFB_2-REFB_1)*REFB_3*ZDRIFTKE(JLON))&
        & /(REFB_2**2*ZMRIFPP(JLON)**2+2.0_JPRB*REFB_2*      &
        & REFB_3*ZMRIFPP(JLON)+REFB_3**2)

        ZDPRI=-((C3TKEFREE**2*EFB_AZ0**2*EFB_UR*REFB_2-C3TKEFREE**2    &
        & *EFB_AZ0**2*REFB_1)*REFB_3*ZDRIFTKE(JLON))/                  &
        & ((C3TKEFREE**2*EFB_AZ0**2*REFB_2**2+2.0_JPRB*C3TKEFREE       &
        &*EFB_AZ0*ETKE_OLAM*REFB_1*REFB_2+ETKE_OLAM**2*REFB_1**2)*     &
        & ZMRIFPP(JLON)**2+((2.0_JPRB*C3TKEFREE*EFB_AZ0*EFB_UR*        &
        & ETKE_OLAM+2*C3TKEFREE**2*EFB_AZ0**2)*REFB_2+(2.0_JPRB*EFB_UR*&
        & ETKE_OLAM**2+2.0_JPRB*C3TKEFREE*EFB_AZ0*ETKE_OLAM)*REFB_1)   &
        &*REFB_3*ZMRIFPP(JLON)+ (EFB_UR**2*ETKE_OLAM**2+2.0_JPRB*      &
        & C3TKEFREE*EFB_AZ0*EFB_UR*ETKE_OLAM+C3TKEFREE**2*EFB_AZ0**2)*REFB_3**2)

        ZDCHI3TKE(JLON)=((ZDRRI*ZMRIFPP(JLON))/ZETKE_R(JLON)**2&
        & -ZDRIFTKE(JLON)/ZETKE_R(JLON))/                      &
        & (1.0_JPRB-ZMRIFPP(JLON))+                            &
        & (ZDRIFTKE(JLON)*(1.0_JPRB-ZMRIFPP(JLON)/             &
        &ZETKE_R(JLON)))/(1.0_JPRB-ZMRIFPP(JLON))**2

        ZDPHI3TKE(JLON)=((ZDPRI*ZMRIFPP(JLON))/ZETKE_P(JLON)**2      &
        & -ZDRIFTKE(JLON)/ZETKE_P(JLON))/                            &
        & (1.0_JPRB-ZMRIFPP(JLON))+(ZDRIFTKE(JLON)*                  &
        & (1.0_JPRB-ZMRIFPP(JLON)/ZETKE_P(JLON)))/                   &
        & (1.0_JPRB-ZMRIFPP(JLON))**2

        ZDRIFTKE(JLON)=ZSIG_AF(JLON)*ZDRIFTKE(JLON)+   &
        & (1.0_JPRB-ZSIG_AF(JLON))*ZDRIFTKE_U
        ZDCHI3TKE(JLON)=ZSIG_AF(JLON)* ZDCHI3TKE(JLON)+&
        &(1.0_JPRB-ZSIG_AF(JLON))* ZDCHI3TKE_U
        ZDPHI3TKE(JLON)=ZSIG_AF(JLON)* ZDPHI3TKE(JLON)+&
        & (1.0_JPRB-ZSIG_AF(JLON))* ZDPHI3TKE_U
    ENDDO

  ENDIF ! type 5
  IF (ITTYPE==6) THEN
    DO JLON=KIDIA,KFDIA
        !Unstable
        ZDRRI_U=RLOUIS_GU1/(RLOUIS_GU2*PMRIPP(JLON,KLEV)+1.0_JPRB) &
         & -(RLOUIS_GU2*(RLOUIS_GU1*PMRIPP(JLON,KLEV)+RLOUIS_S0))/ &
         & (RLOUIS_GU2*PMRIPP(JLON,KLEV)+1)**2
        !Stable
        ZDRRI=(2.0_JPRB*RLOUIS_GS1*PMRIPP(JLON,KLEV)+RLOUIS_GS2)/      &
        & (RLOUIS_GS3*PMRIPP(JLON,KLEV)**2+PMRIPP(JLON,KLEV)*          &
        & RLOUIS_GS4+1.0_JPRB)-((2.0_JPRB*RLOUIS_GS3*PMRIPP(JLON,KLEV)+&
        & RLOUIS_GS4)*(RLOUIS_GS1*PMRIPP(JLON,KLEV)**2+RLOUIS_S0+      &
        & PMRIPP(JLON,KLEV)*RLOUIS_GS2))/(RLOUIS_GS3*                  &
        & PMRIPP(JLON,KLEV)**2+PMRIPP(JLON,KLEV)*RLOUIS_GS4+1.0_JPRB)**2
        ZDRRI=ZSIG_AF(JLON)*ZDRRI+(1.0_JPRB-ZSIG_AF(JLON))*ZDRRI_U

        !Unstable
        ZDPRI_U=PLOUIS_GU1/(PLOUIS_GU2*PMRIPP(JLON,KLEV)+1.0_JPRB)   &
         & -(PLOUIS_GU2*(PLOUIS_GU1*PMRIPP(JLON,KLEV)+PLOUIS_S0))/   &
         & (PLOUIS_GU2*PMRIPP(JLON,KLEV)+1)**2
        !Stable
        ZDPRI=(2.0_JPRB*PLOUIS_GS1*PMRIPP(JLON,KLEV)+PLOUIS_GS2)/      &
        & (PLOUIS_GS3*PMRIPP(JLON,KLEV)**2+PMRIPP(JLON,KLEV)*          &
        & PLOUIS_GS4+1.0_JPRB)-((2.0_JPRB*PLOUIS_GS3*PMRIPP(JLON,KLEV)+&
        & PLOUIS_GS4)*(PLOUIS_GS1*PMRIPP(JLON,KLEV)**2+PLOUIS_S0+      &
        & PMRIPP(JLON,KLEV)*PLOUIS_GS2))/(PLOUIS_GS3*                  &
        & PMRIPP(JLON,KLEV)**2+PMRIPP(JLON,KLEV)*PLOUIS_GS4+1.0_JPRB)**2
        ZDPRI=ZSIG_AF(JLON)*ZDPRI+(1.0_JPRB-ZSIG_AF(JLON))*ZDPRI_U

        ZDRIFTKE(JLON)=(ZMRIFPP(JLON)**3*(ZETKE_P(JLON)*             &
        & ZDRRI-ZETKE_R(JLON)*ZDPRI)+ZMRIFPP(JLON)**2*               &
        & (-ZETKE_P(JLON)**2*ZDRRI+ZETKE_R(JLON)**2*(ZDPRI+1.0_JPRB))&
        &-2.0_JPRB*ZETKE_P(JLON)*ZETKE_R(JLON)**2*ZMRIFPP(JLON)+     &
        & ZETKE_P(JLON)**2*ZETKE_R(JLON)**2)/                        &
        &(ZETKE_P(JLON)*ZETKE_R(JLON)*                               &
        & (ZMRIFPP(JLON)**2-2*ZETKE_P(JLON)*ZMRIFPP(JLON)+           &
        & ZETKE_P(JLON)*ZETKE_R(JLON)))

        ZDCHI3TKE(JLON)=((ZDRRI*ZMRIFPP(JLON))/ZETKE_R(JLON)**2    &
        & -ZDRIFTKE(JLON)/ZETKE_R(JLON))/(1.0_JPRB-ZMRIFPP(JLON))  &
        & +(ZDRIFTKE(JLON)*(1.0_JPRB-ZMRIFPP(JLON)/ZETKE_R(JLON))) &
        & /(1.0_JPRB-ZMRIFPP(JLON))**2
        ZDPHI3TKE(JLON)=(ZDRIFTKE(JLON)*(1.0_JPRB-ZMRIFPP(JLON)/     &
        & ZETKE_P(JLON)))/(1.0_JPRB-ZMRIFPP(JLON))**2-ZDRIFTKE(JLON)/&
        & (ZETKE_P(JLON)*(1.0_JPRB-ZMRIFPP(JLON)))
    ENDDO

  ENDIF ! type 6

  !derivatives of Fm and Fh- surface
  IF(LCOEFK_F1) THEN
    DO JLON=KIDIA,KFDIA
      ZSIG=MAX(0._JPRB,SIGN(1._JPRB,ZMRIFPP(JLON)))
  !    ZFMTKE_DX(JLON)=(1.0_JPRB-ZSIG)*ZDCHI3TKE(JLON)+ZSIG*&
  !    &((1.0_JPRB-ZMRIFPP(JLON))*ZDCHI3TKE(JLON)&
  !     & -ZCHI3TA(JLON)*ZDRIFTKE(JLON))
  !    ZFHTKE_DX(JLON)=(1.0_JPRB-ZSIG)*ZDPHI3TKE(JLON)+ZSIG*&
  !     &((1.0_JPRB-ZMRIFPP(JLON))*ZDPHI3TKE(JLON)&
  !     & -ZPHI3TA(JLON)*ZDRIFTKE(JLON))
      ZFFTKE(JLON)=ZCHI3TA(JLON)*(1.0_JPRB-ZMRIFPP(JLON))
      ZFFTKE_DX(JLON)=(1.0_JPRB-ZMRIFPP(JLON))*ZDCHI3TKE(JLON)&
       & -ZCHI3TA(JLON)*ZDRIFTKE(JLON)
      ZFMTKE_DX(JLON)=(1.0_JPRB-ZSIG)*ZDCHI3TKE(JLON)+ZSIG*&
       &((ZFFTKE(JLON)*ZDCHI3TKE(JLON)+&
       & 0.5_JPRB*ZCHI3TA(JLON)*ZFFTKE_DX(JLON))&
       &/SQRT(ZFFTKE(JLON)))
      ZFHTKE_DX(JLON)=(1.0_JPRB-ZSIG)*ZDPHI3TKE(JLON)+ZSIG*&
       &((ZFFTKE(JLON)*ZDPHI3TKE(JLON)+&
       & 0.5_JPRB*ZPHI3TA(JLON)*ZFFTKE_DX(JLON))&
       &/SQRT(ZFFTKE(JLON)))
    ENDDO
  ELSE
    DO JLON=KIDIA,KFDIA
      !ZFMTKE_DX(JLON)=(1.0_JPRB-ZMRIFPP(JLON))*ZDCHI3TKE(JLON)&
      ! & -ZCHI3TA(JLON)*ZDRIFTKE(JLON)
      !ZFHTKE_DX(JLON)=(1.0_JPRB-ZMRIFPP(JLON))*ZDPHI3TKE(JLON)&
      ! & -ZPHI3TA(JLON)*ZDRIFTKE(JLON)
      ZFFTKE(JLON)=ZCHI3TA(JLON)*(1.0_JPRB-ZMRIFPP(JLON))
      ZFFTKE_DX(JLON)=(1.0_JPRB-ZMRIFPP(JLON))*ZDCHI3TKE(JLON)&
       & -ZCHI3TA(JLON)*ZDRIFTKE(JLON)
      ZFMTKE_DX(JLON)=(ZFFTKE(JLON)*ZDCHI3TKE(JLON)+&
       & 0.5_JPRB*ZCHI3TA(JLON)*ZFFTKE_DX(JLON))&
       &/SQRT(ZFFTKE(JLON))
      ZFHTKE_DX(JLON)=(ZFFTKE(JLON)*ZDPHI3TKE(JLON)+&
       & 0.5_JPRB*ZPHI3TA(JLON)*ZFFTKE_DX(JLON))&
       &/SQRT(ZFFTKE(JLON))
    ENDDO
  ENDIF

  ! COMPUTAION OF alpha_u and alpha_theta - PAUTKE,PATTKE
  DO JLON=KIDIA,KFDIA
    ZAUTKE(JLON)=0.5_JPRB*PMRIPP(JLON,KLEV)*&
    & ZFMTKE_DX(JLON)/ZFMTKE(JLON)
    ZATTKE(JLON)=0.5_JPRB*PMRIPP(JLON,KLEV)*&
    &ZFHTKE_DX(JLON)/ZFTTKE(JLON)
  ENDDO
ENDIF ! LLAFGD

! 'Dry' Antifibrilation scheme
PXDROV(KIDIA:KFDIA)=1.0_JPRB
PXHROV(KIDIA:KFDIA)=1.0_JPRB
IF (LLAFGD) THEN
  IF(XMULAF > 0.0_JPRB) THEN
    DO JLON=KIDIA,KFDIA
      PXDROV(JLON)=MAX(PXDROV(JLON),XMULAF)
      PXHROV(JLON)=MAX(PXHROV(JLON),XMULAF)
    ENDDO
  ELSE
    DO JLON=KIDIA,KFDIA

      !     CALCUL ANTI-FIBRILLATION "GIRARD-DELAGE".
      !     ANTI-FIBRILLATION "GIRARD-DELAGE" COMPUTATION.

      ! alpha_u and alpha_theta - ZAU,ZAT
      ZAU=ZAUTKE(JLON)
      ZAT=ZATTKE(JLON)
      ! CONSTRAINTS FOR 'DRY' ANTIFIBRILLATION SCHEME
      !Check for AF constrains : 2-3.alpha_u+2.alpha_th<=2
      ZSIG=SIGN(0.5_JPRB,1.5_JPRB*ZAU-ZAT)+0.5_JPRB

      ZBE=3._JPRB+ZAT-2.0_JPRB*ZAU
      ZGA=2.0_JPRB+ZSIG*(2.0_JPRB*ZAT-3._JPRB*ZAU)
      ZGAMDZ=4._JPRB*TSPHY*(RG/ZDPHI(JLON))
      ZGKU=ZGAMDZ*PCD(JLON)*ZU(JLON)
      ZGKT=ZGAMDZ*PCH(JLON)*ZU(JLON)
      ZMULA2=XMULAF**2
      ZDENUM=ZGKU*ZGKT
      ZA1=(1.0_JPRB+ZGKU)*(1.0_JPRB+ZGKT)
      ZB1=(1.0_JPRB+ZGKU)*ZGKT*(1.0_JPRB+ZAT)+2.0_JPRB*(1.0_JPRB+ZGKT)*ZGKU*(1.0_JPRB-ZAU)
      ZC1=ZDENUM*ZGA
      ZDELTA1=ZB1**2-4._JPRB*ZA1*ZC1
      ZCRIT1=MAX(SIGN(1.0_JPRB,ZDELTA1),0.0_JPRB)
      ZTAU=-0.5_JPRB * ZCRIT1*(ZB1+SQRT(ABS(ZDELTA1)))/ZA1
      ZCRIT2=MAX(SIGN(1.0_JPRB,XMULAF-ZTAU),0.0_JPRB)
      ZAPRIM=ZMULA2*ZDENUM
      ZBPRIM=ZMULA2*(ZGKU+ZGKT)+XMULAF*ZDENUM*ZBE
      ZCPRIM=ZMULA2+XMULAF*(ZGKT*(1.0_JPRB+ZAT)+2.0_JPRB*ZGKU*(1.0_JPRB-ZAU))&
       & +ZDENUM*ZGA  
      ZSDPRIM=SQRT(ABS(ZBPRIM**2-4._JPRB*ZAPRIM*ZCPRIM))
      ZBETAP= 0.5_JPRB*(-ZBPRIM +ZSDPRIM)/MAX(ZAPRIM,1.0_JPRB-ZCRIT2)
      ZBETA=1.0_JPRB+ZCRIT2*(ZBETAP-1.0_JPRB)

      PXDROV(JLON)=MAX(PXDROV(JLON),ZBETA)
      PXHROV(JLON)=MAX(PXHROV(JLON),ZBETA)
    ENDDO
  ENDIF
ENDIF

!*
!     ------------------------------------------------------------------
!     VI - CORRECTION DES COEFFICIENTS TURBULENTS EN PRESENCE DE PRECIPITATIONS.
!     CORRECTION OF TURBULENT COEFFICIENTS IN CASE OF PRECIPITATIONS.

IF (LRRGUST) THEN
  DO JLON=KIDIA,KFDIA
    ZCD=ZFMTKE(JLON)*PCDN(JLON)
    ZFP=MAX(0.0_JPRB,PFPLSH(JLON,KLEV)+PFPLCH(JLON,KLEV))
    ZRCORR=SQRT(1.0_JPRB+((((ZFP/(ZFP+RRSCALE))**RRGAMMA)*UTILGUST)**2) &
     & /(ZCD*ZU(JLON)**2))
    PCD(JLON)=PCD(JLON)*ZRCORR
    PCH(JLON)=PCH(JLON)*ZRCORR
    PCDN(JLON)=PCDN(JLON)*ZRCORR
    ZCDNH(JLON)=ZCDNH(JLON)*ZRCORR
  ENDDO
ENDIF

IF (LDCLS) THEN
  ! ZDELT     : UN POUR CALCULS DE SATURATION SUR GLACE, ZERO SINON.
  !            : ONE FOR SATURATION CALCULATION FOR ICE, ZERO OTHERWISE.
  IF (LNEIGE) THEN
    DO JLON=KIDIA,KFDIA
      ZDELT(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PTS(JLON)))
    ENDDO
  ELSE
    DO JLON=KIDIA,KFDIA
      ZDELT(JLON)=0.0_JPRB
    ENDDO
  ENDIF

  DO JLON=KIDIA,KFDIA
    ! ZESP      : PRESSION DE VAPEUR SATURANTE SUR PRESSION.
    !            : SATURATION WATER VAPOUR PRESSURE OVER PRESSURE.
    ZEW= FOEW (PTS(JLON),ZDELT(JLON))
    ZESP(JLON)=ZEW/PAPRS(JLON,KLEV)

    !     CALCUL DE LA DERIVEE PAR RAPPORT A TS DE QSATS ET DES COEFFICIENTS
    !     D'ECHANGE MULTIPLIES PAR DENSITE ET MODULE DU VENT.
    !     COMPUTATION OF THE DERIVATIVE OF QSATS WITH RESPECT TO TS AND OF
    !     THE EXCHANGE COEFFICIENTS MULTIPLIED BY DENSITY AND WIND SPEED.

    PDQSTS(JLON)= FODQS (PQSATS(JLON) , ZESP(JLON),&
     & FODLEW (PTS(JLON) , ZDELT(JLON)))  

    PCDROV(JLON)=PCD(JLON)*ZU(JLON)*PAPRS(JLON,KLEV)*PRTI(JLON)
    PCHROV(JLON)=PCH(JLON)*ZU(JLON)*PAPRS(JLON,KLEV)*PRTI(JLON)

    PGWDCS(JLON)=PAPRS(JLON,KLEV)*PRTI(JLON)
  ENDDO
ENDIF

!*
!     ------------------------------------------------------------------
!     VII - INTERPOLATION.

!           INTERPOLATION.

IF (LDHMT) THEN

  DO JLON=KIDIA,KFDIA
    ZCDMR(JLON)=PCD(JLON)
    ZCDNMR(JLON)=PCDN(JLON)
  ENDDO

  CALL ACTKECLS ( KIDIA,KFDIA,KLON,KLEV,&
   & PAPRS,PAPRSF,PCP,PQ,PT,PU,PV,&
   & PCD,PCDN,ZCDMR,ZCDNMR,ZCDNH,PCH,PCPS,ZDPHI,PDPHIT,PDPHIV,&
   & PQS,PSTAB,PTS,&
   & PQCLS,PRHCLS,PTCLS,PUCLS,PVCLS,PUCLN,PVCLN,PZPCLS)

  IF (LRAFTUR) THEN
    DO JLON=KIDIA,KFDIA
      ZSCRAF=(PCD(JLON)*(PU(JLON,KLEV)**2+PV(JLON,KLEV)**2)/ &
       & (PUCLS(JLON)**2+PVCLS(JLON)**2+(GCISMIN*PDPHIV(JLON)/RG)**2))/ &
       & (1.0_JPRB+(LOG(1.0_JPRB+PDPHIV(JLON)/GZ0RAF)/ &
       & LOG(1.0_JPRB+PDPHIV(JLON)/PGZ0(JLON)))**2)  

      ZSCRAF=1.0_JPRB+FACRAF*SQRT(ZSCRAF)
      PURAF(JLON)=ZSCRAF*PUCLS(JLON)
      PVRAF(JLON)=ZSCRAF*PVCLS(JLON)
    ENDDO
  ENDIF

ENDIF

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('ACTKEHMT',1,ZHOOK_HANDLE)
END SUBROUTINE ACTKEHMT
