SUBROUTINE MF_PHYS &
 !---------------------------------------------------------------------
 ! - INPUT and INOUT.
 & (CDCONF,KBL,KGPCOMP,KST,KEND,KGL1,KGL2,KGL3,KGL4,KOFF,KSTGLO, &
 & LDRETCFOU,LDWRTCFOU0,LDLADNMI,PDTPHY, &
 & YDCSGEOM,YDGSGEOM,YDOROG,POROGL,POROGM, &
 & PCUCONVCA,PNLCONVCA, &
 & PGMV,PGMVS,PGFL,PWT0,PWT0L,PWT0M,&
 & PRCP0,PHI0,PHIF0,PRE0,PRE0F,PREPHY0,PREPHY0F,PXYB0,PVCASRS0H, &
 & PWT9,PRCP9,PHI9,PHIF9,PRE9,PRE9F,PREPHY9,PREPHY9F,PXYB9, &
 & PKOZO,&
 & PGP2DSDT, &
 & PCTY0, &
 & PB1,PB2,PGMVT1,PGMVT1S,PGFLT1,&
 & PSP_SB,PSP_SG,PSP_RR,&
 & PSD_VF,PSD_VP,PSD_VV,PSD_VH,PSD_VA,PSD_VC,PSD_VD,PSD_SFL,PSD_SFO,&
 & PEMTD,PEMTU,PTRSW,PRMOON,PGPAR, &
 & PGDEOSI,PGUEOSI,PGMU0,PGMU0_MIN,PGMU0_MAX, &
 & PGDEOTI,PGDEOTI2,PGUEOTI,PGUEOTI2,PGEOLT,PGEOXT, &
 & PGRPROX,PGMIXP,PGFLUXC,PGRSURF, &
 !---------------------------------------------------------------------
 ! - OUTPUT .
 & PDHSF,PALB,PCAPE,PCLCC,PCLCH,PCLCL,PCLCM,PCLCT,PCLPH,PVEIN,PCT, &
 & PDIFCQ,PDIFCQI,PDIFCQL,PDIFCS,PDIFTQ,PDIFTQI,PDIFTQL,PDIFTS, &
 & PFCCQL,PFCCQN,PFCHOZ,PFCHSP,PFCLL,PFCLN,PFCQING,PFCQLNG,PFCQNG,PFCS, &
 & PFCSQL,PFCSQN,PFEVL,PFEVN,PFEVV,PFGEL,PFGELS,PFLWSP,PFONTE, &
 & PFPLCL,PFPLCN,PFPLCG,PFPLCHL,PFPLSL,PFPLSN,PFPLSG,PFPLSHL,&
 & PFRMH,PFRSO,PFRSOC,PFRSODS,PFRSOLU, &
 & PFRSOPS,PFRSOPT,PFRTH,PFRTHC,PFRTHDS,PFTR,PGZ0,PGZ0H,PNEB, &
 & PQCLS,PQICE,PQLI,PQS, &
 & PRH,PRHCLS,PRUISL,PRUISP,PRUISS, &
 & PSTRCU,PSTRCV,PSTRDU,PSTRDV,PSTRMU,PSTRMV,PSTRTU,PSTRTV, &
 & PDIFCQLC,PDIFCQIC,PFIMCC, &
 & PFEDQLC,PFEDQIC,PFEDQRC,PFEDQSC,PFCNEGQLC,PFCNEGQIC,PFCNEGQRC,PFCNEGQSC, &
 & PUCLS,PVCLS,PTCLS,PUGST,PVGST, &
 & PMOCON,PFDIS, &
 & PFHPCL,PFHPCN,PFHPSL,PFHPSN,PFHSCL,PFHSCN,PFHSSL,PFHSSN, &
 & PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL,&
 & PTENDU,PTENDV, &
 & PQSOL, &
 & PFPFPSL,PFPFPSN,PFPFPCL,PFPFPCN,PFPEVPSL,PFPEVPSN, &
 & PFPEVPCL,PFPEVPCN,PDERNSHF,&
 & PFCQRNG, PFCQSNG,PDIAGH)

!**** *MF_PHYS* METEO-FRANCE PHYSICS.

!     Purpose.
!     --------
!         Call METEO-FRANCE physics and physical tendencies.

!**   Interface.
!     ----------
!        *CALL* *MF_PHYS(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        CDCONF    : configuration of work
!        KBL       : NPROMA-packets number
!        KGPCOMP   : total number of grid points in the domain
!        KST       : first element of work.
!        KEND      : last element of work.
!        KGL1,KGL2 : first and last latitude of computations.
!                    - bounds NGPTOT-packets for DM calculations.
!        KGL3,KGL4 : DM-local first and last latitude of computations
!                    - bounds NPROMA-packets for calculations.
!        KOFF      : index offset between NPROMM-arrays and NGPTOT-arrays.
!        KSTGLO    : global offset.
!        LDRETCFOU : .T. -> CPPFTTCINV is called.
!        LDWRTCFOU0: .T. -> CPPFTTCDIR is called.
!        LDLADNMI  : .T. if adiabatic NNMI iteration.
!        PDTPHY    : timestep used in the physics.
!        YDCSGEOM  : structure for computational sphere horizontal geometry
!        YDGSGEOM  : structure for geographical sphere horizontal geometry
!        YDOROG    : structure for orography
!        POROGL,POROGM: components of grad(orography).
!        PCUCONVCA : CA array for interaction with the physics
!        PNLCONVCA : CA array for interaction with the physics
!        PGMV      : GMV at time t and t-dt.
!        PGMVS     : GMVS at time t and t-dt.
!        PGFL      : GFL at time t and t-dt.
!        PWT0      : w-wind time t.
!        PWT0L     : zonal derivative of w-wind at time t.
!        PWT0M     : merid derivative of w-wind at time t.
!        PRCP0     : contains "cp", "R" and "Kap=R/Cp" at t.
!        PHI0      : geopotential height on interlayers at time t.
!        PHIF0     : geopotential height on layers at time t.
!        PRE0      : hydrostatic pressure "prehyd" on interlayers at time t.
!        PRE0F     : hydrostatic pressure "prehyd" on layers at time t.
!        PREPHY0   : input pressure "pre" for AROME on interlayers at time t.
!        PREPHY0F  : input pressure "pre" for AROME on layers at time t.
!        PXYB0     : contains pressure depth, "delta", "alpha" at time t.
!        PVCASRS0H : "a/rs" on interlayers at time t.
!        PRKQVH    : Rasch-Kristjansson scheme - water vapour tendency
!        PRKQCH    : Rasch-Kristjansson scheme - condensates tendency
!        PWT9      : Vertical wind time t-dt.
!        PRCP9     : contains "cp", "R" and "Kap=R/Cp" at t-dt.
!        PHI9      : geopotential height on interlayers at time t-dt.
!        PHIF9     : geopotential height on layers at time t-dt.
!        PRE9      : hydrostatic pressure "prehyd" on interlayers at time t-dt.
!        PRE9F     : hydrostatic pressure "prehyd" on layers at time t-dt.
!        PREPHY9   : input pressure "pre" for AROME on interlayers at time t-dt.
!        PREPHY9F  : input pressure "pre" for AROME on layers at time t-dt.
!        PXYB9     : contains pressure depth, "delta", "alpha" at time t-dt.
!        PKOZO     : fields for photochemistery of ozon.
!        PGP2DSDT  : stochastic physics random pattern

!     INPUT/OUTPUT:
!     -------------
!        PCTY0     : contains vertical velocities, vertical integral of divergence at t.
!        PB1       : "SLB1"-buffer, used for interpolations in the SL scheme.
!        PB2       : "SLB2"-buffer.
!        PGFLT1    : GFL t+dt
!        PGPAR     : surface fields for AROME.
!        PGDEOSI   : DESCENDING INCREMENTAL OPTICAL DEPTHS, SOLAR
!        PGUEOSI   : ASCENDING  INCREMENTAL OPTICAL DEPTHS, SOLAR
!        PGMU0     : COSINE OF SOLAR ZENITH ANGLE, APPROXIMATE ACTUAL VALUE
!        PGMU0_MIN : COSINE OF SOLAR ZENITH ANGLE, MIN VALUE
!        PGMU0_MAX : COSINE OF SOLAR ZENITH ANGLE, MAX VALUE
!        PGDEOTI   : DESCENDING INCREMENTAL OPTICAL DEPTHS, dB/dT(T0) WEIGHTS
!        PGDEOTI2  : DESCENDING INCREMENTAL OPTICAL DEPTHS, B WEIGHTS WITH
!                    LINEAR T_e CORRECTION
!        PGUEOTI   : ASCENDING INCREMENTAL OPTICAL DEPTHS, dB/dT(T0) WEIGHTS
!        PGUEOTI2  : ASCENDING INCREMENTAL OPTICAL DEPTHS, B WEIGHTS WITH
!                    LINEAR T_e CORRECTION
!        PGEOLT    : LOCAL OPTICAL DEPTHS, dB/dT(T0) WEIGHTS
!        PGEOXT    : MAXIMUM OPTICAL DEPTHS FOR EBL-EAL, dB/dT(T0) WEIGHTS
!        PGRPROX   : CORRECTION TERM FOR ADJACENT EXCHANGES
!        PGMIXP    : NON-STATISTICAL WEIGHTS FOR BRACKETING
!        PGFLUXC   : OUT OF BRACKET PART OF CLEARSKY EBL, RESP. EBL-EAL FLUX
!        PGRSURF   : CORRECTIVE RATIO FOR SURFACE CTS CONTRIBUTION

!     OUTPUT:
!     -------
!        PDHSF     : distribution of horizontal mean weights used for
!                    simplified radiation scheme.
!        ---------------------- output of aplpar ------------------------------
!        PALB      : modele surface shortwave albedo (diagnostic).
!        PCAPE     : CAPE.
!        PCLCC     : convective cloud cover (diagnostic).
!        PCLCH     : high cloud cover (diagnostic).
!        PCLCL     : low cloud cover (diagnostic).
!        PCLCM     : medium cloud cover (diagnostic).
!        PCLCT     : total cloud cover (diagnostic).
!        PCLPH     : height (in meters) of the PBL.
!        PVEIN     : ventilation index in the PBL.
!        PCT       : thermical coefficient of soil-vegetation middle.
!        PDIFCQ    : convective flux of specific humidity (not rain/snow).
!        PDIFCQI   : convective flux of solid water (not rain/snow).
!        PDIFCQL   : convective flux of liquid water (not rain/snow).
!        PDIFCS    : convective flux of enthalpy (not rain/snow).
!        PDIFTQ    : turbulent flux (inc. "q" negative) of specific humidity.
!        PDIFTQI   : turbulent flux (inc. "q" negative) of solid water.
!        PDIFTQL   : turbulent flux (inc. "q" negative) of liquid water.
!        PDIFTS    : turbulent flux of enthalpy (or dry static energy).
!        PFCCQL    : convective condensation flux for liquid water.
!        PFCCQN    : convective condensation flux for ice.
!        PFCHOZ    : ozon photo-chemical flux.
!        PFPFPSL   : flux of liquid resol. precipitation: the generation term. 
!        PFPFPSN   : flux of solid resolved precipitation: the generation term. 
!        PFPFPCL   : flux of liquid conv. precipitation: the generation term. 
!        PFPFPCN   : flux of solid conv. precipitation: the generation term. 
!        PFPEVPSL  : resolved precipitation flux due to evaporation.
!        PFPEVPSN  : resolved precipitation flux due to sublimation.
!        PFPEVPCL  : convective precipitation flux due to evaporation.
!        PFPEVPCN  : convective precipitation flux due to sublimation.
!        PFCHSP    : heat flux from surface to deep soil.
!        PFCLL     : latent heat flux over liquid water (or wet soil).
!        PFCLN     : latent heat flux over snow (or ice).
!        PFCQING   : pseudo-flux of ice to correct for "qi"<0.
!        PFCQLNG   : pseudo-flux of liquid water to correct for "ql"<0.
!        PFCQNG    : pseudo-flux of water to correct for Q<0.
!        PFCS      : sensible heat flux at surface level.
!        PFCSQL    : stratiform condensation flux for liquid water.
!        PFCSQN    : stratiform condensation flux for ice.
!        PFEVL     : water vapour flux over liquid water (or wet soil).
!        PFEVN     : water vapour flux over snow (or ice) and frozen soil.
!        PFEVV     : evapotranspiration flux.
!        PFGEL     : freezing flux of soil water.
!        PFGELS    : freezing flux of soil water at surface level.
!        PFLWSP    : water flux from surface to deep soil.
!        PFONTE    : water flux corresponding to surface snow melt.
!        PFPLCL    : convective precipitation as rain.
!        PFPLCN    : convective precipitation as snow.
!        PFPLCG    : convective precipitation as graupel.
!        PFPLCHL   : convective precipitation as hail.
!        PFPLSL    : stratiform precipitation as rain.
!        PFPLSN    : stratiform precipitation as snow.
!        PFPLSG    : stratiform precipitation as graupel.
!        PFPLSHL   : stratiform precipitation as hail.
!        PFRMH     : mesospheric enthalpy flux.
!        PFRSO     : shortwave radiative flux.
!        PFRSOC    : shortwave clear sky radiative flux.
!        PFRSODS   : surface downwards solar flux.
!        PFRSOLU   : downward lunar flux at surface.
!        PFRSOPS   : surface parallel solar flux.
!        PFRSOPT   : top parallel solar flux.
!        PFRTH     : longwave radiative flux.
!        PFRTHC    : longwave clear sky radiative flux.
!        PFRTHDS   : surface downwards IR flux.
!        PFTR      : transpiration flux.
!        PGZ0      : g*roughness length (current).
!        PGZ0H     : current g*thermal roughness length (if KVCLIV >=8).
!        PNEB      : fractional cloudiness for radiation.
!        PQCLS     : specific humidity at 2 meters (diagnostic).
!        PQICE     : specific humidity of solid water for radiation.
!        PQLI      : specific humidity of liquid water for radiation.
!        PQS       : specific humidity at surface level.
!        PRH       : relative humidity.
!        PRHCLS    : relative humidity at 2 meters (diagnostic).
!        PRUISL    : run-off flux out the interception water-tank.
!        PRUISP    : run-off flux in soil.
!        PRUISS    : run-off flux at surface level.
!        PSTRCU    : convective flux of momentum "U".
!        PSTRCV    : convective flux of momentum "V".
!        PSTRDU    : gravity wave drag flux "U".
!        PSTRDV    : gravity wave drag flux "V".
!        PSTRMU    : mesospheric flux for "U"-momentum.
!        PSTRMV    : mesospheric flux for "V"-momentum.
!        PSTRTU    : turbulent flux of momentum "U".
!        PSTRTV    : turbulent flux of momentum "V".
!        PDIFCQLC to PFCNEGQSC:
!        PUCLS     : U-component of wind at 10 meters (diagnostic).
!        PVCLS     : V-component of wind at 10 meters (diagnostic).
!        PTCLS     : temperature at 2 meters (diagnostic).
!        PUGST     : U-component of gusts (diagnostic).
!        PVGST     : V-component of gusts (diagnostic).
!        PDERNSHF  : derivative of the non solar surface with respect to Tsurf
!        ---------------------- end of output of aplpar -----------------------
!        PMOCON    : moisture convergence.
!        PFDIS     : enthalpy flux due to dissipation of kinetic energy. 
!        PFHPCL    : liquid water convective condensation enthalpy flux.
!        PFHPCN    : snow convective condensation enthalpy flux.
!        PFHPSL    : liquid water stratiform condensation enthalpy flux.
!        PFHPSN    : snow stratiform condensation enthalpy flux.
!        PFHSCL    : sensible heat flux due to liquid convective precipitations
!        PFHSCN    : sensible heat flux due to snow convective precipitations.
!        PFHSSL    : sensible heat flux due to liquid stratiform precipitations
!        PFHSSN    : sensible heat flux due to snow stratiform precipitations.
!        PTENDU    : "U"-wind tendency due to physics.
!        PTENDV    : "V"-wind tendency due to physics.
!        PQSOL     : surface specific humidity used in case "delta m=1".
!        PFCQRNG   : pseudo-flux of rain to correct for Q<0
!        PFCQSNG   : pseudo-flux of snow to correct for Q<0
!        PDIAGH    : Add Hail diagnostic PDIAGH (AROME)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!       2000-12-04: F. Bouyssel & J.M. Piriou

!     Modifications.
!     --------------
!       02-06-25 F. Bouyssel : new pseudo-historical variables (RRCV,RRST,CVPP)
!       02-03-29 Y.Bouteloup : Climatological ozone profiles
!       02-04-22 C. Fischer : remove call to wrnhtrajm
!       02-09-03 E. Bazile : new fields for APLPAR (MVVALS, MVVALV)
!       K. YESSAD (10-2002): option lwrtcfou=T: allow to store a temporal
!        average of transmission coefficients.
!       2003-04-20 M. Bellus: calling aplparint instead of aplpar (interface
!                      for PNEBH splitting), changed arguments in aplpar
!                      (aplparint) callings - removed 14 arguments
!       2003-10-03 M. Tudor: MVNNEBH split into 2D arrays MVXCCH and MVPBLH
!       2003-10-06 Y. Bouteloup : Extraction of SCM profile
!       M.Hamrud      01-Oct-2003 CY28 Cleaning
!       K. YESSAD (Jan 2004): physics interpolated at O in the TL and AD code.
!       K. YESSAD (Jan 2004): make the code valid for any value of NFLSA.
!       2003-10-17 M. Tudor: CPF and SPF in GFL, ZFPLXH, PFPLXH
!       2003-12-10 F. Vinit : new fields in output (from APLPAR: PALB,PGZ0,PGZ0H).
!       2004-03-05 Y.Bouteloup : Snow, Rain and TKE 
!       2004-03-26 Y.Bouteloup : introduction of MTS radiance
!       2004-03-01 T. Kovacic : diagnostics on physical tendencies
!       2004-09-15 F.Bouyssel : introduction of INITAPLPAR
!       2004-07-01 K. Yessad: cleaning (WRITEPHYSIO and tests for PC scheme).
!       2004-07-06 K. Yessad: use GFL ozone instead of obsolete pas. scalar one
!       2004-11-18 Y.Seity : AROME new arguments and calls
!       2004-12-20 K. Yessad : Rewrite part 4 (AROME) and add case LGWADV=T.
!       2005-04-21 K. Yessad and Y.Seity: reformulation of "delta m=1".
!       2005-09-21 Y.Seity: add LDLADVEXT and PEXTT9 and PEXTT0 for AROME use
!       2005-10-20 Y. Bouteloup: add facilies for CVGQ
!       2005-10-26 A. Alias : GMGEC/EAC modifs
!            M. Deque : grid point nudging
!            Humidity Mesopheric flux (ZFRMQ) added
!            JF. Royer : Derivative of non solar surface fluxes 'PDERNSHF' added
!       2005-12-22 A. Alias : add sulfate and volcano aerosols
!       2006-03-23 T. Kovacic: new arguments in APL_AROME; PFRTH AND PFRSO
!       2006-02-15 R. Zaaboul: add arguments KBL and KGPCOMP for ARO_GROUND_PARAM
!       2006-03-20 A. Alias : add SDSAT and CVV fields (JF Gueremy)
!       2006-04-15 E. Bazile : TKE flux in CPTEND.
!       2006-06-01 B.H. Sass : Call to HIRLAM physics and modified call of CPCHET
!       2006-06-15 K. Yessad : Bug correction + some indentation cleanings
!       M.Hamrud      01-Jul-2006 Revised surface fields
!       2006-10-02 M. Bellus : ALARO-0 phasing (prognostic convection, p-TKE)
!       2006-10-26 E. Bazile : Corrections of the surface fields for APLPAR.
!       2006-10-25 A. Alias  : Call to APLPAR and CPNUDG modified following
!                              introduction of surface fields 
!       2007-05-07 Y. Seity  : Add graupel and hail precipitation fluxes
!       2007-04-17 S.Ivatek-S: Over dimensioning of PGPAR to NGPAR+1, boundary
!                              checking problem if NGPAR=0
!       2007-03-12-07 A.Alias : grid point nudging in stretched mode (M.Deque)
!       2007-04-03 A.Alias : Ts nudging with land-sea mask (M.Deque)
!       2007-05-07 F. Bouyssel : Change in CPTEND and CPTEND_NEW calling, 
!                                Add PFCLL, PFCLN in some diagnostics
!       2007-05-31 T.Kovacic: New arguments PFCQRNG, PFCQSNG (former ZFCQRNG, 
!                              ZFCQSNG) and  PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL
!       N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!       2007-12-18 Y.Seity : Add Hail for AROME (YH)  
!       4-Mar-2008 Y. Seity : Cleaning IR and WV similated sat pictures
!                            (replaced by Fullpos Calculations)
!       22-Oct-2008 R. Brozkova: Protection of arrays PUNEBH, PENTCH
!       K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!       04-Mar-2009 A.Alias : call CPTEND/INITAPLPAR modified to add
!                         Humidity Mesopheric flux (ZFRMQ).
!                     and IVCLIA removed and call to CPNUDG modified as
!                         Nuding mask is now in SD_VF group
!                         call HL_APLPAR modified to add PFCQNG for acdifus
!                         call APL_AROME modified to add Sulfate/Volcano aerosols for radaer
!       K. Yessad (Jul 2009): remove CDLOCK + some cleanings
!       2009-10-15 Y. Bouteloup : Store radiative cloud water and ice in GFL (YIRAD and YLRAD)
!       F. Vana   15-Oct-2009 : NSPLTHOI option
!       K.Yessad (Feb 2010): use YM_RADTC and RFORADTC
!       2010-03-26 Y. Bouteloup : Store radiative cloud water and ice in GFL (AROME case)
!       2010-04-26 Y. Bouteloup : Only one call to cputqy, cputqys and cputqy_arome
!            This need the use of ZTENDGFL as argument of cptend, cptend_new and apl_arome.
!       2010-05-11 F. Bouyssel : Use of PINDX, PINDY
!       2010-05-28 C. Geijo    : Fix error in IPTR array element referencing 
!       2010-06-21 O.Riviere/F. Bouyssel : Fix to have Ts evolving in Fa files with Surfex
!       Dec 2010 A.Alias   : ZMU0N added to call CPPHINP/APLPAR/APL_AROME/HL_APLPAR
!                            CALL to CPNUDG with or with LMSE (A.Voldoire)
!       K. Yessad (Jan 2011): introduce INTDYN_MOD structures.
!       L. Bengtsson-Sedlar & F. Vana 18-Feb-2011 : CA scheme for convection
!       F. Vana   22-Feb-2011 : 3D turbulence
!       2011-02-01 M. Mokhtari: Add LMDUST and PEXTT9 and PEXTT0 IN APLPAR
!                             (treatment of the desert aerosols) 
!       2011-03 A.Alias  : new argument to  for sunshine hours YSD_VD%YSUND 
!                      CPNUDG if LMSE=.T. or LMSE=.F. (bugfix)
!                      debug ozone GFL (IPO3) (D. St-Martin)
!                      Humidity Mesopheric flux (ZFRMQ) added in CPTEND_NEW
!       F.Bouyssel (26-03-2011): Fix to have Snow in hist file with surfex
!       2011-06: M. Jerczynski - some cleaning to meet norms
!       E. Bazile 2011-08-26 : Output for MUSC 1D with LFA files with WRITEPHYSIO
!         used previously for extracting profiles from 3D (now also available for AROME).
!       K. Yessad (Dec 2011): use YDOROG, YDGSGEOM and YDCSGEOM.
!       2011-11-21 JF Gueremy : dry convective adjustment (LAJUCV)
!   F. Vana  26-Jan-2012 : historic Qs for TOM's BBC.
!       F.Bouttier Jul 2012: stochastic physics for AROME
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLSA    ,NFLEVG   ,NFLSUL   ,NPROMA   ,& 
 & NPROMM   ,NPROMVC 
USE YOMCT0   , ONLY : LSLAG    ,LTWOTL   ,LVERCOR  ,NSTOP    ,LNHDYN ,&
 & LAROME, LSFORCS
USE YOMCT3   , ONLY : NSTEP 
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_GWMPA 
USE YOMDYNA  , ONLY : LGWADV   ,LNH_PDVD ,LNH_GEOGW,L3DTURB
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE YOMNUD   , ONLY : NFNUDG   ,LNUDG 
USE YOMSNU   , ONLY : XPNUDG
USE YOMIOP   , ONLY : NG3D95   ,NG2D95   ,NG2DS5
USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY
USE YOMPHY   , ONLY : NDPSFI   ,LMPHYS   ,LOZONE   ,LRRGUST  ,&
 & LNEBN     ,LNEBR   ,LCVPRO  ,LCDDPRO  ,LCVPGY   ,LAJUCV   ,&
 & LSTRAPRO  , L3MT, LPHSPSH ,LSOLV    ,CGMIXLEN, LPTKE ,LCOEFKTKE,&
 & NSORAYFR
USE YOMPHY2  , ONLY : TSPHY
USE YOMSIMPHL, ONLY : LSIMPH   ,LTRAJPS  ,LRAYSP
USE YOMRCOEF , ONLY : LRCOEF   ,LTLADDIA ,NG3SR
USE YOMDPHY  , ONLY : NVCLIS   ,NTSSG
USE YOMTOPH  , ONLY : NTPLUI   ,NTAJUC
USE SURFACE_FIELDS_MIX  , ONLY :  &
 & YSP_SBD, YSP_SB, YSP_SGD, YSP_SG, &
 & YSP_RRD, YSP_RR,           &
 & YSD_VFD, YSD_VF, YSD_VPD, YSD_VP, YSD_VVD, YSD_VV, YSD_VHD, YSD_VH,&
 & YSD_VAD, YSD_VA, YSD_VCD, YSD_VC, YSD_SFO, YSD_SFL, YSD_SFOD, &
 & YSD_SFLD, YSD_VDD, YSD_VD
USE YOMLDDH  , ONLY : LSDDH
USE YOM_YGFL , ONLY : YGFL     ,YGFLC,  YQ      , YL      , YI      ,&
 & YS        ,YR      ,YG      ,YTKE     , YO3     ,YCPF     ,&
 & YSPF      ,YEXT    ,NGFL_EXT, YCVV    ,YEZDIAG  ,NGFL_EZDIAG ,&
 & YA      ,YH      ,YCVGQ    , YTTE, YMXL, YEFB1, YEFB2, YEFB3 ,&
 & YDAL      ,YDOM    , YUAL   , YUOM    ,YUNEBH   ,YUEN     ,&
 & YRKTH     ,YRKTQV  ,YRKTQC  ,YLRAD    ,YIRAD    ,YSRC     ,&
 & YLCONV, YICONV, YRCONV, YSCONV
USE YOMFFTTC , ONLY : RBASCOTC ,RBASSITC
USE MODULE_RADTC_MIX, ONLY : YM_RADTC ,RFORADTC ,RFOAC
USE PTRSLB1  , ONLY : MSLB1SP9
USE PTRSLB2  , ONLY : MSLB2KAPPAM, MSLB2KAPPAH
USE YOMGMV   , ONLY : YT0      ,YT9      ,YT1      ,NDIMGMV  ,NDIMGMVS
USE YOMSCM   , ONLY : LGSCM
USE YOMARPHY , ONLY : LMPA, LMSE, LMDUST
USE YOMPARAR , ONLY : NRR      ,NGPAR    ,MVTS
USE YOMCST   , ONLY : RG
USE YOMCHET  , ONLY : GCHETN
USE YOMGC    , ONLY : TGSGEOM  ,TOROG
USE YOMGEM   , ONLY : TCSGEOM
USE INTDYN_MOD,ONLY : YYTCTY0  ,YYTRCP0  ,YYTRCP9  ,YYTXYB0_PHY,YYTXYB9_PHY
USE YOMLSFORC, ONLY : LMUSCLFA, IMUSCLFA
USE YOMSPSDT ,  ONLY : YGPSDT

!     -------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF
INTEGER(KIND=JPIM),INTENT(IN)    :: KBL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPCOMP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL3 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL4 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOFF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
LOGICAL           ,INTENT(IN)    :: LDRETCFOU 
LOGICAL           ,INTENT(IN)    :: LDWRTCFOU0 
LOGICAL           ,INTENT(IN)    :: LDLADNMI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTPHY 
TYPE(TCSGEOM)     ,INTENT(IN)    :: YDCSGEOM
TYPE(TGSGEOM)     ,INTENT(IN)    :: YDGSGEOM
TYPE(TOROG)       ,INTENT(IN)    :: YDOROG
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCUCONVCA(NPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNLCONVCA(NPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVS(NPROMA,NDIMGMVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWT0L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWT0M(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRCP0(NPROMA,NFLEVG,YYTRCP0%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE0F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREPHY0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREPHY0F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXYB0(NPROMA,NFLEVG,YYTXYB0_PHY%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCASRS0H(NPROMVC,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRCP9(NPROMA,NFLEVG,YYTRCP9%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHI9(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIF9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE9(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE9F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREPHY9(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREPHY9F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXYB9(NPROMA,NFLEVG,YYTXYB9_PHY%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKOZO(NPROMA,NFLEVG*NVCLIS+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGP2DSDT(NPROMA,YGPSDT%NG2D )
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCTY0(NPROMA,0:NFLEVG,YYTCTY0%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1S(NPROMA,YT1%NDIMS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLT1(NPROMA,NFLEVG,YGFL%NDIM1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP_SB(NPROMA,YSP_SBD%NLEVS,YSP_SBD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP_SG(NPROMA,YSP_SGD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP_RR(NPROMA,YSP_RRD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_VF(NPROMA,YSD_VFD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_VP(NPROMA,YSD_VPD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_VV(NPROMA,YSD_VVD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_VH(NPROMA,YSD_VHD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_VA(NPROMA,YSD_VAD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_VC(NPROMA,YSD_VCD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_VD(NPROMA,YSD_VDD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_SFL(NPROMA,YSD_SFLD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSD_SFO(NPROMA,YSD_SFOD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMTD(NPROMA,1:NFLEVG+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMTU(NPROMA,1:NFLEVG+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTRSW(NPROMA,1:NFLEVG+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRMOON(NPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGPAR(NPROMA,NGPAR+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGDEOSI(NPROMM,0:NFLEVG,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGUEOSI(NPROMM,0:NFLEVG,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMU0(NPROMM,NSORAYFR)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMU0_MIN(NPROMM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMU0_MAX(NPROMM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGDEOTI(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGDEOTI2(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGUEOTI(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGUEOTI2(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOLT(NPROMM,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGEOXT(NPROMM,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGRPROX(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMIXP(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFLUXC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGRSURF(NPROMM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDHSF(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALB(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCAPE(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCC(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCH(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCL(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCM(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLCT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLPH(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVEIN(NPROMM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCT(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFCQ(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFCQI(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFCQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFCS(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTQ(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTQI(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFTS(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCCQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCCQN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCHOZ(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPFPSL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPFPSN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPFPCL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPFPCN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPEVPSL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPEVPSN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPEVPCL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPEVPCN(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCHSP(NPROMM,YSP_SBD%NLEVS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCLL(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCLN(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQING(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCQLNG(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCQNG(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCS(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCSQL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCSQN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEVL(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEVN(NPROMM,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEVV(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFGEL(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFGELS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLWSP(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFONTE(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLCG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLCHL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLSL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLSN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLSG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLSHL(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFRMH(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFCQLC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIFCQIC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFIMCC (NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEDQLC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEDQIC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEDQRC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFEDQSC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCNEGQLC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCNEGQIC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCNEGQRC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFCNEGQSC(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFRSO(NPROMM,0:NFLEVG,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOC(NPROMM,0:1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSODS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOLU(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOPS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOPT(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFRTH(NPROMM,0:NFLEVG,NTSSG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTHC(NPROMM,0:1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTHDS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFTR(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGZ0H(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PNEB(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQICE(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRH(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRHCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRUISL(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRUISP(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRUISS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRCU(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRCV(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRDU(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRDV(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRMU(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRMV(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTU(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSTRTV(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTCLS(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUGST(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVGST(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMOCON(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFDIS(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSCL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSCN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSSL(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHSSN(NPROMM,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFEPFP(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPCQ(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSN(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCMPSL(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENDU(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENDV(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQSOL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDERNSHF(NPROMM) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQRNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQSNG(NPROMM,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIAGH(NPROMA)

!     ------------------------------------------------------------------

LOGICAL :: LL_CALL_CPPFTTCDIR       ! CPPFTTCDIR is called if .T.
LOGICAL :: LLDIAB
LOGICAL :: LL_SAVE_PHSURF
LOGICAL :: LLACTIVE_HV
LOGICAL :: LLACTIVE_Z0F
LOGICAL :: LLACTIVE_Z0H
LOGICAL :: LLACTIVE_PBLH
LOGICAL :: LLACTIVE_SPSH
LOGICAL :: LLACTIVE_QSH
LOGICAL :: LLXFUMSE

INTEGER(KIND=JPIM) :: IFIELDS, IFIELDSS
INTEGER(KIND=JPIM) :: IJL,IBLK
INTEGER(KIND=JPIM) :: IPQ,IPO3,ITDIA,IPTREXT,IPTR_CONT
INTEGER(KIND=JPIM) :: IPGFL(YGFL%NUMFLDS),IPTR(YGFL%NUMFLDS)

INTEGER(KIND=JPIM) :: INSTEP_DEB,INSTEP_FIN
INTEGER(KIND=JPIM) :: JL1, JL2, JLEV, JGFL
INTEGER(KIND=JPIM) :: JROF
INTEGER(KIND=JPIM) :: ISLB1U9  ,ISLB1V9  ,ISLB1T9  ,ISLB1GFL9, ISLB1VD9

INTEGER(KIND=JPIM) :: ICLPH(NPROMA)          ! cf. KCLPH in APLPAR.

!     --- UPPER AIR PHYSICAL TENDENCIES.
REAL(KIND=JPRB) :: ZTENDH(NPROMM,NFLEVG)     ! Enthalpy tendency.
REAL(KIND=JPRB) :: ZTENDQ(NPROMM,NFLEVG)     ! Moisture tendency.
REAL(KIND=JPRB) :: ZTENDPTKE(NPROMM,NFLEVG)  ! Pseudo progn. TKE

REAL(KIND=JPRB) :: ZTENDGFLR(NPROMM,NFLEVG,YGFL%NUMFLDS)  ! GFL tendencies for APL_AROME (assumes YGFL%NUMFLDS>=NRR)
REAL(KIND=JPRB) :: ZTENDGFL(NPROMM,NFLEVG,YGFL%NUMFLDS)   ! GFL tendencies

REAL(KIND=JPRB) :: ZUDOM(NPROMA,NFLEVG),ZUDAL(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZDDOM(NPROMA,NFLEVG),ZDDAL(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUNEBH(NPROMA,NFLEVG),ZENTCH(NPROMA,NFLEVG)

!     --- UPPER AIR PHYSICAL TENDENCIES FOR AROME.
!       (the previous one are not used in AROME)
REAL(KIND=JPRB) :: ZTENDT (NPROMA,NFLEVG)        ! temperature tendency
REAL(KIND=JPRB) :: ZTENDW (NPROMA,NFLEVG)        ! W  tendency
REAL(KIND=JPRB) :: ZTENDD (NPROMA,NFLEVG)        ! d  tendency
REAL(KIND=JPRB) :: ZTENDEXT(NPROMA,NFLEVG,NGFL_EXT)      ! GFL EXTRA tendency
REAL(KIND=JPRB) :: ZTENDEXT_DEP(NPROMA,NFLEVG,NGFL_EXT)  ! GFL EXTRA tendency
REAL(KIND=JPRB) :: ZDIFEXT(NPROMA,NFLEVG,NGFL_EXT)       ! Extra-GFL fluxes.

!     --- SURFACE AND DEEP RESERVOIR PHYSICAL TENDENCIES.
REAL(KIND=JPRB) :: ZTDTS(NPROMM)             ! Surface temperature tendency.
REAL(KIND=JPRB) :: ZTDTP(NPROMM,YSP_SBD%NLEVS)        ! Deep temperature tendency.
REAL(KIND=JPRB) :: ZTDWS(NPROMM)             ! Surface water res. tendency.
REAL(KIND=JPRB) :: ZTDWP(NPROMM)             ! Deep water res. tendency.
REAL(KIND=JPRB) :: ZTDWL(NPROMM)             ! Interception water res. tendency
REAL(KIND=JPRB) :: ZTDSNS(NPROMM)            ! Snow res. tendency. 
REAL(KIND=JPRB) :: ZTDWPI(NPROMM)            ! Deep ice res. tendency.
REAL(KIND=JPRB) :: ZTDWSI(NPROMM)            ! Surface ice res. tendency.
REAL(KIND=JPRB) :: ZTDALBNS(NPROMA)          ! Snow albedo tendency.
REAL(KIND=JPRB) :: ZTDRHONS(NPROMA)          ! Snow density tendency.

!     --- FLUXES FROM PARAMETERISATIONS AND TENDENCIES COMP.
REAL(KIND=JPRB) :: ZFP(NPROMM,0:NFLEVG)      ! Total rainfall flux.
REAL(KIND=JPRB) :: ZFPLCH(NPROMM,0:NFLEVG)   ! convective precipitation flux,
                                             ! local
REAL(KIND=JPRB) :: ZFPLSH(NPROMM,0:NFLEVG)   ! stratiform precipitation flux,
                                             ! local
REAL(KIND=JPRB) :: ZFTKE(NPROMM,0:NFLEVG)    ! TKE flux.

!     --- Fields of potential use for HIRLAM .
REAL(KIND=JPRB) :: ZCVGQL(NPROMM,NFLEVG)     ! local array for cloud water tendency 
REAL(KIND=JPRB) :: ZCVGQI(NPROMM,NFLEVG)     ! local array for cloud ice tendency
REAL(KIND=JPRB) :: ZCVGT(NPROMM,NFLEVG)      ! local array for temperature tendency

!     --- MISCELLANEOUS PARAMETERISATIONS, 2D ARRAYS ---
REAL(KIND=JPRB) :: ZCVGQ(NPROMM,NFLEVG)      ! convergence of humidity
                                             ! ("Kuo" condition).
REAL(KIND=JPRB) :: ZFHP(NPROMM,0:NFLEVG)     ! Total enthalpy flux
                                             ! + sensible heat flux.
REAL(KIND=JPRB) :: ZLCVQ(NPROMA,NFLEVG)      ! limited physical contribution to
                                             ! the diagnostic of moisture
                                             ! convergence (remove
                                             ! large scale precip. contrib.).
REAL(KIND=JPRB) :: ZLSCPE(NPROMM,NFLEVG)     ! cf. PLSCPE in APLPAR.
REAL(KIND=JPRB) :: ZLH(NPROMM,NFLEVG)        ! cf. PLH in APLPAR.
REAL(KIND=JPRB) :: ZQW(NPROMM,NFLEVG)        ! cf. PQW in APLPAR.
REAL(KIND=JPRB) :: ZTW(NPROMM,NFLEVG)        ! cf. PTW in APLPAR.

REAL(KIND=JPRB) :: ZFRMQ(NPROMM,0:NFLEVG)    ! cf. MESOSPHERIC humidity flux in APLPAR.

!     --- 1D DIAGNOSTIC FIELDS, SURFACE FLUXES
REAL(KIND=JPRB) :: ZCD(NPROMM)               ! cf. PCD in APLPAR.
REAL(KIND=JPRB) :: ZCDN(NPROMM)              ! cf. PCDN in APLPAR.
REAL(KIND=JPRB) :: ZCH(NPROMM)               ! cf. PCH in APLPAR.
REAL(KIND=JPRB) :: ZEMIS(NPROMM)             ! cf. PEMIS in APLPAR.
REAL(KIND=JPRB) :: ZFEVI(NPROMM,NTSSG+1)     ! cf. PFEVI in APLPAR.
REAL(KIND=JPRB) :: ZNEIJ(NPROMM)             ! cf. PNEIJ in APLPAR.
REAL(KIND=JPRB) :: ZVEG(NPROMM)              ! cf. PVEG in APLPAR.
REAL(KIND=JPRB) :: ZQSAT(NPROMA,NFLEVG)      ! specific humidity at saturation.
REAL(KIND=JPRB) :: ZQSATS(NPROMA)            ! cf. PQSATS in APLPAR.
REAL(KIND=JPRB) :: ZQS1(NPROMM)              ! cf. PQS1 in APLPARS.

!     --- DIAGNOSTIC FIELDS STATE OF SURFACE AIR ---
REAL(KIND=JPRB) :: ZC1(NPROMM)               ! cf. PC1 in APLPAR.
REAL(KIND=JPRB) :: ZC2(NPROMM)               ! cf. PC2 in APLPAR.
REAL(KIND=JPRB) :: ZCPS(NPROMM)              ! cf. PCPS in APLPAR.
REAL(KIND=JPRB) :: ZLHS(NPROMM)              ! cf. PLHS in APLPAR.
REAL(KIND=JPRB) :: ZRS(NPROMM)               ! cf. PRS in APLPAR.

!     --- RADIATION COEFFICIENTS FOR SIMPLIFIED PHYSICS IN GRID-POINT ---
REAL(KIND=JPRB) :: ZAC(NPROMM,(NFLEVG+1)*(NFLEVG+1))   ! Curtis matrix.
REAL(KIND=JPRB) :: ZAC_HC(NFLEVG+1,NFLEVG+1)           ! horizontally-constant field for ZAC.
REAL(KIND=JPRB) :: ZRADTC(NPROMM,NFLEVG,YM_RADTC%NDIM) ! others than Curtis matrix.

!     --- GEOMETRY FOR RADIATION ---
REAL(KIND=JPRB) :: ZMMU0(NPROMA)  ! mean solar angle (for given day for
                                  ! simpl. rad. scheme)
REAL(KIND=JPRB) :: ZMU0(NPROMA)   ! local cosine of instantaneous solar zenith
                                  ! angle.
REAL(KIND=JPRB) :: ZMU0LU(NPROMM) ! local cosine of instantaneous lunar zenith
                                  ! angle.
REAL(KIND=JPRB) :: ZMU0M(NPROMA)  ! local cosine of averaged solar zenith angle
REAL(KIND=JPRB) :: ZMU0N(NPROMA)  ! same as ZMU0 for next time step (used for LMSE)    

!     ---FOR AROME PHYSICS  ---
REAL(KIND=JPRB) :: ZDT   !pour cputqy_arome, a changer peut etre plus tard...
REAL(KIND=JPRB) :: ZGWT1(NPROMA,0:NFLEVG)     ! vertical velocity calculated by 
                                              ! cputqy_arome before convertion in d
REAL(KIND=JPRB) :: ZTT1(NPROMA,NFLEVG)       ! Temperature at t1 for gnhgw2svd

!     --- Buffers to save the initial value of   ---
!     --- some pseudo-historical surface buffers ---
REAL(KIND=JPRB) :: ZHV(NPROMM)
REAL(KIND=JPRB) :: ZGZ0F(NPROMM)
REAL(KIND=JPRB) :: ZGZ0HF(NPROMM)
REAL(KIND=JPRB) :: ZPBLH(NPROMM)
REAL(KIND=JPRB) :: ZFHPS(NPROMM)
REAL(KIND=JPRB) :: ZQSH (NPROMM)

!  Horizontal exchange coefficients for 3D turbulence
REAL(KIND=JPRB) :: ZKUROV_H(NPROMA,NFLEVG),ZKTROV_H(NPROMA,NFLEVG)

!  Empty arrays for 3TL scheme (to be coded later)
!!later REAL(KIND=JPRB) :: ZDIVT9(NPROMA,NFLEVG)
!!later REAL(KIND=JPRB) :: ZUT9L(NPROMA,NFLEVG)
!!later REAL(KIND=JPRB) :: ZVT9L(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZWT9L(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZWT9M(NPROMA,NFLEVG)

REAL(KIND=JPRB) :: ZEZDIAG(NPROMA,NFLEVG,NGFL_EZDIAG)
REAL(KIND=JPRB) :: ZEXT(NPROMA,NFLEVG,NGFL_EXT)
REAL(KIND=JPRB) :: ZTAUX(NPROMA,NFLEVG), ZDTAJU(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "apl_arome.intfb.h"
#include "aplpar.intfb.h"
#include "aplpars.intfb.h"
#include "aplpassh.intfb.h"
#include "cpchet.intfb.h"
#include "cpmvvps.intfb.h"
#include "cpnudg.intfb.h"
#include "cpozo.intfb.h"
#include "cppfttcdir.intfb.h"
#include "cppfttcinv.intfb.h"
#include "cpphinp.intfb.h"
#include "cppsolan.intfb.h"
#include "cpqsol.intfb.h"
#include "cptend_new.intfb.h"
#include "cptends.intfb.h"
#include "cptendsm.intfb.h"
#include "cputqy_arome.intfb.h"
#include "cputqy.intfb.h"
#include "cputqys.intfb.h"
#include "cpwts.intfb.h"
#include "cp_ptrslb1.intfb.h"
#include "gnhgw2svdarome.intfb.h"
#include "gpinither.intfb.h"
#include "initaplpar.intfb.h"
#include "profilechet.intfb.h"
#include "rdradcoef.intfb.h"
#include "writephysio.intfb.h"
#include "wrphtrajm.intfb.h"
#include "wrphtrsf.intfb.h"
#include "wrradcoef.intfb.h"
#include "acajucv.intfb.h"
#include "open_output_lfa.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('MF_PHYS',0,ZHOOK_HANDLE)
IF (LMUSCLFA) CALL OPEN_OUTPUT_LFA

!     ------------------------------------------------------------------

!        1.    Preliminary calculations necessary
!              for all types of physics.
!              ------------------------------------

IF (LSLAG) CALL CP_PTRSLB1(ISLB1U9,ISLB1V9,ISLB1T9,ISLB1VD9,ISLB1GFL9)

IBLK=(KSTGLO-1)/NPROMA+1
LL_CALL_CPPFTTCDIR=LDWRTCFOU0
IF (LL_CALL_CPPFTTCDIR) THEN
  INSTEP_DEB=1
  INSTEP_FIN=NSTOP-1
ELSE
  INSTEP_DEB=1
  INSTEP_FIN=1
ENDIF

ZEXT(:,:,:)=0.0_JPRB

! initialisation for surfex if XFU
LLXFUMSE=.FALSE.
IF (CDCONF == 'X') THEN
  LLXFUMSE=.TRUE.
ENDIF

! Complete physics is called.
LLDIAB=(LMPHYS.OR.LEPHYS).AND.(.NOT.LAGPHY)

IF (LRAYSP.AND.(NSTEP >= INSTEP_DEB .AND. NSTEP <= INSTEP_FIN) &
   & .AND.(.NOT.LDLADNMI)) THEN  
  CALL CPPSOLAN(KST,KEND,YDGSGEOM%GEMU,YDGSGEOM%GELAM,ZMMU0)
  IF (.NOT.LSDDH) THEN
    CALL GPINITHER(KST,KEND,PDHSF,KSTGLO)
  ENDIF
ENDIF

IF ((LLDIAB.OR.LSIMPH).AND.(.NOT.LDLADNMI)) THEN
  CALL CPPHINP(KST,KEND,&
   & YDGSGEOM%GEMU,YDGSGEOM%GELAM,&
   & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),&
   & PGFL(1,1,YQ%MP),PGFL(1,1,YQ%MPL),PGFL(1,1,YQ%MPM),PGFL(1,1,YCVGQ%MPL),PGFL(1,1,YCVGQ%MPM),&
   & PXYB0(1,1,YYTXYB0_PHY%M_RDELP),PCTY0(1,0,YYTCTY0%M_EVEL),PGFL(1,1,YCVGQ%MP),&
   & ZMU0,ZMU0LU,ZMU0M,ZMU0N,ZCVGQ)  
  ZLCVQ(KST:KEND,1:NFLEVG)=ZCVGQ(KST:KEND,1:NFLEVG)
ENDIF

DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    ZCVGQL(JROF,JLEV)=0._JPRB
    ZCVGQI(JROF,JLEV)=0._JPRB
    ZCVGT (JROF,JLEV)=0._JPRB
  ENDDO
ENDDO

DO JROF=KST,KEND
  ZQSATS(JROF)=0.0_JPRB
  ZFPLCH(JROF,0)=0.0_JPRB
  ZFPLSH(JROF,0)=0.0_JPRB
ENDDO

IF(LNEBN.OR.LNEBR.OR.LRRGUST) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      ZFPLCH(JROF,JLEV)=PGFL(JROF,JLEV,YCPF%MP)
    ENDDO
  ENDDO
ENDIF
IF(LRRGUST) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      ZFPLSH(JROF,JLEV)=PGFL(JROF,JLEV,YSPF%MP)
    ENDDO
  ENDDO
ENDIF

! * In some cases, some pseudo-historic surface buffers (like z0) should
!   not be modified between the entrance and the output of MF_PHYS
!   (this is the case for example for CDCONF='X').
!   For the time being, we must save:
!   - HV: resistance to evapotranspiration
!   - Z0F: gravity * surface roughness length
!   - Z0H: gravity * roughness length for heat
!   - PBLH: PBL height
!   - SPSH: ??? (for ALARO)
!   Value of the keys LLACTIVE_... should be consistent with the tests done
!   in su_surf_flds.F90 (in the future it would be more convenient to add
!   an attribute LACTIVE in the definition of types TYPE_SURF_PTR_3D
!   and TYPE_SURF_PTR_2D).
LL_SAVE_PHSURF=LLDIAB.AND.(.NOT.LDLADNMI).AND.(CDCONF == 'X')
LLACTIVE_HV=LMPHYS.AND.LSOLV
LLACTIVE_Z0F=LMPHYS.AND.(.NOT.(LMSE.OR.LAROME))
LLACTIVE_Z0H=LMPHYS.AND.LSOLV
LLACTIVE_PBLH=LMPHYS.AND.(LNEBR.OR.(TRIM(CGMIXLEN) == 'TM'))
LLACTIVE_SPSH=LMPHYS.AND.LPHSPSH
LLACTIVE_QSH=LMPHYS.AND.LCOEFKTKE
IF (LL_SAVE_PHSURF) THEN
  IF(LLACTIVE_HV) ZHV(1:NPROMA)=PSD_VV(1:NPROMA,YSD_VV%YHV%MP)
  IF(LLACTIVE_Z0F) ZGZ0F(1:NPROMA)=PSD_VF(1:NPROMA,YSD_VF%YZ0F%MP)
  IF(LLACTIVE_Z0H) ZGZ0HF(1:NPROMA)=PSD_VV(1:NPROMA,YSD_VV%YZ0H%MP)
  IF(LLACTIVE_PBLH) ZPBLH(1:NPROMA)=PSD_VH(1:NPROMA,YSD_VH%YPBLH%MP)
  IF(LLACTIVE_SPSH) ZFHPS(1:NPROMA)=PSD_VH(1:NPROMA,YSD_VH%YSPSH%MP)
  IF(LLACTIVE_QSH) ZQSH(1:NPROMA)=PSD_VH(1:NPROMA,YSD_VH%YQSH%MP)
  IF(LCVPRO) THEN
    ZUDAL(:,:)=PGFL(:,:,YUAL%MP)
    ZUDOM(:,:)=PGFL(:,:,YUOM%MP)
    IF(LCDDPRO) THEN
      ZDDAL(:,:)=PGFL(:,:,YDAL%MP)
      ZDDOM(:,:)=PGFL(:,:,YDOM%MP)
    ENDIF
  ENDIF
  IF(YUNEBH%LACTIVE) ZUNEBH(:,:)=PGFL(:,:,YUNEBH%MP)
  IF(YUEN%LACTIVE)   ZENTCH(:,:)=PGFL(:,:,YUEN%MP)
ENDIF

CALL INITAPLPAR ( KST, KEND, NPROMA, NFLEVG, NTSSG, YSP_SBD%NLEVS,&
  & PSD_VF(1,YSD_VF%YVEG%MP  ) ,&
  & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS , ZDIFEXT, PDIFTQ , PDIFTQI, PDIFTQL,&
  & PDIFTS , PFCCQL , PFCCQN , PFCSQL , PFCSQN , PFCQNG , PFCQING,&
  & PFCQLNG, PFCQRNG, PFCQSNG, &
  & PFPLCL , PFPLCN , PFPLCG , PFPLCHL, PFPLSL , PFPLSN ,PFPLSG ,&
  & PFPLSHL, PFRSO  , PFRSOC ,&
  & PFRTH  , PFRTHC , PSTRCU , PSTRCV , PSTRDU , PSTRDV , PSTRTU ,&
  & PSTRTV , PSTRMU , PSTRMV , PFRMH  , ZFRMQ  , &
  & PDIFCQLC,PDIFCQIC,PFIMCC, &
  & PFEDQLC, PFEDQIC, PFEDQRC, PFEDQSC, PFCNEGQLC,PFCNEGQIC,PFCNEGQRC,PFCNEGQSC, &
  & PFCHOZ , ZCPS   , ZLHS   ,&
  & ZRS    , ZLH    , ZLSCPE , PNEB   , PQICE  , PQLI   , ZQSAT  ,&
  & ZQW    , PRH    , ZTW    , PALB   , ZCD    , ZCDN   , ZCH    ,&
  & ZC1    , ZC2    , PCT    , ZEMIS  , PFCHSP , PFCLL  , PFCLN  ,&
  & PFCS   , ZFEVI  , PFEVL  , PFEVN  , PFEVV  , PFTR   , PFLWSP ,&
  & PFONTE , &
  & PFGEL  , PFGELS , PFRSODS, PFRSOPS, PFRSOPT, PFRSOLU, PFRTHDS,&
  & PFPFPSL, PFPFPSN, PFPFPCL, PFPFPCN, PFPEVPSL,PFPEVPSN,&
  & PFPEVPCL, PFPEVPCN, ZFTKE, PGZ0   , PGZ0H  , ZNEIJ  , ZVEG   ,PQS,&
  & ZQSATS , PRUISL , PRUISP , PRUISS , PUCLS  , PVCLS  , PTCLS  ,&
  & PQCLS  , PRHCLS , PCLCT  , PCLCH  , PCLCM  , PCLCL  , PCLCC  ,&
  & PCAPE  , ICLPH  , PCLPH  , PVEIN  , PUGST  , PVGST  , PDIAGH)
!*       2.    Complete physics.
!              -----------------

!        2.2  Complete physics.
!             -----------------

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.(.NOT.LMPA)) THEN

  ! PAS DE TEMPS DE LA PHYSIQUE (/YOMPHY2/)
  ! Dans le cas des iterations de l'initialisation par modes normaux,
  ! le pas de temps pour la physique ne peux pas etre nul pour APLPAR et
  ! CPATY (par contre c'est bien PDTPHY qui est passe en argument aux autres
  ! sous-prog. de la physique). Ceci est du a l'impossibilite de prendre en
  ! compte des flux qui deviennent infinis pour TSPHY=0 (flux de masse du au
  ! reajustement des sursaturations par exemple...). Mais les tendances phys.
  ! sont bien nulles dans le cas de la configuration 'E' (Modes Normaux).
  ! PHYSICS TIME STEP (/YOMPHY2/)
  ! In case of normal mode initialisation iterations, the physics time
  ! step cannot be zero for APLPAR and CPATY (nevertheless it is PDTPHY
  ! which is passed to other physics subroutines). This is due to the
  ! impossibility to take into account fluxes which are infinite for TSPHY=0
  ! (e.g.: mass flux due to oversaturation...). But the physical tendencies
  ! are indeed zero in configuration 'E' (NMI).

  TSPHY = MAX(PDTPHY,1.0_JPRB)

  ! CALL PARAMETERISATIONS

  DO JROF=KST,KEND
    PSD_VF(JROF,YSD_VF%YLSM%MP)=REAL(NINT(PSD_VF(JROF,YSD_VF%YLSM%MP)),JPRB)
  ENDDO
  
    IF (LTWOTL) THEN

      IF (LAJUCV) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KEND
            ZTAUX(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MT)
          ENDDO
        ENDDO
        CALL ACAJUCV(KST,KEND,NPROMA,NTPLUI,NFLEVG,NTAJUC, &
         & PRE0,PXYB0(1,1,YYTXYB0_PHY%M_ALPH),PXYB0(1,1,YYTXYB0_PHY%M_DELP), &
         & PXYB0(1,1,YYTXYB0_PHY%M_LNPR),PGMV(1,1,YT0%MT))
        DO JLEV=1,NFLEVG
          DO JROF=KST,KEND
            ZDTAJU(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MT)-ZTAUX(JROF,JLEV)
          ENDDO
        ENDDO
      ENDIF

      ITDIA=1_JPIM
      CALL APLPAR ( KST    , KEND   , NPROMA ,&
       & ITDIA  , NFLEVG  , KSTGLO,&
       & NVCLIS , YSD_VVD%NUMFLDS ,&
       & NSTEP  ,&
       & NTSSG  , YSP_SBD%NLEVS   ,&
       & KBL    , KGPCOMP, PDTPHY,YDCSGEOM%RINDX,YDCSGEOM%RINDY, LLXFUMSE,&
       & PHI0  , PRE0  , PHIF0 , PRE0F ,PXYB0(1,1,YYTXYB0_PHY%M_ALPH),PSD_VV(1,YSD_VV%YARG%MP),&
       & PSD_VV(1,YSD_VV%YD2%MP)    ,&
       & PXYB0(1,1,YYTXYB0_PHY%M_DELP),PSD_VV(1,YSD_VV%YIVEG%MP),PSD_VV(1,YSD_VV%YLAI%MP),&
       & PXYB0(1,1,YYTXYB0_PHY%M_LNPR),PXYB0(1,1,YYTXYB0_PHY%M_RDELP),&
       & PSD_VV(1,YSD_VV%YRSMIN%MP) , PSD_VV(1,YSD_VV%YSAB%MP  ) ,&
       & PSD_VV(1,YSD_VV%YZ0H%MP  ) , PSD_VA(1,YSD_VA%YSEA%MP  ) , PSD_VA(1,YSD_VA%YLAN%MP ) ,&
       & PSD_VA(1,YSD_VA%YSOO%MP ) , PSD_VA(1,YSD_VA%YDES%MP ) , PSD_VA(1,YSD_VA%YSUL%MP ) ,&
       & PSD_VA(1,YSD_VA%YVOL%MP ) ,YDGSGEOM%RCORI, ZEXT,&
       & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),&
       & PGFL(1,1,YQ%MP),PGFL(1,1,YI%MP),PGFL(1,1,YL%MP),&
       & PGFL(1,1,YLCONV%MP),PGFL(1,1,YICONV%MP),PGFL(1,1,YRCONV%MP),PGFL(1,1,YSCONV%MP),&
       & PGFL(1,1,YS%MP),PGFL(1,1,YR%MP),PGFL(1,1,YTKE%MP),PGFL(1,1,YCVV%MP),&
       & PGFL(1,1,YO3%MP),&
       & PGMV(1,1,YT0%MVOR),&
       & PRCP0(1,1,YYTRCP0%M_CP), ZCVGQ  ,PRCP0(1,1,YYTRCP0%M_R), PKOZO  , ZFPLCH , ZFPLSH ,&
       & PGFL(1,1,YDAL%MP),PGFL(1,1,YDOM%MP),PGFL(1,1,YUEN%MP),PGFL(1,1,YUAL%MP),&
       & PGFL(1,1,YUOM%MP),PGFL(1,1,YUNEBH%MP), PCTY0(1,0,YYTCTY0%M_EVEL), &
       & PGFL(1,1,YRKTH%MP),PGFL(1,1,YRKTQV%MP),PGFL(1,1,YRKTQC%MP), PGFL(1,1,YTTE%MP),&
       & PGFL(1,1,YMXL%MP),PGFL(1,1,YEFB1%MP),PGFL(1,1,YEFB2%MP),PGFL(1,1,YEFB3%MP),&
       & PSD_VH(1,YSD_VH%YTCCH%MP  ) ,  PSD_VH(1,YSD_VH%YSCCH%MP  ) , PSD_VH(1,YSD_VH%YBCCH%MP  ) ,&
       & PSD_VH(1,YSD_VH%YSPSH%MP  ) ,PSD_VH(1,YSD_VH%YPBLH%MP  ) , &
       & PSD_VH(1,YSD_VH%YQSH%MP   ) , &
       & PGPAR , PCUCONVCA, PNLCONVCA, &
       & PSP_SG(1,YSP_SG%YF%MP0) , PSP_SG(1,YSP_SG%YA%MP0), PSP_SG(1,YSP_SG%YR%MP0),&
       & PSP_SB(1,1,YSP_SB%YT%MP0 ) , PSP_RR(1,YSP_RR%YT%MP0 ) , PSP_RR(1,YSP_RR%YFC%MP0) ,&
       & PSP_SB(1,1,YSP_SB%YQ%MP0 ) , PSP_SB(1,1,YSP_SB%YTL%MP0), PSP_RR(1,YSP_RR%YW%MP0)  ,&
       & PSP_RR(1,YSP_RR%YIC%MP0),&
       & PSD_VV(1,YSD_VV%YHV%MP)    , PCTY0(1,1,YYTCTY0%M_VVEL),&
       & PEMTD , PEMTU ,PTRSW ,&
       & PSD_VV(1,YSD_VV%YALV%MP )  ,&
       & PSD_VF(1,YSD_VF%YALBF%MP ) , PSD_VF(1,YSD_VF%YALBSF%MP) , PSD_VF(1,YSD_VF%YEMISF%MP) ,&
       & PSD_VF(1,YSD_VF%YGETRL%MP) , PSD_VF(1,YSD_VF%YLSM%MP  ) , PSD_VF(1,YSD_VF%YVEG%MP  ) ,&
       & PSD_VF(1,YSD_VF%YZ0F%MP  ) , PSD_VF(1,YSD_VF%YZ0RLF%MP ),&
       & PSD_VF(1,YSD_VF%YVRLAN%MP) , PSD_VF(1,YSD_VF%YVRLDI%MP) , PSD_VC(1,YSD_VC%YVC(1)%MP)  ,&
       & PSD_SFL(1,YSD_SFL%YSFL(1)%MP) , PSD_SFO(1,YSD_SFO%YSFO(1)%MP) ,&
       & PRMOON ,&
       & ZMU0   , ZMU0LU , ZMU0M  ,ZMU0N,YDGSGEOM%GELAM,YDGSGEOM%GEMU,YDGSGEOM%GM,&
       & ZAC    , ZAC_HC , ZRADTC(1,1,YM_RADTC%MCOR) , ZMMU0  , PDHSF  ,&
       & ZRADTC(1,1,YM_RADTC%MRAB3C),ZRADTC(1,1,YM_RADTC%MRAB3N), &
       & ZRADTC(1,1,YM_RADTC%MRAB4C),ZRADTC(1,1,YM_RADTC%MRAB4N), &
       & ZRADTC(1,1,YM_RADTC%MRAB6C),ZRADTC(1,1,YM_RADTC%MRAB6N), &
       & ZRADTC(1,1,YM_RADTC%MRAT1C),ZRADTC(1,1,YM_RADTC%MRAT1N), &
       & ZRADTC(1,1,YM_RADTC%MRAT2C),ZRADTC(1,1,YM_RADTC%MRAT2N), &
       & ZRADTC(1,1,YM_RADTC%MRAT3C),ZRADTC(1,1,YM_RADTC%MRAT3N), &
       & ZRADTC(1,1,YM_RADTC%MRAT4C),ZRADTC(1,1,YM_RADTC%MRAT4N), &
       & ZRADTC(1,1,YM_RADTC%MRAT5C),ZRADTC(1,1,YM_RADTC%MRAT5N), &
       & YDOROG%OROG,&
       & PWT0,PGMV(1,1,YT0%MDIV),PGMV(1,1,YT0%MUL),PGMV(1,1,YT0%MVL),PWT0L,PWT0M, &
       & PGDEOSI,PGUEOSI,PGMU0,PGMU0_MIN,PGMU0_MAX, &
       & PGDEOTI,PGDEOTI2,PGUEOTI,PGUEOTI2,PGEOLT,PGEOXT, &
       & PGRPROX,PGMIXP,PGFLUXC,PGRSURF, &
       & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS , ZDIFEXT, PDIFTQ , PDIFTQI,&
       & PDIFTQL,&
       & PDIFTS , PFCCQL , PFCCQN , PFCSQL , PFCSQN , PFCQNG ,&
       & PFCQING, PFCQLNG, PFCQRNG, PFCQSNG,&
       & PFPLCL , PFPLCN , PFPLSL , PFPLSN , PFRSO  ,&
       & PFRSOC ,&
       & PFRTH  , PFRTHC , PSTRCU , PSTRCV , PSTRDU , PSTRDV ,&
       & PSTRTU ,&
       & PSTRTV , PSTRMU , PSTRMV , PFRMH  , ZFRMQ  ,&
       & PDIFCQLC,PDIFCQIC,PFIMCC, &
       & PFEDQLC, PFEDQIC, PFEDQRC, PFEDQSC, PFCNEGQLC,PFCNEGQIC,PFCNEGQRC,PFCNEGQSC, &
       & PFCHOZ ,&
       & ZCPS   , ZLHS   ,&
       & ZRS    , ZLH    , ZLSCPE , PNEB   , PQICE  , PQLI   ,&
       & PGFLT1(1,1,YA%MP1),PGFLT1(1,1,YLRAD%MP1),PGFLT1(1,1,YIRAD%MP1), &
       & ZQSAT  ,&
       & ZQW    , PRH    , ZTW    , PALB   , ZCD    , ZCDN   ,&
       & ZCH    ,&
       & ZC1    , ZC2    , PCT    , ZEMIS  , PFCHSP , PFCLL  ,&
       & PFCLN  ,&
       & PFCS   , ZFEVI  , PFEVL  , PFEVN  , PFEVV  , PFTR   ,&
       & PFLWSP , PFONTE , PSP_SG(1,YSP_SG%YT%MP1), PFGEL  , PFGELS ,&
       & PFRSODS, PFRSOPS, PFRSOPT, PFRSOLU, PFRTHDS, &
       & PFPFPSL, PFPFPSN, PFPFPCL, PFPFPCN, PFPEVPSL,PFPEVPSN,PFPEVPCL,&
       & PFPEVPCN,ZFTKE  , PGZ0   , PGZ0H  , ZNEIJ  , ZVEG   , PQS,&
       & ZQSATS , PRUISL , PRUISP , PRUISS , PUCLS  , PVCLS  ,&
       & PTCLS  ,&
       & PQCLS  , PRHCLS , PCLCT  , PCLCH  , PCLCM  , PCLCL  ,&
       & PCLCC  , PCAPE  , ICLPH  , PCLPH  , PVEIN  , PUGST  , PVGST  ,&
       & ZEZDIAG, PSP_SG(1,YSP_SG%YF%MP1), PDIAGH, ZTENDPTKE, ZKUROV_H,&
       & ZKTROV_H,PDERNSHF, ZTENDEXT_DEP, PSD_VD(1,YSD_VD%YSUND%MP))

      IF (LAJUCV) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KEND
            PGMV(JROF,JLEV,YT0%MT)=ZTAUX(JROF,JLEV)
          ENDDO
        ENDDO
        DO JLEV=1,NFLEVG
          DO JROF=KST,KEND
            PB1(JROF,ISLB1T9+JLEV-NFLSA)=PB1(JROF,ISLB1T9+JLEV-NFLSA)+ZDTAJU(JROF,JLEV)
          ENDDO
        ENDDO
      ENDIF

    ELSE

      ! IF (LAJUCV) THEN
      !   missing code under LAJUCV for leap-frog schemes.
      ! ENDIF

      ITDIA=1_JPIM
      CALL APLPAR ( KST    , KEND   , NPROMA ,&
       & ITDIA  , NFLEVG  , KSTGLO ,&
       & NVCLIS , YSD_VVD%NUMFLDS ,&
       & NSTEP  ,&
       & NTSSG  , YSP_SBD%NLEVS   ,&
       & KBL    , KGPCOMP, PDTPHY,YDCSGEOM%RINDX,YDCSGEOM%RINDY, LLXFUMSE,&
       & PHI9  , PRE9  , PHIF9 , PRE9F , PXYB9(1,1,YYTXYB9_PHY%M_ALPH) , PSD_VV(1,YSD_VV%YARG%MP)   ,&
       & PSD_VV(1,YSD_VV%YD2%MP)    ,&
       & PXYB9(1,1,YYTXYB9_PHY%M_DELP),PSD_VV(1,YSD_VV%YIVEG%MP),PSD_VV(1,YSD_VV%YLAI%MP),&
       & PXYB9(1,1,YYTXYB9_PHY%M_LNPR),PXYB9(1,1,YYTXYB9_PHY%M_RDELP),&
       & PSD_VV(1,YSD_VV%YRSMIN%MP) , PSD_VV(1,YSD_VV%YSAB%MP  ) ,&
       & PSD_VV(1,YSD_VV%YZ0H%MP  ) , PSD_VA(1,YSD_VA%YSEA%MP ) , PSD_VA(1,YSD_VA%YLAN%MP ) ,&
       & PSD_VA(1,YSD_VA%YSOO%MP ) , PSD_VA(1,YSD_VA%YDES%MP ) , PSD_VA(1,YSD_VA%YSUL%MP ) ,&
       & PSD_VA(1,YSD_VA%YVOL%MP ),YDGSGEOM%RCORI, ZEXT,&
       & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),&
       & PGFL(1,1,YQ%MP9),PGFL(1,1,YI%MP9),PGFL(1,1,YL%MP9),&
       & PGFL(1,1,YLCONV%MP),PGFL(1,1,YICONV%MP),PGFL(1,1,YRCONV%MP),PGFL(1,1,YSCONV%MP),&
       & PGFL(1,1,YS%MP9),PGFL(1,1,YR%MP9),PGFL(1,1,YTKE%MP9),PGFL(1,1,YCVV%MP9),&
       & PGFL(1,1,YO3%MP9),&
       & PGMV(1,1,YT9%MVOR),&
       & PRCP9(1,1,YYTRCP9%M_CP), ZCVGQ  ,PRCP9(1,1,YYTRCP9%M_R), PKOZO  , ZFPLCH, ZFPLSH  ,&
       & PGFL(1,1,YDAL%MP),PGFL(1,1,YDOM%MP),PGFL(1,1,YUEN%MP),PGFL(1,1,YUAL%MP),&
       & PGFL(1,1,YUOM%MP),PGFL(1,1,YUNEBH%MP), PCTY0(1,0,YYTCTY0%M_EVEL), &
       & PGFL(1,1,YRKTH%MP),PGFL(1,1,YRKTQV%MP),PGFL(1,1,YRKTQC%MP), PGFL(1,1,YTTE%MP),&
       & PGFL(1,1,YMXL%MP),PGFL(1,1,YEFB1%MP),PGFL(1,1,YEFB2%MP),PGFL(1,1,YEFB3%MP),&
       & PSD_VH(1,YSD_VH%YTCCH%MP  ) ,  PSD_VH(1,YSD_VH%YSCCH%MP  ) , PSD_VH(1,YSD_VH%YBCCH%MP  ) ,&
       & PSD_VH(1,YSD_VH%YSPSH%MP  ) , PSD_VH(1,YSD_VH%YPBLH%MP  ) , &
       & PSD_VH(1,YSD_VH%YQSH%MP   ) , PGPAR, PCUCONVCA, PNLCONVCA, &
       & PSP_SG(1,YSP_SG%YF%MP9) , PSP_SG(1,YSP_SG%YA%MP9), PSP_SG(1,YSP_SG%YR%MP9),&
       & PSP_SB(1,1,YSP_SB%YT%MP9 ) , PSP_RR(1,YSP_RR%YT%MP9 ) , PSP_RR(1,YSP_RR%YFC%MP9) ,&
       & PSP_SB(1,1,YSP_SB%YQ%MP9 ) , PSP_SB(1,1,YSP_SB%YTL%MP9), PSP_RR(1,YSP_RR%YW%MP9 )  ,&
       & PSP_RR(1,YSP_RR%YIC%MP9),&
       & PSD_VV(1,YSD_VV%YHV%MP)    , PCTY0(1,1,YYTCTY0%M_VVEL), &
       & PEMTD , PEMTU ,PTRSW ,&
       & PSD_VV(1,YSD_VV%YALV%MP )  ,&
       & PSD_VF(1,YSD_VF%YALBF%MP ) , PSD_VF(1,YSD_VF%YALBSF%MP) , PSD_VF(1,YSD_VF%YEMISF%MP) ,&
       & PSD_VF(1,YSD_VF%YGETRL%MP) , PSD_VF(1,YSD_VF%YLSM%MP  ) , PSD_VF(1,YSD_VF%YVEG%MP  ) ,&
       & PSD_VF(1,YSD_VF%YZ0F%MP  ) , PSD_VF(1,YSD_VF%YZ0RLF%MP ),&
       & PSD_VF(1,YSD_VF%YVRLAN%MP) , PSD_VF(1,YSD_VF%YVRLDI%MP) , PSD_VC(1,YSD_VC%YVC(1)%MP)  ,&
       & PSD_SFL(1,YSD_SFL%YSFL(1)%MP) , PSD_SFO(1,YSD_SFO%YSFO(1)%MP) , PRMOON ,&
       & ZMU0   , ZMU0LU , ZMU0M  , ZMU0N,YDGSGEOM%GELAM,YDGSGEOM%GEMU,YDGSGEOM%GM,&
       & ZAC    , ZAC_HC , ZRADTC(1,1,YM_RADTC%MCOR) , ZMMU0  , PDHSF  ,&
       & ZRADTC(1,1,YM_RADTC%MRAB3C),ZRADTC(1,1,YM_RADTC%MRAB3N), &
       & ZRADTC(1,1,YM_RADTC%MRAB4C),ZRADTC(1,1,YM_RADTC%MRAB4N), &
       & ZRADTC(1,1,YM_RADTC%MRAB6C),ZRADTC(1,1,YM_RADTC%MRAB6N), &
       & ZRADTC(1,1,YM_RADTC%MRAT1C),ZRADTC(1,1,YM_RADTC%MRAT1N), &
       & ZRADTC(1,1,YM_RADTC%MRAT2C),ZRADTC(1,1,YM_RADTC%MRAT2N), &
       & ZRADTC(1,1,YM_RADTC%MRAT3C),ZRADTC(1,1,YM_RADTC%MRAT3N), &
       & ZRADTC(1,1,YM_RADTC%MRAT4C),ZRADTC(1,1,YM_RADTC%MRAT4N), &
       & ZRADTC(1,1,YM_RADTC%MRAT5C),ZRADTC(1,1,YM_RADTC%MRAT5N), &
       & YDOROG%OROG,&
       & PWT9,PGMV(1,1,YT9%MDIV),PGMV(1,1,YT9%MUL),PGMV(1,1,YT9%MVL),ZWT9L,ZWT9M, &
       & PGDEOSI,PGUEOSI,PGMU0,PGMU0_MIN,PGMU0_MAX, &
       & PGDEOTI,PGDEOTI2,PGUEOTI,PGUEOTI2,PGEOLT,PGEOXT, &
       & PGRPROX,PGMIXP,PGFLUXC,PGRSURF, &
       & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS , ZDIFEXT, PDIFTQ , PDIFTQI,&
       & PDIFTQL,&
       & PDIFTS , PFCCQL , PFCCQN , PFCSQL , PFCSQN , PFCQNG ,&
       & PFCQING, PFCQLNG, PFCQRNG, PFCQSNG, &
       & PFPLCL , PFPLCN , PFPLSL , PFPLSN , PFRSO  ,&
       & PFRSOC ,&
       & PFRTH  , PFRTHC , PSTRCU , PSTRCV , PSTRDU , PSTRDV ,&
       & PSTRTU ,&
       & PSTRTV , PSTRMU , PSTRMV , PFRMH  , ZFRMQ  ,&
       & PDIFCQLC,PDIFCQIC,PFIMCC, &
       & PFEDQLC, PFEDQIC, PFEDQRC, PFEDQSC, PFCNEGQLC,PFCNEGQIC,PFCNEGQRC,PFCNEGQSC, &
       & PFCHOZ ,&
       & ZCPS   , ZLHS   ,&
       & ZRS    , ZLH    , ZLSCPE , PNEB   , PQICE  , PQLI   ,&
       & PGFLT1(1,1,YA%MP1),PGFLT1(1,1,YLRAD%MP1),PGFLT1(1,1,YIRAD%MP1), &
       & ZQSAT  ,&
       & ZQW    , PRH    , ZTW    , PALB   , ZCD    , ZCDN   ,&
       & ZCH    ,&
       & ZC1    , ZC2    , PCT    , ZEMIS  , PFCHSP , PFCLL  ,&
       & PFCLN  ,&
       & PFCS   , ZFEVI  , PFEVL  , PFEVN  , PFEVV  , PFTR   ,&
       & PFLWSP , PFONTE , PSP_SG(1,YSP_SG%YT%MP1), PFGEL  , PFGELS ,&
       & PFRSODS, PFRSOPS, PFRSOPT, PFRSOLU, PFRTHDS, &
       & PFPFPSL, PFPFPSN, PFPFPCL, PFPFPCN, PFPEVPSL,PFPEVPSN,PFPEVPCL,&
       & PFPEVPCN,ZFTKE  , PGZ0   , PGZ0H  , ZNEIJ  , ZVEG   , PQS    ,&
       & ZQSATS , PRUISL , PRUISP , PRUISS , PUCLS  , PVCLS  ,&
       & PTCLS  ,&
       & PQCLS  , PRHCLS , PCLCT  , PCLCH  , PCLCM  , PCLCL  ,&
       & PCLCC  , PCAPE  , ICLPH  , PCLPH  , PVEIN  , PUGST  , PVGST  ,&
       & ZEZDIAG, PSP_SG(1,YSP_SG%YF%MP1), PDIAGH, ZTENDPTKE, ZKUROV_H,&
       & ZKTROV_H,PDERNSHF, ZTENDEXT_DEP, PSD_VD(1,YSD_VD%YSUND%MP))

      ! IF (LAJUCV) THEN
      !   missing code under LAJUCV for leap-frog schemes.
      ! ENDIF

    ENDIF

ENDIF

!        2.3  Computes MOCON in the CLP.
!             --------------------------

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.(.NOT.LMPA)) THEN

  PMOCON(KST:KEND)=0.0_JPRB
  IF (LTWOTL) THEN
    DO JLEV=1,NFLEVG
      DO JROF = KST, KEND
        PMOCON(JROF) = PMOCON(JROF)+(ZLCVQ(JROF,JLEV)-PGFL(JROF,JLEV,YQ%MP)*&
         & PGMV(JROF,JLEV,YT0%MDIV))*PXYB0(JROF,JLEV,YYTXYB0_PHY%M_DELP)&
         & *MAX(0,SIGN(1,JLEV-ICLPH(JROF)))  
      ENDDO
    ENDDO
    DO JROF = KST, KEND
      PMOCON(JROF) = PMOCON(JROF)/(PRE0(JROF,NFLEVG)-PRE0(JROF,ICLPH(JROF)-1))
    ENDDO
  ELSE
    DO JLEV=1,NFLEVG
      DO JROF = KST, KEND
        PMOCON(JROF) = PMOCON(JROF)+(ZLCVQ(JROF,JLEV)-PGFL(JROF,JLEV,YQ%MP9)*&
         & PGMV(JROF,JLEV,YT0%MDIV))*PXYB9(JROF,JLEV,YYTXYB9_PHY%M_DELP)&
         & *MAX(0,SIGN(1,JLEV-ICLPH(JROF)))  
      ENDDO
    ENDDO
    DO JROF = KST, KEND
      PMOCON(JROF) = PMOCON(JROF)/(PRE9(JROF,NFLEVG)-PRE9(JROF,ICLPH(JROF)-1))
    ENDDO
  ENDIF
ENDIF

!        2.4  Stores radiation coefficients.
!             ------------------------------

! * transforms grid-point into Fourier transmission coefficients
!   for simplified physics.

IF (LL_CALL_CPPFTTCDIR) THEN
  CALL CPPFTTCDIR(KGL3,KGL4,KOFF,RBASCOTC(1,KOFF),RBASSITC(1,KOFF),&
   & ZRADTC,ZAC,RFORADTC(1,0,1,KGL3),RFOAC(1,0,KGL3))
ENDIF

! * writes grid-point transmission coefficients
!   for simplified physics.

IF (LRCOEF.AND.(NSTEP == 1).AND.LLDIAB.AND.(.NOT.LDLADNMI) &
   & .AND.(.NOT.LMPA)) THEN
  IFIELDSS=NG3SR*NFLEVG
  CALL WRRADCOEF(KST,KEND,KSTGLO,IFIELDSS,ZRADTC,ZAC_HC)
ENDIF

!       2.5   Ozone
!             -----

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.LOZONE.AND.(.NOT.LMPA)) THEN
  ! * Caution: this part has not been yet validated relative
  !   to the GFL implementation, and LOZONE (the setup of
  !   which has not yet been updated) can be true only if
  !   the GFL ozone is activated as a prognostic and advected
  !   variable.
  IPO3=(YO3%MP_SL1-1)*(NFLEVG+2*NFLSUL)
  IF (LSLAG) THEN
    IF (LTWOTL) THEN
      CALL CPOZO (NPROMA,KST,KEND,NFLEVG,PDTPHY,PFCHOZ,&
       & PB1(1,ISLB1GFL9+IPO3+1-NFLSA),PXYB0(1,1,YYTXYB0_PHY%M_RDELP))  
    ELSE
      CALL CPOZO (NPROMA,KST,KEND,NFLEVG,PDTPHY,PFCHOZ,&
       & PB1(1,ISLB1GFL9+IPO3+1-NFLSA),PXYB9(1,1,YYTXYB9_PHY%M_RDELP))  
    ENDIF
  ELSE
    CALL CPOZO (NPROMA,KST,KEND,NFLEVG,PDTPHY,PFCHOZ,&
     & PGFLT1(1,1,YO3%MP1),PXYB9(1,1,YYTXYB9_PHY%M_RDELP))
  ENDIF
ENDIF

!        2.6   surface specific humidity necessary to compute the vertical
!              advection of q in the case "delta m=1" (unlagged physics only).
!              ---------------------------------------------------------------

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.(NDPSFI == 1)) THEN
  CALL CPQSOL(NPROMA,KST,KEND,PRE0,PSP_RR(1,YSP_RR%YT%MP0),PQS,ZQSATS,PQSOL)
ENDIF

!        2.7  Computation of tendencies T,u,v and Q.
!             --------------------------------------

! Set GFL tendencies to 0

ZTENDGFL(:,:,:) = 0.0_JPRB

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.(.NOT.LSIMPH).AND.(.NOT.LMPA)) THEN

  TSPHY = MAX(PDTPHY,1.0_JPRB)

  ! * CPTEND+CPUTQY = Old( CPATY + CPDUP + CPDTHP )
  ! Calcul des tendances de T , U et de Q et modifications
  ! eventuelles de W et de OMEGA/P

  IF (LTWOTL) THEN

      CALL CPTEND_NEW( NPROMA, KST, KEND, NFLEVG,YDGSGEOM%GNORDL,YDGSGEOM%GNORDM,&
       & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS ,ZDIFEXT,&
       & PDIFTQ , PDIFTQI, PDIFTQL, PDIFTS ,&
       & PFCCQL , PFCCQN , PFCSQL , PFCSQN ,&
       & PFPLSL , PFPLSN , PFPLCL,  PFPLCN, &
       & PFPEVPSL,PFPEVPSN,PFPEVPCL,PFPEVPCN,&
       & PFPFPSL, PFPFPSN, PFPFPCL ,PFPFPCN ,&
       & PFCQLNG, PFCQING, PFCQRNG, PFCQSNG, &
       & PFCQNG , PFRMH  , ZFRMQ  , PFRSO  , PFRTH  ,&
       & PSTRCU , PSTRCV , PSTRDU , PSTRDV ,&
       & PSTRTU , PSTRTV , PSTRMU , PSTRMV ,&
       & PDIFCQLC,PDIFCQIC,PFIMCC, &
       & PFEDQLC, PFEDQIC, PFEDQRC, PFEDQSC, PFCNEGQLC,PFCNEGQIC,PFCNEGQRC,PFCNEGQSC, &
       & ZFTKE  , PXYB0(1,1,YYTXYB0_PHY%M_DELP) ,&
       & PXYB0(1,1,YYTXYB0_PHY%M_RDELP), PHIF0  , PRCP0(1,1,YYTRCP0%M_CP),&
       & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),&
       & PGFL(1,1,YQ%MP),PGFL(1,1,YI%MP),PGFL(1,1,YL%MP),&
       & PGFL(1,1,YLCONV%MP),PGFL(1,1,YICONV%MP),PGFL(1,1,YRCONV%MP),PGFL(1,1,YSCONV%MP),&
       & PGFL(1,1,YR%MP),PGFL(1,1,YS%MP),&
       & ZCPS   , PSP_RR(1,YSP_RR%YT%MP0)  ,&
       & PFHSCL ,PFHSCN,PFHSSL,PFHSSN,&
       & PFHPCL ,PFHPCN,PFHPSL,PFHPSN,&
       & ZFHP   ,ZFP   ,  PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL,&
       & PTENDU , PTENDV , ZTENDH ,&
       & ZTENDGFL(1,1,YQ%MP1),ZTENDGFL(1,1,YI%MP1),ZTENDGFL(1,1,YL%MP1), &
       & ZTENDGFL(1,1,YLCONV%MP1),ZTENDGFL(1,1,YICONV%MP1), &
       & ZTENDGFL(1,1,YRCONV%MP1),ZTENDGFL(1,1,YSCONV%MP1), &
       & ZTENDGFL(1,1,YR%MP1),ZTENDGFL(1,1,YS%MP1),ZTENDGFL(1,1,YTKE%MP1),&
       & ZTENDEXT )
       
  ELSE

      CALL CPTEND_NEW( NPROMA, KST, KEND, NFLEVG,YDGSGEOM%GNORDL,YDGSGEOM%GNORDM,&
       & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS ,ZDIFEXT,&
       & PDIFTQ , PDIFTQI, PDIFTQL, PDIFTS ,&
       & PFCCQL , PFCCQN , PFCSQL , PFCSQN ,&
       & PFPLSL , PFPLSN , PFPLCL,  PFPLCN, &
       & PFPEVPSL,PFPEVPSN,PFPEVPCL,PFPEVPCN,&
       & PFPFPSL, PFPFPSN, PFPFPCL ,PFPFPCN, &
       & PFCQLNG, PFCQING, PFCQRNG, PFCQSNG, &
       & PFCQNG , PFRMH  , ZFRMQ  , PFRSO  , PFRTH  ,&
       & PSTRCU , PSTRCV , PSTRDU , PSTRDV ,&
       & PSTRTU , PSTRTV , PSTRMU , PSTRMV ,&
       & PDIFCQLC,PDIFCQIC,PFIMCC, &
       & PFEDQLC, PFEDQIC, PFEDQRC, PFEDQSC, PFCNEGQLC,PFCNEGQIC,PFCNEGQRC,PFCNEGQSC, &
       & ZFTKE  , PXYB9(1,1,YYTXYB9_PHY%M_DELP) ,&
       & PXYB9(1,1,YYTXYB9_PHY%M_RDELP), PHIF9  , PRCP9(1,1,YYTRCP9%M_CP),&
       & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),&
       & PGFL(1,1,YQ%MP9),PGFL(1,1,YI%MP9),PGFL(1,1,YL%MP9),&
       & PGFL(1,1,YLCONV%MP),PGFL(1,1,YICONV%MP),PGFL(1,1,YRCONV%MP),PGFL(1,1,YSCONV%MP),&
       & PGFL(1,1,YR%MP9),PGFL(1,1,YS%MP9),&
       & ZCPS   , PSP_RR(1,YSP_RR%YT%MP9 ),&
       & PFHSCL ,PFHSCN,PFHSSL,PFHSSN,&
       & PFHPCL ,PFHPCN,PFHPSL,PFHPSN,&
       & ZFHP   ,ZFP   ,  PFEPFP, PFCMPCQ, PFCMPSN, PFCMPSL,&
       & PTENDU , PTENDV , ZTENDH ,&
       & ZTENDGFL(1,1,YQ%MP1),ZTENDGFL(1,1,YI%MP1),ZTENDGFL(1,1,YL%MP1), &
       & ZTENDGFL(1,1,YLCONV%MP1),ZTENDGFL(1,1,YICONV%MP1), &
       & ZTENDGFL(1,1,YRCONV%MP1),ZTENDGFL(1,1,YSCONV%MP1), &
       & ZTENDGFL(1,1,YR%MP1),ZTENDGFL(1,1,YS%MP1),ZTENDGFL(1,1,YTKE%MP1),&
       & ZTENDEXT )

    IF ( L3MT.OR.LSTRAPRO.OR.(NDPSFI==1)) THEN
!     PFEPFP was ZFEPFP in CPTEND_NEW, before, ZFEPFP still in CPFHPFS
      DO JLEV= 0, NFLEVG 
        DO JROF = 1, NPROMA
          PFEPFP(JROF,JLEV) = 0.0_JPRB
          PFCMPCQ(JROF,JLEV) = 0.0_JPRB
          PFCMPSN(JROF,JLEV) = 0.0_JPRB
          PFCMPSL(JROF,JLEV) = 0.0_JPRB
        ENDDO
      ENDDO
    ENDIF

  ENDIF



!        2.7.1  Diagnostics on physical tendencies
!               ----------------------------------

  IF (CDCONF /= 'X') THEN
    IF ((GCHETN%LFREQD).OR.(GCHETN%LCOORD).OR.(GCHETN%LPROFV)) THEN
      IF (LTWOTL) THEN
        CALL CPCHET(  NPROMA, KST, KEND, NFLEVG, NSTEP,&
        & PFHSCL , PFHSCN , PFHSSL , PFHSSN ,&
        & PFHPCL , PFHPCN , PFHPSL , PFHPSN ,&
        & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS ,&
        & PDIFTQ , PDIFTQI, PDIFTQL, PDIFTS ,&
        & PFCCQL , PFCCQN , PFCSQL , PFCSQN ,&
        & PFPLCL , PFPLCN , PFPLSL , PFPLSN ,&
        & PFPFPSL, PFPFPSN, PFPFPCL, PFPFPCN,&
        & PFPEVPSL,PFPEVPSN,PFPEVPCL,PFPEVPCN,&
        & PFRMH  , ZFRMQ  , PFRSO  , PFRTH  ,&
        & PSTRCU , PSTRCV , PSTRDU , PSTRDV ,&
        & PSTRTU , PSTRTV , PSTRMU , PSTRMV ,&
        & PXYB0(1,1,YYTXYB0_PHY%M_RDELP), PRCP0(1,1,YYTRCP0%M_CP),  &
        & PGMV(1,1,YT0%MT),PGFL(1,1,YQ%MP),PGFL(1,1,YI%MP),PGFL(1,1,YL%MP),&
        & ZCPS   , PSP_RR(1,YSP_RR%YT%MP0 ) , PQS  , &
        & PTENDU , PTENDV , ZTENDH ,         &
        & ZTENDGFL(1,1,YQ%MP1),ZTENDGFL(1,1,YI%MP1),ZTENDGFL(1,1,YL%MP1),  &
        & ZTENDGFL(1,1,YR%MP1),ZTENDGFL(1,1,YS%MP1),&
        & YDGSGEOM%GEMU,YDGSGEOM%GELAM, PHI0   , PHIF0  )
      ELSE
        CALL CPCHET(  NPROMA, KST, KEND, NFLEVG, NSTEP,&
        & PFHSCL , PFHSCN , PFHSSL , PFHSSN ,&
        & PFHPCL , PFHPCN , PFHPSL , PFHPSN ,&
        & PDIFCQ , PDIFCQI, PDIFCQL, PDIFCS ,&
        & PDIFTQ , PDIFTQI, PDIFTQL, PDIFTS ,&
        & PFCCQL , PFCCQN , PFCSQL , PFCSQN ,&
        & PFPLCL , PFPLCN , PFPLSL , PFPLSN ,&
        & PFPFPSL, PFPFPSN, PFPFPCL, PFPFPCN,&
        & PFPEVPSL,PFPEVPSN,PFPEVPCL,PFPEVPCN,&
        & PFRMH  , ZFRMQ  , PFRSO  , PFRTH  ,&
        & PSTRCU , PSTRCV , PSTRDU , PSTRDV ,&
        & PSTRTU , PSTRTV , PSTRMU , PSTRMV ,&
        & PXYB9(1,1,YYTXYB9_PHY%M_RDELP), PRCP9(1,1,YYTRCP9%M_CP),  &
        & PGMV(1,1,YT9%MT),PGFL(1,1,YQ%MP9),PGFL(1,1,YI%MP9),PGFL(1,1,YL%MP9),&
        & ZCPS   , PSP_RR(1,YSP_RR%YT%MP9)  , PQS    , &
        & PTENDU , PTENDV , ZTENDH ,         &
        & ZTENDGFL(1,1,YQ%MP1),ZTENDGFL(1,1,YI%MP1),ZTENDGFL(1,1,YL%MP1),  &
        & ZTENDGFL(1,1,YR%MP1),ZTENDGFL(1,1,YS%MP1),&
        & YDGSGEOM%GEMU,YDGSGEOM%GELAM, PHI9   , PHIF9  )
      ENDIF
    ENDIF
    
    IF (GCHETN%LPROFV) &
     & CALL PROFILECHET &
     & (KEND, &
     & YDGSGEOM%GELAM,YDGSGEOM%GEMU, &
     & YDGSGEOM%GM,ZMU0,YDOROG%OROG,POROGL,POROGM,YDGSGEOM%RCORI,YDCSGEOM%RATATH,YDCSGEOM%RATATX, &
     & PSD_VF(1,YSD_VF%YLSM%MP ), PSD_VV(1,YSD_VV%YARG%MP), PSD_VV(1,YSD_VV%YSAB%MP), PSD_VF(1,YSD_VF%YALBF%MP), &
     & PSD_VF(1,YSD_VF%YALBSF%MP), PSD_VF(1,YSD_VF%YEMISF%MP), PSD_VV(1,YSD_VV%YD2%MP), PSD_VV(1,YSD_VV%YIVEG%MP), &
     & PSD_VV(1,YSD_VV%YLAI%MP), PCT, PSD_VF(1,YSD_VF%YZ0F%MP), PSD_VV(1,YSD_VV%YZ0H%MP), PSD_VF(1,YSD_VF%YZ0RLF%MP), &
     & PSD_VF(1,YSD_VF%YGETRL%MP), PSD_VF(1,YSD_VF%YVRLAN%MP), PSD_VF(1,YSD_VF%YVRLDI%MP), PSD_VV(1,YSD_VV%YRSMIN%MP), &
     & PSD_VF(1,YSD_VF%YVEG%MP), PSD_VV(1,YSD_VV%YHV%MP), PSD_VA(1,YSD_VA%YSEA%MP), PSD_VA(1,YSD_VA%YLAN%MP), &
     & PSD_VA(1,YSD_VA%YSOO%MP), PSD_VA(1,YSD_VA%YDES%MP), PSD_VC(1,YSD_VC%YVC(1)%MP), &
     & PGMVS(1,YT0%MSP),PGMVS(1,YT0%MSPL),PGMVS(1,YT0%MSPM),&
     & PGMV(1,1,YT0%MT),PGMV(1,1,YT0%MTL),PGMV(1,1,YT0%MTM),&
     & PGFL(1,1,YQ%MP),PGFL(1,1,YQ%MPL),PGFL(1,1,YQ%MPM), &
     & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MVOR),PGMV(1,1,YT0%MDIV), ZCVGQ, ZLCVQ, &
     & PSP_RR(1,YSP_RR%YT%MP9), PSP_SB(1,1,YSP_SB%YT%MP9), PSP_RR(1,YSP_RR%YFC%MP9), PSP_RR(1,YSP_RR%YW%MP9), &
     & PSP_RR(1,YSP_RR%YIC%MP9), PSP_SB(1,1,YSP_SB%YQ%MP9), PSP_SB(1,1,YSP_SB%YTL%MP9), PSP_SG(1,YSP_SG%YF%MP9), &
     & PDIFCQ, PDIFCQI, PDIFCQL, PDIFCS, PDIFTQ, PDIFTQI, PDIFTQL, &
     & PDIFTS, PFCCQL, PFCCQN, PFCSQL, PFCSQN, PFCQNG, PFCQING, &
     & PFCQLNG, PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFRSO, PFRTH, &
     & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV, PSTRMU, PSTRMV, &
     & PFRMH, PFCHOZ, PNEB, PQICE, PQLI, PRH, &
     & PFCS, PFCLL, PFCLN, PFEVL, PFEVN, PFEVV, PFTR, PFLWSP, PFONTE, &
     & PFGEL, PFGELS, PFRSODS, PFRSOPS, PFRSOPT, PFRTHDS, &
     & PQS, PRUISL, PRUISP, PRUISS, &
     & PUCLS, PVCLS, PTCLS, PQCLS, PRHCLS, &
     & PCLCT, PCLCH, PCLCM, PCLCL, PCLCC, &
     & ZFPLCH, ZFPLSH, &
     & PSD_VH(1,YSD_VH%YTCCH%MP),  PSD_VH(1,YSD_VH%YSCCH%MP), PSD_VH(1,YSD_VH%YBCCH%MP), PSD_VH(1,YSD_VH%YPBLH%MP) )

  ENDIF

ENDIF

!        2.8  Modification of vertical velocities
!             by some physics output when required.
!             -------------------------------------

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.(.NOT.LSIMPH)) THEN

  ! * MODIFICATION DE LA VITESSE VERTICALE ET DE LA TENDANCE DE
  ! PRESSION DE SURFACE SI NDPSFI=1 ( MASSE VARIABLE ).
  ! Ajout de la physique dans l'equation de continuite/Add physics
  ! in continuity equation.

  IF (NDPSFI == 1) THEN
    IF (LSLAG .AND. LTWOTL) THEN
      CALL CPMVVPS(LVERCOR,NPROMA,NPROMVC,KST,KEND,NFLEVG,PDTPHY,&
       & PVCASRS0H,ZFP,PRE0(1,NFLEVG),PFEVL,PFEVN,&
       & PCTY0(1,0,YYTCTY0%M_EVEL),PCTY0(1,0,YYTCTY0%M_PSDVBC),PB1(1,MSLB1SP9))
    ELSEIF (LSLAG .AND. (.NOT.LTWOTL)) THEN
      CALL CPMVVPS(LVERCOR,NPROMA,NPROMVC,KST,KEND,NFLEVG,PDTPHY,&
       & PVCASRS0H,ZFP,PRE9(1,NFLEVG),PFEVL,PFEVN,&
       & PCTY0(1,0,YYTCTY0%M_EVEL),PCTY0(1,0,YYTCTY0%M_PSDVBC),PB1(1,MSLB1SP9))
    ELSE
      CALL CPMVVPS(LVERCOR,NPROMA,NPROMVC,KST,KEND,NFLEVG,PDTPHY,&
       & PVCASRS0H,ZFP,PRE9(1,NFLEVG),PFEVL,PFEVN,&
       & PCTY0(1,0,YYTCTY0%M_EVEL),PCTY0(1,0,YYTCTY0%M_PSDVBC),PGMVT1S(1,YT1%MSP))
    ENDIF
  ENDIF

ENDIF

!        2.9  Computation of evolution of T, u, v and Q.
!             ------------------------------------------

! * Calculation of IPGFL, since the old pointers
!   MSLB1[X]9 (=MSLB1GFL9+IP[X]) do not exist any longer in PTRSLB1.

! usefull pointer for new version of cputqy

DO JGFL=1,YGFL%NUMFLDS
  IF ((YGFLC(JGFL)%MP1 > 0) .AND. (YGFLC(JGFL)%MP_SL1 > 0)) THEN
     IPGFL(YGFLC(JGFL)%MP1) = (YGFLC(JGFL)%MP_SL1-1)*(NFLEVG+2*NFLSUL)
  ENDIF   
ENDDO  

!  ALARO does not respect the coding rules, tendency of pseudo-TKE is computed in APLPAR and not
!  in CPTEND_NEW. To use the new version of cputqy it is then necessary to write it in GFL tendencies array.
! This memory transfer is not necessary, please respect coding rules to avoid it.

IF (LPTKE) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      ZTENDGFL(JROF,JLEV,YTKE%MP1) = ZTENDPTKE(JROF,JLEV)
    ENDDO
  ENDDO    
ENDIF
! Extra-GFL
IF(LMDUST.AND.(NGFL_EXT/=0)) THEN
  DO JGFL=1, NGFL_EXT
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZTENDGFL(JROF,JLEV,YEXT(JGFL)%MP1) = ZTENDEXT(JROF,JLEV,JGFL)+ &  ! turbulent tendency
      &                                      ZTENDEXT_DEP(JROF,JLEV,JGFL) ! moist tendency
      ENDDO
    ENDDO 
  ENDDO   
ENDIF

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.(.NOT.LSIMPH).AND.(.NOT.LMPA)) THEN
  ! Calcul de T , Q et du Vent a l'instant 1

  CALL CPUTQY( NPROMA, KST,KEND,NFLEVG, PDTPHY, IPGFL, &
   & ISLB1U9,ISLB1V9,ISLB1T9,ISLB1GFL9, &  
   & ZTENDH, ZTENDT, PTENDU, PTENDV, ZTENDGFL, &
   & PRCP0(1,1,YYTRCP0%M_CP),PXYB0(1,1,YYTXYB0_PHY%M_DELP),PGMV(1,1,YT0%MT),PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),&
   & PRCP9(1,1,YYTRCP9%M_CP),PXYB9(1,1,YYTXYB9_PHY%M_DELP),PGMV(1,1,YT9%MT),PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),&
   & PB1, PGMVT1, PGFLT1, &
   & PFDIS)

ENDIF

!        2.9a Evolution of precipitation fluxes
!             ------------------------------------------

IF(LNEBN.OR.LNEBR.OR.LRRGUST) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      PGFLT1(JROF,JLEV,YCPF%MP1)=ZFPLCH(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF
IF(LRRGUST) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      PGFLT1(JROF,JLEV,YSPF%MP1)=ZFPLSH(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!       2.9b Prognostic convection etc.
!            --------------------------

! TRANSFER NOT ADVECTED VARIABLES INTO PGFLT1
IF (LCVPRO) THEN
   IF (.NOT.YUAL%LADV) THEN
      PGFLT1(KST:KEND,1:NFLEVG,YUAL%MP1)=PGFL(KST:KEND,1:NFLEVG,YUAL%MP)
      PGFLT1(KST:KEND,1:NFLEVG,YUOM%MP1)=PGFL(KST:KEND,1:NFLEVG,YUOM%MP)
   ENDIF
   IF (LCDDPRO) THEN
    IF (.NOT.YDAL%LADV) THEN
      PGFLT1(KST:KEND,1:NFLEVG,YDAL%MP1)=PGFL(KST:KEND,1:NFLEVG,YDAL%MP)
      PGFLT1(KST:KEND,1:NFLEVG,YDOM%MP1)=PGFL(KST:KEND,1:NFLEVG,YDOM%MP)
    ENDIF
   ENDIF
ENDIF
IF (YUNEBH%LACTIVE.AND..NOT.YUNEBH%LADV)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YUNEBH%MP1)=PGFL(KST:KEND,1:NFLEVG,YUNEBH%MP)
ENDIF
IF (YUEN%LACTIVE.AND..NOT.YUEN%LADV) THEN
   PGFLT1(KST:KEND,1:NFLEVG,YUEN%MP1)=PGFL(KST:KEND,1:NFLEVG,YUEN%MP)
ENDIF
IF (YRKTH%LACTIVE)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YRKTH%MP1)=PGFL(KST:KEND,1:NFLEVG,YRKTH%MP)
ENDIF
IF (YRKTQV%LACTIVE)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YRKTQV%MP1)=PGFL(KST:KEND,1:NFLEVG,YRKTQV%MP)
ENDIF
IF (YRKTQC%LACTIVE)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YRKTQC%MP1)=PGFL(KST:KEND,1:NFLEVG,YRKTQC%MP)
ENDIF
IF (YTTE%LACTIVE.AND..NOT.YTTE%LADV)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YTTE%MP1)=PGFL(KST:KEND,1:NFLEVG,YTTE%MP)
ENDIF
IF (YMXL%LACTIVE)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YMXL%MP1)=PGFL(KST:KEND,1:NFLEVG,YMXL%MP)
ENDIF
IF (YEFB1%LACTIVE)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YEFB1%MP1)=PGFL(KST:KEND,1:NFLEVG,YEFB1%MP)
ENDIF
IF (YEFB2%LACTIVE)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YEFB2%MP1)=PGFL(KST:KEND,1:NFLEVG,YEFB2%MP)
ENDIF
IF (YEFB3%LACTIVE)THEN
   PGFLT1(KST:KEND,1:NFLEVG,YEFB3%MP1)=PGFL(KST:KEND,1:NFLEVG,YEFB3%MP)
ENDIF

!        2.10  Surface variables.
!              ------------------

IF (LLDIAB.AND.(.NOT.LDLADNMI).AND.LMPHYS.AND.(.NOT.LMPA).AND.(.NOT.LSFORCS)) THEN
  
  IF(.NOT.LMSE) THEN
    CALL CPTENDS( NPROMA, KST, KEND, NFLEVG, YSP_SBD%NLEVS, PDTPHY,&
     & PFPLCL, PFPLSL, PFPLCN, PFPLSN,&
     & PFRSO, PFRTH,&
     & PSP_SG(1,YSP_SG%YA%MP1),PCT, ZC1, ZC2,&
     & PFCHSP, PFCLL, PFCLN, PFCS,&
     & ZFEVI,PFEVL, PFEVN,&
     & PFEVV, PFGEL, PFGELS, PFLWSP, PFONTE, PFTR,&
     & PSD_VF(1,YSD_VF%YLSM%MP), PSP_SG(1,YSP_SG%YR%MP1),&
     & PRUISL, PRUISP, PRUISS, PSP_SG(1,YSP_SG%YF%MP1), ZVEG,&
     & ZTDTS, ZTDTP, ZTDWS, ZTDWSI, ZTDWP, ZTDWPI, ZTDWL,&
     & ZTDSNS, ZTDALBNS, ZTDRHONS)  

    CALL CPWTS ( NPROMA, KST,KEND, YSP_SBD%NLEVS, PDTPHY,&
     & ZTDTS, ZTDTP, ZTDWS, ZTDWSI, ZTDWP, ZTDWPI, ZTDWL,&
     & ZTDSNS, ZTDALBNS, ZTDRHONS,&
     & PSD_VP(1,YSD_VP%YTPC%MP),PSD_VP(1,YSD_VP%YWPC%MP),PSD_VF(1,YSD_VF%YLSM%MP),PSD_VV(1,YSD_VV%YIVEG%MP),&
     & PSP_RR(1,YSP_RR%YT%MP1),PSP_SB(1,1,YSP_SB%YT%MP1),PSP_RR(1,YSP_RR%YW%MP1),&
     & PSP_RR(1,YSP_RR%YIC%MP1),&
     & PSP_SB(1,1,YSP_SB%YQ%MP1),PSP_SB(1,1,YSP_SB%YTL%MP1),PSP_RR(1,YSP_RR%YFC%MP1),&
     & PSP_SG(1,YSP_SG%YF%MP1),PSP_SG(1,YSP_SG%YA%MP1),PSP_SG(1,YSP_SG%YR%MP1))  
  ELSE
    IF (LLXFUMSE) THEN
      DO JROF=KST,KEND
        PSP_RR(JROF,YSP_RR%YT%MP0)=PGPAR(JROF,MVTS)
      ENDDO
    ELSE
      DO JROF=KST,KEND
        PSP_RR(JROF,YSP_RR%YT%MP1)=PGPAR(JROF,MVTS)
      ENDDO
    ENDIF
  ENDIF
  IF(LNUDG)THEN

    ! * Calculation of IPQ since the old pointers
    !   MSLB1[X]9 (=MSLB1GFL9+IP[X]) do not exist any longer in PTRSLB1.
    IPQ=(YQ%MP_SL1-1)*(NFLEVG+2*NFLSUL)

    CALL CPNUDG ( NPROMA, KST,KEND, NFNUDG, NFLEVG, IBLK,&
     & XPNUDG,&
     & PSD_VF(1,YSD_VF%YNUDM%MP),&
     & PSP_RR(1,YSP_RR%YT%MP1),PSP_RR(1,YSP_RR%YW%MP1),&
     & PSP_SB(1,1,YSP_SB%YQ%MP1),PSP_SG(1,YSP_SG%YF%MP1),&
     & PB1(1,ISLB1T9+1-NFLSA),PB1(1,ISLB1GFL9+IPQ+1-NFLSA),&
     & PB1(1,ISLB1U9+1-NFLSA),PB1(1,ISLB1V9+1-NFLSA),&
     & PB1(1,MSLB1SP9),&
     & PGMV(1,1,YT0%MT),PGFL(1,1,YQ%MP),PGMV(1,1,YT0%MU),&
     & PGMV(1,1,YT0%MV),PRE0(1,NFLEVG),YDGSGEOM%GM,PSD_VF(1,YSD_VF%YLSM%MP))
  ENDIF
ENDIF

!        2.11 Evolution of CVV (GY)
!             ---------------------

IF(LCVPGY) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      PGFLT1(JROF,JLEV,YCVV%MP1)=PGFL(JROF,JLEV,YCVV%MP)
    ENDDO
  ENDDO
ENDIF

!    -------------------------------------------------------------------

!*       3.    Simplified physics.
!              -------------------

!        3.1  Preliminary calculations necessary for simplified physics.
!             ----------------------------------------------------------

IF (.NOT.LDLADNMI) THEN

  ! * transforms Fourier into grid-point transmission coefficients
  !   for simplified physics.
  !   Can be called even if lsimph=.false. for validation, but output
  !   of CPPFTTCINV is needed only if lsimph=.true.

  IF (LDRETCFOU) THEN
    CALL CPPFTTCINV(KGL3,KGL4,KOFF,RBASCOTC(1,KOFF),RBASSITC(1,KOFF),&
     & RFORADTC(1,0,1,KGL3),RFOAC(1,0,KGL3),ZRADTC,ZAC)  
    ! * Simplified physics now reads a horizontally-constant field for ZAC:
    !   take RFOAC(..,0,kgl3) to initialize ZAC_HC.
    DO JL2=1,NFLEVG+1
      DO JL1=1,NFLEVG+1
        IJL=JL1+(JL2-1)*(NFLEVG+1)
        ZAC_HC(JL2,JL1)=RFOAC(IJL,0,KGL3)
      ENDDO
    ENDDO
  ENDIF

ENDIF

IF (LSIMPH.AND.(.NOT.LDLADNMI)) THEN

  ! * read grid-point transmission coefficients
  !   for simplified physics.

  IF (LRAYSP.AND.LRCOEF.AND.(NSTEP > 1.OR.LTLADDIA)) THEN
    IFIELDSS=NG3SR*NFLEVG
    CALL RDRADCOEF(KST,KEND,KSTGLO,IFIELDSS,ZRADTC,ZAC_HC)  
  ENDIF

ENDIF

!        3.2  Simplified physics.
!             -------------------

IF (LSIMPH.AND.(.NOT.LDLADNMI)) THEN

  TSPHY = MAX(PDTPHY,1.0_JPRB)

  IF (LTWOTL) THEN

    CALL APLPASSH (KST,KEND,NPROMA,NFLEVG,&
     & PRE0,PGFL(1,1,YQ%MP),&
     & PSP_SG(1,YSP_SG%YF%MP1),PSP_RR(1,YSP_RR%YT%MP1),PSP_RR(1,YSP_RR%YW%MP1),&
     & PSD_VF(1,YSD_VF%YLSM%MP),PSD_VF(1,YSD_VF%YVEG%MP),&
     & ZQS1)  

    IF (.NOT.LLDIAB) THEN
      CALL APLPASSH (KST,KEND,NPROMA,NFLEVG,&
       & PRE0,PGFL(1,1,YQ%MP),&
       & PSP_SG(1,YSP_SG%YF%MP0),PSP_RR(1,YSP_RR%YT%MP0),PSP_RR(1,YSP_RR%YW%MP0),&
       & PSD_VF(1,YSD_VF%YLSM%MP),PSD_VF(1,YSD_VF%YVEG%MP),&
       & PQS)  
    ENDIF

    CALL APLPARS (KST,KEND,NPROMA,1,NFLEVG,NTSSG,NSTEP,&
     & PHI0,PRE0,PHIF0,PRE0F,PXYB0(1,1,YYTXYB0_PHY%M_DELP),PXYB0(1,1,YYTXYB0_PHY%M_RDELP),&
     & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),PGFL(1,1,YQ%MP),PRCP0(1,1,YYTRCP0%M_CP),ZCVGQ,&
     & PSP_SG(1,YSP_SG%YF%MP0),PSP_RR(1,YSP_RR%YT%MP0),PQS,&
     & PSP_RR(1,YSP_RR%YT%MP1),ZQS1,&
     & PSD_VF(1,YSD_VF%YGETRL%MP),PSD_VF(1,YSD_VF%YLSM%MP),&
     & PSD_VF(1,YSD_VF%YZ0F%MP),PSD_VF(1,YSD_VF%YVRLAN%MP),PSD_VF(1,YSD_VF%YVRLDI%MP),&
     & PSD_VF(1,YSD_VF%YALBF%MP),ZMU0,YDGSGEOM%GM,&
     & ZAC_HC,ZRADTC(1,1,YM_RADTC%MCOR),&
     & ZRADTC(1,1,YM_RADTC%MRAB3C),ZRADTC(1,1,YM_RADTC%MRAB3N), &
     & ZRADTC(1,1,YM_RADTC%MRAB4C),ZRADTC(1,1,YM_RADTC%MRAB4N), &
     & ZRADTC(1,1,YM_RADTC%MRAB6C),ZRADTC(1,1,YM_RADTC%MRAB6N), &
     & ZRADTC(1,1,YM_RADTC%MRAT1C),ZRADTC(1,1,YM_RADTC%MRAT1N), &
     & ZRADTC(1,1,YM_RADTC%MRAT2C),ZRADTC(1,1,YM_RADTC%MRAT2N), &
     & ZRADTC(1,1,YM_RADTC%MRAT3C),ZRADTC(1,1,YM_RADTC%MRAT3N), &
     & ZRADTC(1,1,YM_RADTC%MRAT4C),ZRADTC(1,1,YM_RADTC%MRAT4N), &
     & ZRADTC(1,1,YM_RADTC%MRAT5C),ZRADTC(1,1,YM_RADTC%MRAT5N), &
     & PDIFCQ,PDIFCS,PDIFTQ,PDIFTS,&
     & PFCCQL,PFCCQN,PFCSQL,PFCSQN,&
     & PFPLCL,PFPLCN,PFPLSL,PFPLSN,PFRSO,PFRTH,&
     & PSTRCU,PSTRCV,PSTRDU,PSTRDV,PSTRTU,PSTRTV,&
     & PSTRMU,PSTRMV,PFRMH)  

  ELSE

    CALL APLPASSH (KST,KEND,NPROMA,NFLEVG,&
     & PRE0,PGFL(1,1,YQ%MP),&
     & PSP_SG(1,YSP_SG%YF%MP1),PSP_RR(1,YSP_RR%YT%MP1),PSP_RR(1,YSP_RR%YW%MP1),&
     & PSD_VF(1,YSD_VF%YLSM%MP),PSD_VF(1,YSD_VF%YVEG%MP),&
     & ZQS1)  

    IF (.NOT.LLDIAB) THEN
      CALL APLPASSH (KST,KEND,NPROMA,NFLEVG,&
       & PRE9,PGFL(1,1,YQ%MP9),&
       & PSP_SG(1,YSP_SG%YF%MP0),PSP_RR(1,YSP_RR%YT%MP9),PSP_RR(1,YSP_RR%YW%MP9),&
       & PSD_VF(1,YSD_VF%YLSM%MP),PSD_VF(1,YSD_VF%YVEG%MP),&
       & PQS)  
    ENDIF

    CALL APLPARS (KST,KEND,NPROMA,1,NFLEVG,NTSSG,NSTEP,&
     & PHI9,PRE9,PHIF9,PRE9F,PXYB9(1,1,YYTXYB9_PHY%M_DELP),PXYB9(1,1,YYTXYB9_PHY%M_RDELP),&
     & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),PGFL(1,1,YQ%MP9),PRCP9(1,1,YYTRCP9%M_CP),ZCVGQ,&
     & PSP_SG(1,YSP_SG%YF%MP0),PSP_RR(1,YSP_RR%YT%MP9),PQS,&
     & PSP_RR(1,YSP_RR%YT%MP1),ZQS1,&
     & PSD_VF(1,YSD_VF%YGETRL%MP),PSD_VF(1,YSD_VF%YLSM%MP),&
     & PSD_VF(1,YSD_VF%YZ0F%MP),PSD_VF(1,YSD_VF%YVRLAN%MP),PSD_VF(1,YSD_VF%YVRLDI%MP),&
     & PSD_VF(1,YSD_VF%YALBF%MP),ZMU0,YDGSGEOM%GM,&
     & ZAC_HC,ZRADTC(1,1,YM_RADTC%MCOR),&
     & ZRADTC(1,1,YM_RADTC%MRAB3C),ZRADTC(1,1,YM_RADTC%MRAB3N), &
     & ZRADTC(1,1,YM_RADTC%MRAB4C),ZRADTC(1,1,YM_RADTC%MRAB4N), &
     & ZRADTC(1,1,YM_RADTC%MRAB6C),ZRADTC(1,1,YM_RADTC%MRAB6N), &
     & ZRADTC(1,1,YM_RADTC%MRAT1C),ZRADTC(1,1,YM_RADTC%MRAT1N), &
     & ZRADTC(1,1,YM_RADTC%MRAT2C),ZRADTC(1,1,YM_RADTC%MRAT2N), &
     & ZRADTC(1,1,YM_RADTC%MRAT3C),ZRADTC(1,1,YM_RADTC%MRAT3N), &
     & ZRADTC(1,1,YM_RADTC%MRAT4C),ZRADTC(1,1,YM_RADTC%MRAT4N), &
     & ZRADTC(1,1,YM_RADTC%MRAT5C),ZRADTC(1,1,YM_RADTC%MRAT5N), &
     & PDIFCQ,PDIFCS,PDIFTQ,PDIFTS,&
     & PFCCQL,PFCCQN,PFCSQL,PFCSQN,&
     & PFPLCL,PFPLCN,PFPLSL,PFPLSN,PFRSO,PFRTH,&
     & PSTRCU,PSTRCV,PSTRDU,PSTRDV,PSTRTU,PSTRTV,&
     & PSTRMU,PSTRMV,PFRMH)  

  ENDIF

ENDIF

!        3.3  Store the model trajectory at t-dt (leap-frog) or t (sl2tl).
!             ------------------------------------------------------------

IF (LTRAJPS.AND.(.NOT.LDLADNMI)) THEN
  IFIELDS=NG3D95*NFLEVG+NG2D95
  IF (LTWOTL) THEN
    CALL WRPHTRSF(NSTEP,KST,KEND,KSTGLO,NG2DS5,&
     & PQS,PSP_RR(1,YSP_RR%YT%MP0),PSP_SG(1,YSP_SG%YF%MP0))  
  ELSE
    CALL WRPHTRAJM(NSTEP,KST,KEND,KSTGLO,IFIELDS,&
     & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),&
     & PGFL(1,1,YQ%MP9),PGFL(1,1,YL%MP9),PGFL(1,1,YI%MP9),PGMVS(1,YT9%MSP))  
    CALL WRPHTRSF(NSTEP,KST,KEND,KSTGLO,NG2DS5,&
     & PQS,PSP_RR(1,YSP_RR%YT%MP9),PSP_SG(1,YSP_SG%YF%MP9))  
  ENDIF
ENDIF

!        3.4  Computation of tendencies T,u,v and Q.
!             --------------------------------------

IF (LSIMPH.AND.(.NOT.LDLADNMI)) THEN

  TSPHY = MAX(PDTPHY,1.0_JPRB)

  IF (LTWOTL) THEN
    CALL CPTENDSM (NPROMA,KST,KEND,NFLEVG,&
     & PDIFCQ,PDIFCS,PDIFTQ,PDIFTS,&
     & PFCCQL,PFCCQN,PFCSQL,PFCSQN,&
     & PFPLCL,PFPLCN,PFPLSL,PFPLSN,&
     & PFRSO,PFRTH,&
     & PSTRCU,PSTRCV,PSTRDU,PSTRDV,&
     & PSTRTU,PSTRTV,&
     & PSTRMU,PSTRMV,PFRMH,&
     & PXYB0(1,1,YYTXYB0_PHY%M_RDELP),PHIF0,&
     & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MT),PGFL(1,1,YQ%MP),&
     & PQS,PSP_RR(1,YSP_RR%YT%MP0),YDOROG%OROG,&
     & PTENDU,PTENDV,ZTENDH,ZTENDQ,&
     & PFHPCL,PFHPCN,PFHPSL,PFHPSN,&
     & PFHSCL,PFHSCN,PFHSSL,PFHSSN)  
  ELSE
    CALL CPTENDSM (NPROMA,KST,KEND,NFLEVG,&
     & PDIFCQ,PDIFCS,PDIFTQ,PDIFTS,&
     & PFCCQL,PFCCQN,PFCSQL,PFCSQN,&
     & PFPLCL,PFPLCN,PFPLSL,PFPLSN,&
     & PFRSO,PFRTH,&
     & PSTRCU,PSTRCV,PSTRDU,PSTRDV,&
     & PSTRTU,PSTRTV,&
     & PSTRMU,PSTRMV,PFRMH,&
     & PXYB9(1,1,YYTXYB9_PHY%M_RDELP),PHIF9,&
     & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),PGMV(1,1,YT9%MT),PGFL(1,1,YQ%MP9),&
     & PQS,PSP_RR(1,YSP_RR%YT%MP9),YDOROG%OROG,&
     & PTENDU,PTENDV,ZTENDH,ZTENDQ,&
     & PFHPCL,PFHPCN,PFHPSL,PFHPSN,&
     & PFHSCL,PFHSCN,PFHSSL,PFHSSN)  
  ENDIF

ENDIF

!        3.5  Computation of evolution of T, u, v and Q.
!             ------------------------------------------

IF (LSIMPH.AND.(.NOT.LDLADNMI)) THEN

  CALL CPUTQYS( NPROMA, KST,KEND,NFLEVG, PDTPHY, IPGFL, &
   & ISLB1U9,ISLB1V9,ISLB1T9,ISLB1GFL9, &  
   & ZTENDH, ZTENDQ, PTENDU, PTENDV, &
   & PRCP0(1,1,YYTRCP0%M_CP),PXYB0(1,1,YYTXYB0_PHY%M_DELP),PGMV(1,1,YT0%MT),PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),&
   & PRCP9(1,1,YYTRCP9%M_CP),PXYB9(1,1,YYTXYB9_PHY%M_DELP),PGMV(1,1,YT9%MT),PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),&
   & PB1, PGMVT1, PGFLT1, &
   & PFDIS)

ENDIF

!     ------------------------------------------------------------------

!*       4.    AROME  physics.
!              ---------------

IF (LMPA.AND.(.NOT.LDLADNMI)) THEN

  !      4.1  CALL APL_AROME
  !           --------------

  !  Define AROME's pointer for ZTENDGFLR
  ! * default value (to avoid uninitialised values):
  DO JGFL=1,YGFL%NUMFLDS
    IPTR(JGFL)=JGFL
  ENDDO

  ! * ZTENDR     
  IPTR_CONT = 1
  IPTR(YQ%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1
  IPTR(YL%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1
  IPTR(YR%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1
  IPTR(YI%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1
  IPTR(YS%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1
  IPTR(YG%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1
  IF (YH%LACTIVE) THEN
    IPTR(YH%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1
  ENDIF   

  ! * ZTENDTKE   
  IPTR(YTKE%MP1) = IPTR_CONT ; IPTR_CONT = IPTR_CONT+1

  ! * ZTENDEXT
  DO JGFL=1,NGFL_EXT
    IPTR(YEXT(JGFL)%MP1)=IPTR_CONT
    IPTR_CONT = IPTR_CONT+1   
  ENDDO  
 
  IF ( NGFL_EXT > 0 ) THEN
    IPTREXT=IPTR(YEXT(1)%MP1)  
  ELSE
    IPTREXT=YEXT(1)%MP1 
  ENDIF

  ZTENDGFLR(1:NPROMM,1:NFLEVG,1:YGFL%NUMFLDS)=0.0_JPRB

  IF(LTWOTL) THEN
    CALL APL_AROME( KBL, KGPCOMP, KST  , KEND   , NPROMA ,&
     & 1      , NFLEVG , NSTEP  ,&
     & NTSSG   ,  PDTPHY, LLXFUMSE,YDCSGEOM%RINDX,YDCSGEOM%RINDY, &
     & YDGSGEOM%GEMU,YDGSGEOM%GELAM,YDOROG%OROG,YDGSGEOM%GM,ZMU0,ZMU0M,ZMU0N,&
     & YDGSGEOM%GECLO,YDGSGEOM%GESLO,PSD_VC(1,YSD_VC%YVC(1)%MP),PSD_VF(1,YSD_VF%YLSM%MP),&
     & PSD_VA(1,YSD_VA%YSEA%MP ) , PSD_VA(1,YSD_VA%YLAN%MP ) ,PSD_VA(1,YSD_VA%YSOO%MP ) , PSD_VA(1,YSD_VA%YDES%MP ) ,&
     & PSD_VA(1,YSD_VA%YSUL%MP ) , PSD_VA(1,YSD_VA%YVOL%MP ) , &
     & PGP2DSDT, &
     & PHI0,PHIF0,PREPHY0,PREPHY0F,PXYB0(1,1,YYTXYB0_PHY%M_RDELP),PXYB0(1,1,YYTXYB0_PHY%M_DELP),&
     & PGMV(1,1,YT0%MT),PGFL(1,1,YQ%MP),&
     & PRCP0(1,1,YYTRCP0%M_CP),PRCP0(1,1,YYTRCP0%M_R),PXYB0(1,1,YYTXYB0_PHY%M_ALPH),PXYB0(1,1,YYTXYB0_PHY%M_LNPR), &
     & PGFL(1,1,YL%MP),PGFL(1,1,YI%MP),PGFL(1,1,YR%MP),PGFL(1,1,YS%MP),&
     & PGFL(1,1,YG%MP),PGFL(1,1,YH%MP),PGFL(1,1,YTKE%MP),PGFL(1,1,YSRC%MP),PGFL(1,1,YEXT(1)%MP), &
     & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV), PWT0, &
     & PGPAR,&
     & PEMTD , PEMTU ,PTRSW , &
     ! outputs
     & PGFLT1(1,1,YLRAD%MP1),PGFLT1(1,1,YIRAD%MP1),PGFLT1(1,1,YA%MP1),PGFLT1(1,1,YSRC%MP1),&
     & ZTENDT, ZTENDGFLR(1,1,1), PTENDU, PTENDV,ZTENDW, ZTENDGFLR(1,1,IPTR(YTKE%MP1)),ZTENDGFLR(1,1,IPTREXT),&
     & PFRTH, PFRSO,PFRTHDS, PFRSODS,PFRTHC, PFRSOC, &
     & PSD_VF(1,YSD_VF%YALBF%MP), PSD_VF(1,YSD_VF%YEMISF%MP),PGFL(1,1,YEZDIAG(1)%MP),&
     & PCLCH,PCLCL,PCLCM,PCLCT,PFPLSL,PFPLSN,PFPLSG,PFPLSHL,PSTRTU,PSTRTV,&
     & PFCS,PFCLL,PFCLN,PUCLS,PVCLS,PTCLS,PQCLS,PRHCLS,PUGST,PVGST,&
     & PFEVL,PFEVN,PCLPH,PSP_SG(1,YSP_SG%YF%MP1),PDIAGH, PSD_SFO(1,YSD_SFO%YSFO(1)%MP) )
  ELSE    
    CALL APL_AROME( KBL, KGPCOMP, KST  , KEND   , NPROMA ,&
     & 1      , NFLEVG , NSTEP  ,&
     & NTSSG   ,  PDTPHY, LLXFUMSE,YDCSGEOM%RINDX,YDCSGEOM%RINDY, &
     & YDGSGEOM%GEMU,YDGSGEOM%GELAM,YDOROG%OROG,YDGSGEOM%GM,ZMU0,ZMU0M,ZMU0N,&
     & YDGSGEOM%GECLO,YDGSGEOM%GESLO,PSD_VC(1,YSD_VC%YVC(1)%MP),PSD_VF(1,YSD_VF%YLSM%MP),&
     & PSD_VA(1,YSD_VA%YSEA%MP ) , PSD_VA(1,YSD_VA%YLAN%MP ) ,PSD_VA(1,YSD_VA%YSOO%MP ) , PSD_VA(1,YSD_VA%YDES%MP ) ,&
     & PSD_VA(1,YSD_VA%YSUL%MP ) , PSD_VA(1,YSD_VA%YVOL%MP ) , &
     & PGP2DSDT, &
     & PHI9,PHIF9,PREPHY9,PREPHY9F,PXYB9(1,1,YYTXYB9_PHY%M_RDELP),PXYB9(1,1,YYTXYB9_PHY%M_DELP),&
     & PGMV(1,1,YT9%MT),PGFL(1,1,YQ%MP9),&
     & PRCP9(1,1,YYTRCP9%M_CP),PRCP9(1,1,YYTRCP9%M_R),PXYB9(1,1,YYTXYB9_PHY%M_ALPH),PXYB9(1,1,YYTXYB9_PHY%M_LNPR), &
     & PGFL(1,1,YL%MP9),PGFL(1,1,YI%MP9),PGFL(1,1,YR%MP9),PGFL(1,1,YS%MP9),&
     & PGFL(1,1,YG%MP9),PGFL(1,1,YH%MP9),PGFL(1,1,YTKE%MP9),PGFL(1,1,YSRC%MP9),PGFL(1,1,YEXT(1)%MP9), &
     & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV), PWT9, &
     & PGPAR,&
     & PEMTD , PEMTU ,PTRSW ,&
     ! outputs
     & PGFLT1(1,1,YLRAD%MP1),PGFLT1(1,1,YIRAD%MP1),PGFLT1(1,1,YA%MP1),PGFLT1(1,1,YSRC%MP1),&
     & ZTENDT, ZTENDGFLR(1,1,1), PTENDU, PTENDV,ZTENDW, ZTENDGFLR(1,1,IPTR(YTKE%MP1)),ZTENDGFLR(1,1,IPTREXT),&
     & PFRTH, PFRSO, PFRTHDS, PFRSODS,PFRTHC,PFRSOC,&
     & PSD_VF(1,YSD_VF%YALBF%MP), PSD_VF(1,YSD_VF%YEMISF%MP),PGFL(1,1,YEZDIAG(1)%MP), &
     & PCLCH,PCLCL,PCLCM,PCLCT,PFPLSL,PFPLSN,PFPLSG,PFPLSHL,PSTRTU,PSTRTV,&
     & PFCS,PFCLL,PFCLN,PUCLS,PVCLS,PTCLS,PQCLS,PRHCLS,PUGST,PVGST,&
     & PFEVL,PFEVN,PCLPH,PSP_SG(1,YSP_SG%YF%MP1),PDIAGH, PSD_SFO(1,YSD_SFO%YSFO(1)%MP) )
  ENDIF 
  !Save surface temperature
  IF (LMSE) THEN
    IF (LLXFUMSE) THEN
      DO JROF=KST,KEND
        PSP_RR(JROF,YSP_RR%YT%MP0)=PGPAR(JROF,MVTS)
      ENDDO
    ELSE
      DO JROF=KST,KEND
        PSP_RR(JROF,YSP_RR%YT%MP1)=PGPAR(JROF,MVTS)
      ENDDO
    ENDIF 
  ENDIF  
  !      4.2  COMPUTE THE PHYS. TENDENCY FOR "T" AND "w"
  !           ------------------------------------------


  IF ((.NOT.LNH_GEOGW).AND.LVERTFE.AND.LVFE_GWMPA) THEN
    ! * case LVFE_GWMPA not yet coded.
    !   (in this case ZGWT1 must be computed at full levels and
    !   not at half levels)
    CALL ABOR1(' MF_PHYS: case LVFE_GWMPA not yet coded if LMPA=T!')
  ENDIF
  IF (LNH_GEOGW.AND.(.NOT.LVERTFE)) THEN
    CALL ABOR1(' MF_PHYS: case LNH_GEOGW.AND.(.NOT.LVERTFE) not yet coded!')
  ENDIF

  ZDT = PDTPHY

  ZTENDD=0.0_JPRB

  ! * compute ZTT1:
  IF (LSLAG.AND.LTWOTL) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZTT1(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MT)+ZDT*ZTENDT(JROF,JLEV)
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZTT1(JROF,JLEV)=PGMV(JROF,JLEV,YT9%MT)+ZDT*ZTENDT(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF

  ! * compute ZGWT1 = tendency of gw:
  IF (LNHDYN) THEN
    IF (LNH_PDVD) THEN
      ! Valid for LVFE_GWMPA=F only; ZGWT1 assumed to be half level values.
      DO JLEV=1,NFLEVG-1
        DO JROF=KST,KEND
          ZGWT1(JROF,JLEV)=0.5_JPRB*RG*(ZTENDW(JROF,JLEV)+ZTENDW(JROF,JLEV+1))
        ENDDO
      ENDDO
      DO JROF=KST,KEND
        ZGWT1(JROF,NFLEVG)=0.0_JPRB
        ZGWT1(JROF,0)=0.0_JPRB
      ENDDO
    ELSEIF (LNH_GEOGW) THEN
      ! Valid for LVERTFE=T only; ZGWT1 assumed to be full level values.
      ! Surface value of ZTENDW is assumed to be 0.
      ZGWT1(KST:KEND,1:NFLEVG)=RG*ZTENDW(KST:KEND,1:NFLEVG)
      ZGWT1(KST:KEND,0)=0.0_JPRB
    ENDIF
  ENDIF

  ! * convert gw tendency in d tendency:
  IF(LNHDYN) THEN
    IF ((LNH_PDVD.AND.LGWADV).OR.LNH_GEOGW) THEN
      ZTENDD(KST:KEND,1:NFLEVG)=ZGWT1(KST:KEND,1:NFLEVG)
    ELSE
      IF (LSLAG.AND.LTWOTL) THEN
        CALL GNHGW2SVDAROME(KST,KEND,PRE0F,PXYB0(1,1,YYTXYB0_PHY%M_LNPR),ZTT1,PREPHY0F,ZGWT1,&
         & ZTENDD)  
      ELSE
        CALL GNHGW2SVDAROME(KST,KEND,PRE9F,PXYB9(1,1,YYTXYB9_PHY%M_LNPR),ZTT1,PREPHY9F,ZGWT1,&
         & ZTENDD)  
      ENDIF
    ENDIF
  ELSE
    ZTENDD=0.0_JPRB
  ENDIF

  !      4.3  PUT THE TENDENCIES IN PB1/GFLT1/GMVT1.
  !           --------------------------------------

  ZTENDGFL(1:NPROMM,1:NFLEVG,1:YGFL%NUMFLDS)=0.0_JPRB
  DO JGFL=1,YGFL%NUMFLDS
    IF (YGFLC(JGFL)%LT1) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEND
          ZTENDGFL(JROF,JLEV,YGFLC(JGFL)%MP1)=ZTENDGFLR(JROF,JLEV,IPTR(YGFLC(JGFL)%MP1))
        ENDDO
      ENDDO
    ENDIF
  ENDDO

  CALL CPUTQY_AROME(NPROMA,KST,KEND,NFLEVG,ZDT,IPGFL,&
       & ISLB1U9,ISLB1V9,ISLB1T9,ISLB1GFL9,ISLB1VD9 , &  
       & ZTENDT, ZTENDGFL, PTENDU, PTENDV, ZTENDD ,&
       & PB1, PGMVT1, PGFLT1)

ENDIF

!     ------------------------------------------------------------------

!*       5.    Final calculations.
!              -------------------

! * Restore the initial value of some pseudo-historical surface buffers
!   if relevant.
IF (LL_SAVE_PHSURF) THEN
  IF(LLACTIVE_HV) PSD_VV(1:NPROMA,YSD_VV%YHV%MP)=ZHV(1:NPROMA)
  IF(LLACTIVE_Z0F) PSD_VF(1:NPROMA,YSD_VF%YZ0F%MP)=ZGZ0F(1:NPROMA)
  IF(LLACTIVE_Z0H) PSD_VV(1:NPROMA,YSD_VV%YZ0H%MP)=ZGZ0HF(1:NPROMA)
  IF(LLACTIVE_PBLH) PSD_VH(1:NPROMA,YSD_VH%YPBLH%MP)=ZPBLH(1:NPROMA)
  IF(LLACTIVE_SPSH) PSD_VH(1:NPROMA,YSD_VH%YSPSH%MP)=ZFHPS(1:NPROMA)
  IF(LLACTIVE_QSH)  PSD_VH(1:NPROMA,YSD_VH%YQSH%MP)=ZQSH(1:NPROMA)
  IF(LCVPRO) THEN
    PGFL(:,:,YUAL%MP)=ZUDAL(:,:)
    PGFL(:,:,YUOM%MP)=ZUDOM(:,:)
    IF(LCDDPRO) THEN
      PGFL(:,:,YDAL%MP)=ZDDAL(:,:)
      PGFL(:,:,YDOM%MP)=ZDDOM(:,:)
    ENDIF
  ENDIF
  IF(YUNEBH%LACTIVE) PGFL(:,:,YUNEBH%MP)=ZUNEBH(:,:)
  IF(YUEN%LACTIVE)   PGFL(:,:,YUEN%MP)=ZENTCH(:,:)
ENDIF

! Store horizontal exchange coefficients (3D turbulence) to SL2 buffers
IF (L3DTURB) THEN
  DO JLEV=1,NFLEVG
    PB2(KST:KEND,MSLB2KAPPAM+JLEV-1)=ZKUROV_H(KST:KEND,JLEV)
    PB2(KST:KEND,MSLB2KAPPAH+JLEV-1)=ZKTROV_H(KST:KEND,JLEV)
  ENDDO
ENDIF

!-------------------------------------------------
! Extract Single Column Model profiles from 3D run or 
! write LFA file for MUSC (1D model)
!-------------------------------------------------
IF(LGSCM.OR.LMUSCLFA) THEN
  CALL WRITEPHYSIO &
       & (KEND, &
       & KST, KGL1, KGL2, KSTGLO, &
       & NSTEP  , NTSSG  , YSP_SBD%NLEVS   ,&
       & YDGSGEOM%GELAM,YDGSGEOM%GEMU, &
       & YDGSGEOM%GM, ZMU0,YDOROG%OROG,POROGL,POROGM,YDGSGEOM%RCORI,YDCSGEOM%RATATH,YDCSGEOM%RATATX, &
       & PHI0  , PRE0  , PHIF0 , PRE0F , PXYB0(1,1,YYTXYB0_PHY%M_ALPH), PXYB0(1,1,YYTXYB0_PHY%M_DELP),&
       & PXYB0(1,1,YYTXYB0_PHY%M_LNPR), PXYB0(1,1,YYTXYB0_PHY%M_RDELP),&
       & PSD_VF(1,YSD_VF%YLSM%MP ), PSD_VV(1,YSD_VV%YARG%MP), PSD_VV(1,YSD_VV%YSAB%MP), &
       & PSD_VV(1,YSD_VV%YD2%MP), PSD_VV(1,YSD_VV%YIVEG%MP), PSD_VV(1,YSD_VV%YLAI%MP), PCT, &
       & PSD_VV(1,YSD_VV%YALV%MP ), PALB, PSD_VF(1,YSD_VF%YALBF%MP), PSD_VF(1,YSD_VF%YALBSF%MP), PSP_SG(1,YSP_SG%YT%MP1),&
       & PSD_VF(1,YSD_VF%YEMISF%MP), &
       & PSD_VF(1,YSD_VF%YZ0F%MP), PSD_VV(1,YSD_VV%YZ0H%MP), PSD_VF(1,YSD_VF%YZ0RLF%MP), &
       & PSD_VF(1,YSD_VF%YGETRL%MP), PSD_VF(1,YSD_VF%YVRLAN%MP), PSD_VF(1,YSD_VF%YVRLDI%MP), PSD_VV(1,YSD_VV%YRSMIN%MP), &
       & PSD_VF(1,YSD_VF%YVEG%MP), PSD_VV(1,YSD_VV%YHV%MP), PSD_VA(1,YSD_VA%YSEA%MP), PSD_VA(1,YSD_VA%YLAN%MP), &
       & PSD_VA(1,YSD_VA%YSOO%MP), PSD_VA(1,YSD_VA%YDES%MP), PSD_VC(1,YSD_VC%YVC(1)%MP), &
       & PGMVS(1,YT0%MSP),PGMVS(1,YT0%MSPL),PGMVS(1,YT0%MSPM),&
       & PGMV(1,1,YT0%MT),PGMV(1,1,YT0%MTL),PGMV(1,1,YT0%MTM),&
       & PGFL(1,1,YQ%MP),PGFL(1,1,YQ%MPL),PGFL(1,1,YQ%MPM), &
       & PGFL(1,1,YI%MP),PGFL(1,1,YL%MP), PGFL(1,1,YS%MP),PGFL(1,1,YR%MP),PGFL(1,1,YG%MP),PGFL(1,1,YTKE%MP),&
       & PRCP0(1,1,YYTRCP0%M_CP), PRCP0(1,1,YYTRCP0%M_R),&
       & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MVOR),PGMV(1,1,YT0%MDIV), ZCVGQ, ZLCVQ, &
       & PSP_RR(1,YSP_RR%YT%MP9), PSP_SB(1,1,YSP_SB%YT%MP9), PSP_RR(1,YSP_RR%YFC%MP9), PSP_RR(1,YSP_RR%YW%MP9), &
       & PSP_RR(1,YSP_RR%YIC%MP9), PSP_SB(1,1,YSP_SB%YQ%MP9), PSP_SB(1,1,YSP_SB%YTL%MP9), PSP_SG(1,YSP_SG%YF%MP9), &
       & PSP_SG(1,YSP_SG%YA%MP0), PSP_SG(1,YSP_SG%YR%MP0), PCTY0(1,1,YYTCTY0%M_VVEL),&
       & PEMTD , PEMTU ,PTRSW ,YDGSGEOM%GECLO,YDGSGEOM%GESLO, &
       & PDIFCQ, PDIFCQI, PDIFCQL, PDIFCS, PDIFTQ, PDIFTQI, PDIFTQL, &
       & PDIFTS, PFCCQL, PFCCQN, PFCSQL, PFCSQN, PFCQNG, PFCQING, &
       & PFCQLNG, PFCQRNG, PFCQSNG, PFPLCL, PFPLCN, PFPLSL, PFPLSN, PFPLSG, PFPLSHL, &
       & PFPFPSL, PFPFPSN, PFPFPCL, PFPFPCN, PFPEVPSL,PFPEVPSN,PFPEVPCL, PFPEVPCN,&
       & ZFTKE, PFRSO, PFRSOC, PFRTH, PFRTHC, PFRSOLU, &
       & PSTRCU, PSTRCV, PSTRDU, PSTRDV, PSTRTU, PSTRTV, PSTRMU, PSTRMV, &
       & PFRMH, ZFRMQ, PFCHOZ, PNEB, PQICE, PQLI, PRH, &
       & PFCS, PFCLL, PFCLN, PFEVL, PFEVN, ZFEVI, PFEVV, PFTR, PFLWSP, PFONTE, &
       & PFGEL, PFGELS, PFCHSP, PFRSODS, PFRSOPS, PFRSOPT, PFRTHDS, &
       & ZCD, ZCDN, ZCH, ZC1, ZC2, ZEMIS, PGZ0   , PGZ0H  , ZNEIJ  , ZVEG, &
       & ZCPS, ZLHS, ZRS, ZLH, ZLSCPE, ZQSAT, ZQW, ZTW, &
       & PQS, ZQSATS, PRUISL, PRUISP, PRUISS, &
       & PUCLS, PVCLS, PTCLS, PQCLS, PRHCLS, &
       & PCLCT, PCLCH, PCLCM, PCLCL, PCLCC, &
       & PCAPE  , ICLPH  , PCLPH  , PUGST  , PVGST, &
       & ZFPLCH, ZFPLSH, &
       & PSD_VH(1,YSD_VH%YTCCH%MP),  PSD_VH(1,YSD_VH%YSCCH%MP), PSD_VH(1,YSD_VH%YBCCH%MP), PSD_VH(1,YSD_VH%YPBLH%MP) )
ENDIF

!     ------------------------------------------------------------------
IF (LMUSCLFA) CALL LFAFER(IMUSCLFA)

IF (LHOOK) CALL DR_HOOK('MF_PHYS',1,ZHOOK_HANDLE)
END SUBROUTINE MF_PHYS
