!OPTIONS XOPT(NOEVAL)
SUBROUTINE SUPHY0(KULOUT)

!**** *SUPHY0*   - Initialize common YOMPHY0 physics atmospheric
!                  constants

!     Purpose.
!     --------
!           Initialize YOMPHY0, the common that contains the parameters
!           for the atmospheric part of the physics of the model.

!**   Interface.
!     ----------
!        *CALL* *SUPHY0(...)

!        Explicit arguments :
!        --------------------
!        KULOUT : Logical unit for the output

!        Implicit arguments :
!        --------------------
!        COMMON YOMPHY0

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Documentation ARPEGE

!     Author.
!     -------
!        J.-F. Geleyn .
!        Original : 90-9-1

!     Modifications.
!     --------------
!        Modified by J.M. Piriou: 2001-04-05 (introduce GCVMLT).
!        Modified by Y. Bouteloup:2002-06-14 (introduce RCVEVAP).
!        Modified by F. Bouyssel: 2002-06-25 (introduce UTILGUST, RRGAMMA, RRSCALE).
!        Modified by J.M. Piriou: 2002-08-19 (introduce GPBLH*).
!        Modified by D. Banciu:   2002-12-09 (introduce GCVPSIE).
!        R. El Khatib : 2001-08-07 Pruning options
!        J.M. Piriou  : 2002-01-10 set default values to operational ones.
!        03-06, move rnlcurv rnegat into yomphy0 (F. Bouyssel, C. Fischer)
!        E. Bazile : 2004-02-24 (introduce EDK).
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        E. Bazile : 2004-06-30 (inroduces XKLM).
!        R. Brozkova : 2004-11 modifs for Xue-Randall cloud. scheme
!        P. Marquet & F.Bouyssel : 2004-08-18 (Lopez)
!        F.Bouyssel : 2005-07-18 (new defaults and new parameters for Lopez)
!        F.Bouyssel : 2006-01-25 (new defaults for Lopez!)
!        F. Vana    : 2006-01-30 tunables for pTKE.
!        R.Brozkova : 2006-03-03 : tuning constants for mixing lengths and Charnock formulae
!        A.Alias    : 2006-03-10 renaming KDN in RKDN
!        Modified by GMGEC/EAC  : 2006-05 list of modif.
!                    V. Lorant  : 99-01-05 (new mixing length profile).
!                    M. Deque   : 00-03-21 (new entrainment rate VL  ).
!                    P. Marquet : 2002-11-05 TRENTRVL=0. if not ACPBLH
!                    P. Marquet : 2004-05-27 TFVR and TFVS for ADVPRC.
!                    P. Marquet : 2004-10-13 RAUTEFR for snow (ACMICRO).
!                    P. Marquet : 2004-10-14 RAUTSBET for snow (ACMICRO).
!                    A. Alias   : 2005-06-23 (param. for ACCVIMPGDY)
!        E. Bazile & P. Marquet : 2006-04-11 AGRE1,AGRERICR,AJBUMIN,RCOFLM
!                                  for LPBLE.
!        F.Bouyssel : 2006-10-30 RQLCV,RQICVMAX,RQICVMIN,RHEVAP
!        M. Bellus  : 03-Oct-2006 : ALARO-0 phasing (defaults for prognostic
!                                   convection, pTKE and PIL microphysics)
!        A.Alias     : 2006-07   FEVAPC added for ACCVIMPGY (JF Gueremy)
!        F.Bouyssel : 2007-05-21 Change writing format of some variables
!        E. Bazile   : 2006-10   RTKFBCVPP added for LCVPPKF and GCVHMIN.
!        A.Alias     : 2007-03   RLMLHx tuning for ACNEBR (P.Marquet)
!        A.Alias     : 2007-10   Add printing TRENTRV/TENTRVL
!        E. Bazile   : 2008-04-28 Coef. for thermal production (LECT)
!        F. Bouyssel : 2008-06 Coef. for KFB shallow clouds
!        Y. Bouteloup: 2008-07 Change implicit value for TKE tuning parameters
!                              RTKFBCVPP becomes RKFBTAU
!        F. Bouyssel : 2008-10 Coef. for ECUME (RFLCHCE)
!        F. Vana     : 21-Jan-2009 tuning parameters for p/eTKE
!        C. Wittmann : 2009-03 WMXOV for LACPANMX
!        F. Bouyssel : 2009-11 ECTMAX for CBR scheme
!        I. Bastak Duran + F. Vana : 12-Oct-2009  p/eTKE tunables update
!        K. Yessad (Jan 2010): remove useless variables.
!        F. Bouyssel : 2010-04 New sedimentation coefficients
!        F. Bouyssel : 2010-05 Parameters for saturation deficit
!        I. Bastak Duran + F. Vana : 22-Feb-2011  TOUCANS defaults
!        F. Bouyssel : 2011-01 Intro. GCVOMGE, GCVOMGQ
!        F. Bouyssel : 2011-03 Intro. GCVOMGS, GCVOMGP
!        O. Riviere  : 2011-06 Intro. RSMOOTHMELT
!        J.M. Piriou : 2011-09-21 Introduce PCMT convection scheme parameters.
!        JF. Gueremy : 2011-11 Dry Conv. Adj. (AJ1PEPS,AJ1MEPS,NAJITER)
!        S.Martinez  : 2011-09 put writes related to "Alaro TOUCANS/pTKE scheme" under key LCOEFKTKE
!        I. Bastak-Duran, F. Vana : 26-Jan-2012 Update of Toucans/pTKE setup
! ------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULNAM
USE YOMCST   , ONLY : ROMEGA
USE YOMCT0B  , ONLY : LECMWF
USE YOMSIMPHL, ONLY : LSIMPH   ,LSMOOTHD ,LSMOOTHA
USE YOMPHY   , ONLY : LCVCAS   ,LCVLIS   ,LCVRA    ,LMPHYS   ,LCONDWT  ,&
 & LPROCLD  ,LNEBCO   ,LNEBR    ,LECT     ,LNEBGR   ,LCVRAV3 ,L3MT,&
 & LENTCH   ,CGTURS ,LNEBNXR,LCOEFKTKE
USE YOMPHY0  , ONLY : TYM      ,NPCLO1   ,NPCLO2   ,AERCS1   ,AERCS3   ,&
 & AERCS5   ,ALMAV    ,BEDIFV   ,ECMNP    ,EDB      ,EDC      ,&
 & EDD      ,EVAP     ,FONT     ,GALP     ,GCCSV    ,GCOMOD   ,&
 & GCVADS   ,GCVALFA  ,GCVBETA  ,GCVPSI   ,GDDEVA   ,GDDSDE   ,&
 & GRCVPP   ,GWDAMP   ,GWDBC    ,GWDCCO   ,GWDCD    ,GWDLT    ,&
 & AHCLPV   ,GWDSE    ,HOBST    ,HUCOE    ,HUTIL    ,&
 & HUTIL1   ,HUTIL2   ,QSMODC   ,&
 & QSSC     ,QSMIN    ,QSNEBC   ,QSNEBS   ,QSSUSC   ,QSSUSS   ,&
 & QXRAL    ,QXRDEL   ,QXRR     ,REVGSL   ,RTCAPE   ,SCO      ,&
 & SENSL    ,SNNBCO   ,SPNBCO   ,SXNBCO   ,TCA      ,TCT      ,&
 & TCW      ,TENTR    ,TENTRX   ,TDDGP    ,TUDGP    ,TURB     ,&
 & TVF      ,UHDIFV   ,USDMLT   ,USUPRC   ,USURIC   ,USURID   ,&
 & USURIDE  ,USURICE  ,USURICL  ,VCHRNK   ,VKARMN   ,VZ0CM    ,&
 & XNBMAX   ,RICRLM   ,XBLM     ,XMINLM   ,XMAXLM   ,XWSALM   ,&
 & XWSBLM   ,GCVNU    ,GCVMLT   ,RCVEVAP  ,GPBLHK0  ,GPBLHRA  ,&
 & UTILGUST ,RRGAMMA  ,RRSCALE  ,GCVPSIE  ,QSUSXC   ,QSUSXS   ,&
 & ETACUT   ,RNEGAT   ,RNLCURV  ,QSSUSV   ,QXRHX    ,GCISMIN  ,&
 & GWDPROF  ,GWDVALI  ,RCIN     ,EDK      ,XKLM     ,RPHI0    ,&
 & RPHIR    ,QXRTGH   ,ADISE    ,ADISI    ,AECLS3   ,AECLS4   ,&
 & AKN      ,ALD      ,ALPHAE   ,ALPHAT   ,ECTMIN   ,UCWSTAR  ,&
 & UDECT    ,USHEARM  ,UPRETMIN ,UPRETMAX ,ARSCH    ,ARSCQ    ,&
 & ARSC1    ,ARSB2    ,ACBRPHIM ,ALMAVE   ,ALMAVX   ,RICRET   ,&
 & RDTFAC   ,RAUTEFS  ,RNINTR   ,RNINTS   ,RQLCR    ,&
 & RQICRMAX ,RQICRMIN ,RACCEF   ,RRIMEF   ,RHCRIT1  ,RHCRIT2  ,&
 & RETAMIN  ,TFVR     ,TFVS     ,RAUTEFR  ,RAUTSBET ,GRHCMOD  ,&
 & RQICRT1  ,RQICRT2  ,RQICRSN  ,RQCRNS   ,RFACNSM  ,RAGGEF   ,&
 & RQLCV    ,RQICVMAX ,RQICVMIN ,REVASX   ,STTBMIN  ,RHCEXPDX ,&
 & AGRE1    ,AGRERICR ,AJBUMIN  ,RCOFLM   ,QXRAL_ADJ,&
 & A0ML_AU  ,A0ML_AT  ,A0ML_BU  ,A0ML_BT  ,VZIUSTAR0,&
 & TENTRVL  ,TRENTRV  ,&
 & AGRE2    ,AGREF    ,RGELSFON ,RDECRD   ,&
 & AJ1PEPS  ,AJ1MEPS  ,NAJITER  ,ADJTAU   ,REVAPN    ,&
 & ALFX     ,TCTC     ,TVFC     ,GAMAP1   ,RKDN      ,&
 & VVN      ,VVX      ,FENTRT   ,FQLIC    ,FNEBC    ,&
 & NUPTKE   ,TDDBU    ,RMULACVG ,&
 & TDDFR    ,TUDBU    ,TUDFR    ,GCVACHI  ,GCVALMX  ,&
 & GCVBEE   ,GCVEEX   ,GCVTAUDE ,ECMNPI   ,&
 & GRRMINA  ,GDDBETA  ,GDDDP    ,GDDFXM   ,&
 & GDDEVF   ,GDDWPF   ,TFVL     ,TFVI     ,&
 & GPEIPHI  ,GPETAU   ,GENVSRH  ,GPEFDC   ,&
 & TENTRD   ,RDPHIC   ,RWBF1    ,RWBF2    ,&
 & RSMDNEBX ,RSMDTX   ,NSMTPA   ,NSMTPB   ,&
 & SCLESPR  ,SCLESPS  ,HUCRED   ,FSPRAIN  ,EFFCOLL  ,&
 & FEVAPC   ,GCVHMIN  ,RLMLH1   ,&
 & TKEMULT  ,C_EPSILON,C3TKEFREE,ETKE_RIFC,ETKE_R   ,ETKE_LAM1,ETKE_LAM2  ,&
 & ETKE_LAM3,ETKE_LAM4,ETKE_CTHETA,ENT_LAMBDA,ETKE_RIFC_MAF,&
 & ETKE_KA1,ETKE_KA2,ETKE_KA3,ETKE_CG01,ETKE_CG02,ETKE_CG03,&
 & ETKE_BETA_EPS,ETKE_GAMMA_EPS,ETKE_KLM,ETKE_TAULM,&
 & ETKE_GAMD,ETKE_TALPH,ETKE_ALPH0,ETKE_OLAM,ETKE_LAM0,ETKE_ENTRX,ETKE_ENTRN,&
 & RPRTH    ,RKFBTAU  ,RKFBNBX  ,RFLCHCE ,WMXOV ,&
 & ECTMAX   ,GCVOMGE  ,GCVOMGQ  ,GCVOMGP  ,GCVOMGS,RSMOOTHMELT, &
 & LGMT     ,LGDDD    ,LGCRUC   ,LGPSMI   ,NCVENTR   ,GCVENTRN , GCVENTR_MIN, &
 & GCVENTRX ,ADOE,RFRAC_ENTR_N,RFRAC_ENTR_X,THPCMT, &
 & EFB_AZ0, EFB_UR, REFB_1, REFB_2, REFB_3
USE YOMQNSE  , ONLY : RQNSE_S0
USE YOMLOUIS , ONLY : RLOUIS_S0

! ------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT
INTEGER(KIND=JPIM) :: ITTYPE
REAL(KIND=JPRB) :: ZNUPTKE4,ZLAMBDA,ZSF,ZLAMF_X,ZSM0
REAL(KIND=JPRB) :: ZHOOK_HANDLE
CHARACTER(LEN=200) :: CLERR

! ------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"

#include "namphy0.h"

! ------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUPHY0',0,ZHOOK_HANDLE)
! ------------------------------------------------------------------------

!*       1.    Set default values.
!              -------------------

!        1.1 Set implicit default values

A0ML_AU=4.5_JPRB
A0ML_AT=5.0_JPRB
A0ML_BU=3.0_JPRB
A0ML_BT=0.8_JPRB
AERCS1=0.2726_JPRB
AERCS3=-0.4239_JPRB
AERCS5=0.3595_JPRB
ALMAV=300._JPRB
BEDIFV=0.05_JPRB
ECMNP=3000._JPRB
EDB=5._JPRB
EDC=5._JPRB
EDD=5._JPRB
EDK=1.0_JPRB
EFB_AZ0=0.2_JPRB
EFB_UR=0.4545_JPRB
ETACUT=1.0_JPRB
EVAP=4.8E+06_JPRB
FONT=2.4E+04_JPRB
GALP=0.53_JPRB
GCISMIN=6.7E-05_JPRB
GCCSV=0._JPRB
GCOMOD=1._JPRB
GCVADS=0.8_JPRB
GCVALFA=4.5E-05_JPRB
GCVBETA=0.2_JPRB
GCVMLT=0.00016_JPRB
GCVNU=2.5E-05_JPRB
GCVPSI=0.5_JPRB
GCVPSIE=0.0_JPRB
GDDEVA=0.25_JPRB
GDDSDE=0.5_JPRB
GPBLHK0=0.25_JPRB
GPBLHRA=4._JPRB
GRCVPP=1._JPRB
GWDAMP=0.6_JPRB
GWDBC=2._JPRB
GWDCCO=1._JPRB
GWDCD=6._JPRB
GWDLT=0._JPRB
GWDSE=3.5E-03_JPRB
GWDPROF=1._JPRB
GWDVALI=0._JPRB
HOBST=3._JPRB
HUCOE=2._JPRB
HUTIL=1.8_JPRB
HUTIL1=-0.6_JPRB
HUTIL2=1.1_JPRB
NPCLO1=1
NPCLO2=1
QSSC=1600._JPRB
QSMIN=1.E-4_JPRB
QSMODC=1._JPRB
QSNEBC=26000._JPRB
QSNEBS=0.7_JPRB
QSSUSC=1._JPRB
QSSUSS=0.25_JPRB
QSSUSV=0._JPRB
QSUSXC=3.3E-05_JPRB
QSUSXS=3.3E-05_JPRB
RDECRD=2.E+04_JPRB
RGELSFON=0.0125_JPRB
REVAPN=1._JPRB
RPHI0=0._JPRB
RPHIR=1750._JPRB
QXRAL=130._JPRB
QXRAL_ADJ=130._JPRB
QXRDEL=0._JPRB
QXRHX=1._JPRB
QXRR=0.5_JPRB
QXRTGH=3.5_JPRB
RCIN=0._JPRB
RCVEVAP=0._JPRB
REVGSL=80._JPRB
REFB_1=9.0_JPRB
REFB_2=5.0_JPRB
REFB_3=-11.0_JPRB
RFLCHCE=0._JPRB
RTCAPE=10800._JPRB
SCO=-20._JPRB
SENSL=1._JPRB
SNNBCO=0._JPRB
SPNBCO=3000._JPRB
SXNBCO=0.5_JPRB
TCA=1._JPRB
TCT=1.E-4_JPRB
TCW=8.E-4_JPRB
TENTR=2.5E-06_JPRB
TENTRX=8.E-05_JPRB
TDDGP=0.8_JPRB
TUDGP=0.8_JPRB
TURB=1._JPRB
TVF=1._JPRB
TYM(1)=0.92_JPRB
TYM(2)=0.74_JPRB
TYM(3)=16.6_JPRB
TYM(4)=10.1_JPRB
TYM(5)=0.08_JPRB
UHDIFV=8.E-04_JPRB
USDMLT=1.25E+04_JPRB
USUPRC=0.0_JPRB
USURIC=1.0_JPRB
USURID=0.035_JPRB
USURIDE=1.0_JPRB
USURICE=0.5_JPRB
USURICL=4._JPRB
UTILGUST=0.125_JPRB
VCHRNK=0.021_JPRB
VKARMN=0.4_JPRB
VZ0CM=1.5E-04_JPRB
VZIUSTAR0=0._JPRB
XNBMAX=1._JPRB
AHCLPV=1000._JPRB
RICRLM=0.5_JPRB
RRGAMMA=0.8_JPRB
RRSCALE=1.15E-4_JPRB
XBLM=6.5_JPRB
XKLM=1.0_JPRB
XMAXLM=3000._JPRB
XMINLM=500._JPRB
XWSALM=0.1_JPRB
XWSBLM=7.0_JPRB
RNEGAT = -7.E-05_JPRB
RNLCURV = 7.E+04_JPRB
WMXOV=1.0_JPRB

!     - - - - - - - - - - - - - -
!     The old Convective scheme :
!     - - - - - - - - - - - - - -
TRENTRV=1._JPRB
TENTRVL=-1.0_JPRB

IF ( LCONDWT.AND.LPROCLD ) THEN
  RDTFAC=0.5_JPRB
ELSE
  RDTFAC=1.0_JPRB
ENDIF

!        - - - - - - - - - -
!        Lopez Microphysics :
!        - - - - - - - - - -
RAUTEFR=1.E-03_JPRB
RAUTEFS=1.E-03_JPRB
RAUTSBET=0.025_JPRB
RNINTR=8.E+06_JPRB
RNINTS=2.E+06_JPRB
RQLCR=2.E-04_JPRB
RQICRMAX=0.3E-04_JPRB
RQICRMIN=0.2E-06_JPRB
RQLCV=2.E-04_JPRB
RQICVMAX=0.3E-04_JPRB
RQICVMIN=0.2E-06_JPRB
RQICRT1=-80._JPRB
RQICRT2=30._JPRB
RQICRSN=0.5_JPRB
RQCRNS=0.03_JPRB
RACCEF=1._JPRB
RAGGEF=0.2_JPRB
RRIMEF=1._JPRB
REVASX=0.0_JPRB
RHCRIT1=0.5_JPRB
RHCRIT2=0.91_JPRB
RETAMIN=0.4_JPRB
RFACNSM=1.4_JPRB
TFVL=0.0_JPRB
TFVI=0.0_JPRB
TFVR=5.0_JPRB
TFVS=0.6_JPRB
GRHCMOD=0.3_JPRB
RHCEXPDX=0.3_JPRB


!--------------------------
! PIL MICROPHYSICS
!--------------------------
ADJTAU=900._JPRB
RDPHIC=10000._JPRB
RWBF1=300._JPRB
RWBF2=4._JPRB
SCLESPR=248000._JPRB
SCLESPS=15500._JPRB
HUCRED=1.0_JPRB
FSPRAIN=13.4_JPRB
EFFCOLL=6.7E-02_JPRB
RSMDNEBX=0.2_JPRB
RSMDTX=1.0_JPRB
NSMTPA=2
NSMTPB=3
!--------------------------
! Prognostic convection physical parameters
!--------------------------
TUDBU=0.5_JPRB
TDDBU=0.5_JPRB
TUDFR=0.0012_JPRB
TDDFR=0.0006_JPRB
GCVALMX=0.95_JPRB
RMULACVG=25._JPRB
GCVACHI=1.E9_JPRB
GCVBEE=0._JPRB
GCVEEX=1._JPRB
GCVTAUDE=900._JPRB
ECMNPI=3000._JPRB
GPEIPHI=0._JPRB
GPETAU=1._JPRB/ROMEGA
GENVSRH=0._JPRB
GPEFDC=0.18_JPRB
GRRMINA=1.E-5_JPRB
GDDBETA=2.0_JPRB
GDDDP=10000._JPRB
GDDFXM=1.E+5_JPRB
GDDEVF=0.5_JPRB
GDDWPF=0._JPRB
TENTRD=1.E-4_JPRB

!-------------------------------------------------
! PCMT convection scheme parameters.
!-------------------------------------------------
LGMT=.TRUE.
LGDDD=.FALSE.
LGCRUC=.FALSE.
LGPSMI=.FALSE.
NCVENTR=1
GCVENTRN=1.2E-4_JPRB
GCVENTRX=6.0E-4_JPRB
GCVENTR_MIN=0._JPRB
THPCMT=-1._JPRB
ADOE=1.0_JPRB
RFRAC_ENTR_N=1.E-4_JPRB
RFRAC_ENTR_X=3.E-3_JPRB

!     - - - - - - - -
!     For TKE scheme :
!     - - - - - - - -
ADISE=-0.5_JPRB
ADISI=1.5_JPRB
AECLS3=3.75_JPRB
AECLS4=0.3_JPRB
AKN=0.126_JPRB
ALD=1.18_JPRB
ALPHAE=2.7_JPRB
ALPHAT=1.13_JPRB
ECTMIN=1.E-6_JPRB
ECTMAX=100._JPRB

UCWSTAR=1.0_JPRB/3._JPRB
UDECT=5._JPRB
USHEARM=1.E-04_JPRB
UPRETMIN=60000._JPRB
UPRETMAX=97500._JPRB
ARSCH=4._JPRB
ARSCQ=1.2_JPRB
ARSC1=2.0_JPRB/(3._JPRB*ARSCH*ARSCQ)
ARSB2=3._JPRB*ARSCH*ARSC1/2.0_JPRB
ACBRPHIM=2.2_JPRB
ALMAVE=10._JPRB
ALMAVX=1000._JPRB
RICRET=0.195_JPRB
GCVHMIN=0.0_JPRB
GCVOMGE=0.0_JPRB
GCVOMGQ=0.0_JPRB
GCVOMGS=0.5_JPRB
GCVOMGP=2000._JPRB

!     ---------------------------
!     For KFB CVPP d'AROME
!     ---------------------------
RKFBTAU=0._JPRB
RKFBNBX=0.3_JPRB

STTBMIN=SQRT(3._JPRB)
! A minimum value for the adimentional jump in boyancy : d(Theta)/Theta
AJBUMIN=0.005_JPRB
! The "Master Length" is equal to "RCOFLM*Z_PBL"
RCOFLM=0.085_JPRB

!     - - - - - - - - - - - - - - - -
!     For dry conv. adjustment scheme :
!     - - - - - - - - - - - - - - - -
AJ1MEPS=0.99_JPRB
AJ1PEPS=10.0_JPRB
NAJITER=30

!     - - - - - - - - - - - - - - - - - - - - - - -
!     For Grenier (2000) top-PBL entrainment scheme :
!     - - - - - - - - - - - - - - - - - - - - - - -
AGRE1=0.16_JPRB
AGRE2=100._JPRB
AGREF=1._JPRB
AGRERICR=50._JPRB

! Pseudo prognostic TKE scheme
NUPTKE=0.52_JPRB
! tuning par for TKE-related mixing length
TKEMULT=1.0_JPRB
! TOUCANS scheme
C_EPSILON =0.871_JPRB
ETKE_CTHETA =1.0_JPRB
ETKE_RIFC_MAF =0.5_JPRB
ETKE_RIFC =0.186_JPRB
ETKE_R    =0.367_JPRB
ETKE_LAM1 =0.107_JPRB
ETKE_LAM2 =0.00032_JPRB
ETKE_LAM3 =0.0864_JPRB
ETKE_LAM4 =0.1_JPRB
ETKE_OLAM =2.0_JPRB/3.0_JPRB
ETKE_LAM0 =2.0_JPRB/3.0_JPRB
!  RV, RESTT and RCPD to be used from from YOMCST
!ENT_LAMBDA=(3545._JPRB+RV*LOG(100000._JPRB/RESTT))/RCPD
ENT_LAMBDA=5.87_JPRB
ETKE_CG01 =1.0_JPRB
ETKE_CG02 =1.0_JPRB
ETKE_CG03 =1.0_JPRB
ETKE_BETA_EPS =0.02_JPRB
ETKE_GAMMA_EPS =3E-2_JPRB
ETKE_GAMD =2.0_JPRB
ETKE_TALPH =200.0_JPRB
ETKE_ALPH0 =100.0_JPRB
ETKE_ENTRX=1E-3_JPRB
ETKE_ENTRN=1E-4_JPRB
ETKE_KLM=1.0_JPRB
ETKE_TAULM=1.0_JPRB
C3TKEFREE =1.184_JPRB

!     - - - - - - - - - - - - -
!     For ACCVIMPGY and PCMT :
!     - - - - - - - - - - - - -
ALFX=10.E-02_JPRB
TCTC=5.0E-04_JPRB
TVFC=1._JPRB
GAMAP1=1.5_JPRB
RKDN=30.E-06_JPRB
VVN=-2.0_JPRB
VVX=-38.0_JPRB
FENTRT=2.5_JPRB
FQLIC=1.0_JPRB
FNEBC=7.0_JPRB
FEVAPC=4.0_JPRB

!     For the "Lenderink & Holtslag" mixing Length : (ACNEBR)
RLMLH1  = 0.4_JPRB

RPRTH   = 0.0_JPRB

! Smoothing of melting of snow in advprcs.F90

RSMOOTHMELT=0.6

!        1.2 Modify default values according to LECMWF

IF (LECMWF) THEN
ELSE
ENDIF

!     ------------------------------------------------------------------

!*       2.    Modify default values.
!              ----------------------

CALL POSNAM(NULNAM,'NAMPHY0')
READ(NULNAM,NAMPHY0)


IF(.NOT.(LNEBCO.AND.(LNEBR.OR.LNEBGR.OR.LECT)) .AND.LCVRAV3) THEN
  WRITE(UNIT=KULOUT,FMT='(A)') ' '
  WRITE(UNIT=KULOUT,FMT='(A)') ' !'
  WRITE(UNIT=KULOUT,FMT='(A)') ' ! TENTRVL is set to 0. in SUPHY0  !!'
  WRITE(UNIT=KULOUT,FMT='(A)') ' !'
  WRITE(UNIT=KULOUT,FMT='(A)') ' '
  TENTRVL = 0.0_JPRB
ENDIF

! Computation of constants for TOUCANS scheme (Alaro physics)

IF(LCOEFKTKE) THEN
 IF     (TRIM(CGTURS) == 'RMC') THEN
  ITTYPE=1
 ELSEIF (TRIM(CGTURS) == 'MD1') THEN
  ITTYPE=2
 ELSEIF (TRIM(CGTURS) == 'MD2') THEN
  ITTYPE=3
 ELSEIF (TRIM(CGTURS) == 'QNSE') THEN
  ITTYPE=4
 ELSEIF (TRIM(CGTURS) == 'EFB') THEN
  ITTYPE=5
 ELSEIF (TRIM(CGTURS) == 'LOUIS') THEN
  ITTYPE=6
 ELSE
  CLERR='SUPHY0: UNEXPECTED VALUE FOR CGTURS: '//TRIM(CGTURS)
  CALL ABOR1(CLERR)
 ENDIF
ENDIF

ZNUPTKE4=NUPTKE**4
ZSM0=ZNUPTKE4/2._JPRB
ZLAMF_X=SQRT(375.0_JPRB*C3TKEFREE**2*(C3TKEFREE**2+2._JPRB*C3TKEFREE+1._JPRB)*ZSM0**2 &
  & -20.0_JPRB*(C3TKEFREE**2+C3TKEFREE-1._JPRB)*ZSM0+1.0_JPRB)
ZLAMBDA=SQRT(5.0_JPRB)*3.0_JPRB/4.0_JPRB*ZLAMF_X-30.0_JPRB*C3TKEFREE &
  & *(C3TKEFREE+1._JPRB)*ZSM0+0.25_JPRB
ZSF=(10.0_JPRB-75.0_JPRB*C3TKEFREE*(C3TKEFREE+1._JPRB)*ZSM0)/(8.0_JPRB*ZLAMBDA)

ETKE_LAM1=4.0_JPRB/15.0_JPRB*ZLAMBDA
ETKE_LAM3=(14.0_JPRB*ZLAMBDA*ZSF+5.0_JPRB*ZLAMBDA)/75.0_JPRB
ETKE_LAM2=(5.0_JPRB*ZLAMBDA-6.0_JPRB*ZLAMBDA*ZSF)/25.0_JPRB

IF(ITTYPE==1) THEN
  !RMC01
  ETKE_R=1.0_JPRB
  ETKE_LAM4=0.0_JPRB
  ETKE_RIFC=0.544_JPRB
ELSE IF (ITTYPE==2) THEN
  ! Model I
  ETKE_R=1.0_JPRB-2.0_JPRB*ZLAMBDA*(44.0_JPRB*ZSF**2+160.0_JPRB*ZSF-25.0_JPRB)/375.0_JPRB
  ETKE_LAM4=0.0_JPRB
  ETKE_RIFC=(15.0_JPRB*ZSM0*C3TKEFREE**2*(C3TKEFREE+1.0_JPRB))&
  & /(6.0_JPRB*ETKE_OLAM+2.0_JPRB*C3TKEFREE)
ELSE IF ((ITTYPE==3).OR.(ITTYPE==4)) THEN
  ! Model II or QNSE
  ETKE_R=-4.0_JPRB*C3TKEFREE/75.0_JPRB*&
    &(88.0_JPRB*ZLAMBDA*ZSF**2+320.0_JPRB*ZLAMBDA*ZSF-50.0_JPRB*ZLAMBDA-375.0_JPRB)/&
    &(8.0_JPRB*ZLAMBDA*C3TKEFREE*ZSF+10.0_JPRB*ZLAMBDA*C3TKEFREE+20.0_JPRB*C3TKEFREE+15.0_JPRB)
  ETKE_LAM4=ZLAMBDA/4.0_JPRB
  ETKE_RIFC=(30_JPRB*C3TKEFREE**3*ZSM0)/&
  &(12.0_JPRB*ETKE_OLAM+(-60.0_JPRB*C3TKEFREE**2*ETKE_LAM3-20.0_JPRB*C3TKEFREE**2&
  &*ETKE_LAM2+4.0_JPRB*C3TKEFREE)*ETKE_LAM4-15.0_JPRB*C3TKEFREE**2*ETKE_LAM1     &
  & +4.0_JPRB*C3TKEFREE)
ELSE IF (ITTYPE==5) THEN
!EFB
  ETKE_R=EFB_UR
  ETKE_LAM4=0.0_JPRB
  ETKE_RIFC=0.25_JPRB
ELSE IF (ITTYPE==6) THEN
!LOUIS
  ETKE_R=RLOUIS_S0
  ETKE_LAM4=0.0_JPRB
  ETKE_RIFC=0.91_JPRB
ENDIF

IF ((ITTYPE==1.OR.ITTYPE==2.OR.ITTYPE==3).AND.ETKE_R<ETKE_RIFC) THEN
  CLERR='SUPHY0: ETKE_R must be greater than ETKE_RIFC. &
  & Modify NUPTKE, ETKE_OLAM, C3TKEFREE'
  CALL ABOR1(CLERR)
ENDIF

ETKE_CTHETA=C_EPSILON*C3TKEFREE/ETKE_OLAM

ETKE_KA1 = ZLAMBDA/2.0_JPRB
ETKE_KA2 = ETKE_OLAM/(2.0_JPRB*C3TKEFREE)
ETKE_KA3 = 1.0_JPRB


!*       3.    Print final values.
!              -------------------

WRITE(UNIT=KULOUT,FMT='('' COMMON YOMPHY0 '')')
WRITE(UNIT=KULOUT,FMT='('' AERCS1 = '',E11.4,'' AERCS3 = '',E11.4 &
 & ,'' AERCS5 = '',E11.4,'' ALMAV = '',E11.4,'' ECMNP = '',E11.4 &
 & ,'' EDB = '',E11.4,/,'' EDC = '',E11.4,'' EDD = '',E11.4 &
 & ,'' EDK = '',E11.4,'' ETACUT = '',E11.4 &
 & ,'' EVAP = '',E11.4,'' FONT = '',E11.4,'' GWDAMP = '',E11.4 &
 & ,'' GWDSE = '',E11.4,'' GWDBC = '',E11.4,/&
 & ,'' GWDCD = '',E11.4,/&
 & ,'' GWDPROF = '',E11.4,'' GWDVALI = '',E11.4,/&
 & ,'' HUCOE = '',E11.4,'' HUTIL = '',E11.4 ,'' HUTIL1 = '',E11.4 ,'' HUTIL2 = '',E11.4 &
 & ,'' VCHRNK = '',E11.4,'' VKARMN = '',E11.4 &
 & ,'' SNNBCO = '',E11.4,/,'' SPNBCO = '',E11.4 &
 & ,'' SXNBCO = '',E11.4,'' HOBST = '',E11.4,'' NPCLO1 = '',I4 &
 & ,'' NPCLO2 = '',I4,'' RCIN = '',E11.4,'' RCVEVAP = '',E11.4 &
 & ,'' REVGSL = '',E11.4,'' RFLCHCE = '',E11.4,/&
 & ,'' RTCAPE= '',E11.4,'' GCOMOD= '',E11.4,/&
 & ,'' UHDIFV = '',E11.4,'' VZ0CM = '',E11.4,'' VZIUSTAR0 = '',E11.4 &
 & ,'' XNBMAX = '',E11.4,'' GALP = '',E11.4,'' SENSL = '',E11.4 &
 & ,'' TCA = '',E11.4,/,'' TCT = '',E11.4,'' TCW = '',E11.4 &
 & ,'' TURB = '',E11.4,'' TVF = '',E11.4,/&
 & ,'' TYM = '',5E11.4,'' QSMIN = '',E11.4,/&
 & ,'' QSSC = '',E11.4,'' RPHI0 = '',E11.4,'' RPHIR = '',E11.4 &
 & ,'' BEDIFV = '',E11.4,'' SCO = '',E11.4 &
 & ,'' USDMLT = '',E11.4,/&
 & ,'' GDDEVA = '',E11.4,'' GDDSDE = '',E11.4,/&
 & ,'' GWDCCO = '',E11.4,/&
 & ,'' TENTR = '',E11.4,'' TENTRX = '',E11.4,'' TENTRD = '',E11.4 &
 & ,'' TUDGP = '',E11.4,'' TDDGP = '',E11.4 &
 & ,'' GRCVPP = '',E11.4,'' GCVHMIN= '',E11.4,/&
 & ,'' GCVOMGE = '',E11.4,'' GCVOMGQ= '',E11.4 &
 & ,'' GCVOMGS = '',E11.4,'' GCVOMGP= '',E11.4,/&
 & ,'' RKFBTAU = '',E11.4,'' RKFBNBX = '',E11.4 &
 & )')&
 & AERCS1,AERCS3,AERCS5,ALMAV,ECMNP,EDB,EDC,EDD,EDK,ETACUT,EVAP,FONT,&
 & GWDAMP,GWDSE,GWDBC,GWDCD,GWDPROF,GWDVALI,&
 & HUCOE,HUTIL,HUTIL1,HUTIL2,VCHRNK,VKARMN,SNNBCO,SPNBCO,SXNBCO,HOBST,&
 & NPCLO1,NPCLO2,RCIN,RCVEVAP,REVGSL,RFLCHCE,RTCAPE,GCOMOD,&
 & UHDIFV,VZ0CM,VZIUSTAR0,XNBMAX,GALP,SENSL,TCA,TCT,TCW,&
 & TURB,TVF,TYM,QSMIN,QSSC,RPHI0,RPHIR,BEDIFV,SCO,USDMLT,&
 & GDDEVA,GDDSDE,GWDCCO,&
 & TENTR,TENTRX,TENTRD,TUDGP,TDDGP,GRCVPP,GCVHMIN,&
 & GCVOMGE,GCVOMGQ,GCVOMGS,GCVOMGP,RKFBTAU,RKFBNBX

WRITE(UNIT=KULOUT,FMT='('' USUPRC = '',E11.4,'' USURIC = '',E11.4 &
 & ,'' QSNEBC = '',E11.4,'' QSNEBS = '',E11.4,'' QSMODC = '',E11.4 &
 & ,'' QSSUSC = '',E11.4,'' QSSUSS = '',E11.4,'' QSSUSV = '',E11.4 &
 & ,'' QSUSXC = '',E11.4,'' QSUSXS = '',E11.4 &
 & ,'' GCCSV  = '',E11.4,/&
 & ,'' QXRAL  = '',E11.4,'' QXRAL_ADJ  = '',E11.4,'' QXRDEL = '',E11.4 &
 & ,'' QXRHX  = '',E11.4,'' QXRR   = '',E11.4,'' QXRTGH = '',E11.4,/&
 & ,'' GWDLT  = '',E11.4,'' AHCLPV = '',E11.4,/&
 & ,'' GCVADS = '',E11.4,'' GCVBETA= '',E11.4 &
 & ,'' RICRLM = '',E11.4,'' XBLM   = '',E11.4,'' XKLM = '',E11.4 &
 & ,'' XMINLM = '',E11.4,'' XMAXLM = '',E11.4 &
 & ,'' WMXOV  = '',E11.4 &
 & )')&
 & USUPRC,USURIC,QSNEBC,QSNEBS,QSMODC,QSSUSC,QSSUSS,QSSUSV,QSUSXC,QSUSXS,GCCSV &
 & ,QXRAL,QXRAL_ADJ,QXRDEL,QXRHX,QXRR,QXRTGH,GWDLT,AHCLPV,GCVADS,GCVBETA,RICRLM &
 & ,XBLM,XKLM,XMINLM,XMAXLM,WMXOV
WRITE(UNIT=KULOUT,FMT='('' XWSALM = '',E11.4,'' XWSBLM = '',E11.4 &
 & ,'' GCVALFA= '',E11.4,'' GCVPSI = '',E11.4,'' GCVPSIE = '',E11.4 &
 & ,'' USURICL= '',E11.4,'' USURICE= '',E11.4,/ &
 & ,'' USURID= '',E11.4,'' USURIDE= '',E11.4,'' GCVNU= '',E11.4 &
 & ,'' GCVMLT= '',E11.4,'' GPBLHK0= '',E11.4,'' GPBLHRA = '',E11.4 &
 & ,'' UTILGUST= '',E11.4,'' RRGAMMA= '',E11.4,'' RRSCALE = '',E11.4 &
 & )')&
 & XWSALM,XWSBLM,GCVALFA,GCVPSI,GCVPSIE,USURICL,USURICE,USURID,USURIDE,GCVNU,GCVMLT,GPBLHK0 &
 & ,GPBLHRA,UTILGUST,RRGAMMA,RRSCALE
WRITE(UNIT=KULOUT,FMT='('' RNEGAT == '',E11.4,'' RNLCURV = '',E11.4 &
 & )') RNEGAT,RNLCURV
WRITE(UNIT=KULOUT,FMT='('' RDTFAC == '',E11.4,'' TRENTRV= '',E11.4 &
 &,'' TENTRVL= '',E11.4 )') RDTFAC,TRENTRV,TENTRVL
WRITE(UNIT=KULOUT,FMT='(9(5(A,E11.4),/))') 'GCISMIN=',GCISMIN
WRITE(UNIT=KULOUT,FMT='('' A0ML_AU == '',E11.4,'' A0ML_AT == '',E11.4 &
 &,'' A0ML_BU == '',E11.4,'' A0ML_BT == '',E11.4 )') A0ML_AU,A0ML_AT&
 &,A0ML_BU,A0ML_BT

!     - - - - - - - -
!     For TKE scheme :
!     - - - - - - - -
WRITE(UNIT=KULOUT,FMT='('' ADISE   = '',E11.4,&
 & '' ADISI   = '',E11.4,/&
 &,'' AECLS3  = '',E11.4,'' AECLS4  = '',E11.4,'' AKN     = '',E11.4,/&
 &,'' ALD     = '',E11.4,'' ALPHAE  = '',E11.4,'' ALPHAT  = '',E11.4,/&
 &,'' ECTMIN  = '',E11.4,'' ECTMAX  = '',E11.4,'' UCWSTAR = '',E11.4,/&
 &,'' UDECT   = '',E11.4,'' USHEARM = '',E11.4,/&
 &,'' UPRETMIN= '',E11.4,'' UPRETMAX= '',E11.4,'' ARSCH   = '',E11.4,/&
 &,'' ARSCQ   = '',E11.4,'' ARSC1   = '',E11.4,/&
 &,'' ARSB2   = '',E11.4,'' STTBMIN = '',E11.4,/&
 &,'' ACBRPHIM= '',E11.4,'' ALMAVE  = '',E11.4,'' ALMAVX  = '',E11.4,/&
 &,'' RICRET  = '',E11.4,/&
 &,'' AJBUMIN = '',E11.4,'' RCOFLM  = '',E11.4 &
     &)')&
 &ADISE,ADISI,AECLS3,AECLS4,AKN,ALD,ALPHAE,ALPHAT,ECTMIN,ECTMAX,UCWSTAR,UDECT,&
 &USHEARM,UPRETMIN,UPRETMAX,ARSCH,ARSCQ,ARSC1,ARSB2,STTBMIN,&
 &ACBRPHIM,ALMAVE,ALMAVX,RICRET,AJBUMIN,RCOFLM

!       - - - - - - - - - -
!       Lopez Microphysics :
!       - - - - - - - - - -
WRITE(UNIT=KULOUT,FMT='('' - - - - - - - - - - - '')')
WRITE(UNIT=KULOUT,FMT='(''  Microphysics scheme  '')')
WRITE(UNIT=KULOUT,FMT='('' - - - - - - - - - - - '')')
WRITE(UNIT=KULOUT,FMT='( '' RAUTEFR = '',E11.4   &
 &,'' RAUTEFS = '',E11.4,'' RAUTSBET= '',E11.4,/ &
 &,'' RNINTR  = '',E11.4,'' RNINTS  = '',E11.4,/ &
 &,'' RQLCR   = '',E11.4,'' RQCRNS  = '',E11.4,/ &
 &,'' RQICRMIN= '',E11.4,'' RQICRMAX= '',E11.4,/ &
 &,'' RQLCV   = '',E11.4 &
 &,'' RQICVMIN= '',E11.4,'' RQICVMAX= '',E11.4,/ &
 &,'' RQICRT1 = '',E11.4,'' RQICRT2 = '',E11.4,/ &
 &,'' RQICRSN = '',E11.4,'' RACCEF  = '',E11.4,/ &
 &,'' RAGGEF  = '',E11.4,'' RRIMEF  = '',E11.4,/ &
 &,'' RHCRIT1 = '',E11.4,'' RHCRIT2 = '',E11.4,/ &
 &,'' RETAMIN = '',E11.4,'' RFACNSM = '',E11.4,/ &
 &,'' TFVL    = '',E11.4,'' TFVI    = '',E11.4,/ &
 &,'' TFVR    = '',E11.4,'' TFVS    = '',E11.4,/ &
 &,'' GRHCMOD = '',E11.4,'' REVASX  = '',E11.4,/ &
 &,'' RHCEXPDX= '',E11.4,/ &
 &    )') &
 &RAUTEFR,RAUTEFS,RAUTSBET,RNINTR,RNINTS,RQLCR,RQCRNS,RQICRMIN,RQICRMAX, &
 &RQLCV,RQICVMIN,RQICVMAX,RQICRT1,RQICRT2,RQICRSN,RACCEF,RAGGEF,RRIMEF, &
 &RHCRIT1,RHCRIT2,RETAMIN,RFACNSM,TFVL,TFVI,TFVR,TFVS,GRHCMOD,REVASX,RHCEXPDX

!     - - - - - - - - - - -
!     For Grenier scheme :
!     - - - - - - - - - - -
WRITE(UNIT=KULOUT,FMT='('' AGRE1   = '',E11.4, &
 & '' AGRE2   = '',E11.4,'' AGREF   = '',E11.4, &
 & '' AGRERICR= '',E11.4,'' RPRTH   = '',E11.4 &
 & )')&
 & AGRE1,    AGRE2,   AGREF, AGRERICR, RPRTH

!     - - - - - - - - - - - - - - - -
!     For dry conv. adjustment scheme :                                -
!     - - - - - - - - - - - - - - - -
WRITE(UNIT=KULOUT,FMT='(&
 & '' AJ1PEPS = '',E11.4,'' AJ1MEPS = '',E11.4,'' NAJITER = '',I3,8X &
 & )')&
 & AJ1PEPS, AJ1MEPS, NAJITER

!     - - - - - - - - - - - - -
!     For ACCVIMPGY and PCMT :
!     - - - - - - - - - - - - -
WRITE(UNIT=KULOUT,FMT='( '' ALFX    = '',E11.4  &
 &,'' TCTC    = '',E11.4,'' TVFC    = '',F11.6,/ &
 &,'' GAMAP1  = '',F11.6,'' RKDN    = '',E11.4,/ &
 &,'' VVN     = '',F11.6,'' VVX     = '',F11.6,/ &
 &,'' FENTRT  = '',F11.6  &
 &,'' FQLIC   = '',F11.6,'' FNEBC   = '',F11.6,/ &
 &,'' FEVAPC  = '',F11.6)')&
 & ALFX, TCTC, TVFC, GAMAP1, RKDN, VVN, VVX, FENTRT, FQLIC, FNEBC, FEVAPC

!     - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!     For the "Lenderink & Holtslag" mixing Length : (ACNEBR)
!     - - - - - - - - - - - - - - - - - - - - - - - - - - - -
WRITE(UNIT=KULOUT,FMT='('' RLMLH1  = '',E10.4)') RLMLH1


WRITE(UNIT=KULOUT,FMT='(''  Alaro TOUCANS/pTKE scheme '')')
WRITE(UNIT=KULOUT,FMT='( '' NUPTKE = '',E11.4  &
 & ,'' TKEMULT = '',E11.4,'' C_EPSILON = '',E11.4,'' ETKE_RIFC = '',E11.4 & 
 & ,'' ETKE_LAM2 = '',E11.4,'' ETKE_LAM3 = '',E11.4,'' C3TKEFREE = '',E11.4,&
 &'' ETKE_R = '',E11.4,'' ETKE_LAM4 = '',E11.4 ,'' ETKE_OLAM = '',E11.4,&
 &'' ETKE_LAM0 = '',E11.4,'' ETKE_LAM1 = '',E11.4,&
 &'' ETKE_CTHETA = '',E11.4,'' ETKE_KA1 = '',E11.4,'' ETKE_KA2 = '',E11.4,&
 &'' ETKE_ETKE_KA3 = '',E11.4,'' ETKE_CG01 = '',E11.4,'' ETKE_CG02 = '',E11.4,&
 & '' ETKE_CG03='',E11.4,'' ETKE_BETA_EPS='',E11.4,'' ETKE_GAMMA_EPS = '',E11.4,&
 & '' ENT_LAMBDA = '',E11.4, '' ETKE_RIFC_MAF = '',E11.4,&
 & '' ETKE_GAMD = '',E11.4, ''  ETKE_TALPH = '',E11.4,'' ETKE_ALPH0 = '',E11.4,&
 & '' ETKE_ENTRX = '',E11.4,'' ETKE_ENTRN = '',E11.4,'' ETKE_KLM = '',E11.4,&
 & '' ETKE_TAULM = '',E11.4)')& 
 & NUPTKE, TKEMULT, C_EPSILON, ETKE_RIFC, ETKE_LAM2, ETKE_LAM3, C3TKEFREE, &
 & ETKE_R,ETKE_LAM4,ETKE_OLAM,ETKE_LAM0,ETKE_LAM1,ETKE_CTHETA,ETKE_KA1,&
 & ETKE_KA2,ETKE_KA3, ETKE_CG01, ETKE_CG02,ETKE_CG03,ETKE_BETA_EPS,&
 & ETKE_GAMMA_EPS,ENT_LAMBDA,ETKE_RIFC_MAF,ETKE_GAMD,&
 & ETKE_TALPH,ETKE_ALPH0,ETKE_ENTRX,ETKE_ENTRN,ETKE_KLM,ETKE_TAULM
WRITE(UNIT=KULOUT,FMT='(''  Alaro TOUCANS/EFB constants '')')
WRITE(UNIT=KULOUT,FMT='( ''EFB_AZ0  = '',E11.4,  &
 &'' EFB_UR = '',E11.4,'' REFB_1 = '',E11.4,'' REFB_2 = '',E11.4,&
 &'' REFB_3 = '',E11.4)') &
 & EFB_AZ0,EFB_UR,REFB_1,REFB_2,REFB_3
WRITE(UNIT=KULOUT,FMT='(''  Intermediate TOUCANS/pTKE constants '')')
WRITE(UNIT=KULOUT,FMT='( ''LAMBDA  = '',E11.4,  &
 &'' F = '',E11.4,'' lambda_8 = '',E11.4,'' Q = '',E11.4,'' SM0 = '',E11.4,&
 & '' C_K = '',E11.4, '' C_H = '',E11.4, '' A/kappa = '',E11.4)') &
 & ZLAMBDA,ZSF**2,ETKE_OLAM/C3TKEFREE,&
 & 1.0_JPRB-3.0_JPRB*ETKE_LAM3+ETKE_LAM2,&
 & ZSM0,ZNUPTKE4/C_EPSILON,ZNUPTKE4/C_EPSILON*C3TKEFREE,NUPTKE**3*C_EPSILON


WRITE(KULOUT,'(/'' PIL MICROPHYSICS : '')')
WRITE(UNIT=KULOUT,FMT='('' RDPHIC = '',F8.2   &
 &,'' RWBF1 = '',F6.2,'' RWBF2 = '',F6.2,/ &
 &,'' SCLESPR = '',F10.1,'' SCLESPS = '',F10.1,'' HUCRED = '',F5.2 &
 &,'' FSPRAIN = '',F9.3,'' EFFCOLL = '',F10.5,'' REVAPN = '',F5.3 &
 &,'' ADJTAU = '',F6.1, '' RSMDNEBX = '',F5.2 &
 &,'' RSMDTX = '',F5.2, '' RDECRD = '',F8.1, '' RGELSFON = '',F10.6 &
 &,'' NSMTPA = '',I2 &
 &,'' NSMTPB = '',I2 &
 &    )') &
 &RDPHIC,RWBF1,RWBF2,SCLESPR,SCLESPS,HUCRED,FSPRAIN,&
 &EFFCOLL,REVAPN,ADJTAU,RSMDNEBX,RSMDTX,RDECRD,RGELSFON,NSMTPA,NSMTPB

WRITE(KULOUT,'(/'' PROGNOSTIC CONVECTION '')')
WRITE(KULOUT,&
 &'('' TUDBU='',E16.6,'' TUDFR='',E16.6,'' TDDBU='',E16.6 &
 &,'' TDDFR='',E16.6,''RMULACVG= '',E16.6 &
 &,'' GCVALMX='',G10.4,'' GCVACHI='',E16.6 &
 &,'' GCVEEX ='',G10.4,'' GCVTAUDE='',G10.4 &
 &,'' GCVBEE ='',G10.4 &
 &,'' ECMNPI ='',G10.4,/ &
 &,'' GPEIPHI ='',G10.4 &
 &,'' GPETAU ='',G10.4 &
 &,'' GENVSRH ='',G10.4 &
 &,'' GPEFDC ='',G10.4 &
 &   )') &
 &  TUDBU,TUDFR,TDDBU,TDDFR,RMULACVG,GCVALMX,GCVACHI,&
 &  GCVEEX,GCVTAUDE,GCVBEE,ECMNPI,&
 &  GPEIPHI, GPETAU, GENVSRH, GPEFDC
WRITE(KULOUT,&
 &'('' GDDEVF='',G10.4,'' GDDWPF='',G10.4,'' GDDBETA='',G10.4,&
 &'' GDDDP='',G10.4,'' GDDFXM='',G10.4)')&
 & GDDEVF, GDDWPF, GDDBETA, GDDDP, GDDFXM
WRITE(KULOUT,'('' GRRMINA ='',G10.4)') GRRMINA
WRITE(KULOUT,FMT=*) 'LGMT=',LGMT,' LGDDD=',LGDDD,' LGCRUC=',LGCRUC,' NCVENTR=',NCVENTR
WRITE(KULOUT,FMT='(A,L5,9(A,G10.4))') 'LGPSMI=',LGPSMI,' GCVENTRN=',GCVENTRN,' GCVENTRX=',GCVENTRX,' GCVENTR_MIN=',GCVENTR_MIN
WRITE(KULOUT,FMT='(9(A,G10.4))') ' ADOE=',ADOE,' RFRAC_ENTR_N=',RFRAC_ENTR_N &
 &,' RFRAC_ENTR_X=',RFRAC_ENTR_X,' THPCMT=',THPCMT
!     ------------------------------------------------------------------

!*       4.    Check consistency between logical and real namelist inputs.
!              -------------------

IF (LMPHYS) THEN

  IF(LCVRA.AND.((TUDGP /= 0.0_JPRB.OR.TDDGP /= 0.0_JPRB).AND..NOT.LCVCAS)) THEN
    WRITE(UNIT=KULOUT,FMT='(A)') &
     & 'INCONSISTENCY BETWEEN TUDGP, TDDGP AND LCVCAS!...'
    CALL ABOR1('TUDP<>0. OR TDDGP<>0. IMPLIES LCVCAS=T!...')
  ENDIF

  IF(.NOT.LCVLIS.AND.GCVPSI /= 0.0_JPRB) THEN
    WRITE(UNIT=KULOUT,FMT='(A)') 'INCONSISTENCY BETWEEN LCVLIS AND GCVPSI!...'
    CALL ABOR1('LCVLIS=F IMPLIES GCVPSI=0.!...')
  ENDIF

  IF(L3MT.AND.LNEBNXR.AND..NOT.(QXRDEL==0.5_JPRB.AND.QXRR==0.25_JPRB)) THEN
    WRITE(UNIT=KULOUT,FMT='(A)') 'IF L3MT IT MUST BE QXRDEL=0.5 AND QXRR=0.25!'
    CALL ABOR1('3MT && X-R settings inconsistency')
  ENDIF

  IF(L3MT.AND.GCOMOD/=0._JPRB) THEN
    WRITE(UNIT=KULOUT,FMT='(A)') 'IF L3MT IT MUST BE GCOMOD=0.0!'
    CALL ABOR1('3MT && GCOMOD settings inconsistency')
  ENDIF

ENDIF

!*  Consistency check for simplified physics keys

IF ((LMPHYS.OR.LSIMPH).AND.(LSMOOTHD.OR.LSMOOTHA)) THEN

  IF(RNLCURV == 0.0_JPRB) THEN
    WRITE(UNIT=KULOUT,FMT='(A)') 'SMOOTHING IN SIM. PH. BUT RNLCURV = ZERO'
    CALL ABOR1('PHYSICS AND SMOOTHING IMPLY RNLCURV /= ZERO')
  ENDIF

ENDIF

! ------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUPHY0',1,ZHOOK_HANDLE)
END SUBROUTINE SUPHY0
