!OPTIONS XOPT(NOEVAL)
SUBROUTINE SUPHY3(KULOUT)

!**** *SUPHY3*   - Initialize common YOMPHY3 physics radiative
!                  constants

! Purpose:
! --------
!   Initialize YOMPHY3, module that contains parameters
!   for ACRANEB/ACRANEB2 radiation schemes.

! Interface:
! ----------
! INPUT:
!   KULOUT - logical unit for the output

! Method:
! -------

! Externals:
! ----------

! Reference:
! ----------
!   Documentation ARPEGE

! Author:
! -------
!   1990-09, J.-F. Geleyn

! Modifications:
! --------------
!   2003-10, M.Hamrud
!   CY28 Cleaning.

!   2006-09, R.Brozkova
!   ALARO-0 (new cloud model).

!   2008-10, J. Masek
!   Parameters for NER statistical model.

!   2008-10, T. Kral
!   Parameters for new gaseous transmission functions.

!   2010-01, K. Yessad
!   Removed useless variables.

!   2013-11, J. Masek
!   Settings for ACRANEB2 scheme, backphasing to cy38t1.
! End Modifications
!-------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULNAM
USE YOMPHY   , ONLY : LCLSATUR ,LRPROX   ,LRSTAER  ,NRAY     ,NRAUTOEV
USE YOMPHY3  , ONLY : GCA      ,GCB      ,GCC      ,VDP      ,&
 & VNP      ,BSFSA    ,BSFSI    ,BSFSN    ,BSFTA    ,&
 & BSFTI    ,BSFTN    ,EARRT    ,EOASA    ,EOASI    ,&
 & EOASN    ,EOATA    ,EOATI    ,EOATN    ,EODSA    ,&
 & EODSI    ,EODSN    ,EODTA    ,EODTI    ,EODTN    ,&
 & EORAY    ,GCD4     ,GCE4     ,QCO2     ,RII0     ,&
 & USAA     ,USAI     ,USAN     ,USBA     ,USBI     ,&
 & USBN     ,GIREC1   ,GIREC2   ,GIREC3   ,GIREC4   ,&
 & FCM_AI   ,FCM_AL                                 ,&
 & FCM_B_AI ,FCM_B_AL ,FCM_B_BI ,FCM_B_BL           ,&
 & FCM_IWC2DE         ,FCM_LWC2RE                   ,&
 & FCM_DEL_AI         ,FCM_DEL_AL                   ,&
 & FCM_DEL_DI         ,FCM_DEL_DL                   ,&
 & FCM_MU_AI,FCM_MU_AL,FCM_MU_DI,FCM_MU_DL          ,&
 & FCM_NU_AI,FCM_NU_AL,FCM_NU_DI,FCM_NU_DL          ,&
 & FCM_DEL_A,FCM_DEL_D,FCM_MU_A ,FCM_MU_D           ,&
 & FCM_N_I  ,FCM_N_L  ,FCM_P_AI ,FCM_P_AL ,FCM_P_DI ,&
 & FCM_P_DL ,FCM_P_GI ,FCM_P_GL ,FCM_Q_AI ,FCM_Q_AL ,&
 & FCM_Q_DI ,FCM_Q_DL ,FCM_Q_GI ,FCM_Q_GL ,N_SPBAND ,&
 & REXP_NEB ,FSM_AA   ,FSM_BB   ,FSM_CC   ,FSM_DD   ,&
 & FSM_EE   ,FSM_FF   ,FSM_GG   ,FSM_HH   ,FSM_II   ,&
 & GOLCA    ,GOLCB    ,GOLCC                        ,&
 & FGTS_A   ,FGTS_B   ,FGTS_C   ,FGTS_ALPHA         ,&
 & FGTS_DELTA0        ,FGTS_D   ,FGTS_P00 ,FGTS_P   ,&
 & FGTS_Q   ,FGTS_OA  ,FGTS_OB  ,FGTS_OC  ,FGTS_OD  ,&
 & FGTC_A   ,FGTC_B   ,FGTC_C   ,FGTC_ALPHA         ,&
 & FGTC_DELTA0        ,FGTC_D   ,FGTC_P00 ,FGTC_P   ,&
 & FGTC_Q   ,FGTC_OA  ,FGTC_OB  ,FGTC_OC  ,FGTC_OD  ,&
 & FGTT_A   ,FGTT_B   ,FGTT_C   ,FGTT_ALPHA         ,&
 & FGTT_DELTA0        ,FGTT_D   ,FGTT_P00 ,FGTT_P   ,&
 & FGTT_Q   ,FGTT_OA  ,FGTT_OB  ,FGTT_OC  ,FGTT_OD  ,&
 & FRS_K_SCAT0        ,FRS_P_CRIT         ,FRS_BETA ,&
 & RTL      ,RLAMB_SOLID        ,RLAMB_WATER

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"

#include "namphy3.h"
!     ------------------------------------------------------------------

!*       1.    Set default values.
!              -------------------

!        1.1 Set implicit default values

IF (LHOOK) CALL DR_HOOK('SUPHY3',0,ZHOOK_HANDLE)

IF (LRSTAER) THEN
  BSFSA(1)=.3490E+00_JPRB
  BSFTA(1)=.3471E+00_JPRB
  EOASA(1)=.6727E-01_JPRB
  EOATA(1)=.5037E-01_JPRB
  EODSA(1)=.3665E+00_JPRB
  EODTA(1)=.1766E-01_JPRB
  USAA(1)=-.3020E+00_JPRB
  BSFSA(2:6)=0._JPRB 
  BSFTA(2:6)=0._JPRB
  EOASA(2:6)=0._JPRB
  EOATA(2:6)=0._JPRB
  EODSA(2:6)=0._JPRB
  EODTA(2:6)=0._JPRB
  USAA(2:6)=0._JPRB
ELSE
  EOATA(1:6)=(/.2774E-01_JPRB,.2469E-01_JPRB,.2024E-01_JPRB,&
               & .3318E-02_JPRB,.1537E-01_JPRB,.2459E-01_JPRB/)
  EODTA(1:6)=(/.1140E-01_JPRB,.8529E-02_JPRB,.1197E-01_JPRB,&
               & .3212E-03_JPRB,.7916E-03_JPRB,.1127E-03_JPRB/)
  BSFTA(1:6)=(/.3439E+00_JPRB,.3499E+00_JPRB,.3421E+00_JPRB,&
               & .3550E+00_JPRB,.4569E+00_JPRB,.4929E+00_JPRB/)
  EOASA(1:6)=(/.9678E-01_JPRB,.8146E-02_JPRB,.1707E+00_JPRB,&
               & .2121E+00_JPRB,.4081E-01_JPRB,.1124E-02_JPRB/)
  EODSA(1:6)=(/.4545E+00_JPRB,.3465E+00_JPRB,.3863E+00_JPRB,&
               & .4131E+00_JPRB,.3399E+00_JPRB,.2855E+00_JPRB/)
  BSFSA(1:6)=(/.3448E+00_JPRB,.3317E+00_JPRB,.3427E+00_JPRB,&
               & .3441E+00_JPRB,.3443E+00_JPRB,.3361E+00_JPRB/)
  USAA(1:6)=(/-.3104E+00_JPRB,-.3366E+00_JPRB,-.3146E+00_JPRB,&
               &-.3118E+00_JPRB,-.3114E+00_JPRB,-.3278E+00_JPRB/)
ENDIF

BSFSI=.3238_JPRB
BSFSN=.3300_JPRB
BSFTI=.3231_JPRB
BSFTN=.3584_JPRB
EARRT=0.001324_JPRB
EOASI=.1692E-01_JPRB
EOASN=.3353E-01_JPRB
EOATI=.3455E+01_JPRB
EOATN=.9077E+01_JPRB
EODSI=.1406E+01_JPRB
EODSN=.6558E+01_JPRB
EODTI=.8435E+00_JPRB
EODTN=.4319E+01_JPRB
USAI=-.3524_JPRB
USAN=-.3400_JPRB
USBA=0._JPRB
USBI=0._JPRB
USBN=0._JPRB

IF ( NRAY == 1 ) THEN

  ! default values for old gaseous transmissions based on AFGL 1980 data;
  ! CO2+ composition according to IPCC 1990 dry air concentrations:
  !   gas :      CO2       N2O        CO       CH4        O2
  !   ppmv:    353.2     0.310     0.120     1.725  209480.6
  GCA(1)=.8041E-01_JPRB
  GCA(2)=.1456E+00_JPRB
  GCA(3)=.4787E+01_JPRB
  GCA(4)=.2102E+04_JPRB
  GCA(5)=.1334E+01_JPRB
  GCA(6)=.1551E-04_JPRB
  GCB(1)=.8968E+07_JPRB
  GCB(2)=.2413E+10_JPRB
  GCB(3)=.3548E+05_JPRB
  GCB(4)=.6370E+10_JPRB
  GCB(5)=.8499E+11_JPRB
  GCB(6)=.1012E+06_JPRB
  GCC(1)=.5925E-10_JPRB
  GCC(2)=.1842E-10_JPRB
  GCC(3)=.2532E-07_JPRB
  GCC(4)=.1953E+07_JPRB
  GCC(5)=.1734E-11_JPRB
  GCC(6)=.1225E-16_JPRB
  GCD4=.3608E-69_JPRB
  GCE4=.7563E+04_JPRB
  GIREC1=0.0_JPRB
  GIREC2=0.0_JPRB
  GIREC3=0.0_JPRB
  GIREC4=0.0_JPRB
  VDP(1,1)=.21868E+02_JPRB
  VDP(2,1)=.17453E+02_JPRB
  VDP(3,1)=.68918E+00_JPRB
  VDP(4,1)=.23456E-03_JPRB
  VDP(5,1)=.22317E-09_JPRB
  VDP(1,2)=.48401E+02_JPRB
  VDP(2,2)=.18648E+02_JPRB
  VDP(3,2)=.35199E-01_JPRB
  VDP(4,2)=.63691E-06_JPRB
  VDP(5,2)=.86395E-13_JPRB
  VDP(1,3)=.76948E+02_JPRB
  VDP(2,3)=.41056E+01_JPRB
  VDP(3,3)=.42667E-03_JPRB
  VDP(4,3)=0._JPRB
  VDP(5,3)=0._JPRB
  VDP(1,4)=.27241E+03_JPRB
  VDP(2,4)=.57091E+04_JPRB
  VDP(3,4)=.14393E+05_JPRB
  VDP(4,4)=.29879E+04_JPRB
  VDP(5,4)=.25382E+02_JPRB
  VDP(1,5)=.86942E+02_JPRB
  VDP(2,5)=.32186E+03_JPRB
  VDP(3,5)=.10775E+03_JPRB
  VDP(4,5)=.21261E+01_JPRB
  VDP(5,5)=.40003E-02_JPRB
  VDP(1,6)=.24408E+02_JPRB
  VDP(2,6)=.81919E+01_JPRB
  VDP(3,6)=.72193E+01_JPRB
  VDP(4,6)=.56230E+00_JPRB
  VDP(5,6)=.25384E-02_JPRB
  VNP(1,1)=.69926E+01_JPRB
  VNP(2,1)=.63915E+00_JPRB
  VNP(3,1)=.28896E-03_JPRB
  VNP(4,1)=.10050E-08_JPRB
  VNP(5,1)=.99037E-16_JPRB
  VNP(1,2)=.31105E+01_JPRB
  VNP(2,2)=.14225E-01_JPRB
  VNP(3,2)=.69355E-06_JPRB
  VNP(4,2)=.36087E-12_JPRB
  VNP(5,2)=.44113E-20_JPRB
  VNP(1,3)=.23659E+01_JPRB
  VNP(2,3)=.11139E-02_JPRB
  VNP(3,3)=.10618E-05_JPRB
  VNP(4,3)=0._JPRB
  VNP(5,3)=0._JPRB
  VNP(1,4)=.19810E+03_JPRB
  VNP(2,4)=.46954E+04_JPRB
  VNP(3,4)=.22512E+04_JPRB
  VNP(4,4)=.52461E+02_JPRB
  VNP(5,4)=.11645E+00_JPRB
  VNP(1,5)=.46348E+02_JPRB
  VNP(2,5)=.35630E+02_JPRB
  VNP(3,5)=.33005E+01_JPRB
  VNP(4,5)=.18045E-01_JPRB
  VNP(5,5)=.88667E-05_JPRB
  VNP(1,6)=.47413E+01_JPRB
  VNP(2,6)=.16334E+01_JPRB
  VNP(3,6)=.48164E+00_JPRB
  VNP(4,6)=.56140E-02_JPRB
  VNP(5,6)=.67790E-04_JPRB
  GOLCA(1)=0._JPRB
  GOLCB(1)=0._JPRB
  GOLCC(1)=0._JPRB
  GOLCA(2)=0._JPRB
  GOLCB(2)=0._JPRB
  GOLCC(2)=0._JPRB
  GOLCA(3)=0._JPRB
  GOLCB(3)=0._JPRB
  GOLCC(3)=0._JPRB
  GOLCA(4)=0._JPRB
  !GOLCA(4)=9.04151E-02_JPRB  ! to be activated later
  GOLCB(4)=1.66625E+03_JPRB
  GOLCC(4)=5.02361E+09_JPRB
  GOLCA(5)=0._JPRB
  !GOLCA(5)=8.25057E-02_JPRB  ! to be activated later
  GOLCB(5)=9.54721E+03_JPRB
  GOLCC(5)=6.34968E+13_JPRB
  GOLCA(6)=0._JPRB
  !GOLCA(6)=1.16030E-02_JPRB  ! to be activated later
  GOLCB(6)=1.48917E+10_JPRB
  GOLCC(6)=7.38043E+13_JPRB

  ! mass fraction of CO2 in dry air corresponding to 353.2ppmv
  QCO2=.5366E-03_JPRB

  ! mean coefficient for Rayleigh scattering
  EORAY=.8606E-06_JPRB

ELSE

  ! default values for new gaseous transmissions based on HITRAN 2008 data;
  ! CO2+ composition according to WDCGG 2010 dry air concentrations:
  !   gas :      CO2       N2O        CO       CH4        O2
  !   ppmv:    389.0     0.323   omitted     1.808  209460.0
  FGTS_A(1,0)   = 5.928E-02_JPRB
  FGTS_A(1,1)   = 8.931E-06_JPRB
  FGTS_A(1,2)   = 3.971E-10_JPRB
  FGTS_A(2,0)   = 1.262E-01_JPRB
  FGTS_A(2,1)   = 0.000E+00_JPRB
  FGTS_A(2,2)   = 0.000E+00_JPRB
  FGTS_A(3,0)   = 1.198E+01_JPRB
  FGTS_A(3,1)   = 5.991E-05_JPRB
  FGTS_A(3,2)   = 0.000E+00_JPRB
  FGTS_B(1,0)   = 1.301E+06_JPRB
  FGTS_B(1,1)   = 0.000E+00_JPRB
  FGTS_B(1,2)   = 2.483E-03_JPRB
  FGTS_B(2,0)   = 1.504E+08_JPRB
  FGTS_B(2,1)   = 0.000E+00_JPRB
  FGTS_B(2,2)   = 9.991E-03_JPRB
  FGTS_B(3,0)   = 1.000E-20_JPRB
  FGTS_B(3,1)   = 0.000E+00_JPRB
  FGTS_B(3,2)   = 0.000E+00_JPRB
  FGTS_C(1)     = 0.000E+00_JPRB
  FGTS_C(2)     = 0.000E+00_JPRB
  FGTS_C(3)     = 0.000E+00_JPRB
  FGTS_C(4)     = 0.000E+00_JPRB
  FGTS_C(5)     = 1.000E+00_JPRB
  FGTS_ALPHA(1) = 2.411E-01_JPRB
  FGTS_ALPHA(2) = 6.755E-01_JPRB
  FGTS_ALPHA(3) = 3.697E-01_JPRB
  FGTS_DELTA0(1)= 1.972E-02_JPRB
  FGTS_DELTA0(2)= 2.002E-04_JPRB
  FGTS_DELTA0(3)= 1.145E-02_JPRB
  FGTS_D(1)     = 1.958E-01_JPRB
  FGTS_D(2)     = 8.762E-02_JPRB
  FGTS_D(3)     = 1.009E-02_JPRB
  FGTS_P00(1,0) = 9.937E-01_JPRB
  FGTS_P00(1,1) =-9.554E-05_JPRB
  FGTS_P00(1,2) = 7.381E-08_JPRB
  FGTS_P00(2,0) = 1.208E+00_JPRB
  FGTS_P00(2,1) =-7.589E-04_JPRB
  FGTS_P00(2,2) = 1.640E-06_JPRB
  FGTS_P00(3,0) = 9.576E-01_JPRB
  FGTS_P00(3,1) = 1.317E-04_JPRB
  FGTS_P00(3,2) =-1.601E-07_JPRB
  FGTS_P(1,0,0) = 2.391E+00_JPRB
  FGTS_P(1,1,0) =-6.446E-01_JPRB
  FGTS_P(1,2,0) =-5.688E-01_JPRB
  FGTS_P(1,3,0) = 1.047E-01_JPRB
  FGTS_P(1,4,0) = 8.456E-02_JPRB
  FGTS_P(1,5,0) = 1.048E-02_JPRB
  FGTS_P(1,0,1) =-8.194E-04_JPRB
  FGTS_P(1,1,1) =-4.135E-04_JPRB
  FGTS_P(1,2,1) = 3.675E-04_JPRB
  FGTS_P(1,3,1) =-5.008E-05_JPRB
  FGTS_P(1,4,1) =-8.338E-06_JPRB
  FGTS_P(1,5,1) =-3.917E-07_JPRB
  FGTS_P(1,0,2) =-2.428E-06_JPRB
  FGTS_P(1,1,2) =-2.396E-07_JPRB
  FGTS_P(1,2,2) = 3.662E-07_JPRB
  FGTS_P(1,3,2) = 1.000E-07_JPRB
  FGTS_P(1,4,2) =-5.778E-08_JPRB
  FGTS_P(1,5,2) =-6.344E-09_JPRB
  FGTS_P(2,0,0) =-3.725E+00_JPRB
  FGTS_P(2,1,0) =-2.302E+00_JPRB
  FGTS_P(2,2,0) = 2.814E-01_JPRB
  FGTS_P(2,3,0) = 1.956E-03_JPRB
  FGTS_P(2,4,0) = 1.210E-03_JPRB
  FGTS_P(2,5,0) =-4.342E-03_JPRB
  FGTS_P(2,0,1) =-8.642E-04_JPRB
  FGTS_P(2,1,1) = 8.351E-04_JPRB
  FGTS_P(2,2,1) =-5.712E-04_JPRB
  FGTS_P(2,3,1) = 1.570E-04_JPRB
  FGTS_P(2,4,1) =-2.871E-05_JPRB
  FGTS_P(2,5,1) = 3.713E-06_JPRB
  FGTS_P(2,0,2) = 1.333E-05_JPRB
  FGTS_P(2,1,2) =-1.368E-06_JPRB
  FGTS_P(2,2,2) = 6.625E-08_JPRB
  FGTS_P(2,3,2) =-5.724E-08_JPRB
  FGTS_P(2,4,2) = 4.738E-08_JPRB
  FGTS_P(2,5,2) =-1.776E-08_JPRB
  FGTS_P(3,0,0) = 1.974E-01_JPRB
  FGTS_P(3,1,0) = 6.929E-01_JPRB
  FGTS_P(3,2,0) = 1.609E-01_JPRB
  FGTS_P(3,3,0) =-6.706E-03_JPRB
  FGTS_P(3,4,0) =-3.971E-03_JPRB
  FGTS_P(3,5,0) =-2.186E-04_JPRB
  FGTS_P(3,0,1) =-3.538E-04_JPRB
  FGTS_P(3,1,1) = 8.533E-05_JPRB
  FGTS_P(3,2,1) =-6.721E-07_JPRB
  FGTS_P(3,3,1) =-1.052E-06_JPRB
  FGTS_P(3,4,1) =-1.103E-06_JPRB
  FGTS_P(3,5,1) =-3.021E-07_JPRB
  FGTS_P(3,0,2) = 9.482E-07_JPRB
  FGTS_P(3,1,2) =-7.418E-08_JPRB
  FGTS_P(3,2,2) = 4.101E-08_JPRB
  FGTS_P(3,3,2) =-1.057E-08_JPRB
  FGTS_P(3,4,2) =-3.502E-09_JPRB
  FGTS_P(3,5,2) = 1.130E-10_JPRB
  FGTS_Q(1,0)   = 5.407E-01_JPRB
  FGTS_Q(1,1)   =-3.555E-06_JPRB
  FGTS_Q(1,2)   = 2.133E-11_JPRB
  FGTS_Q(2,0)   = 1.196E-01_JPRB
  FGTS_Q(2,1)   =-7.917E-06_JPRB
  FGTS_Q(2,2)   = 5.061E-11_JPRB
  FGTS_Q(3,0)   =-5.909E-01_JPRB
  FGTS_Q(3,1)   = 1.142E-09_JPRB
  FGTS_Q(3,2)   =-8.870E-15_JPRB

  ! solar overlaps switched off for efficiency reasons
  FGTS_OA(1)    = 0.000E+00_JPRB
  FGTS_OA(2)    = 0.000E+00_JPRB
  FGTS_OA(3)    = 0.000E+00_JPRB
  !FGTS_OA(1)    =-1.860E-01_JPRB
  !FGTS_OA(2)    = 5.113E-01_JPRB
  !FGTS_OA(3)    = 4.048E-01_JPRB
  FGTS_OB(1)    = 1.000E+00_JPRB
  FGTS_OB(2)    = 1.000E+00_JPRB
  FGTS_OB(3)    = 1.000E+00_JPRB
  FGTS_OC(1)    = 1.237E+00_JPRB
  FGTS_OC(2)    = 2.204E+00_JPRB
  FGTS_OC(3)    = 2.143E+00_JPRB
  FGTS_OD(1)    = 0.000E+00_JPRB
  FGTS_OD(2)    = 0.000E+00_JPRB
  FGTS_OD(3)    = 0.000E+00_JPRB

  FGTC_A(1,0)   = 2.937E-01_JPRB
  FGTC_A(1,1)   = 7.585E-02_JPRB
  FGTC_A(1,2)   = 0.000E+00_JPRB
  FGTC_A(2,0)   = 6.113E-01_JPRB
  FGTC_A(2,1)   = 4.347E-12_JPRB
  FGTC_A(2,2)   = 3.000E-09_JPRB
  FGTC_A(3,0)   = 8.333E-01_JPRB
  FGTC_A(3,1)   = 2.705E-04_JPRB
  FGTC_A(3,2)   = 2.715E-07_JPRB
  FGTC_B(1,0)   = 3.198E+07_JPRB
  FGTC_B(1,1)   = 0.000E+00_JPRB
  FGTC_B(1,2)   = 1.671E-12_JPRB
  FGTC_B(2,0)   = 1.482E+08_JPRB
  FGTC_B(2,1)   = 0.000E+00_JPRB
  FGTC_B(2,2)   = 3.783E-01_JPRB
  FGTC_B(3,0)   = 1.586E+09_JPRB
  FGTC_B(3,1)   = 0.000E+00_JPRB
  FGTC_B(3,2)   = 5.655E+01_JPRB
  FGTC_C(1)     = 4.948E-03_JPRB
  FGTC_C(2)     = 2.107E-02_JPRB
  FGTC_C(3)     = 4.272E-02_JPRB
  FGTC_C(4)     = 8.737E-01_JPRB
  FGTC_C(5)     = 8.407E-01_JPRB
  FGTC_ALPHA(1) = 5.886E-01_JPRB
  FGTC_ALPHA(2) = 7.365E-01_JPRB
  FGTC_ALPHA(3) = 5.359E-01_JPRB
  FGTC_DELTA0(1)= 1.751E-01_JPRB
  FGTC_DELTA0(2)= 7.645E-04_JPRB
  FGTC_DELTA0(3)= 2.119E-02_JPRB
  FGTC_D(1)     = 1.669E-01_JPRB
  FGTC_D(2)     = 5.933E-02_JPRB
  FGTC_D(3)     = 5.777E-02_JPRB
  FGTC_P00(1,0) = 7.799E-01_JPRB
  FGTC_P00(1,1) = 4.897E-04_JPRB
  FGTC_P00(1,2) = 3.617E-07_JPRB
  FGTC_P00(2,0) = 1.351E+00_JPRB
  FGTC_P00(2,1) =-9.627E-04_JPRB
  FGTC_P00(2,2) = 3.055E-06_JPRB
  FGTC_P00(3,0) = 1.149E+00_JPRB
  FGTC_P00(3,1) =-3.841E-04_JPRB
  FGTC_P00(3,2) = 1.347E-06_JPRB
  FGTC_P(1,0,0) =-4.076E-01_JPRB
  FGTC_P(1,1,0) = 1.922E-01_JPRB
  FGTC_P(1,2,0) = 2.291E-01_JPRB
  FGTC_P(1,3,0) = 7.504E-02_JPRB
  FGTC_P(1,4,0) = 2.660E-02_JPRB
  FGTC_P(1,5,0) = 2.693E-03_JPRB
  FGTC_P(1,0,1) =-7.336E-04_JPRB
  FGTC_P(1,1,1) =-2.094E-03_JPRB
  FGTC_P(1,2,1) =-1.253E-03_JPRB
  FGTC_P(1,3,1) =-3.890E-04_JPRB
  FGTC_P(1,4,1) =-8.797E-05_JPRB
  FGTC_P(1,5,1) =-4.437E-06_JPRB
  FGTC_P(1,0,2) = 1.997E-06_JPRB
  FGTC_P(1,1,2) = 3.369E-06_JPRB
  FGTC_P(1,2,2) = 1.652E-06_JPRB
  FGTC_P(1,3,2) = 5.345E-07_JPRB
  FGTC_P(1,4,2) = 7.608E-08_JPRB
  FGTC_P(1,5,2) = 6.344E-09_JPRB
  FGTC_P(2,0,0) = 3.240E-01_JPRB
  FGTC_P(2,1,0) = 6.357E-01_JPRB
  FGTC_P(2,2,0) =-3.365E-02_JPRB
  FGTC_P(2,3,0) =-1.395E-02_JPRB
  FGTC_P(2,4,0) = 1.070E-02_JPRB
  FGTC_P(2,5,0) = 4.126E-04_JPRB
  FGTC_P(2,0,1) =-2.226E-03_JPRB
  FGTC_P(2,1,1) =-2.806E-04_JPRB
  FGTC_P(2,2,1) = 5.763E-04_JPRB
  FGTC_P(2,3,1) =-6.421E-05_JPRB
  FGTC_P(2,4,1) =-3.447E-05_JPRB
  FGTC_P(2,5,1) = 7.357E-06_JPRB
  FGTC_P(2,0,2) =-9.578E-07_JPRB
  FGTC_P(2,1,2) =-2.530E-07_JPRB
  FGTC_P(2,2,2) = 8.162E-07_JPRB
  FGTC_P(2,3,2) =-5.242E-08_JPRB
  FGTC_P(2,4,2) =-3.313E-08_JPRB
  FGTC_P(2,5,2) =-2.277E-08_JPRB
  FGTC_P(3,0,0) =-2.926E+00_JPRB
  FGTC_P(3,1,0) =-5.583E-01_JPRB
  FGTC_P(3,2,0) = 6.293E-02_JPRB
  FGTC_P(3,3,0) = 9.520E-03_JPRB
  FGTC_P(3,4,0) =-3.912E-03_JPRB
  FGTC_P(3,5,0) =-5.541E-04_JPRB
  FGTC_P(3,0,1) = 1.420E-03_JPRB
  FGTC_P(3,1,1) =-2.318E-04_JPRB
  FGTC_P(3,2,1) = 3.479E-06_JPRB
  FGTC_P(3,3,1) = 1.319E-05_JPRB
  FGTC_P(3,4,1) =-4.933E-06_JPRB
  FGTC_P(3,5,1) = 1.092E-06_JPRB
  FGTC_P(3,0,2) =-1.310E-07_JPRB
  FGTC_P(3,1,2) =-1.080E-08_JPRB
  FGTC_P(3,2,2) =-2.144E-09_JPRB
  FGTC_P(3,3,2) =-5.236E-09_JPRB
  FGTC_P(3,4,2) = 6.809E-09_JPRB
  FGTC_P(3,5,2) =-3.393E-09_JPRB
  FGTC_Q(1,0)   = 4.937E-01_JPRB
  FGTC_Q(1,1)   =-2.052E-06_JPRB
  FGTC_Q(1,2)   = 1.543E-11_JPRB
  FGTC_Q(2,0)   = 8.016E-01_JPRB
  FGTC_Q(2,1)   =-3.096E-06_JPRB
  FGTC_Q(2,2)   = 1.857E-11_JPRB
  FGTC_Q(3,0)   = 1.386E-02_JPRB
  FGTC_Q(3,1)   = 4.135E-07_JPRB
  FGTC_Q(3,2)   =-2.954E-12_JPRB
  FGTC_OA(1)    = 3.394E+00_JPRB
  FGTC_OA(2)    = 0.000E+00_JPRB  ! H2O-O3 thermal overlap switched off
  !FGTC_OA(2)    = 3.174E+00_JPRB
  FGTC_OA(3)    =-6.596E-01_JPRB
  FGTC_OB(1)    = 8.659E-01_JPRB
  FGTC_OB(2)    = 5.000E-01_JPRB
  FGTC_OB(3)    = 1.235E+00_JPRB
  FGTC_OC(1)    = 3.514E+00_JPRB
  FGTC_OC(2)    = 3.360E+00_JPRB
  FGTC_OC(3)    = 2.538E+00_JPRB
  FGTC_OD(1)    = 1.229E+00_JPRB
  FGTC_OD(2)    = 1.000E+00_JPRB
  FGTC_OD(3)    = 3.119E+00_JPRB

  FGTT_A(1,0)   = 1.955E-01_JPRB
  FGTT_A(1,1)   = 7.973E-02_JPRB
  FGTT_A(1,2)   = 2.269E-11_JPRB
  FGTT_A(2,0)   = 5.619E-01_JPRB
  FGTT_A(2,1)   = 0.000E+00_JPRB
  FGTT_A(2,2)   = 0.000E+00_JPRB
  FGTT_A(3,0)   = 1.217E+00_JPRB
  FGTT_A(3,1)   = 2.331E-04_JPRB
  FGTT_A(3,2)   = 0.000E+00_JPRB
  FGTT_B(1,0)   = 1.746E+07_JPRB
  FGTT_B(1,1)   = 0.000E+00_JPRB
  FGTT_B(1,2)   = 0.000E+00_JPRB
  FGTT_B(2,0)   = 1.295E+08_JPRB
  FGTT_B(2,1)   = 0.000E+00_JPRB
  FGTT_B(2,2)   = 3.261E-01_JPRB
  FGTT_B(3,0)   = 2.235E+12_JPRB
  FGTT_B(3,1)   = 0.000E+00_JPRB
  FGTT_B(3,2)   = 7.027E+04_JPRB
  FGTT_C(1)     = 5.217E-03_JPRB
  FGTT_C(2)     = 2.062E-02_JPRB
  FGTT_C(3)     = 3.607E-02_JPRB
  FGTT_C(4)     = 4.162E-01_JPRB
  FGTT_C(5)     = 8.254E-01_JPRB
  FGTT_ALPHA(1) = 8.038E-01_JPRB
  FGTT_ALPHA(2) = 7.362E-01_JPRB
  FGTT_ALPHA(3) = 5.005E-01_JPRB
  FGTT_DELTA0(1)= 1.404E-03_JPRB
  FGTT_DELTA0(2)= 8.527E-04_JPRB
  FGTT_DELTA0(3)= 5.245E-02_JPRB
  FGTT_D(1)     = 2.269E-01_JPRB
  FGTT_D(2)     = 5.748E-02_JPRB
  FGTT_D(3)     = 6.004E-02_JPRB
  FGTT_P00(1,0) = 5.548E-01_JPRB
  FGTT_P00(1,1) = 1.959E-05_JPRB
  FGTT_P00(1,2) = 8.256E-07_JPRB
  FGTT_P00(2,0) = 1.342E+00_JPRB
  FGTT_P00(2,1) =-9.558E-04_JPRB
  FGTT_P00(2,2) = 3.048E-06_JPRB
  FGTT_P00(3,0) = 1.120E+00_JPRB
  FGTT_P00(3,1) =-3.516E-04_JPRB
  FGTT_P00(3,2) = 1.261E-06_JPRB
  FGTT_P(1,0,0) = 7.075E-02_JPRB
  FGTT_P(1,1,0) = 4.313E-01_JPRB
  FGTT_P(1,2,0) = 3.426E-01_JPRB
  FGTT_P(1,3,0) = 4.490E-02_JPRB
  FGTT_P(1,4,0) = 3.492E-02_JPRB
  FGTT_P(1,5,0) = 3.685E-03_JPRB
  FGTT_P(1,0,1) =-1.804E-03_JPRB
  FGTT_P(1,1,1) =-3.681E-03_JPRB
  FGTT_P(1,2,1) =-1.567E-03_JPRB
  FGTT_P(1,3,1) =-4.267E-04_JPRB
  FGTT_P(1,4,1) =-1.151E-04_JPRB
  FGTT_P(1,5,1) =-6.676E-07_JPRB
  FGTT_P(1,0,2) = 4.438E-06_JPRB
  FGTT_P(1,1,2) = 4.958E-06_JPRB
  FGTT_P(1,2,2) = 2.333E-06_JPRB
  FGTT_P(1,3,2) = 6.923E-07_JPRB
  FGTT_P(1,4,2) = 9.715E-08_JPRB
  FGTT_P(1,5,2) =-5.359E-09_JPRB
  FGTT_P(2,0,0) = 3.579E-01_JPRB
  FGTT_P(2,1,0) = 6.159E-01_JPRB
  FGTT_P(2,2,0) =-3.498E-02_JPRB
  FGTT_P(2,3,0) =-2.019E-02_JPRB
  FGTT_P(2,4,0) = 8.147E-03_JPRB
  FGTT_P(2,5,0) = 3.948E-04_JPRB
  FGTT_P(2,0,1) =-2.120E-03_JPRB
  FGTT_P(2,1,1) =-2.172E-04_JPRB
  FGTT_P(2,2,1) = 4.937E-04_JPRB
  FGTT_P(2,3,1) =-4.819E-05_JPRB
  FGTT_P(2,4,1) =-3.050E-05_JPRB
  FGTT_P(2,5,1) = 6.105E-06_JPRB
  FGTT_P(2,0,2) =-6.443E-07_JPRB
  FGTT_P(2,1,2) =-3.228E-07_JPRB
  FGTT_P(2,2,2) = 8.116E-07_JPRB
  FGTT_P(2,3,2) =-5.903E-08_JPRB
  FGTT_P(2,4,2) =-2.735E-08_JPRB
  FGTT_P(2,5,2) =-1.926E-08_JPRB
  FGTT_P(3,0,0) =-1.948E+00_JPRB
  FGTT_P(3,1,0) =-3.644E-01_JPRB
  FGTT_P(3,2,0) = 2.913E-02_JPRB
  FGTT_P(3,3,0) = 6.445E-03_JPRB
  FGTT_P(3,4,0) =-2.560E-03_JPRB
  FGTT_P(3,5,0) =-4.321E-04_JPRB
  FGTT_P(3,0,1) = 7.917E-04_JPRB
  FGTT_P(3,1,1) =-8.277E-05_JPRB
  FGTT_P(3,2,1) = 4.005E-05_JPRB
  FGTT_P(3,3,1) = 1.399E-05_JPRB
  FGTT_P(3,4,1) =-4.348E-06_JPRB
  FGTT_P(3,5,1) = 9.689E-07_JPRB
  FGTT_P(3,0,2) = 4.307E-07_JPRB
  FGTT_P(3,1,2) = 4.382E-09_JPRB
  FGTT_P(3,2,2) =-1.461E-08_JPRB
  FGTT_P(3,3,2) =-5.728E-09_JPRB
  FGTT_P(3,4,2) = 6.934E-09_JPRB
  FGTT_P(3,5,2) =-2.788E-09_JPRB
  FGTT_Q(1,0)   = 6.578E-01_JPRB
  FGTT_Q(1,1)   =-1.986E-06_JPRB
  FGTT_Q(1,2)   = 1.423E-11_JPRB
  FGTT_Q(2,0)   = 9.047E-01_JPRB
  FGTT_Q(2,1)   =-3.611E-06_JPRB
  FGTT_Q(2,2)   = 2.157E-11_JPRB
  FGTT_Q(3,0)   = 2.739E-02_JPRB
  FGTT_Q(3,1)   = 3.674E-07_JPRB
  FGTT_Q(3,2)   =-2.626E-12_JPRB
  FGTT_OA(1)    = 3.564E+00_JPRB
  FGTT_OA(2)    = 0.000E+00_JPRB  ! H2O-O3 thermal overlap switched off
  !FGTT_OA(2)    = 2.407E+00_JPRB
  FGTT_OA(3)    = 2.033E+00_JPRB
  FGTT_OB(1)    = 1.238E+00_JPRB
  FGTT_OB(2)    = 5.000E-01_JPRB
  FGTT_OB(3)    = 5.000E-01_JPRB
  FGTT_OC(1)    = 3.407E+00_JPRB
  FGTT_OC(2)    = 3.135E+00_JPRB
  FGTT_OC(3)    = 4.285E+00_JPRB
  FGTT_OD(1)    = 1.455E+00_JPRB
  FGTT_OD(2)    = 1.000E+00_JPRB
  FGTT_OD(3)    = 1.000E+00_JPRB

  ! mass fraction of CO2 in dry air corresponding to 389.0ppmv
  QCO2=.5911E-03_JPRB

  ! fitting parameters for Rayleigh scattering
  FRS_K_SCAT0=1.126E-06_JPRB
  FRS_P_CRIT =1.680E+05_JPRB
  FRS_BETA   =2.444E-01_JPRB

ENDIF

! Default values for cloud model - first index denotes spectral band:
!   1 - solar
!   2 - thermal
IF ( NRAY == 1 ) THEN
  ! liquid clouds - Stephens 1978, ice clouds - Rockel et al. 1991
  FCM_DEL_A(1) = 9.55496E-01_JPRB
  FCM_DEL_A(2) = 1.79152E+02_JPRB
  FCM_DEL_D(1) = 5.72959E+01_JPRB
  FCM_DEL_D(2) = 2.00159E+02_JPRB
  FCM_MU_A(1)  = 8.66200E-01_JPRB
  FCM_MU_A(2)  = 6.65532E-01_JPRB
  FCM_MU_D(1)  = 1.00000E+00_JPRB
  FCM_MU_D(2)  = 6.70517E-01_JPRB
  FCM_N_I      = 1.00000E-01_JPRB
  FCM_N_L      = 1.00000E-01_JPRB
  FCM_P_AI(1,0)= 1.48920E+01_JPRB
  FCM_P_AI(1,1)=-3.71379E+01_JPRB
  FCM_P_AI(1,2)= 0.00000E+00_JPRB
  FCM_P_AI(1,3)= 0.00000E+00_JPRB
  FCM_P_AI(2,0)= 9.76920E+00_JPRB
  FCM_P_AI(2,1)=-7.54840E+00_JPRB
  FCM_P_AI(2,2)= 0.00000E+00_JPRB
  FCM_P_AI(2,3)= 0.00000E+00_JPRB
  FCM_P_AL(1,0)= 1.42062E+00_JPRB
  FCM_P_AL(1,1)=-3.54037E+00_JPRB
  FCM_P_AL(1,2)= 1.73535E+00_JPRB
  FCM_P_AL(1,3)= 0.00000E+00_JPRB
  FCM_P_AL(2,0)= 5.08522E+00_JPRB
  FCM_P_AL(2,1)=-1.08144E+01_JPRB
  FCM_P_AL(2,2)= 5.16331E+00_JPRB
  FCM_P_AL(2,3)= 0.00000E+00_JPRB
  FCM_P_DI(1,0)= 1.30487E+01_JPRB
  FCM_P_DI(1,1)= 1.98670E+01_JPRB
  FCM_P_DI(1,2)=-5.08578E+01_JPRB
  FCM_P_DI(1,3)= 0.00000E+00_JPRB
  FCM_P_DI(2,0)= 1.23173E+01_JPRB
  FCM_P_DI(2,1)=-3.32808E+01_JPRB
  FCM_P_DI(2,2)= 5.77295E+01_JPRB
  FCM_P_DI(2,3)=-7.35728E+01_JPRB
  FCM_P_DL(1,0)= 4.68646E+00_JPRB
  FCM_P_DL(1,1)=-8.77652E+00_JPRB
  FCM_P_DL(1,2)= 2.12789E+00_JPRB
  FCM_P_DL(1,3)= 0.00000E+00_JPRB
  FCM_P_DL(2,0)= 4.52292E+00_JPRB
  FCM_P_DL(2,1)=-1.43327E+01_JPRB
  FCM_P_DL(2,2)= 1.16650E+01_JPRB
  FCM_P_DL(2,3)= 0.00000E+00_JPRB
  FCM_P_GI(1,0)=-2.01472E+00_JPRB
  FCM_P_GI(1,1)= 9.16284E+01_JPRB
  FCM_P_GI(1,2)= 7.82275E+01_JPRB
  FCM_P_GI(1,3)= 0.00000E+00_JPRB
  FCM_P_GI(2,0)=-7.55124E-01_JPRB
  FCM_P_GI(2,1)=-4.68837E+00_JPRB
  FCM_P_GI(2,2)= 6.72877E+01_JPRB
  FCM_P_GI(2,3)= 0.00000E+00_JPRB
  FCM_P_GL(1,0)= 8.82686E-01_JPRB
  FCM_P_GL(1,1)=-2.02445E+00_JPRB
  FCM_P_GL(1,2)= 1.45259E+00_JPRB
  FCM_P_GL(1,3)= 0.00000E+00_JPRB
  FCM_P_GL(2,0)= 1.93784E-01_JPRB
  FCM_P_GL(2,1)=-6.07676E-01_JPRB
  FCM_P_GL(2,2)= 5.37734E-01_JPRB
  FCM_P_GL(2,3)= 0.00000E+00_JPRB
  FCM_Q_AI(1,1)= 1.86588E+01_JPRB
  FCM_Q_AI(1,2)= 0.00000E+00_JPRB
  FCM_Q_AI(1,3)= 0.00000E+00_JPRB
  FCM_Q_AI(2,1)= 2.78063E+00_JPRB
  FCM_Q_AI(2,2)= 0.00000E+00_JPRB
  FCM_Q_AI(2,3)= 0.00000E+00_JPRB
  FCM_Q_AL(1,1)=-2.81210E+00_JPRB
  FCM_Q_AL(1,2)= 2.35758E+00_JPRB
  FCM_Q_AL(1,3)= 0.00000E+00_JPRB
  FCM_Q_AL(2,1)=-2.31252E+00_JPRB
  FCM_Q_AL(2,2)= 1.56848E+00_JPRB
  FCM_Q_AL(2,3)= 0.00000E+00_JPRB
  FCM_Q_DI(1,1)= 1.04582E+01_JPRB
  FCM_Q_DI(1,2)= 6.15641E+00_JPRB
  FCM_Q_DI(1,3)= 0.00000E+00_JPRB
  FCM_Q_DI(2,1)= 1.18370E+00_JPRB
  FCM_Q_DI(2,2)= 2.23187E+00_JPRB
  FCM_Q_DI(2,3)= 7.00424E+00_JPRB
  FCM_Q_DL(1,1)=-2.18205E+00_JPRB
  FCM_Q_DL(1,2)= 1.34831E+00_JPRB
  FCM_Q_DL(1,3)= 0.00000E+00_JPRB
  FCM_Q_DL(2,1)=-3.12310E+00_JPRB
  FCM_Q_DL(2,2)= 2.57959E+00_JPRB
  FCM_Q_DL(2,3)= 0.00000E+00_JPRB
  FCM_Q_GI(1,1)= 7.89999E+01_JPRB
  FCM_Q_GI(1,2)= 5.24929E+01_JPRB
  FCM_Q_GI(1,3)= 0.00000E+00_JPRB
  FCM_Q_GI(2,1)= 4.93801E+00_JPRB
  FCM_Q_GI(2,2)= 2.38096E+01_JPRB
  FCM_Q_GI(2,3)= 0.00000E+00_JPRB
  FCM_Q_GL(1,1)=-2.20830E+00_JPRB
  FCM_Q_GL(1,2)= 1.39832E+00_JPRB
  FCM_Q_GL(1,3)= 0.00000E+00_JPRB
  FCM_Q_GL(2,1)=-3.27770E+00_JPRB
  FCM_Q_GL(2,2)= 2.78446E+00_JPRB
  FCM_Q_GL(2,3)= 0.00000E+00_JPRB
  REXP_NEB     = 8._JPRB
ELSE
  ! SW/LW liquid clouds - Hu and Stamnes 1993
  ! SW    ice    clouds - Key  et al. 2002
  ! LW    ice    clouds - Yang et al. 2005
  FCM_IWC2DE(1,-2)= 1.000e+01_JPRB
  FCM_IWC2DE(1,-1)= 1.500e+02_JPRB
  FCM_IWC2DE(1, 0)= 0.000e+00_JPRB
  FCM_IWC2DE(1, 1)= 1.646e+02_JPRB
  FCM_IWC2DE(1, 2)= 0.000e+00_JPRB
  FCM_IWC2DE(1, 3)= 9.800e-02_JPRB
  FCM_IWC2DE(2,-2)= 1.000e+01_JPRB
  FCM_IWC2DE(2,-1)= 1.500e+02_JPRB
  FCM_IWC2DE(2, 0)= 0.000e+00_JPRB
  FCM_IWC2DE(2, 1)= 1.646e+02_JPRB
  FCM_IWC2DE(2, 2)= 0.000e+00_JPRB
  FCM_IWC2DE(2, 3)= 9.800e-02_JPRB
  FCM_LWC2RE(1,-2)= 4.000e+00_JPRB
  FCM_LWC2RE(1,-1)= 6.000e+01_JPRB
  FCM_LWC2RE(1, 0)= 5.200e+00_JPRB
  FCM_LWC2RE(1, 1)= 5.860e+00_JPRB
  FCM_LWC2RE(1, 2)= 0.000e+00_JPRB
  FCM_LWC2RE(1, 3)= 3.000e-01_JPRB
  FCM_LWC2RE(2,-2)= 4.000e+00_JPRB
  FCM_LWC2RE(2,-1)= 6.000e+01_JPRB
  FCM_LWC2RE(2, 0)= 5.200e+00_JPRB
  FCM_LWC2RE(2, 1)= 5.860e+00_JPRB
  FCM_LWC2RE(2, 2)= 0.000e+00_JPRB
  FCM_LWC2RE(2, 3)= 3.000e-01_JPRB
  FCM_AI          = 3.751e+00_JPRB
  FCM_AL          = 7.670e+00_JPRB
  FCM_B_AI        = 5.291e+00_JPRB
  FCM_B_AL        = 1.029e+01_JPRB
  FCM_B_BI        = 2.438e-02_JPRB
  FCM_B_BL        = 2.862e-02_JPRB
  FCM_DEL_AI      = 1.351e+00_JPRB
  FCM_DEL_AL      = 4.370e-01_JPRB
  FCM_DEL_DI      = 9.440e+00_JPRB
  FCM_DEL_DL      = 2.656e+01_JPRB
  FCM_MU_AI       = 1.110e+00_JPRB
  FCM_MU_AL       = 1.112e+00_JPRB
  FCM_MU_DI       = 2.502e+00_JPRB
  FCM_MU_DL       = 1.545e+00_JPRB
  FCM_NU_AI       = 7.968e-01_JPRB
  FCM_NU_AL       = 6.170e-01_JPRB
  FCM_NU_DI       = 1.972e-01_JPRB
  FCM_NU_DL       = 4.286e-01_JPRB
  FCM_P_AI(1,0)   = 2.220e+00_JPRB
  FCM_P_AI(1,1)   =-5.987e-01_JPRB
  FCM_P_AI(1,2)   = 1.424e-02_JPRB
  FCM_P_AI(1,3)   = 0.000e+00_JPRB
  FCM_P_AI(2,0)   = 5.358e+00_JPRB
  FCM_P_AI(2,1)   =-3.019e-01_JPRB
  FCM_P_AI(2,2)   =-6.257e-02_JPRB
  FCM_P_AI(2,3)   = 0.000e+00_JPRB
  FCM_P_AL(1,0)   = 2.269e+00_JPRB
  FCM_P_AL(1,1)   =-7.348e-01_JPRB
  FCM_P_AL(1,2)   = 1.324e-02_JPRB
  FCM_P_AL(1,3)   = 0.000e+00_JPRB
  FCM_P_AL(2,0)   = 4.556e+00_JPRB
  FCM_P_AL(2,1)   = 1.136e+00_JPRB
  FCM_P_AL(2,2)   =-2.839e-01_JPRB
  FCM_P_AL(2,3)   = 0.000e+00_JPRB
  FCM_P_DI(1,0)   = 7.317e+00_JPRB
  FCM_P_DI(1,1)   =-1.027e+00_JPRB
  FCM_P_DI(1,2)   =-2.098e-03_JPRB
  FCM_P_DI(1,3)   = 0.000e+00_JPRB
  FCM_P_DI(2,0)   = 3.459e+00_JPRB
  FCM_P_DI(2,1)   = 5.067e-01_JPRB
  FCM_P_DI(2,2)   =-2.246e-01_JPRB
  FCM_P_DI(2,3)   = 0.000e+00_JPRB
  FCM_P_DL(1,0)   = 6.738e+00_JPRB
  FCM_P_DL(1,1)   =-1.401e+00_JPRB
  FCM_P_DL(1,2)   = 4.148e-02_JPRB
  FCM_P_DL(1,3)   = 0.000e+00_JPRB
  FCM_P_DL(2,0)   = 2.597e+00_JPRB
  FCM_P_DL(2,1)   = 7.839e-01_JPRB
  FCM_P_DL(2,2)   =-3.055e-01_JPRB
  FCM_P_DL(2,3)   = 0.000e+00_JPRB
  FCM_P_GI(1,0)   = 8.456e-01_JPRB
  FCM_P_GI(1,1)   = 3.288e-02_JPRB
  FCM_P_GI(1,2)   =-1.781e-03_JPRB
  FCM_P_GI(1,3)   = 0.000e+00_JPRB
  FCM_P_GI(2,0)   = 1.235e+00_JPRB
  FCM_P_GI(2,1)   =-5.668e-01_JPRB
  FCM_P_GI(2,2)   = 1.839e-01_JPRB
  FCM_P_GI(2,3)   = 0.000e+00_JPRB
  FCM_P_GL(1,0)   = 8.522e-01_JPRB
  FCM_P_GL(1,1)   = 2.299e-01_JPRB
  FCM_P_GL(1,2)   =-2.660e-02_JPRB
  FCM_P_GL(1,3)   = 0.000e+00_JPRB
  FCM_P_GL(2,0)   =-1.323e-01_JPRB
  FCM_P_GL(2,1)   = 4.596e-01_JPRB
  FCM_P_GL(2,2)   =-4.061e-02_JPRB
  FCM_P_GL(2,3)   = 0.000e+00_JPRB
  FCM_Q_AI(1,1)   = 2.187e-04_JPRB
  FCM_Q_AI(1,2)   = 2.271e-08_JPRB
  FCM_Q_AI(1,3)   = 0.000e+00_JPRB
  FCM_Q_AI(2,1)   =-1.382e-07_JPRB
  FCM_Q_AI(2,2)   = 9.871e-11_JPRB
  FCM_Q_AI(2,3)   = 0.000e+00_JPRB
  FCM_Q_AL(1,1)   = 3.618e-07_JPRB
  FCM_Q_AL(1,2)   = 9.838e-11_JPRB
  FCM_Q_AL(1,3)   = 0.000e+00_JPRB
  FCM_Q_AL(2,1)   = 4.615e-02_JPRB
  FCM_Q_AL(2,2)   = 3.117e-02_JPRB
  FCM_Q_AL(2,3)   = 0.000e+00_JPRB
  FCM_Q_DI(1,1)   = 6.151e-08_JPRB
  FCM_Q_DI(1,2)   = 4.742e-10_JPRB
  FCM_Q_DI(1,3)   = 0.000e+00_JPRB
  FCM_Q_DI(2,1)   =-4.750e-02_JPRB
  FCM_Q_DI(2,2)   = 1.181e-02_JPRB
  FCM_Q_DI(2,3)   = 0.000e+00_JPRB
  FCM_Q_DL(1,1)   = 7.615e-05_JPRB
  FCM_Q_DL(1,2)   = 2.878e-09_JPRB
  FCM_Q_DL(1,3)   = 0.000e+00_JPRB
  FCM_Q_DL(2,1)   =-2.634e-01_JPRB
  FCM_Q_DL(2,2)   = 8.519e-02_JPRB
  FCM_Q_DL(2,3)   = 0.000e+00_JPRB
  FCM_Q_GI(1,1)   =-1.118e-07_JPRB
  FCM_Q_GI(1,2)   = 1.007e-10_JPRB
  FCM_Q_GI(1,3)   = 0.000e+00_JPRB
  FCM_Q_GI(2,1)   =-1.310e-01_JPRB
  FCM_Q_GI(2,2)   = 5.860e-02_JPRB
  FCM_Q_GI(2,3)   = 0.000e+00_JPRB
  FCM_Q_GL(1,1)   = 2.816e-07_JPRB
  FCM_Q_GL(1,2)   = 1.001e-10_JPRB
  FCM_Q_GL(1,3)   = 0.000e+00_JPRB
  FCM_Q_GL(2,1)   =-1.056e-01_JPRB
  FCM_Q_GL(2,2)   = 6.589e-03_JPRB
  FCM_Q_GL(2,3)   = 0.000e+00_JPRB
ENDIF

! default values for NER statistical model
IF ( LRPROX ) THEN
  
  ! new (requires LRPROX, tuning for 87 levels)
  FSM_AA(0)= 2.484E-01_JPRB
  FSM_AA(1)=-3.614E-01_JPRB
  FSM_AA(2)= 3.958E-01_JPRB
  FSM_BB(0)= 1.717E-01_JPRB
  FSM_BB(1)= 1.414E-02_JPRB
  FSM_BB(2)=-1.986E-02_JPRB

  ! old
  FSM_CC=0.14_JPRB
  FSM_DD=3.5_JPRB
  FSM_EE=0.07_JPRB
  FSM_FF=1.5_JPRB
  FSM_GG=0.14_JPRB
  FSM_HH=20._JPRB
  FSM_II=0._JPRB

ELSE

  ! old
  FSM_CC=0.29_JPRB
  FSM_DD=2.0_JPRB
  FSM_EE=0.07_JPRB
  FSM_FF=1.5_JPRB
  FSM_GG=0.14_JPRB
  FSM_HH=20._JPRB
  FSM_II=0._JPRB

ENDIF

! default values for proportions of Lambertian scattering
RLAMB_SOLID=0._JPRB
RLAMB_WATER=0._JPRB

!     Remark : the values or RII0 is calculated and not set up

!     ------------------------------------------------------------------

!*       2.    Modify default values.
!              ----------------------

CALL POSNAM(NULNAM,'NAMPHY3')
READ(NULNAM,NAMPHY3)
!     ------------------------------------------------------------------

!*       3.    Print final values.
!              -------------------

WRITE(UNIT=KULOUT,FMT='('' COMMON YOMPHY3 '')')
WRITE(UNIT=KULOUT,FMT='('' BSFSA = '',6(1X,E10.4),'' BSFSI = '',E10.4 &
 & ,'' BSFSN = '',E10.4,'' BSFTA = '',6(1X,E10.4),'' BSFTI = '',E10.4,/&
 & ,'' BSFTN = '',E10.4,'' EARRT = '',E10.4,'' EOASA = ''&
 & ,6(1X,E10.4),'' EOASI = '',E10.4,'' EOASN = '',E10.4 &
 & ,'' EOATA = '',6(1X,E10.4),/,'' EOATI = '',E10.4,'' EOATN = '',E10.4 &
 & ,'' EODSA = '',6(1X,E10.4),'' EODSI = '',E10.4,'' EODSN = '',E10.4 &
 & ,'' EODTA = '',6(1X,E10.4),/,'' EODTI = '',E10.4 &
 & ,'' EODTN = '',E10.4,/&
 & ,'' QCO2 = '',E10.4 &
 & ,'' USAA = '',6(1X,E10.4),'' USAI = '',E10.4,'' USAN = '',E10.4 &
 & ,'' USBA = '',E10.4,/,'' USBI = '',E10.4,'' USBN = '',E10.4,/&
 & ,'' GIREC1 = '',E10.4,'' GIREC2 = '',E10.4,'' GIREC3 = '',E10.4 &
 & ,'' GIREC4 = '',E10.4)')&
 & BSFSA,BSFSI,BSFSN,BSFTA,BSFTI,BSFTN,EARRT,EOASA,EOASI,EOASN,&
 & EOATA,EOATI,EOATN,EODSA,EODSI,EODSN,EODTA,EODTI,EODTN,&
 & QCO2,USAA,USAI,USAN,USBA,USBI,USBN,&
 & GIREC1,GIREC2,GIREC3,GIREC4

IF ( NRAY == 1 ) THEN

  WRITE(UNIT=KULOUT,FMT= &
   & '('' EORAY = '',E10.4,/&
   & ,'' GCA(6) = '',6E11.4,/,'' GCB(6) = '',6E11.4,/,'' GCC(6) = ''&
   & ,6E11.4,/,'' GCD4 = '',E11.4,'' GCE4 = '',E11.4,/&
   & ,'' VDP(5,6) = '',5E12.5,/,5(12X,5E12.5,/)&
   & ,'' VNP(5,6) = '',5E12.5,/,5(12X,5E12.5,/)&
   & ,'' GIREC1 = '',E10.4,'' GIREC2 = '',E10.4,'' GIREC3 = '',E10.4 &
   & ,'' GIREC4 = '',E10.4,/ &
   & ,'' GOLCA = '',6(1X,E10.4),/ &
   & ,'' GOLCB = '',6(1X,E10.4),/ &
   & ,'' GOLCC = '',6(1X,E10.4) &
   & )')&
   & EORAY,GCA,GCB,GCC,GCD4,GCE4,VDP,VNP,GOLCA,GOLCB,GOLCC


ELSE

  WRITE(KULOUT,'('' RTL                ='',1X,ES12.5)')    RTL
  WRITE(KULOUT,'('' FRS_K_SCAT0        ='',1X,ES12.5)')    FRS_K_SCAT0
  WRITE(KULOUT,'('' FRS_P_CRIT         ='',1X,ES12.5)')    FRS_P_CRIT 
  WRITE(KULOUT,'('' FRS_BETA           ='',1X,ES12.5)')    FRS_BETA
  WRITE(KULOUT,'('' FGTS_A     (:,0)   ='',3(1X,ES12.5))') FGTS_A     (:,0)
  WRITE(KULOUT,'('' FGTS_A     (:,1)   ='',3(1X,ES12.5))') FGTS_A     (:,1)
  WRITE(KULOUT,'('' FGTS_A     (:,2)   ='',3(1X,ES12.5))') FGTS_A     (:,2)
  WRITE(KULOUT,'('' FGTS_B     (:,0)   ='',3(1X,ES12.5))') FGTS_B     (:,0)
  WRITE(KULOUT,'('' FGTS_B     (:,1)   ='',3(1X,ES12.5))') FGTS_B     (:,1)
  WRITE(KULOUT,'('' FGTS_B     (:,2)   ='',3(1X,ES12.5))') FGTS_B     (:,2)
  WRITE(KULOUT,'('' FGTS_C     (:)     ='',5(1X,ES12.5))') FGTS_C     (:)
  WRITE(KULOUT,'('' FGTS_ALPHA (:)     ='',3(1X,ES12.5))') FGTS_ALPHA (:)
  WRITE(KULOUT,'('' FGTS_DELTA0(:)     ='',3(1X,ES12.5))') FGTS_DELTA0(:)
  WRITE(KULOUT,'('' FGTS_D     (:)     ='',3(1X,ES12.5))') FGTS_D     (:)
  WRITE(KULOUT,'('' FGTS_P00   (:,0)   ='',3(1X,ES12.5))') FGTS_P00   (:,0)
  WRITE(KULOUT,'('' FGTS_P00   (:,1)   ='',3(1X,ES12.5))') FGTS_P00   (:,1)
  WRITE(KULOUT,'('' FGTS_P00   (:,2)   ='',3(1X,ES12.5))') FGTS_P00   (:,2)
  WRITE(KULOUT,'('' FGTS_P     (:,0,0) ='',3(1X,ES12.5))') FGTS_P     (:,0,0)
  WRITE(KULOUT,'('' FGTS_P     (:,1,0) ='',3(1X,ES12.5))') FGTS_P     (:,1,0)
  WRITE(KULOUT,'('' FGTS_P     (:,2,0) ='',3(1X,ES12.5))') FGTS_P     (:,2,0)
  WRITE(KULOUT,'('' FGTS_P     (:,3,0) ='',3(1X,ES12.5))') FGTS_P     (:,3,0)
  WRITE(KULOUT,'('' FGTS_P     (:,4,0) ='',3(1X,ES12.5))') FGTS_P     (:,4,0)
  WRITE(KULOUT,'('' FGTS_P     (:,5,0) ='',3(1X,ES12.5))') FGTS_P     (:,5,0)
  WRITE(KULOUT,'('' FGTS_P     (:,0,1) ='',3(1X,ES12.5))') FGTS_P     (:,0,1)
  WRITE(KULOUT,'('' FGTS_P     (:,1,1) ='',3(1X,ES12.5))') FGTS_P     (:,1,1)
  WRITE(KULOUT,'('' FGTS_P     (:,2,1) ='',3(1X,ES12.5))') FGTS_P     (:,2,1)
  WRITE(KULOUT,'('' FGTS_P     (:,3,1) ='',3(1X,ES12.5))') FGTS_P     (:,3,1)
  WRITE(KULOUT,'('' FGTS_P     (:,4,1) ='',3(1X,ES12.5))') FGTS_P     (:,4,1)
  WRITE(KULOUT,'('' FGTS_P     (:,5,1) ='',3(1X,ES12.5))') FGTS_P     (:,5,1)
  WRITE(KULOUT,'('' FGTS_P     (:,0,2) ='',3(1X,ES12.5))') FGTS_P     (:,0,2)
  WRITE(KULOUT,'('' FGTS_P     (:,1,2) ='',3(1X,ES12.5))') FGTS_P     (:,1,2)
  WRITE(KULOUT,'('' FGTS_P     (:,2,2) ='',3(1X,ES12.5))') FGTS_P     (:,2,2)
  WRITE(KULOUT,'('' FGTS_P     (:,3,2) ='',3(1X,ES12.5))') FGTS_P     (:,3,2)
  WRITE(KULOUT,'('' FGTS_P     (:,4,2) ='',3(1X,ES12.5))') FGTS_P     (:,4,2)
  WRITE(KULOUT,'('' FGTS_P     (:,5,2) ='',3(1X,ES12.5))') FGTS_P     (:,5,2)
  WRITE(KULOUT,'('' FGTS_Q     (:,0)   ='',3(1X,ES12.5))') FGTS_Q     (:,0)
  WRITE(KULOUT,'('' FGTS_Q     (:,1)   ='',3(1X,ES12.5))') FGTS_Q     (:,1)
  WRITE(KULOUT,'('' FGTS_Q     (:,2)   ='',3(1X,ES12.5))') FGTS_Q     (:,2)
  WRITE(KULOUT,'('' FGTS_OA    (:)     ='',3(1X,ES12.5))') FGTS_OA    (:)
  WRITE(KULOUT,'('' FGTS_OB    (:)     ='',3(1X,ES12.5))') FGTS_OB    (:)
  WRITE(KULOUT,'('' FGTS_OC    (:)     ='',3(1X,ES12.5))') FGTS_OC    (:)
  WRITE(KULOUT,'('' FGTS_OD    (:)     ='',3(1X,ES12.5))') FGTS_OD    (:)

  WRITE(KULOUT,'('' FGTC_A     (:,0)   ='',3(1X,ES12.5))') FGTC_A     (:,0)
  WRITE(KULOUT,'('' FGTC_A     (:,1)   ='',3(1X,ES12.5))') FGTC_A     (:,1)
  WRITE(KULOUT,'('' FGTC_A     (:,2)   ='',3(1X,ES12.5))') FGTC_A     (:,2)
  WRITE(KULOUT,'('' FGTC_B     (:,0)   ='',3(1X,ES12.5))') FGTC_B     (:,0)
  WRITE(KULOUT,'('' FGTC_B     (:,1)   ='',3(1X,ES12.5))') FGTC_B     (:,1)
  WRITE(KULOUT,'('' FGTC_B     (:,2)   ='',3(1X,ES12.5))') FGTC_B     (:,2)
  WRITE(KULOUT,'('' FGTC_C     (:)     ='',5(1X,ES12.5))') FGTC_C     (:)
  WRITE(KULOUT,'('' FGTC_ALPHA (:)     ='',3(1X,ES12.5))') FGTC_ALPHA (:)
  WRITE(KULOUT,'('' FGTC_DELTA0(:)     ='',3(1X,ES12.5))') FGTC_DELTA0(:)
  WRITE(KULOUT,'('' FGTC_D     (:)     ='',3(1X,ES12.5))') FGTC_D     (:)
  WRITE(KULOUT,'('' FGTC_P00   (:,0)   ='',3(1X,ES12.5))') FGTC_P00   (:,0)
  WRITE(KULOUT,'('' FGTC_P00   (:,1)   ='',3(1X,ES12.5))') FGTC_P00   (:,1)
  WRITE(KULOUT,'('' FGTC_P00   (:,2)   ='',3(1X,ES12.5))') FGTC_P00   (:,2)
  WRITE(KULOUT,'('' FGTC_P     (:,0,0) ='',3(1X,ES12.5))') FGTC_P     (:,0,0)
  WRITE(KULOUT,'('' FGTC_P     (:,1,0) ='',3(1X,ES12.5))') FGTC_P     (:,1,0)
  WRITE(KULOUT,'('' FGTC_P     (:,2,0) ='',3(1X,ES12.5))') FGTC_P     (:,2,0)
  WRITE(KULOUT,'('' FGTC_P     (:,3,0) ='',3(1X,ES12.5))') FGTC_P     (:,3,0)
  WRITE(KULOUT,'('' FGTC_P     (:,4,0) ='',3(1X,ES12.5))') FGTC_P     (:,4,0)
  WRITE(KULOUT,'('' FGTC_P     (:,5,0) ='',3(1X,ES12.5))') FGTC_P     (:,5,0)
  WRITE(KULOUT,'('' FGTC_P     (:,0,1) ='',3(1X,ES12.5))') FGTC_P     (:,0,1)
  WRITE(KULOUT,'('' FGTC_P     (:,1,1) ='',3(1X,ES12.5))') FGTC_P     (:,1,1)
  WRITE(KULOUT,'('' FGTC_P     (:,2,1) ='',3(1X,ES12.5))') FGTC_P     (:,2,1)
  WRITE(KULOUT,'('' FGTC_P     (:,3,1) ='',3(1X,ES12.5))') FGTC_P     (:,3,1)
  WRITE(KULOUT,'('' FGTC_P     (:,4,1) ='',3(1X,ES12.5))') FGTC_P     (:,4,1)
  WRITE(KULOUT,'('' FGTC_P     (:,5,1) ='',3(1X,ES12.5))') FGTC_P     (:,5,1)
  WRITE(KULOUT,'('' FGTC_P     (:,0,2) ='',3(1X,ES12.5))') FGTC_P     (:,0,2)
  WRITE(KULOUT,'('' FGTC_P     (:,1,2) ='',3(1X,ES12.5))') FGTC_P     (:,1,2)
  WRITE(KULOUT,'('' FGTC_P     (:,2,2) ='',3(1X,ES12.5))') FGTC_P     (:,2,2)
  WRITE(KULOUT,'('' FGTC_P     (:,3,2) ='',3(1X,ES12.5))') FGTC_P     (:,3,2)
  WRITE(KULOUT,'('' FGTC_P     (:,4,2) ='',3(1X,ES12.5))') FGTC_P     (:,4,2)
  WRITE(KULOUT,'('' FGTC_P     (:,5,2) ='',3(1X,ES12.5))') FGTC_P     (:,5,2)
  WRITE(KULOUT,'('' FGTC_Q     (:,0)   ='',3(1X,ES12.5))') FGTC_Q     (:,0)
  WRITE(KULOUT,'('' FGTC_Q     (:,1)   ='',3(1X,ES12.5))') FGTC_Q     (:,1)
  WRITE(KULOUT,'('' FGTC_Q     (:,2)   ='',3(1X,ES12.5))') FGTC_Q     (:,2)
  WRITE(KULOUT,'('' FGTC_OA    (:)     ='',3(1X,ES12.5))') FGTC_OA    (:)
  WRITE(KULOUT,'('' FGTC_OB    (:)     ='',3(1X,ES12.5))') FGTC_OB    (:)
  WRITE(KULOUT,'('' FGTC_OC    (:)     ='',3(1X,ES12.5))') FGTC_OC    (:)
  WRITE(KULOUT,'('' FGTC_OD    (:)     ='',3(1X,ES12.5))') FGTC_OD    (:)

  WRITE(KULOUT,'('' FGTT_A     (:,0)   ='',3(1X,ES12.5))') FGTT_A     (:,0)
  WRITE(KULOUT,'('' FGTT_A     (:,1)   ='',3(1X,ES12.5))') FGTT_A     (:,1)
  WRITE(KULOUT,'('' FGTT_A     (:,2)   ='',3(1X,ES12.5))') FGTT_A     (:,2)
  WRITE(KULOUT,'('' FGTT_B     (:,0)   ='',3(1X,ES12.5))') FGTT_B     (:,0)
  WRITE(KULOUT,'('' FGTT_B     (:,1)   ='',3(1X,ES12.5))') FGTT_B     (:,1)
  WRITE(KULOUT,'('' FGTT_B     (:,2)   ='',3(1X,ES12.5))') FGTT_B     (:,2)
  WRITE(KULOUT,'('' FGTT_C     (:)     ='',5(1X,ES12.5))') FGTT_C     (:)
  WRITE(KULOUT,'('' FGTT_ALPHA (:)     ='',3(1X,ES12.5))') FGTT_ALPHA (:)
  WRITE(KULOUT,'('' FGTT_DELTA0(:)     ='',3(1X,ES12.5))') FGTT_DELTA0(:)
  WRITE(KULOUT,'('' FGTT_D     (:)     ='',3(1X,ES12.5))') FGTT_D     (:)
  WRITE(KULOUT,'('' FGTT_P00   (:,0)   ='',3(1X,ES12.5))') FGTT_P00   (:,0)
  WRITE(KULOUT,'('' FGTT_P00   (:,1)   ='',3(1X,ES12.5))') FGTT_P00   (:,1)
  WRITE(KULOUT,'('' FGTT_P00   (:,2)   ='',3(1X,ES12.5))') FGTT_P00   (:,2)
  WRITE(KULOUT,'('' FGTT_P     (:,0,0) ='',3(1X,ES12.5))') FGTT_P     (:,0,0)
  WRITE(KULOUT,'('' FGTT_P     (:,1,0) ='',3(1X,ES12.5))') FGTT_P     (:,1,0)
  WRITE(KULOUT,'('' FGTT_P     (:,2,0) ='',3(1X,ES12.5))') FGTT_P     (:,2,0)
  WRITE(KULOUT,'('' FGTT_P     (:,3,0) ='',3(1X,ES12.5))') FGTT_P     (:,3,0)
  WRITE(KULOUT,'('' FGTT_P     (:,4,0) ='',3(1X,ES12.5))') FGTT_P     (:,4,0)
  WRITE(KULOUT,'('' FGTT_P     (:,5,0) ='',3(1X,ES12.5))') FGTT_P     (:,5,0)
  WRITE(KULOUT,'('' FGTT_P     (:,0,1) ='',3(1X,ES12.5))') FGTT_P     (:,0,1)
  WRITE(KULOUT,'('' FGTT_P     (:,1,1) ='',3(1X,ES12.5))') FGTT_P     (:,1,1)
  WRITE(KULOUT,'('' FGTT_P     (:,2,1) ='',3(1X,ES12.5))') FGTT_P     (:,2,1)
  WRITE(KULOUT,'('' FGTT_P     (:,3,1) ='',3(1X,ES12.5))') FGTT_P     (:,3,1)
  WRITE(KULOUT,'('' FGTT_P     (:,4,1) ='',3(1X,ES12.5))') FGTT_P     (:,4,1)
  WRITE(KULOUT,'('' FGTT_P     (:,5,1) ='',3(1X,ES12.5))') FGTT_P     (:,5,1)
  WRITE(KULOUT,'('' FGTT_P     (:,0,2) ='',3(1X,ES12.5))') FGTT_P     (:,0,2)
  WRITE(KULOUT,'('' FGTT_P     (:,1,2) ='',3(1X,ES12.5))') FGTT_P     (:,1,2)
  WRITE(KULOUT,'('' FGTT_P     (:,2,2) ='',3(1X,ES12.5))') FGTT_P     (:,2,2)
  WRITE(KULOUT,'('' FGTT_P     (:,3,2) ='',3(1X,ES12.5))') FGTT_P     (:,3,2)
  WRITE(KULOUT,'('' FGTT_P     (:,4,2) ='',3(1X,ES12.5))') FGTT_P     (:,4,2)
  WRITE(KULOUT,'('' FGTT_P     (:,5,2) ='',3(1X,ES12.5))') FGTT_P     (:,5,2)
  WRITE(KULOUT,'('' FGTT_Q     (:,0)   ='',3(1X,ES12.5))') FGTT_Q     (:,0)
  WRITE(KULOUT,'('' FGTT_Q     (:,1)   ='',3(1X,ES12.5))') FGTT_Q     (:,1)
  WRITE(KULOUT,'('' FGTT_Q     (:,2)   ='',3(1X,ES12.5))') FGTT_Q     (:,2)
  WRITE(KULOUT,'('' FGTT_OA    (:)     ='',3(1X,ES12.5))') FGTT_OA    (:)
  WRITE(KULOUT,'('' FGTT_OB    (:)     ='',3(1X,ES12.5))') FGTT_OB    (:)
  WRITE(KULOUT,'('' FGTT_OC    (:)     ='',3(1X,ES12.5))') FGTT_OC    (:)
  WRITE(KULOUT,'('' FGTT_OD    (:)     ='',3(1X,ES12.5))') FGTT_OD    (:)

ENDIF

IF (LCLSATUR) THEN
  WRITE(KULOUT,'('' N_SPBAND = '',I2)') N_SPBAND
  IF ( NRAY == 1 ) THEN
    WRITE(KULOUT,'('' FCM_DEL_A(:)   ='',2(1X,ES12.5))') FCM_DEL_A(:)
    WRITE(KULOUT,'('' FCM_DEL_D(:)   ='',2(1X,ES12.5))') FCM_DEL_D(:)
    WRITE(KULOUT,'('' FCM_MU_A (:)   ='',2(1X,ES12.5))') FCM_MU_A (:)
    WRITE(KULOUT,'('' FCM_MU_D (:)   ='',2(1X,ES12.5))') FCM_MU_D (:)
    WRITE(KULOUT,'('' FCM_N_I        ='',1X,ES12.5)')    FCM_N_I
    WRITE(KULOUT,'('' FCM_N_L        ='',1X,ES12.5)')    FCM_N_L
    WRITE(KULOUT,'('' REXP_NEB       ='',1X,ES12.5)')    REXP_NEB
  ELSE
    WRITE(KULOUT,'('' FCM_IWC2DE(1,:)='',6(1X,ES12.5))') FCM_IWC2DE(1,:)
    WRITE(KULOUT,'('' FCM_IWC2DE(2,:)='',6(1X,ES12.5))') FCM_IWC2DE(2,:)
    WRITE(KULOUT,'('' FCM_LWC2RE(1,:)='',6(1X,ES12.5))') FCM_LWC2RE(1,:)
    WRITE(KULOUT,'('' FCM_LWC2RE(2,:)='',6(1X,ES12.5))') FCM_LWC2RE(2,:)
    WRITE(KULOUT,'('' FCM_AI         ='',1X,ES12.5)')    FCM_AI
    WRITE(KULOUT,'('' FCM_AL         ='',1X,ES12.5)')    FCM_AL
    WRITE(KULOUT,'('' FCM_B_AI       ='',1X,ES12.5)')    FCM_B_AI
    WRITE(KULOUT,'('' FCM_B_AL       ='',1X,ES12.5)')    FCM_B_AL
    WRITE(KULOUT,'('' FCM_B_BI       ='',1X,ES12.5)')    FCM_B_BI
    WRITE(KULOUT,'('' FCM_B_BL       ='',1X,ES12.5)')    FCM_B_BL
    WRITE(KULOUT,'('' FCM_DEL_AI     ='',1X,ES12.5)')    FCM_DEL_AI
    WRITE(KULOUT,'('' FCM_DEL_AL     ='',1X,ES12.5)')    FCM_DEL_AL
    WRITE(KULOUT,'('' FCM_DEL_DI     ='',1X,ES12.5)')    FCM_DEL_DI
    WRITE(KULOUT,'('' FCM_DEL_DL     ='',1X,ES12.5)')    FCM_DEL_DL
    WRITE(KULOUT,'('' FCM_MU_AI      ='',1X,ES12.5)')    FCM_MU_AI
    WRITE(KULOUT,'('' FCM_MU_AL      ='',1X,ES12.5)')    FCM_MU_AL
    WRITE(KULOUT,'('' FCM_MU_DI      ='',1X,ES12.5)')    FCM_MU_DI
    WRITE(KULOUT,'('' FCM_MU_DL      ='',1X,ES12.5)')    FCM_MU_DL
    WRITE(KULOUT,'('' FCM_NU_AI      ='',1X,ES12.5)')    FCM_NU_AI
    WRITE(KULOUT,'('' FCM_NU_AL      ='',1X,ES12.5)')    FCM_NU_AL
    WRITE(KULOUT,'('' FCM_NU_DI      ='',1X,ES12.5)')    FCM_NU_DI
    WRITE(KULOUT,'('' FCM_NU_DL      ='',1X,ES12.5)')    FCM_NU_DL
  ENDIF
  WRITE(KULOUT,'('' FCM_P_AI (:,0) ='',2(1X,ES12.5))') FCM_P_AI (:,0)
  WRITE(KULOUT,'('' FCM_P_AI (:,1) ='',2(1X,ES12.5))') FCM_P_AI (:,1)
  WRITE(KULOUT,'('' FCM_P_AI (:,2) ='',2(1X,ES12.5))') FCM_P_AI (:,2)
  WRITE(KULOUT,'('' FCM_P_AI (:,3) ='',2(1X,ES12.5))') FCM_P_AI (:,3)
  WRITE(KULOUT,'('' FCM_P_AL (:,0) ='',2(1X,ES12.5))') FCM_P_AL (:,0)
  WRITE(KULOUT,'('' FCM_P_AL (:,1) ='',2(1X,ES12.5))') FCM_P_AL (:,1)
  WRITE(KULOUT,'('' FCM_P_AL (:,2) ='',2(1X,ES12.5))') FCM_P_AL (:,2)
  WRITE(KULOUT,'('' FCM_P_AL (:,3) ='',2(1X,ES12.5))') FCM_P_AL (:,3)
  WRITE(KULOUT,'('' FCM_P_DI (:,0) ='',2(1X,ES12.5))') FCM_P_DI (:,0)
  WRITE(KULOUT,'('' FCM_P_DI (:,1) ='',2(1X,ES12.5))') FCM_P_DI (:,1)
  WRITE(KULOUT,'('' FCM_P_DI (:,2) ='',2(1X,ES12.5))') FCM_P_DI (:,2)
  WRITE(KULOUT,'('' FCM_P_DI (:,3) ='',2(1X,ES12.5))') FCM_P_DI (:,3)
  WRITE(KULOUT,'('' FCM_P_DL (:,0) ='',2(1X,ES12.5))') FCM_P_DL (:,0)
  WRITE(KULOUT,'('' FCM_P_DL (:,1) ='',2(1X,ES12.5))') FCM_P_DL (:,1)
  WRITE(KULOUT,'('' FCM_P_DL (:,2) ='',2(1X,ES12.5))') FCM_P_DL (:,2)
  WRITE(KULOUT,'('' FCM_P_DL (:,3) ='',2(1X,ES12.5))') FCM_P_DL (:,3)
  WRITE(KULOUT,'('' FCM_P_GI (:,0) ='',2(1X,ES12.5))') FCM_P_GI (:,0)
  WRITE(KULOUT,'('' FCM_P_GI (:,1) ='',2(1X,ES12.5))') FCM_P_GI (:,1)
  WRITE(KULOUT,'('' FCM_P_GI (:,2) ='',2(1X,ES12.5))') FCM_P_GI (:,2)
  WRITE(KULOUT,'('' FCM_P_GI (:,3) ='',2(1X,ES12.5))') FCM_P_GI (:,3)
  WRITE(KULOUT,'('' FCM_P_GL (:,0) ='',2(1X,ES12.5))') FCM_P_GL (:,0)
  WRITE(KULOUT,'('' FCM_P_GL (:,1) ='',2(1X,ES12.5))') FCM_P_GL (:,1)
  WRITE(KULOUT,'('' FCM_P_GL (:,2) ='',2(1X,ES12.5))') FCM_P_GL (:,2)
  WRITE(KULOUT,'('' FCM_P_GL (:,3) ='',2(1X,ES12.5))') FCM_P_GL (:,3)
  WRITE(KULOUT,'('' FCM_Q_AI (:,1) ='',2(1X,ES12.5))') FCM_Q_AI (:,1)
  WRITE(KULOUT,'('' FCM_Q_AI (:,2) ='',2(1X,ES12.5))') FCM_Q_AI (:,2)
  WRITE(KULOUT,'('' FCM_Q_AI (:,3) ='',2(1X,ES12.5))') FCM_Q_AI (:,3)
  WRITE(KULOUT,'('' FCM_Q_AL (:,1) ='',2(1X,ES12.5))') FCM_Q_AL (:,1)
  WRITE(KULOUT,'('' FCM_Q_AL (:,2) ='',2(1X,ES12.5))') FCM_Q_AL (:,2)
  WRITE(KULOUT,'('' FCM_Q_AL (:,3) ='',2(1X,ES12.5))') FCM_Q_AL (:,3)
  WRITE(KULOUT,'('' FCM_Q_DI (:,1) ='',2(1X,ES12.5))') FCM_Q_DI (:,1)
  WRITE(KULOUT,'('' FCM_Q_DI (:,2) ='',2(1X,ES12.5))') FCM_Q_DI (:,2)
  WRITE(KULOUT,'('' FCM_Q_DI (:,3) ='',2(1X,ES12.5))') FCM_Q_DI (:,3)
  WRITE(KULOUT,'('' FCM_Q_DL (:,1) ='',2(1X,ES12.5))') FCM_Q_DL (:,1)
  WRITE(KULOUT,'('' FCM_Q_DL (:,2) ='',2(1X,ES12.5))') FCM_Q_DL (:,2)
  WRITE(KULOUT,'('' FCM_Q_DL (:,3) ='',2(1X,ES12.5))') FCM_Q_DL (:,3)
  WRITE(KULOUT,'('' FCM_Q_GI (:,1) ='',2(1X,ES12.5))') FCM_Q_GI (:,1)
  WRITE(KULOUT,'('' FCM_Q_GI (:,2) ='',2(1X,ES12.5))') FCM_Q_GI (:,2)
  WRITE(KULOUT,'('' FCM_Q_GI (:,3) ='',2(1X,ES12.5))') FCM_Q_GI (:,3)
  WRITE(KULOUT,'('' FCM_Q_GL (:,1) ='',2(1X,ES12.5))') FCM_Q_GL (:,1)
  WRITE(KULOUT,'('' FCM_Q_GL (:,2) ='',2(1X,ES12.5))') FCM_Q_GL (:,2)
  WRITE(KULOUT,'('' FCM_Q_GL (:,3) ='',2(1X,ES12.5))') FCM_Q_GL (:,3)
ENDIF

IF ( NRAY == 1 ) THEN
  WRITE(KULOUT,'('' FSM_CC ='',ES12.5)') FSM_CC
  WRITE(KULOUT,'('' FSM_DD ='',ES12.5)') FSM_DD
  WRITE(KULOUT,'('' FSM_EE ='',ES12.5)') FSM_EE
  WRITE(KULOUT,'('' FSM_FF ='',ES12.5)') FSM_FF
  WRITE(KULOUT,'('' FSM_GG ='',ES12.5)') FSM_GG
  WRITE(KULOUT,'('' FSM_HH ='',ES12.5)') FSM_HH
  WRITE(KULOUT,'('' FSM_II ='',ES12.5)') FSM_II
ELSE
  IF ( NRAUTOEV == 0.AND.LRPROX ) THEN
    WRITE(KULOUT,'('' FSM_AA ='',3(1X,ES12.5))') FSM_AA(:)
    WRITE(KULOUT,'('' FSM_BB ='',3(1X,ES12.5))') FSM_BB(:)
  ENDIF
ENDIF

WRITE(KULOUT,'('' RLAMB_SOLID ='',ES12.5)') RLAMB_SOLID
WRITE(KULOUT,'('' RLAMB_WATER ='',ES12.5)') RLAMB_WATER

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPHY3',1,ZHOOK_HANDLE)
END SUBROUTINE SUPHY3
