!OCL  SCALAR
SUBROUTINE SUGFL3

!------------------------------------------------------------------------------
!**** *SUGFL3*   - Setting up GFL environment: part 3

!     Purpose.
!     --------
!      Initialization of YOM_YGFL, part 3.
!      Set GFL attributes.

!**   Interface.
!     ----------
!        *CALL* *SUGFL3

!        Explicit arguments :
!        --------------------
!         none

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!      Mats Hamrud *ECMWF*
!      Original : 2002-03-11
!      Code in SUDYN until Feb 2007
!      Code in SUDYN_SETGFLATTR until 2011
!      Creation of SUGFL3 in Nov 2011.

!     Modifications.
!     --------------
!      K. YESSAD (Nov 2011) former SUDYN_SETGFLATTR code put in new SUGFL3; + minor cleanings
!      J.M. Piriou and E. Bazile (Sep 2011): new GFL variables (LCONV, ICONV, RCONV, SCONV)
!     End Modifications
!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LSLAG    ,LTWOTL   ,LPC_FULL , LPC_CHEAP
USE YOMPHY   , ONLY : LMPHYS
USE YOEPHY   , ONLY : NEPHYS_PCFULL      ,LEPHYS
USE YOMSIMPHL, ONLY : LSIMPH

! ky: this variable must probably be renamed (LRPHYLINGFL would be better) and put in YOM_YGFL
!     value can be T only if LSLAG=T
!     it simply means that "there is at least one advected GFL variable
!      which has two different interpolations
!      for X(origin) and diabatic term at the origin point,
!      and the diabatic term is interpolated by a linear interpolation".
!     But the new GFL set-up can be tested with LSPLTHOIGFL remaining in YOMDYN.
USE YOMDYN   , ONLY : LSPLTHOIGFL

USE YOM_YGFL , ONLY : YL, YLCONV, YICONV,YRCONV,YSCONV, YI, &
 & YLRAD, YIRAD, YS, YR, YG, YH, YTKE,YEFB1,YEFB2,YEFB3, YA, YO3, YSRC, YEXT,YQ, &
 & YCPF, YSPF, YAERO, YGRG, YGRGTEND, YFORC, YEZDIAG, YGHG, &
 & YCVGQ, YTRAC, YSDSAT, YCVV, YQVA, YTTE, YMXL, &
 & YUOM, YUAL, YDOM, YDAL, YUEN, YUNEBH, YERA40, YNOGW, YSLDIA, &
 & YRKTH, YRKTQV, YRKTQC, &
 & YQ_NL, YI_NL, YIRAD_NL, YS_NL, YR_NL, YG_NL, YH_NL, YTKE_NL, YA_NL, YO3_NL, YSRC_NL,&
 & YL_NL, YLCONV_NL, YICONV_NL, YRCONV_NL, YSCONV_NL,YEFB1_NL,YEFB2_NL,YEFB3_NL, &
 & YLRAD_NL, YEXT_NL, YCPF_NL, YSPF_NL, YAERO_NL, YGRG_NL, YFORC_NL, YTTE_NL, YMXL_NL,&
 & YEZDIAG_NL, YGHG_NL, YCVGQ_NL, YTRAC_NL, YSDSAT_NL, YCVV_NL, YQVA_NL, &
 & YUOM_NL, YUAL_NL, YDOM_NL, YDAL_NL, YUEN_NL, YUNEBH_NL, YERA40_NL, YSLDIA_NL, YNOGW_NL, &
 & NGFL_EXT, NAERO, NGRG, NGFL_FORC, NGFL_EZDIAG, NGHG, NTRAC, NERA40, NSLDIA, NNOGW, &
 & YLRCH4_NL, YLRCH4, YCH4S_NL, YCH4S, NGRG_CPLO4, YGRGTEND_NL, &
 & YRKTH_NL,YRKTQV_NL,YRKTQC_NL,YAEROUT,YAEROUT_NL,NAEROUT,YUVP,YUVP_NL,NUVP
USE GFL_SUBS_MOD , ONLY : SET_GFL_ATTR,PRINT_GFL

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JGFL

LOGICAL :: LLT9,LLPT,LLPC,LLPC0
LOGICAL :: LLAD
LOGICAL :: LLDIAB, LLPHYLIN

CHARACTER(LEN=12) :: CLSLINT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "sucslint.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGFL3',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    SET GFL ATTRIBUTES
!              ------------------

! * Note that in this part the order of the GFL variables is no longer
!   the "SUGFL2-standard order" (cf. in SUGFL2) but a random order which reduces
!   the risk of "hidden" bugs.
!   This SUGFL3-order is used to order the GFL variables at time "t-dt".

IF (LPC_FULL.AND..NOT.LPC_CHEAP) THEN
  ! EUL: true; SL3TL: not yet coded; SL2TL: false.
  LLPC0=.NOT.LTWOTL
ELSEIF (LPC_FULL.AND.LPC_CHEAP.AND.LTWOTL) THEN
  ! EUL: not coded; SL3TL: not yet coded; SL2TL: true.
  LLPC0=.TRUE.
ELSE
  LLPC0=.FALSE.
ENDIF
LLT9 = (.NOT. LTWOTL) .OR. LPC_FULL

! ky: quid for HIRLAM physics? LLDIAB must be probably T in this case too.
LLDIAB=LEPHYS.OR.LMPHYS.OR.LSIMPH

LLPHYLIN=.FALSE.

! * Ozone:
IF(YO3%LACTIVE) THEN
  CALL SUCSLINT(YO3_NL%LSLHD,YO3_NL%LVSPLIP,YO3_NL%LHV, &
   & YO3_NL%LQM,YO3_NL%LQMH,YO3_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YO3_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YO3_NL%LPC
  CALL SET_GFL_ATTR(YO3,LDADV=YO3_NL%LADV,LDADV5=YO3_NL%LADV5,&
   & LDPHYLIN=YO3_NL%LPHYLIN,LDHORTURB=YO3_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YO3_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YO3_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YO3_NL%REFVALC)  
  LLPHYLIN=LLPHYLIN.OR.YO3_NL%LPHYLIN
ENDIF

! * Specific humidity:
IF(YQ%LACTIVE) THEN
  CALL SUCSLINT(YQ_NL%LSLHD,YQ_NL%LVSPLIP,YQ_NL%LHV, &
   & YQ_NL%LQM,YQ_NL%LQMH,YQ_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YQ_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YQ_NL%LPC 
  CALL SET_GFL_ATTR(YQ,LDADV=YQ_NL%LADV,LDADV5=YQ_NL%LADV5,&
   & LDPHYLIN=YQ_NL%LPHYLIN,LDHORTURB=YQ_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YQ_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YQ_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YQ_NL%REFVALC)  
  LLPHYLIN=LLPHYLIN.OR.YQ_NL%LPHYLIN
ENDIF

! * Cloud fraction:
IF(YA%LACTIVE) THEN
  CALL SUCSLINT(YA_NL%LSLHD,YA_NL%LVSPLIP,YA_NL%LHV, &
   & YA_NL%LQM,YA_NL%LQMH,YA_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YA_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YA_NL%LPC
  CALL SET_GFL_ATTR(YA,LDADV=YA_NL%LADV,LDADV5=YA_NL%LADV5,&
   & LDPHYLIN=YA_NL%LPHYLIN,LDHORTURB=YA_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YA_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YA_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YA_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YA_NL%LPHYLIN
ENDIF

! * Second order flux (for AROME physics):
IF(YSRC%LACTIVE) THEN
  CALL SUCSLINT(YSRC_NL%LSLHD,YSRC_NL%LVSPLIP,YSRC_NL%LHV, &
   & YSRC_NL%LQM,YSRC_NL%LQMH,YSRC_NL%LINTLIN,CLSLINT)
  ! Notice : this field is not supposed to be pronostic !
  LLPT=LPC_FULL .AND. LLDIAB .AND. YSRC_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YSRC_NL%LPC
  CALL SET_GFL_ATTR(YSRC,LDADV=YSRC_NL%LADV,LDADV5=YSRC_NL%LADV5,&
   & LDPHYLIN=YSRC_NL%LPHYLIN,LDHORTURB=YSRC_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YSRC_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YSRC_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YSRC_NL%REFVALC)  
  LLPHYLIN=LLPHYLIN.OR.YSRC_NL%LPHYLIN
ENDIF

! * Liquid water:
IF(YL%LACTIVE) THEN
  CALL SUCSLINT(YL_NL%LSLHD,YL_NL%LVSPLIP,YL_NL%LHV, &
   & YL_NL%LQM,YL_NL%LQMH,YL_NL%LINTLIN,CLSLINT) 
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YL_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YL_NL%LPC
  CALL SET_GFL_ATTR(YL,LDADV=YL_NL%LADV,LDADV5=YL_NL%LADV5,&
   & LDPHYLIN=YL_NL%LPHYLIN,LDHORTURB=YL_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YL_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YL_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YL_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YL_NL%LPHYLIN
ENDIF

! * Liquid water (convective part):
IF(YLCONV%LACTIVE) THEN
  CALL SUCSLINT(YLCONV_NL%LSLHD,YLCONV_NL%LVSPLIP,YLCONV_NL%LHV, &
   & YLCONV_NL%LQM,YLCONV_NL%LQMH,YLCONV_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YLCONV_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YLCONV_NL%LPC
  CALL SET_GFL_ATTR(YLCONV,LDADV=YLCONV_NL%LADV,LDADV5=YLCONV_NL%LADV5,&
   & LDPHYLIN=YLCONV_NL%LPHYLIN,LDHORTURB=YLCONV_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YLCONV_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YLCONV_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YLCONV_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YLCONV_NL%LPHYLIN
ENDIF

! * Ice water (convective part):
IF(YICONV%LACTIVE) THEN
  CALL SUCSLINT(YICONV_NL%LSLHD,YICONV_NL%LVSPLIP,YICONV_NL%LHV, &
   & YICONV_NL%LQM,YICONV_NL%LQMH,YICONV_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YICONV_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YICONV_NL%LPC
  CALL SET_GFL_ATTR(YICONV,LDADV=YICONV_NL%LADV,LDADV5=YICONV_NL%LADV5,&
   & LDPHYLIN=YICONV_NL%LPHYLIN,LDHORTURB=YICONV_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YICONV_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YICONV_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YICONV_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YICONV_NL%LPHYLIN
ENDIF

! * Rain (convective part):
IF(YRCONV%LACTIVE) THEN
  CALL SUCSLINT(YRCONV_NL%LSLHD,YRCONV_NL%LVSPLIP,YRCONV_NL%LHV, &
   & YRCONV_NL%LQM,YRCONV_NL%LQMH,YRCONV_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YRCONV_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YRCONV_NL%LPC
  CALL SET_GFL_ATTR(YRCONV,LDADV=YRCONV_NL%LADV,LDADV5=YRCONV_NL%LADV5,&
   & LDPHYLIN=YRCONV_NL%LPHYLIN,LDHORTURB=YRCONV_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YRCONV_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YRCONV_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YRCONV_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YRCONV_NL%LPHYLIN
ENDIF

! * Snow (convective part):
IF(YSCONV%LACTIVE) THEN
  CALL SUCSLINT(YSCONV_NL%LSLHD,YSCONV_NL%LVSPLIP,YSCONV_NL%LHV, &
   & YSCONV_NL%LQM,YSCONV_NL%LQMH,YSCONV_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YSCONV_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YSCONV_NL%LPC
  CALL SET_GFL_ATTR(YSCONV,LDADV=YSCONV_NL%LADV,LDADV5=YSCONV_NL%LADV5,&
   & LDPHYLIN=YSCONV_NL%LPHYLIN,LDHORTURB=YSCONV_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YSCONV_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YSCONV_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YSCONV_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YSCONV_NL%LPHYLIN
ENDIF

! * Radiative Liquid water:
IF(YLRAD%LACTIVE) THEN
  CALL SUCSLINT(YLRAD_NL%LSLHD,YLRAD_NL%LVSPLIP,YLRAD_NL%LHV, &
   & YLRAD_NL%LQM,YLRAD_NL%LQMH,YLRAD_NL%LINTLIN,CLSLINT) 
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YLRAD_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YLRAD_NL%LPC
  CALL SET_GFL_ATTR(YLRAD,LDADV=YLRAD_NL%LADV,LDADV5=YLRAD_NL%LADV5,&
   & LDPHYLIN=YLRAD_NL%LPHYLIN,LDHORTURB=YLRAD_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YLRAD_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YLRAD_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YLRAD_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YLRAD_NL%LPHYLIN
ENDIF

! * Ice:
IF(YI%LACTIVE) THEN
  CALL SUCSLINT(YI_NL%LSLHD,YI_NL%LVSPLIP,YI_NL%LHV, &
   & YI_NL%LQM,YI_NL%LQMH,YI_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YI_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YI_NL%LPC
  CALL SET_GFL_ATTR(YI,LDADV=YI_NL%LADV,LDADV5=YI_NL%LADV5,&
   & LDPHYLIN=YI_NL%LPHYLIN,LDHORTURB=YI_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YI_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YI_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YI_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YI_NL%LPHYLIN
ENDIF

! * Radiative Ice:
IF(YIRAD%LACTIVE) THEN
  CALL SUCSLINT(YIRAD_NL%LSLHD,YIRAD_NL%LVSPLIP,YIRAD_NL%LHV, &
   & YIRAD_NL%LQM,YIRAD_NL%LQMH,YIRAD_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. (YIRAD_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YIRAD_NL%LPC
  CALL SET_GFL_ATTR(YIRAD,LDADV=YIRAD_NL%LADV,LDADV5=YIRAD_NL%LADV5,&
   & LDPHYLIN=YIRAD_NL%LPHYLIN,LDHORTURB=YIRAD_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YIRAD_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YIRAD_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YIRAD_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YIRAD_NL%LPHYLIN
ENDIF

! * Snow:
IF(YS%LACTIVE) THEN
  CALL SUCSLINT(YS_NL%LSLHD,YS_NL%LVSPLIP,YS_NL%LHV, &
   & YS_NL%LQM,YS_NL%LQMH,YS_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YS_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YS_NL%LPC
  CALL SET_GFL_ATTR(YS,LDADV=YS_NL%LADV,LDADV5=YS_NL%LADV5,&
   & LDPHYLIN=YS_NL%LPHYLIN,LDHORTURB=YS_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YS_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YS_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YS_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YS_NL%LPHYLIN
ENDIF

! * Rain:
IF(YR%LACTIVE) THEN
  CALL SUCSLINT(YR_NL%LSLHD,YR_NL%LVSPLIP,YR_NL%LHV, &
   & YR_NL%LQM,YR_NL%LQMH,YR_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YR_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YR_NL%LPC
  CALL SET_GFL_ATTR(YR,LDADV=YR_NL%LADV,LDADV5=YR_NL%LADV5, &
   & LDPHYLIN=YR_NL%LPHYLIN,LDHORTURB=YR_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YR_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YR_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YR_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YR_NL%LPHYLIN
ENDIF

! * Graupels:
IF(YG%LACTIVE) THEN
  CALL SUCSLINT(YG_NL%LSLHD,YG_NL%LVSPLIP,YG_NL%LHV, &
   & YG_NL%LQM,YG_NL%LQMH,YG_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YG_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YG_NL%LPC
  CALL SET_GFL_ATTR(YG,LDADV=YG_NL%LADV,LDADV5=YG_NL%LADV5,&
   & LDPHYLIN=YG_NL%LPHYLIN,LDHORTURB=YG_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YG_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YG_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YG_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YG_NL%LPHYLIN
ENDIF

! * Hail:
IF(YH%LACTIVE) THEN
  CALL SUCSLINT(YH_NL%LSLHD,YH_NL%LVSPLIP,YH_NL%LHV, &
   & YH_NL%LQM,YH_NL%LQMH,YH_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YH_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YH_NL%LPC
  CALL SET_GFL_ATTR(YH,LDADV=YH_NL%LADV,LDADV5=YH_NL%LADV5,&
   & LDPHYLIN=YH_NL%LPHYLIN,LDHORTURB=YH_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YH_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YH_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YH_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YH_NL%LPHYLIN
ENDIF

! * CVGQ:
LLAD=YCVGQ%LGP
IF(YCVGQ%LACTIVE) THEN
  CALL SUCSLINT(YCVGQ_NL%LSLHD,YCVGQ_NL%LVSPLIP, &
   & YCVGQ_NL%LHV,YCVGQ_NL%LQM,YCVGQ_NL%LQMH,YCVGQ_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YCPF_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YCPF_NL%LPC
  CALL SET_GFL_ATTR(YCVGQ,LDADV=LLAD,LDADV5=YCVGQ_NL%LADV5,&
   & LDPHYLIN=YCVGQ_NL%LPHYLIN,LDHORTURB=YCVGQ_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YCVGQ_NL%LPHY,LDPT=LLPT,&
   & LDPC=LLPC,KCOUPLING=0,CDSLINT=CLSLINT,PREFVALC=YCVGQ_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YCVGQ_NL%LPHYLIN
ENDIF

! * TKE:
IF(YTKE%LACTIVE) THEN
  CALL SUCSLINT(YTKE_NL%LSLHD,YTKE_NL%LVSPLIP, & 
   & YTKE_NL%LHV,YTKE_NL%LQM,YTKE_NL%LQMH,YTKE_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YTKE_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YTKE_NL%LPC
  CALL SET_GFL_ATTR(YTKE,LDADV=YTKE_NL%LADV,LDADV5=YTKE_NL%LADV5,&
   & LDPHYLIN=YTKE_NL%LPHYLIN,LDHORTURB=YTKE_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YTKE_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YTKE_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YTKE_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YTKE_NL%LPHYLIN
ENDIF

! * TTE:
IF(YTTE%LACTIVE) THEN
  CALL SUCSLINT(YTTE_NL%LSLHD,YTTE_NL%LVSPLIP, & 
   & YTTE_NL%LHV,YTTE_NL%LQM,YTTE_NL%LQMH,YTTE_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YTTE_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YTTE_NL%LPC
  CALL SET_GFL_ATTR(YTTE,LDADV=YTTE_NL%LADV,LDADV5=YTTE_NL%LADV5,&
   & LDPHYLIN=YTTE_NL%LPHYLIN,LDHORTURB=YTTE_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YTTE_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YTTE_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YTTE_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YTTE_NL%LPHYLIN
ENDIF

! * MXL
IF(YMXL%LACTIVE) THEN
  CALL SUCSLINT(YMXL_NL%LSLHD,YMXL_NL%LVSPLIP, & 
   & YMXL_NL%LHV,YMXL_NL%LQM,YMXL_NL%LQMH,YMXL_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YMXL_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YMXL_NL%LPC
  CALL SET_GFL_ATTR(YMXL,LDADV=YMXL_NL%LADV,LDADV5=YMXL_NL%LADV5,&
   & LDPHYLIN=YMXL_NL%LPHYLIN,LDHORTURB=YMXL_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YMXL_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YMXL_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YMXL_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YMXL_NL%LPHYLIN
ENDIF

! * EFB1
IF(YEFB1%LACTIVE) THEN
  CALL SUCSLINT(YEFB1_NL%LSLHD,YEFB1_NL%LVSPLIP, & 
   & YEFB1_NL%LHV,YEFB1_NL%LQM,YEFB1_NL%LQMH,YEFB1_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YEFB1_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YEFB1_NL%LPC
  CALL SET_GFL_ATTR(YEFB1,LDADV=YEFB1_NL%LADV,LDADV5=YEFB1_NL%LADV5,&
   & LDPHYLIN=YEFB1_NL%LPHYLIN,LDHORTURB=YEFB1_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YEFB1_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YEFB1_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YEFB1_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YEFB1_NL%LPHYLIN
ENDIF

! * EFB2
IF(YEFB2%LACTIVE) THEN
  CALL SUCSLINT(YEFB2_NL%LSLHD,YEFB2_NL%LVSPLIP, & 
   & YEFB2_NL%LHV,YEFB2_NL%LQM,YEFB2_NL%LQMH,YEFB2_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YEFB2_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YEFB2_NL%LPC
  CALL SET_GFL_ATTR(YEFB2,LDADV=YEFB2_NL%LADV,LDADV5=YEFB2_NL%LADV5,&
   & LDPHYLIN=YEFB2_NL%LPHYLIN,LDHORTURB=YEFB2_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YEFB2_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YEFB2_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YEFB2_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YEFB2_NL%LPHYLIN
ENDIF

! * EFB3
IF(YEFB3%LACTIVE) THEN
  CALL SUCSLINT(YEFB3_NL%LSLHD,YEFB3_NL%LVSPLIP, & 
   & YEFB3_NL%LHV,YEFB3_NL%LQM,YEFB3_NL%LQMH,YEFB3_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YEFB3_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YEFB3_NL%LPC
  CALL SET_GFL_ATTR(YEFB3,LDADV=YEFB3_NL%LADV,LDADV5=YEFB3_NL%LADV5,&
   & LDPHYLIN=YEFB3_NL%LPHYLIN,LDHORTURB=YEFB3_NL%LHORTURB,LDT9=LLT9, &
   & LDPHY=YEFB3_NL%LPHY,LDPT=LLPT,LDPC=LLPC,KCOUPLING=YEFB3_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YEFB3_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YEFB3_NL%LPHYLIN
ENDIF

! * Convective precipitation flux:
IF(YCPF%LACTIVE) THEN
  CALL SUCSLINT(YCPF_NL%LSLHD,YCPF_NL%LVSPLIP, &
   & YCPF_NL%LHV,YCPF_NL%LQM,YCPF_NL%LQMH,YCPF_NL%LINTLIN,CLSLINT)
  ! Notice : this field is not supposed to be pronostic !
  LLPT=LPC_FULL .AND. LLDIAB .AND. YCPF_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YCPF_NL%LPC
  CALL SET_GFL_ATTR(YCPF,LDADV=.FALSE.,LDT9=LLT9,LDPHY=YCPF_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YCPF_NL%NCOUPLING, &
   & PREFVALC=YCPF_NL%REFVALC)
ENDIF

! * Stratiform precipitation flux:
IF(YSPF%LACTIVE) THEN
  CALL SUCSLINT(YSPF_NL%LSLHD,YSPF_NL%LVSPLIP, &
   & YSPF_NL%LHV,YSPF_NL%LQM,YSPF_NL%LQMH,YSPF_NL%LINTLIN,CLSLINT)
  ! Notice : this field is not supposed to be pronostic !
  LLPT=LPC_FULL .AND. LLDIAB .AND. YSPF_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YSPF_NL%LPC
  CALL SET_GFL_ATTR(YSPF,LDADV=.FALSE.,LDT9=LLT9,LDPHY=YSPF_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YSPF_NL%NCOUPLING, &
   & PREFVALC=YSPF_NL%REFVALC)
ENDIF

! * Standard deviation of the saturation depression:
IF(YSDSAT%LACTIVE) THEN
  CALL SUCSLINT(YSDSAT_NL%LSLHD,YSDSAT_NL%LVSPLIP, &
   & YSDSAT_NL%LHV,YSDSAT_NL%LQM,YSDSAT_NL%LQMH,YSDSAT_NL%LINTLIN,CLSLINT)
  ! Notice : this field is not supposed to be pronostic !
  LLPT=LPC_FULL .AND. LLDIAB .AND. YSDSAT_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YSDSAT_NL%LPC
  CALL SET_GFL_ATTR(YSDSAT,LDADV=YSDSAT_NL%LADV,&
   & LDPHYLIN=YSDSAT_NL%LPHYLIN,LDHORTURB=YSDSAT_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YSDSAT_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YSDSAT_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YSDSAT_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YSDSAT_NL%LPHYLIN
ENDIF

! * Convective vertical velocity:
IF(YCVV%LACTIVE) THEN
  CALL SUCSLINT(YCVV_NL%LSLHD,YCVV_NL%LVSPLIP, &
   & YCVV_NL%LHV,YCVV_NL%LQM,YCVV_NL%LQMH,YCVV_NL%LINTLIN,CLSLINT)
  ! Notice : this field is not supposed to be pronostic !
  LLPT=LPC_FULL .AND. LLDIAB .AND. YCVV_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YCVV_NL%LPC
  CALL SET_GFL_ATTR(YCVV,LDADV=YCVV_NL%LADV,&
   & LDPHYLIN=YCVV_NL%LPHYLIN,LDHORTURB=YCVV_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YCVV_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YCVV_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YCVV_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YCVV_NL%LPHYLIN
ENDIF

! * Total humidity variation (for HIRLAM physics):
IF(YQVA%LACTIVE) THEN
  ! prepare variable YQVA used in HIRLAM cloud- and condensation. 
  CALL SUCSLINT(YQVA_NL%LSLHD,YQVA_NL%LVSPLIP, &
   & YQVA_NL%LHV,YQVA_NL%LQM,YQVA_NL%LQMH,YQVA_NL%LINTLIN,CLSLINT)
  ! Notice : this field is not supposed to be pronostic 
  LLPT=LPC_FULL .AND. LLDIAB .AND. YQVA_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YQVA_NL%LPC
  CALL SET_GFL_ATTR(YQVA,LDADV=.FALSE.,LDT9=LLT9,LDPHY=YQVA_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YQVA_NL%NCOUPLING, &
   & PREFVALC=YQVA_NL%REFVALC)
ENDIF

! * "Prognostic convection" variables (for ALARO physics):
!   interpolations like for q.
IF(YUOM%LACTIVE) THEN
  CALL SUCSLINT(YUOM_NL%LSLHD,YUOM_NL%LVSPLIP, &
   & YUOM_NL%LHV,YUOM_NL%LQM,YUOM_NL%LQMH,YUOM_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YUOM_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YUOM_NL%LPC 
  CALL SET_GFL_ATTR(YUOM,LDADV=YUOM_NL%LADV,&
   & LDPHYLIN=YUOM_NL%LPHYLIN,LDHORTURB=YUOM_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YUOM_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YUOM_NL%NCOUPLING, & 
   & CDSLINT=CLSLINT,PREFVALC=YUOM_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YUOM_NL%LPHYLIN
ENDIF
IF(YUAL%LACTIVE) THEN
  CALL SUCSLINT(YUAL_NL%LSLHD,YUAL_NL%LVSPLIP, &
   & YUAL_NL%LHV,YUAL_NL%LQM,YUAL_NL%LQMH,YUAL_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YUAL_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YUAL_NL%LPC 
  CALL SET_GFL_ATTR(YUAL,LDADV=YUAL_NL%LADV,&
   & LDPHYLIN=YUAL_NL%LPHYLIN,LDHORTURB=YUAL_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YUAL_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YUAL_NL%NCOUPLING, & 
   & CDSLINT=CLSLINT,PREFVALC=YUAL_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YUAL_NL%LPHYLIN
ENDIF
IF(YDOM%LACTIVE) THEN
  CALL SUCSLINT(YDOM_NL%LSLHD,YDOM_NL%LVSPLIP, &
   & YDOM_NL%LHV,YDOM_NL%LQM,YDOM_NL%LQMH,YDOM_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YDOM_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YDOM_NL%LPC 
  CALL SET_GFL_ATTR(YDOM,LDADV=YDOM_NL%LADV,&
   & LDPHYLIN=YDOM_NL%LPHYLIN,LDHORTURB=YDOM_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YDOM_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YDOM_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YDOM_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YDOM_NL%LPHYLIN
ENDIF
IF(YDAL%LACTIVE) THEN
  CALL SUCSLINT(YDAL_NL%LSLHD,YDAL_NL%LVSPLIP, &
   & YDAL_NL%LHV,YDAL_NL%LQM,YDAL_NL%LQMH,YDAL_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YDAL_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YDAL_NL%LPC 
  CALL SET_GFL_ATTR(YDAL,LDADV=YDAL_NL%LADV,&
   & LDPHYLIN=YDAL_NL%LPHYLIN,LDHORTURB=YDAL_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YDAL_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YDAL_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YDAL_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YDAL_NL%LPHYLIN
ENDIF
IF(YUEN%LACTIVE) THEN
  CALL SUCSLINT(YUEN_NL%LSLHD,YUEN_NL%LVSPLIP, &
   & YUEN_NL%LHV,YUEN_NL%LQM,YUEN_NL%LQMH,YUEN_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YUEN_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YUEN_NL%LPC 
  CALL SET_GFL_ATTR(YUEN,LDADV=YUEN_NL%LADV,&
   & LDPHYLIN=YUEN_NL%LPHYLIN,LDHORTURB=YUEN_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YUEN_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YUEN_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YUEN_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YUEN_NL%LPHYLIN
ENDIF
IF(YUNEBH%LACTIVE) THEN
  CALL SUCSLINT(YUNEBH_NL%LSLHD,YUNEBH_NL%LVSPLIP, &
   & YUNEBH_NL%LHV,YUNEBH_NL%LQM,YUNEBH_NL%LQMH,YUNEBH_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YUNEBH_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YUNEBH_NL%LPC 
  CALL SET_GFL_ATTR(YUNEBH,LDADV=YUNEBH_NL%LADV,&
   & LDPHYLIN=YUNEBH_NL%LPHYLIN,LDHORTURB=YUNEBH_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YUNEBH_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YUNEBH_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YUNEBH_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YUNEBH_NL%LPHYLIN
ENDIF
IF(YRKTH%LACTIVE) THEN
  CALL SUCSLINT(YRKTH_NL%LSLHD,YRKTH_NL%LVSPLIP, &
   & YRKTH_NL%LHV,YRKTH_NL%LQM,YRKTH_NL%LQMH,YRKTH_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YRKTH_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YRKTH_NL%LPC
  CALL SET_GFL_ATTR(YRKTH,LDADV=YRKTH_NL%LADV,&
   & LDPHYLIN=YRKTH_NL%LPHYLIN,LDHORTURB=YRKTH_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YRKTH_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YRKTH_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YRKTH_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YRKTH_NL%LPHYLIN
ENDIF
IF(YRKTQV%LACTIVE) THEN
  CALL SUCSLINT(YRKTQV_NL%LSLHD,YRKTQV_NL%LVSPLIP, &
   & YRKTQV_NL%LHV,YRKTQV_NL%LQM,YRKTQV_NL%LQMH,YRKTQV_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YRKTQV_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YRKTQV_NL%LPC
  CALL SET_GFL_ATTR(YRKTQV,LDADV=YRKTQV_NL%LADV,&
   & LDPHYLIN=YRKTQV_NL%LPHYLIN,LDHORTURB=YRKTQV_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YRKTQV_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YRKTQV_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YRKTQV_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YRKTQV_NL%LPHYLIN
ENDIF
IF(YRKTQC%LACTIVE) THEN
  CALL SUCSLINT(YRKTQC_NL%LSLHD,YRKTQC_NL%LVSPLIP, &
   & YRKTQC_NL%LHV,YRKTQC_NL%LQM,YRKTQC_NL%LQMH,YRKTQC_NL%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YRKTQC_NL%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YRKTQC_NL%LPC
  CALL SET_GFL_ATTR(YRKTQC,LDADV=YRKTQC_NL%LADV,&
   & LDPHYLIN=YRKTQC_NL%LPHYLIN,LDHORTURB=YRKTQC_NL%LHORTURB,&
   & LDT9=LLT9,LDPHY=YRKTQC_NL%LPHY, &
   & LDPT=LLPT,LDPC=LLPC,KCOUPLING=YRKTQC_NL%NCOUPLING, &
   & CDSLINT=CLSLINT,PREFVALC=YRKTQC_NL%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YRKTQC_NL%LPHYLIN
ENDIF

! * LRCH4:
IF(NGHG > 0 .AND. YLRCH4%LACTIVE) THEN
  CALL SUCSLINT(YLRCH4_NL%LSLHD,YLRCH4_NL%LVSPLIP,YLRCH4_NL%LHV, &
   & YLRCH4_NL%LQM,YLRCH4_NL%LQMH,YLRCH4_NL%LINTLIN,CLSLINT) 
  LLPT=LPC_FULL.AND.LLDIAB.AND.LSLAG.AND.(YLRCH4_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YLRCH4_NL%LPC
  CALL SET_GFL_ATTR(YLRCH4,LDADV=YLRCH4_NL%LADV,LDADV5=YLRCH4_NL%LADV5, &
   & LDT9=LLT9,LDPHY=YLRCH4_NL%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YLRCH4_NL%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YLRCH4_NL%REFVALC)
ENDIF

! * LCH4S:
IF(NGHG > 0 .AND. YCH4S%LACTIVE) THEN
  CALL SUCSLINT(YCH4S_NL%LSLHD,YCH4S_NL%LVSPLIP,YCH4S_NL%LHV, &
   & YCH4S_NL%LQM,YCH4S_NL%LQMH,YCH4S_NL%LINTLIN,CLSLINT) 
  LLPT=LPC_FULL.AND.LLDIAB.AND.LSLAG.AND.(YCH4S_NL%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YCH4S_NL%LPC
  CALL SET_GFL_ATTR(YCH4S,LDADV=YCH4S_NL%LADV,LDADV5=YCH4S_NL%LADV5, &
   & LDT9=LLT9,LDPHY=YCH4S_NL%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YCH4S_NL%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YCH4S_NL%REFVALC)
ENDIF

! Aerosols, greenhouse gases, tracers, reactive gases:
!   Remark k.y.: these variables are currently used for ECMWF applications
!    and LDADV=LSLAG in the CALL of SET_GFL_ATTR
!    (that means that the namelist value of Y[X]_NL(JGFL)%LADV and
!    Y[X]_NL(JGFL)%LPHY is not taken into account). The calls to SET_GFL_ATTR
!    must be probably revised if some other teams wanted to use such GFL for
!    their own applications.
DO JGFL=1,NAERO
  CALL SUCSLINT(YAERO_NL(JGFL)%LSLHD, &
   & YAERO_NL(JGFL)%LVSPLIP,YAERO_NL(JGFL)%LHV, & 
   & YAERO_NL(JGFL)%LQM,YAERO_NL(JGFL)%LQMH,YAERO_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT = LPC_FULL .AND. LLDIAB .AND. YAERO_NL(JGFL)%LPT .AND. LSLAG
  LLPC = LLPC0.AND.YAERO_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YAERO(JGFL),LDADV=YAERO_NL(JGFL)%LADV,LDADV5=YAERO_NL(JGFL)%LADV5, &
   & LDT9=LLT9,LDPHY=YAERO_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YAERO_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YAERO_NL(JGFL)%REFVALC)
ENDDO
DO JGFL=1,NGHG
  CALL SUCSLINT(YGHG_NL(JGFL)%LSLHD, &
   & YGHG_NL(JGFL)%LVSPLIP,YGHG_NL(JGFL)%LHV, & 
   & YGHG_NL(JGFL)%LQM,YGHG_NL(JGFL)%LQMH,YGHG_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YGHG_NL(JGFL)%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YGHG_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YGHG(JGFL),LDADV=YGHG_NL(JGFL)%LADV,LDADV5=YGHG_NL(JGFL)%LADV5,&
   & LDT9=LLT9,LDPHY=YGHG_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YGHG_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YGHG_NL(JGFL)%REFVALC)
ENDDO
DO JGFL=1,NTRAC
  CALL SUCSLINT(YTRAC_NL(JGFL)%LSLHD, &
   & YTRAC_NL(JGFL)%LVSPLIP,YTRAC_NL(JGFL)%LHV, &
   & YTRAC_NL(JGFL)%LQM,YTRAC_NL(JGFL)%LQMH,YTRAC_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YTRAC_NL(JGFL)%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YTRAC_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YTRAC(JGFL),LDADV=YTRAC_NL(JGFL)%LADV,LDADV5=YTRAC_NL(JGFL)%LADV5, &
   & LDT9=LLT9,LDPHY=YTRAC_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YTRAC_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YTRAC_NL(JGFL)%REFVALC)
ENDDO
DO JGFL=1,NGRG
  CALL SUCSLINT(YGRG_NL(JGFL)%LSLHD, &
   & YGRG_NL(JGFL)%LVSPLIP,YGRG_NL(JGFL)%LHV, & 
   & YGRG_NL(JGFL)%LQM,YGRG_NL(JGFL)%LQMH,YGRG_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YGRG_NL(JGFL)%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YGRG_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YGRG(JGFL),LDADV=YGRG_NL(JGFL)%LADV,LDADV5=YGRG_NL(JGFL)%LADV5,&
   & LDT9=LLT9,LDPHY=YGRG_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YGRG_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YGRG_NL(JGFL)%REFVALC)
ENDDO
DO JGFL=1,NGRG_CPLO4
  CALL SUCSLINT(YGRGTEND_NL(JGFL)%LSLHD, &
   & YGRGTEND_NL(JGFL)%LVSPLIP,YGRGTEND_NL(JGFL)%LHV, & 
   & YGRGTEND_NL(JGFL)%LQM,YGRGTEND_NL(JGFL)%LQMH,YGRGTEND_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. YGRGTEND_NL(JGFL)%LPT.AND.LSLAG
  LLPC = LLPC0.AND.YGRGTEND_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YGRGTEND(JGFL),LDADV=YGRGTEND_NL(JGFL)%LADV, &
   & LDADV5=YGRGTEND_NL(JGFL)%LADV5,LDT9=LLT9,&
   & LDPHY=YGRGTEND_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YGRGTEND_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YGRGTEND_NL(JGFL)%REFVALC)
ENDDO

! ERA40 fields: these variables are currently used for ECMWF applications.
DO JGFL=1,NERA40
  LLPT = .FALSE.
  LLPC = .FALSE. 
  LLT9 = .FALSE.
  CALL SET_GFL_ATTR(YERA40(JGFL),LDADV=.FALSE.,LDT9=LLT9,&
   & LDPHY=YERA40_NL(JGFL)%LPHY,LDPT=LLPT, LDPC=LLPC,&
   & KCOUPLING=YERA40_NL(JGFL)%NCOUPLING,&
   & PREFVALC=YERA40_NL(JGFL)%REFVALC)
ENDDO

! NOGW fields: noro gwd scheme fields used for ECMWF applications
DO JGFL=1,NNOGW
  LLPT = .FALSE.
  LLPC = .FALSE.
  LLT9 = .FALSE.
  CALL SET_GFL_ATTR(YNOGW(JGFL),LDADV=.FALSE.,LDT9=LLT9,&
   & LDPHY=YNOGW_NL(JGFL)%LPHY,LDPT=LLPT, LDPC=LLPC,&
   & KCOUPLING=YNOGW_NL(JGFL)%NCOUPLING,&
   & PREFVALC=YNOGW_NL(JGFL)%REFVALC)
ENDDO

! NUVP fields: UV processor output fields used for ECMWF applications
DO JGFL=1,NUVP
  LLPT = .FALSE.
  LLPC = .FALSE.
  LLT9 = .FALSE.
  CALL SET_GFL_ATTR(YUVP(JGFL),LDADV=.FALSE.,LDT9=LLT9,&
   & LDPHY=YUVP_NL(JGFL)%LPHY,LDPT=LLPT, LDPC=LLPC,&
   & KCOUPLING=YUVP_NL(JGFL)%NCOUPLING,&
   & PREFVALC=YUVP_NL(JGFL)%REFVALC)
ENDDO

! NAEROUT fields: aerosol output fields used for ECMWF applications
DO JGFL=1,NAEROUT
  LLPT = .FALSE.
  LLPC = .FALSE.
  LLT9 = .FALSE.
  CALL SET_GFL_ATTR(YAEROUT(JGFL),LDADV=.FALSE.,LDT9=LLT9,&
   & LDPHY=YAEROUT_NL(JGFL)%LPHY,LDPT=LLPT, LDPC=LLPC,&
   & KCOUPLING=YAEROUT_NL(JGFL)%NCOUPLING,&
   & PREFVALC=YAEROUT_NL(JGFL)%REFVALC)
ENDDO

! Semi-Lagrangian diagnostic fields: these variables are currently used for ECMWF applications.
DO JGFL=1,NSLDIA
  LLPT = .FALSE.
  LLPC = .FALSE. 
  LLT9 = .FALSE.
  CALL SET_GFL_ATTR(YSLDIA(JGFL),LDADV=.FALSE.,LDT9=LLT9,&
   & LDPHY=YSLDIA_NL(JGFL)%LPHY,LDPT=LLPT, LDPC=LLPC,&
   & KCOUPLING=YSLDIA_NL(JGFL)%NCOUPLING,&
   & PREFVALC=YSLDIA_NL(JGFL)%REFVALC)
ENDDO

! * Forcings (FORC):
DO JGFL=1,NGFL_FORC
  CALL SUCSLINT(YFORC_NL(JGFL)%LSLHD, &
   & YFORC_NL(JGFL)%LVSPLIP,YFORC_NL(JGFL)%LHV, &
   & YFORC_NL(JGFL)%LQM,YFORC_NL(JGFL)%LQMH,YFORC_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. &
   & (YFORC_NL(JGFL)%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YFORC_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YFORC(JGFL),LDADV=YFORC_NL(JGFL)%LADV,LDT9=LLT9,&
   & LDPHY=YFORC_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YFORC_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YFORC_NL(JGFL)%REFVALC)
ENDDO

! * Easy diagnostics (EZDIAG):
DO JGFL=1,NGFL_EZDIAG
  CALL SUCSLINT(YEZDIAG_NL(JGFL)%LSLHD, &
   & YEZDIAG_NL(JGFL)%LVSPLIP,YEZDIAG_NL(JGFL)%LHV, &
   & YEZDIAG_NL(JGFL)%LQM,YEZDIAG_NL(JGFL)%LQMH,YEZDIAG_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. &
   & (YEZDIAG_NL(JGFL)%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YEZDIAG_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YEZDIAG(JGFL),LDADV=YEZDIAG_NL(JGFL)%LADV,LDT9=.FALSE.,&
   & LDPHY=YEZDIAG_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YEZDIAG_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT,&
   & PREFVALC=YEZDIAG_NL(JGFL)%REFVALC)
ENDDO

! * Extra-GFL variables:
DO JGFL=1,NGFL_EXT
  CALL SUCSLINT(YEXT_NL(JGFL)%LSLHD, &
   & YEXT_NL(JGFL)%LVSPLIP,YEXT_NL(JGFL)%LHV, &
   & YEXT_NL(JGFL)%LQM,YEXT_NL(JGFL)%LQMH,YEXT_NL(JGFL)%LINTLIN,CLSLINT)
  LLPT=LPC_FULL .AND. LLDIAB .AND. LSLAG .AND. &
   & (YEXT_NL(JGFL)%LPT.OR.NEPHYS_PCFULL==2)
  LLPC = LLPC0.AND.YEXT_NL(JGFL)%LPC
  CALL SET_GFL_ATTR(YEXT(JGFL),LDADV=YEXT_NL(JGFL)%LADV,&
   & LDPHYLIN=YEXT_NL(JGFL)%LPHYLIN,&
   & LDHORTURB=YEXT_NL(JGFL)%LHORTURB,LDT9=LLT9,&
   & LDPHY=YEXT_NL(JGFL)%LPHY,LDPT=LLPT,LDPC=LLPC,&
   & KCOUPLING=YEXT_NL(JGFL)%NCOUPLING,CDSLINT=CLSLINT, &
   & PREFVALC=YEXT_NL(JGFL)%REFVALC)
  LLPHYLIN=LLPHYLIN.OR.YEXT_NL(JGFL)%LPHYLIN
ENDDO

!     ------------------------------------------------------------------

!*       2.    PRINT GFL ATTRIBUTES
!              --------------------

CALL PRINT_GFL

!     ------------------------------------------------------------------

!*       3.    Redefine LSPLTHOIGFL when required 
!              ----------------------------------

LSPLTHOIGFL=LLPHYLIN

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SUGFL3',1,ZHOOK_HANDLE)
END SUBROUTINE SUGFL3
